#
# this module look for HDF5 (http://hdf.ncsa.uiuc.edu) support
# it will define the following values
#
# HDF5_INCLUDE_PATH = where hdf5.h can be found
# HDF5_LIBRARY      = the library to link against (hdf5 etc)
#

#SET(HDF5_DIR $ENV{HDF5_DIR})

SET(HDF5_LIBRARY_PATHS
  /usr/lib 
  /opt/lib
  /usr/local/lib
  /sw/lib
  lib
)

SET(HDF5_INCLUDE_PATHS
  /usr/include
  /opt/include
  /usr/local/include
  /sw/include
  include
)

IF($ENV{HDF5_DIR} MATCHES "hdf")
  MESSAGE(STATUS "Using environment variable HDF5_DIR.")
  SET(HDF5_LIBRARY_PATHS ${HDF5_LIBRARY_PATHS} $ENV{HDF5_DIR}/lib)
  SET(HDF5_INCLUDE_PATHS ${HDF5_INCLUDE_PATHS} $ENV{HDF5_DIR}/include)
ENDIF($ENV{HDF5_DIR} MATCHES "hdf")

FIND_LIBRARY(HDF5_LIBRARY hdf5 ${HDF5_LIBRARY_PATHS})
FIND_PATH(HDF5_INCLUDE_PATH hdf5.h ${HDF5_INCLUDE_PATHS} )

IF(NOT HDF5_LIBRARY)
  MESSAGE(STATUS "hdf5 is not found in pre-defined paths.")
  MESSAGE(STATUS "We are using the version packaged with OHMMS instead.")
  MESSAGE(STATUS "NOTE!!! Don't worry.  We are not installing anything")
  MESSAGE(STATUS "to anywhere except to the OHMMS project directory.")
  IF(UNIX)
    IF(RUN_CONFIGURE)
      EXEC_PROGRAM(
       "${AUXPACKAGES}/hdf5/configure --prefix=${PROJECT_BINARY_DIR} --with-cc=${CMAKE _CC_COMPILER}"
        ${AUXPACKAGES}/hdf5
      )
      EXEC_PROGRAM(make ${AUXPACKAGES}/hdf5)
      EXEC_PROGRAM(make ${AUXPACKAGES}/hdf5 ARGS install)
      FIND_LIBRARY(HDF5_LIBRARY hdf5 ${PROJECT_BINARY_DIR}/lib)
      FIND_PATH(HDF5_INCLUDE_PATH H5Ipublic.h ${PROJECT_BINARY_DIR}/include)
    ENDIF(RUN_CONFIGURE)
  ENDIF(UNIX)
ENDIF(NOT HDF5_LIBRARY)

IF(HDF5_LIBRARY)
  MESSAGE(STATUS "Found HDF5")
  INCLUDE_DIRECTORIES(${HDF5_INCLUDE_PATH})
  SET(HDF5_LIBRARY ${HDF5_LIBRARY})
  SET(HAVE_LIBHDF5 1)
ENDIF(HDF5_LIBRARY) 

#LINK_LIBRARIES(${HDF5_LIBRARY})
