#!/usr/bin/env perl
#//////////////////////////////////////////////////////////////////
#// (c) Copyright 1998-2002 by Jeongnim Kim
#//////////////////////////////////////////////////////////////////
#//////////////////////////////////////////////////////////////////
#//   Jeongnim Kim
#//   National Center for Supercomputing Applications &
#//   Materials Computation Center
#//   University of Illinois, Urbana-Champaign
#//   Urbana, IL 61801
#//   e-mail: jnkim@ncsa.uiuc.edu
#//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
#//////////////////////////////////////////////////////////////////
####################
#Usgae: bandplot.pl --in file-name --begin nbi --end nbf --xmgr xmgrfile
#plot bandstructures [nbi,nbf) for a given set of k-points
####################
#declare data
####################
@ene = (); # energies to be plotted
$nkpts=0;
$nstates=0;
$efermi = 0.0;

$helpnm = "-h";	   	       # print out help message and exit
$infilenm = "--in";
$outfilenm = "--out";
$xmgrnm = "--xmgr";
$beginnm = "--begin";
$endnm = "--end";
$plotnm = "--plot";

## setting default values
$infile = "";
$outfile = "band.agr";
$plotengine = "xmgrace";
$xmgrpar = "bandXmgr.par";
$bstart = 0; #starting from the lowest band
$bend = -1;  #all the input bands

### value specified will be used.
@arglist = (
  [$infilenm,"<file>",  "an input file to be processed."],
  [$outfilenm,"<file>", "an output file [default=$outfile]."],
  [$beginnm,"<int>", "start-index for [start,end) to be plotted [default=0]."],
  [$endnm,"<int>", "end-index for [start,end) to be plotted [default=-1]."],
  [$xmgrnm,"<file>","using a Xmgr parameter file [default=bandXmgr.par]."],
  [$plotnm,"<pakcage>","plot using a plot pacakge[default=xmgrace]."],
  [$helpnm,"",         "display this help message."],
);

### create a hash of the arguments, where we will store results.  Each
### argument entry has an array of scalars, where the first scalar is
### the name of the required argument (if any), and the
### rest of the list is the set of values for the argument.
%arghash = ();
for $i (0 .. $#arglist)
{
  my @arginfo = @{$arglist[$i]};
  $arghash{$arginfo[0]} = [ $arginfo[1] ];
}

### a procedure to print an error message and quit
sub printerror
{
  if (@_)
    {
      print "\nERROR: @_\n\n";
    }
  printusage;
}

### a procedure which will parse the command-line arguments and set the
### values in arglist
sub parseargs
{
  my @argvlist = @ARGV;
  while (@argvlist)
    {
      my $arg = $argvlist[0];
      if (exists $arghash{$arg})
	{
	  my $argval = 1;
	  if ($arghash{$arg}[0])
	    {
	      shift @argvlist;
	      if (@argvlist)
		{
		  $argval = $argvlist[0];
		}
	      else
		{
		  printerror "You must provide a value for $arghash{$arg}[1]";
		}
	    }
	  print "Found argument $arg, value $argval.\n" if $dbgprnt;
	  push @{$arghash{$arg}}, $argval;
	}
      else
	{
	  printerror "Unknown command-line argument \"$arg\".";
	}
      shift @argvlist;
    }
}

#################################################33
#print usage with "-h" option 
#################################################33
sub printusage
{
  print "Usage: bandplot.pl \n";
  for $i (0 .. $#arglist)
  {
    my @arginfo = @{$arglist[$i]};
    my $outstring = $arginfo[0];
    my $desc = $arginfo[2];
    if ($arginfo[1])
    {
      $outstring .= " $arginfo[1]";
    }
    my $dots = 77 - (length $outstring) - (length $desc);
    print $outstring . " " . "." x $dots . " $desc\n";
  }

  print "\n";
  exit(1);
}


#################################################33
### process the arguments
#################################################33
sub setvariables
{
  #get the input file
  if (scalar @{$arghash{$infilenm}} > 1)
    {
      $infile = $arghash{$infilenm}[1];
    }

  #get the output file
  if (scalar @{$arghash{$outfilenm}} > 1)
    {
      $outfile = $arghash{$outfilenm}[1];
    }

  #get the Xmgr parameter file
  if (scalar @{$arghash{$xmgrnm}} > 1)
    {
      $outfile = $arghash{$xmgrnm}[1];
    }

  # reset names of compilers and applications, if necessary
  if (scalar @{$arghash{$beginnm}} > 1)
    {
	$bstart = $arghash{$beginnm}[1];
    }
  # reset names of compilers and applications, if necessary
  if (scalar @{$arghash{$endnm}} > 1)
    {
	$bend = $arghash{$endnm}[1];
    }

}

########################################
# report to screen
########################################
sub reportSTD 
{
    printf("Input File  = $infile\n");
    printf("Output File = $outfile\n");
    printf("Band        = [$bstart,$bend)\n");
    printf("number of kpoints = $nkpts\n");
    printf("number of states = $nstates\n");
}

#################################################33
# read data from a file
#################################################33
sub readData
{
    open(MyFile,$infile);

    my $toread = 1;
    while($toread) {
	$line = <MyFile>;
	$where = index($line,"fermi");
	if($where >= 0) {
	    @nw = split(/\s+/,$line);
	    $efermi = $nw[1];
	}
	$where = index($line,"eigvalues");
	if($where >= 0) {
	    $toread = 0;
	}
    }

    @nw = split(/\s+/,$line);
    foreach $a (@nw) {
      $where = index($a,"nkpts");
      if($where >=0) {
        ($dump,$nkpts) = split('=',$a);
      } 
     $where = index($a,"nstates");
     if($where >=0) {
       ($dump,$nstates) = split('[=<>]',$a);
     } 
    }

    for($ik = 0; $ik<$nkpts; $ik++) {
	$line = <MyFile>;
	chop($line);
	push @ene,split(/\s+/,$line);	
    }
    $size = @ene;
}

#################################################33
#create a Xmgrace file using a template file
#################################################33
sub saveXmgr
{

    if($bend < $bstart) {
	$bend = $nstates;#overwrite the bend
    }

    system("cp -r $xmgrtemp $outfile");
    open(MFILE,">>$outfile") or die "Cannot open $mfile for write.\n";
    my $di = 1.0/($nkpts-1.0);
    my $ibc = 0;
    for($ib = $bstart; $ib<$bend; $ib++,$ibc++)
    {
	print MFILE "\@target G0.S$ibc\n";
	print MFILE "\@type xy\n";
	my $k= 0.0;
	my $i = $ib;
	for($ik = 0; $ik<$nkpts; $ik++) 
	{
	    print MFILE "$k $ene[$i]\n";
	    $i += $nstates;
	    $k += $di;
	}
	print MFILE "&\n";
    }
}


#################################################33
#create a Xmgrace file using a template file
#################################################33
sub plotXmgr
{

    if($bend < $bstart) {
	$bend = $nstates;#overwrite the bend
    }

    open(MFILE,">$outfile") or die "Cannot open $mfile for write.\n";
    my $k= 0.0;
    my $dk = 1.0/($nkpts-1.0);
    for($ik = 0; $ik<$nkpts; $ik++) 
    {
	print MFILE "$k ";

	$i = $ik*$nstates+$bstart;
	for($ib = $bstart; $ib<$bend; $ib++, $i++) {
	    $val = $ene[$i]-$efermi;
	    print MFILE "$val ";	
	}
	print MFILE "\n";
	$k += $dk;
    }

    print("Plot : xmgrace -mono -param $xmgrpar -nxy $outfile\n");
    system("xmgrace -mono -param $xmgrpar -nxy $outfile");
}


#######################################################################
foreach $a (@ARGV)
  {
      if ($a eq $helpnm or $a eq $questionnm)
      {
	printusage;
      }
  }


### parse the command-line arguments
parseargs;
setvariables;
readData;
reportSTD;
plotXmgr;
