// -*- C++ -*-

////////////////////////////////////////////////////////////////////////////
// Do experiment by removing add for each block.
////////////////////////////////////////////////////////////////////////////

#include <vector>
#include <iostream>
#include <fstream>
#include <iomanip>
using namespace std;
#include "Utilities/SimpleParser.h"
#include "ScaleFunctors/NRLScaleFunction.h"

const double bohr    = 0.529177e0; // AA
const double bohrinv = 1.0/bohr;
const double bohrinv2 = bohrinv*bohrinv;
const double bohrinv3 = bohrinv2*bohrinv;
const double rydberg  = 13.6058e0;  // eV
const double rydbergoverbohr  = rydberg/bohr;
const double rydbergoverbohr2 = rydberg/bohr/bohr;

// convert NRL format to OHMMS format
// overlap = old (or metal)
void print_old(ostream& os1, ostream& os2,
               double he0, double hf0, double hfb0, double hg0,
               double se0, double sf0, double sfb0, double sg0,
               double rcut, double screen, 
               double ri = 1.0, double del = 0.01){

  double he  = he0*rydberg;
  double hf  = hf0*rydbergoverbohr;
  double hfb = hfb0*rydbergoverbohr2;
  double hg  = hg0*hg0*bohrinv;
  double se  = se0;
  double sf  = sf0*bohrinv;
  double sfb = sfb0*bohrinv2;
  double sg  = sg0*sg0*bohrinv;

  NRLScaleFunction<double> h(he,hf,hfb,hg,rcut,screen);
  NRLScaleFunction<double> s(se,sf,sfb,sg,rcut,screen);

  os1 << setw(20) << setprecision(12) << he
      << setw(20) << setprecision(12) << hf
      << setw(20) << setprecision(12) << hfb
      << setw(20) << setprecision(12) << hg << endl;
  os1 << setw(20) << setprecision(12) << se
      << setw(20) << setprecision(12) << sf
      << setw(20) << setprecision(12) << sfb
      << setw(20) << setprecision(12) << sg << endl;

  double xxx= ri;
  double dx = del;
  while(xxx<=h.Rcut) {
    os2 << setw(20) << xxx << setw(20) << h(xxx)  << setw(20) << s(xxx) << endl;
    xxx+= dx;
  }          
}

// convert NRL format to OHMMS format
// overlap = old (or metal)
void print_new(ostream& os1, ostream& os2,
               double he0, double hf0, double hfb0, double hg0,
               double d0, double se0, double sf0, double sfb0, double sg0,
               double rcut, double screen, 
               double ri = 1.0, double del = 0.01){

  double he  = he0*rydberg;
  double hf  = hf0*rydbergoverbohr;
  double hfb = hfb0*rydbergoverbohr2;
  double hg  = hg0*hg0*bohrinv;
  double se  = se0*bohrinv;
  double sf  = sf0*bohrinv2;
  double sfb = sfb0*bohrinv3;
  double sg  = sg0*sg0*bohrinv;

  NRLScaleFunction<double> h(he,hf,hfb,hg,rcut,screen);
  NRLScaleFunction2<double> s(d0,se,sf,sfb,sg,rcut,screen);

  os1 << setw(20) << setprecision(12) << he
      << setw(20) << setprecision(12) << hf
      << setw(20) << setprecision(12) << hfb
      << setw(20) << setprecision(12) << hg << endl;
  os1 << setw(20) << setprecision(12) << se
      << setw(20) << setprecision(12) << sf
      << setw(20) << setprecision(12) << sfb
      << setw(20) << setprecision(12) << sg << endl;

  double xxx= ri;
  double dx = del;
  while(xxx<=h.Rcut) {
    os2 << setw(20) << xxx << setw(20) << h(xxx)  << setw(20) << s(xxx) << endl;
    xxx+= dx;
  }          
}

int main(int argc, char** argv) {

  typedef double T;
  double ri = 1.0;
  double del = 0.01;

  if(argc<2) {
    cout << "usage : convertnrltb.x filename [ri AA dr AA] " << endl;
    cout << "filemame = file containing TB Parameters in NRL format" << endl;
    cout << "ri = starting distance for spline data " << endl;
    cout << "dr = distance strided  for spline data " << endl;
    cout << "mv TBPARAM.NRL TBPARAM.Species.NRL " << endl;
    cout << "mv TBPARAM.NRL TBPARAM.Species.NRL.Spline " << endl;
    return 1;
  }

  if(argc > 2) {
    int ic=0;
    while(ic<argc) {
      if(!strcmp(argv[ic],"ri")) {
        ri = atof(argv[++ic]);
      } else if(!strcmp(argv[ic],"dr")) {
        del = atof(argv[++ic]);
      }
      ic++;
    }
  } 

  vector<T> val(100);
  ifstream fin(argv[1]);
  ofstream tb0("TBPARAM.NRL");
  ofstream tb1("TBPARAM.NRL.Spline");
  tb0.setf(ios::scientific, ios::floatfield);
  tb1.setf(ios::scientific, ios::floatfield);

  vector<string> slist;
  getwords(slist,fin);//format
  bool oldformat = true;
  if(slist[0] == "NN00001") {
    oldformat = false;
  } 

  tb0 << "<SK>" << endl << slist[0] << endl;
  tb1 << "<SK>" << endl << slist[0] << endl;
  getwords(slist,fin);//species
  char species[2];  species[0] = slist[0][0]; species[1] = slist[0][1];

  getwords(slist,fin);//number of species
  getwords(slist,fin);//rcut and screenl 
  T rcut =atof(slist[0].c_str())*bohr; 
  T screenl = atof(slist[1].c_str())*bohr;

  getwords(slist,fin);
  getwords(slist,fin);
  T mass = atof(slist[0].c_str());
  getwords(slist,fin);
  T e = atof(slist[0].c_str())+atof(slist[1].c_str())+atof(slist[2].c_str());
  
  tb0 << "species = " << species << " -m "<< mass << " -q " << e << " -L 0 1 2 " << endl;
  tb1 << "species = " << species << " -m "<< mass << " -q " << e << " -L 0 1 2 " << endl;
  tb0 << "bond " << species << " " << species << endl;
  tb1 << "bond " << species << " " << species << endl;

  tb0 << "rcut    ="  << setw(20) << setprecision(12) << rcut << " AA " << endl;
  tb0 << "screenl =" << setw(20) << setprecision(12)  << screenl << " AA " << endl;
  tb1 << "rcut    ="  << setw(20) << setprecision(12) << rcut << " AA " << endl;
  tb1 << "screenl =" << setw(20) << setprecision(12)  << screenl << " AA " << endl;
  int np = static_cast<int>((rcut-ri)/del)+1; // number of spline point


  for(int i=0; i<97; i++) {
    getwords(slist,fin);
    val[i] = atof(slist[0].c_str());
  }

  tb0 << "lambda  = " << val[0]*val[0]/bohr << " 1/AA " <<  endl; // lambda = 1/au
  tb1 << "lambda  = " << val[0]*val[0]/bohr << " 1/AA " <<  endl; // lambda = 1/au

  tb0 << "0 0 -1 0 0 "  << " # s oniste" << endl
       << setw(20) << setprecision(12) << val[1]*rydberg 
       << setw(20) << setprecision(12) << val[2]*rydberg 
       << setw(20) << setprecision(12) << val[3]*rydberg
       << setw(20) << setprecision(12) << val[4]*rydberg << endl;
  tb1 << "0 0 -1 0 0 " << " # s onsite " << endl
       << setw(20) << setprecision(12) << val[1]*rydberg 
       << setw(20) << setprecision(12) << val[2]*rydberg 
       << setw(20) << setprecision(12) << val[3]*rydberg
       << setw(20) << setprecision(12) << val[4]*rydberg << endl;

  tb0 << "0 0  0 0 0 " << "#ss-sig" << endl;
  tb1 << "0 0  0 0 0 " << np <<  " #ss-sig" << endl;
  if(oldformat) 
    print_old(tb0,tb1,val[17],val[18],val[19],val[20],val[57],val[58],val[59],val[60],rcut,screenl,ri,del);
  else
    print_new(tb0,tb1, val[17],val[18],val[19], val[20], 
              1.0, val[57],val[58],val[59],val[60],
              rcut,screenl,ri,del);

  tb0 << "0 1  0 0 1 " << "#sp-sig" << endl;
  tb1 << "0 1  0 0 1 " << np << " #sp-sig" << endl;
  if(oldformat) 
    print_old(tb0,tb1,val[21],val[22],val[23],val[24],val[61],val[62],val[63],val[64],rcut,screenl,ri,del);
  else
    print_new(tb0,tb1,val[21],val[22],val[23],val[24],
              0.0,val[61],val[62],val[63],val[64],rcut,screenl,ri,del);

  tb0 << "1 1 -1 1 1 "  << " # p onsite " << endl
       << setw(20) << setprecision(12) << val[5]*rydberg 
       << setw(20) << setprecision(12) << val[6]*rydberg 
       << setw(20) << setprecision(12) << val[7]*rydberg
       << setw(20) << setprecision(12) << val[8]*rydberg << endl;
  tb1 << "1 1 -1 1 1 " << " # p onsite " << endl
       << setw(20) << setprecision(12) << val[5]*rydberg 
       << setw(20) << setprecision(12) << val[6]*rydberg 
       << setw(20) << setprecision(12) << val[7]*rydberg
       << setw(20) << setprecision(12) << val[8]*rydberg << endl;

  tb0 << "1 1  0 1 1 " << "#pp-sig" << endl;
  tb1 << "1 1  0 1 1 " << np <<  " #pp-sig" << endl;
  if(oldformat) 
    print_old(tb0,tb1,val[25],val[26],val[27],val[28],val[65],val[66],val[67],val[68],rcut,screenl,ri,del);
  else
    print_new(tb0,tb1,val[25],val[26],val[27],val[28],
              1.0,val[65],val[66],val[67],val[68],rcut,screenl,ri,del);

  tb0 << "1 1 1 1 1 " << "#pp-pi" << endl;
  tb1 << "1 1 1 1 1 " << np << " #pp-pi" << endl;
  if(oldformat) 
    print_old(tb0,tb1,val[29],val[30],val[31],val[32],val[69],val[70],val[71],val[72],rcut,screenl,ri,del);
  else
    print_new(tb0,tb1,val[29],val[30],val[31],val[32],
              1.0,val[69],val[70],val[71],val[72],rcut,screenl,ri,del);

  tb0 << "0 2 0 0 4 " << "#sd-sig" << endl;
  tb1 << "0 2 0 0 4 " << np << " #sd-sig" << endl;
  if(oldformat) 
    print_old(tb0,tb1,val[33],val[34],val[35],val[36],val[73],val[74],val[75],val[76],rcut,screenl,ri,del);
  else
    print_new(tb0,tb1,val[33],val[34],val[35],val[36],
              0.0,val[73],val[74],val[75],val[76],rcut,screenl,ri,del);

  tb0 << "1 2 0 1 4 " << "#pd-sig" << endl;
  tb1 << "1 2 0 1 4 " << np << " #pd-sig" << endl;
  if(oldformat) 
    print_old(tb0,tb1,val[37],val[38],val[39],val[40],val[77],val[78],val[79],val[80],rcut,screenl,ri,del);
  else
    print_new(tb0,tb1,val[37],val[38],val[39],val[40],
              0.0,val[77],val[78],val[79],val[80],rcut,screenl,ri,del);

  tb0 << "1 2 1 1 4 " << "#pd-pi" << endl;
  tb1 << "1 2 1 1 4 " << np << " #pd-pi" << endl;
  if(oldformat) 
    print_old(tb0,tb1,val[41],val[42],val[43],val[44],val[81],val[82],val[83],val[84],rcut,screenl,ri,del);
  else
    print_new(tb0,tb1,val[41],val[42],val[43],val[44],
              0.0,val[81],val[82],val[83],val[84],rcut,screenl,ri,del);

  tb0 << "2 2 -1 4 4 " << " # d onsite " << endl
       << setw(20) << setprecision(12) << val[9]*rydberg 
       << setw(20) << setprecision(12) << val[10]*rydberg 
       << setw(20) << setprecision(12) << val[11]*rydberg
       << setw(20) << setprecision(12) << val[12]*rydberg << endl;
  tb1 << "2 2 -1 4 4 " << " # d onsite " << endl
       << setw(20) << setprecision(12) << val[9]*rydberg 
       << setw(20) << setprecision(12) << val[10]*rydberg 
       << setw(20) << setprecision(12) << val[11]*rydberg
       << setw(20) << setprecision(12) << val[12]*rydberg << endl;

  tb0 << "2 2 0 4 4 " << "#dd-sig" << endl;
  tb1 << "2 2 0 4 4 " << np << " #dd-sig" << endl;
  if(oldformat) 
    print_old(tb0,tb1,val[45],val[46],val[47],val[48],val[85],val[86],val[87],val[88],rcut,screenl,ri,del);
  else
    print_new(tb0,tb1,val[45],val[46],val[47],val[48],
              1.0,val[85],val[86],val[87],val[88],rcut,screenl,ri,del);

  tb0 << "2 2 1 4 4 " << "#dd-pi" << endl;
  tb1 << "2 2 1 4 4 " << np << " #dd-pi" << endl;
  if(oldformat) 
    print_old(tb0,tb1,val[49],val[50],val[51],val[52],val[89],val[90],val[91],val[92],rcut,screenl,ri,del);
  else
    print_new(tb0,tb1,val[49],val[50],val[51],val[52],
              1.0,val[89],val[90],val[91],val[92],rcut,screenl,ri,del);

  tb0 << "2 2 2 4 4 " << "#dd-del" << endl;
  tb1 << "2 2 2 4 4 " << np <<  " #dd-del" << endl;
  if(oldformat) 
    print_old(tb0,tb1,val[53],val[54],val[55],val[56],val[93],val[94],val[95],val[96],rcut,screenl,ri,del);
  else
    print_new(tb0,tb1,val[53],val[54],val[55],val[56],
              1.0,val[93],val[94],val[95],val[96],rcut,screenl,ri,del);

  tb0 << "</SK>" << endl;
  tb1 << "</SK>" << endl;

  return 0;
}

  /***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: convertnrltb.cpp 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/

