#!/usr/bin/env perl
# convert from NRL to OHMMS format
# units are converted to eV and AA
# only single species
$i=0;
$iv = 0;
$ia = 0;
$rydberg = 13.6058;
$bohr = 0.529177;
$bohrinv = 1.0/$bohr;
$zero = 0.0;

while(<>) {
chop($_);
@pos = split(' ');
if($i == 0) {
    $nrltype=$pos[0];
}
if($i == 1) {
    $speciesname = $pos[1];
    $speciesname =~ s/\W(\w+)\W/\1/
}
if($i == 2) {
    $nspecies = $pos[0];
} 
if($i == 3) {
    $rcut = $pos[0]*$bohr;
    $screen = $pos[1]*$bohr;
} 
if($i == 4) {
    $norb = $pos[0];
} 
if($i == 5) {
    $mass = $pos[0];
}
if($i == 6) {
    $valence = $pos[0] + $pos[1] + $pos[2];
}
if($i == 7) {
    $lam = $pos[0]*$pos[0]*$bohrinv;
}
if($i > 7 && $i<24) {
    $ax[$ia] = $pos[0]*$rydberg;
    $ia++;
}
if($i>=24) {
  if($iv<40) {
      $h[$iv] = $pos[0];
  } else {
      $s[$iv-40] = $pos[0];
  }
    $iv++;
}
$i++;
}

print "\#TBPARAM for NRL potential\n";
print "\#Converted for OHMMS package\n";
print "\#Generated by nrltb-dallas.ev.pl written by Jeongnim Kim / editted by Dallas Trinkle\n";

$hu[0] = $rydberg;
$hu[1] = $rydberg*$bohrinv;
$hu[2] = $rydberg*$bohrinv*$bohrinv; 
$hu[3] = $bohrinv;
if($nrltype eq "NN00001") {
    $su[0] = $bohrinv;
    $su[1] = $bohrinv*$bohrinv;
    $su[2] = $bohrinv*$bohrinv*$bohrinv;
} else {
    $su[0] = 1.0;
    $su[1] = $bohrinv;
    $su[2] = $bohrinv*$bohrinv;
}
$su[3] = $bohrinv;

# Determine if there's crystal-field splitting
# (i.e., are the t2g and eg elements the same?)
$crystalsplit = ($norb == 9) && (($ax[8] != $ax[12]) || ($ax[9] != $ax[13]) || ($ax[10] != $ax[14]) || ($ax[11] != $ax[15])); 

#Output routines:

#species
printf "<species>\n";
printf "<name> $speciesname </name>\n";
printf "<mass> $mass </mass>\n";
printf "<valence> $valence </valence>\n";
printf "<orbital> 0 </orbital>\n";
printf "<orbital> 1 </orbital>\n";
    if($norb == 9) {
	printf "<orbital> 2 </orbital>\n";
    }
printf "</species>\n";

#onsite terms are separated 
printf "<onsite name=\"nrl\">\n";
printf "bond $speciesname $speciesname\n";
printf "rcut    =  $rcut\n";
printf "screenl =  $screen\n";
printf "lambda  =  $lam\n";
if($norb == 9) {
  if ($crystalsplit) {
    printf "density 4\n";
  } else {
    printf "density 3\n";
  }
} else {
  printf "density 2\n";
}
printf ("0 0 %20.12e %20.12e %20.12e %20.12e\n", $zero, $ax[1], $ax[2], $ax[3]);
printf ("1 1 %20.12e %20.12e %20.12e %20.12e\n", $zero, $ax[5], $ax[6], $ax[7]);

if($norb == 9) {
    printf ("2 4 %20.12e %20.12e %20.12e %20.12e\n", $zero, $ax[9], $ax[10], $ax[11]);
    if ($crystalsplit) {
      $crystal = $ax[12]-$ax[8];
      printf ("2 6 %20.12e %20.12e %20.12e %20.12e\n", $crystal, $ax[13], $ax[14], $ax[15]);
    }
}
printf "</onsite>\n";

#slater-koster parameters start here
printf "<TBPARAM name=\"nrl\">\n";
printf "$nrltype\n";
printf "bond $speciesname $speciesname\n";
printf "rcut     =  $rcut\n";
printf "screenl  =  $screen\n";

$ic=0;
sub print_hs
{
    my $va, $vb, $vc, $vd;
    my $sa, $sb, $sc, $sd;
    $va = $h[$ic]*$hu[0]; 
    $sa = $s[$ic]*$su[0];
    $ic++;
    $vb = $h[$ic]*$hu[1]; 
    $sb = $s[$ic]*$su[1];
    $ic++;
    $vc = $h[$ic]*$hu[2]; 
    $sc = $s[$ic]*$su[2];
    $ic++;
    $vd = $h[$ic]*$h[$ic]*$hu[3]; 
    $sd = $s[$ic]*$s[$ic]*$su[3];
    $ic++;
    printf ("%20.12e %20.12e %20.12e %20.12e\n", $va, $vb, $vc, $vd);
    printf ("%20.12e %20.12e %20.12e %20.12e\n", $sa, $sb, $sc, $sd);
}
printf ("0 0 -1 0 0 %20.12e \#s-onsite\n", $ax[0]);
printf "0 0 0 0 0 \#ss-sig\n";
print_hs;
printf "0 1 0 0 1 \#sp-sig\n";
print_hs;

printf ("1 1 -1 1 1 %20.12e \#p-onsite\n", $ax[4]);
printf "1 1 0 1 1 \#pp-sig\n";
print_hs;


printf "1 1 1 1 1 \#pp-pi\n";
print_hs;

if($norb == 9) {
    printf "0 2 0 0 4 \#sd-sig\n";
    print_hs;

    printf "1 2 0 1 4 \#pd-sig\n";
    print_hs;

    printf "1 2 1 1 4 \#pd-pi\n";
    print_hs;

    printf ("2 2 -1 4 4 %20.12e \#d-onsite\n", $ax[8]);

    printf "2 2 0 4 4 \#dd-sig\n";
    print_hs;

    printf "2 2 1 4 4 \#dd-pi\n";
    print_hs;

    printf "2 2 2 4 4 \#dd-del\n";
    print_hs;
}

printf "</TBPARAM>\n";
