//////////////////////////////////////////////////////////////////
// (c) Copyright 2003- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "Particle/OhmmsParticle.h"
#include "Message/Communicate.h"
#include "Particle/BulkSystemBuilder.h"
#include "Particle/NNEngine.h"
#include "Particle/NNRegGridEngine.h"
#include "ParticleIO/XMLParticleIO.h"
#include "Utilities/SpeciesCollection.h"
using namespace OHMMS;

int main(int argc, char** argv) {

  Controller->initialize(argc,argv);
  OhmmsInfo ohmms(argc,argv,Controller->mycontext());

  if(argc<2) {
    CreateBulkSystem::printUsage(cout);
    return 1;
  }

  //====================================================
  //create a Particle Set for torque
  //====================================================
  Particle_t sys;

  //====================================================
  //create ParticleAttribute for torque
  //====================================================
  Particle_t::ParticlePos_t Torque(POSSTR,"torque");
  typedef Particle_t::Tensor_t Tensor_t;
  ParticleAttrib<Tensor_t> MassTensor("TensorType","mz");
  sys.addAttribute(Torque);
  sys.addAttribute(MassTensor);

  //====================================================
  //initialize OhmmsParticle using the options
  //====================================================
  CreateBulkSystem::apply(&sys,argc,argv);
   
  //====================================================
  //get pair list with a negative interaction radius
  //====================================================
  sys.getPairlist(3.0);
  sys.getNNEngine()->get(cout);

  //====================================================
  //Get a reference to SpeciesBase to access mass
  //====================================================
  SpeciesBase& species = *(SpeciesCollection::getSpecies());

  //====================================================
  //Add C(arbon) and Si(licon) to SpeciesCollection
  //Typically, species are added by an input configuration file or interatomic potentials
  //====================================================
  SpeciesCollection::addSpecies("C");
  SpeciesCollection::addSpecies("Si");

  //save the index of an  attribute with the name of "mass"
  int imass = SpeciesCollection::addAttrib("mass");
  for(int i=0; i<species.getTotalNum(); i++) species(imass,i) = 12.0;

  ///assign a diagonal tensor as the inverse of mass
  for(int i=0; i<sys.getLocalNum(); i++) {
    Scalar_t massinv = 1.0/species(imass,sys.IonID[i]);
    MassTensor[i].diagonal(massinv);
  }

  //====================================================
  //write Particle in xml-format
  //====================================================
  XMLSaveParticle ptclsave(sys);
  ptclsave.reset("bulk");
  ptclsave.report(0);

  return 0;
}  
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: mdsim.cpp 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
  
