//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "Utilities/OhmmsInfo.h"
#include "AppBase/TransformParticle.h"
#include "AppBase/DistributeParticle.h"
#ifdef HAVE_METIS
#include "Particle/MetisGraphPartition.h"
#endif
#include "Message/OpenMP.h"

namespace OHMMS {

  bool DistributeParticle::put(xmlNodePtr cur) {

#ifdef HAVE_METIS
    typedef Particle_t::ParticleLayout_t      ParticleLayout_t;
    typedef Particle_t::ParticleIndex_t       ParticleIndex_t;
    typedef Particle_t::ParticlePos_t         ParticlePos_t;
    typedef Particle_t::SingleParticleIndex_t SingleParticleIndex_t;
    typedef Particle_t::SingleParticlePos_t   SingleParticlePos_t;
    typedef Particle_t::Scalar_t              Scalar_t;

    int npart = omp_get_max_threads();

    Scalar_t rmax=3.0;
    const int mpigrid  = ParticleLayout_t::MPI_GRID;
    const int ompgrid  = ParticleLayout_t::OMP_GRID;
    const int finegrid = ParticleLayout_t::SPATIAL_GRID;
    const int gl       = ParticleLayout_t::GridLevel;
    vector<SingleParticleIndex_t> grid(gl,SingleParticleIndex_t(1));

    ParticleLayout_t& lattice = in_.Lattice;
    for(int m=0; m<gl; m++) grid[m]=lattice.Grid[m];

    cur = cur->xmlChildrenNode;
    while (cur != NULL) {
      if(!xmlStrcmp(cur->name, (const xmlChar *)"parameter")) {
        string aname = (const char*)(xmlGetProp(cur, (const xmlChar *) "name"));
	if(aname == "rcut") {
	  putContent(rmax,cur);
	}  else if(aname == "grid") {
	  putContent(grid[finegrid],cur);
	} else if(aname == "ompgrid") {
	  putContent(grid[ompgrid],cur);
	} else if(aname == "mppgrid") {
	  putContent(grid[mpigrid],cur);
	} 
      }
      cur=cur->next;
    }

    //int npart = grid[ompgrid][0]*grid[ompgrid][1]*grid[ompgrid][2];
    lattice.makeGrid(grid);

    if(npart==1) return true;

    int MaxConnectedCells = lattice.connectGrid(rmax);
    lattice.createData();
    int ng = lattice.ncontexts();

    typedef std::vector<int> IndexList_t;
    IndexList_t wgt(ng),cellid(ng);
    for(int ig=0; ig<lattice.ncontexts(); ig++)
      wgt[ig]=lattice.last_connected(ig)-lattice.first_connected(ig);
    
    MetisGraphPartition<IndexList_t> graph(npart);
    graph.partition(lattice.c_offset,
		    lattice.c_id,
		    wgt,cellid);
    
    int r_unit = in_.R.getUnit();
    
    in_.curR.setUnit(PosUnit::LatticeUnit);
    in_.Lattice.applyBC(in_.R,in_.curR);
    int nmax = in_.updateLayout();
    
    ParticleIndex_t GroupIDsave(in_.GroupID);
    in_.R.setUnit(PosUnit::LatticeUnit);
    int nloc=0;
    int cell_offset=0;

    lattice.resizeGroup(ng);

    //CellID has the map to the grid-based index to the physical index
    for(int ipart=0; ipart<npart; ipart++) {
      for(int k=0;k<graph.gSet[ipart].size(); k++,cell_offset++) {
	int ig= graph.gSet[ipart][k];
	//cellid[cell_offset]=ig;
	lattice.group(ig)=ipart;
	ParticleLayout_t::const_iterator jt = lattice.beginData(ig);
	ParticleLayout_t::const_iterator jt_end = lattice.endData(ig);
	while(jt != jt_end) {
	  int jat = *jt;
	  in_.R[nloc]=in_.curR[jat];
	  in_.GroupID[nloc]=GroupIDsave[jat];
	  in_.ID[nloc]=nloc;
	  in_.Cell[nloc]=ig;
	  jt++; nloc++;
	}
      }
    }
    
    lattice.clearData();
    for(int i=0;i<in_.getLocalNum(); i++)  lattice.addData(in_.Cell[i],i);
    
    //update curR with the reorded position 
    in_.curR = in_.R;

    //restore R to the original unit
    if(r_unit == PosUnit::CartesianUnit) {
      lattice.convert2Cart(in_.curR,in_.R);
    } 

   #endif 
    return true;
  }
}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 158 $   $Date: 2004-11-03 16:11:26 -0600 (Wed, 03 Nov 2004) $
 * $Id: DistributeParticle.cpp 158 2004-11-03 22:11:26Z jnkim $ 
 ***************************************************************************/
