//////////////////////////////////////////////////////////////////
// (c) Copyright 2003-  by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#ifndef OHMMS_OHMMSAPPBASE_H
#define OHMMS_OHMMSAPPBASE_H

#include "AppBase/OhmmsElementSet.h"

namespace OHMMS {

  /** Base class for applicaton codes using common xml vocabulrary.
   *
   * Valid xml documents should contain
   *\xmlonly
   <simulation>
     <process>
       <random/>
       <particlesets/>
       <main-app/>
     <process>
   </simulation>
   \endxmlonly
   *main-app is up to the application. 
   *For instance, OhmmsApp package can handle mdsim, nebsim, dimersim (optional) 
   *and phonon (optional) as main-app.
   */
  class OhmmsAppBase {

  public:

    ///constructor
    OhmmsAppBase(int argc, char** argv);

    ///destructor
    virtual ~OhmmsAppBase();
  
    ///parse a file
    bool parse(const char* fname);

    ///dump the current status
    virtual bool dump() = 0;

    /**initialize the content
     *
     *The main function of this functin is to add OhmmsElementBase* for specific applications.
     */
    virtual void initialize() = 0;

  protected:

    /** A set of OhmmsElementBase* corresponding any xml element within process.
     *
     *Currently, non-validating parser is used. Some nodes can be placed in several places,
     *which leads to rather messy parsing process. Tight object-to-xml mapping may be desirable.
     */
    OhmmsElementSet *NodeList;    
    xmlDocPtr m_doc;
    xmlXPathContextPtr m_context;

    bool put(xmlNodePtr cur);
  };
}
#endif
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 164 $   $Date: 2004-11-04 17:35:36 -0600 (Thu, 04 Nov 2004) $
 * $Id: OhmmsAppBase.h 164 2004-11-04 23:35:36Z jnkim $ 
 ***************************************************************************/
