//////////////////////////////////////////////////////////////////
// (c) Copyright 2003  by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "AppBase/OhmmsElementSet.h"
#include "AppBase/RandomNumberControl.h"
#include "AppBase/ProjectData.h"
#include "AppBase/ParticleSets.h"
#include "OhmmsData/RecordProperty.h"
#include "Utilities/OhmmsInfo.h"

//initialize the static data member of RecordPropertyList
std::map<string, RecordProperty*> RecordPropertyList::RegisteredRecords;

namespace OHMMS {

  OhmmsElementSet::OhmmsElementSet() {

    ///Default nodes: the order is related to the  MainKeys
    ///\warning When this order is changed, the enumeration should be also changed.
    addNode(new ProjectData);
    addNode(new RandomNumberControl);

    ParticleSets* pset = new ParticleSets;
    addNode(pset);
    //addNode(new ParticleLayoutManager(*pset));
    //addNode(new WorkingPropagator);
    //addNode(new IonicPotential);
  }

  OhmmsElementSet::~OhmmsElementSet() {
    for(int i=0; i<Nodes.size(); i++) delete Nodes[i];
  }

  OhmmsElementBase* OhmmsElementSet::getNode(const std::string& aname){
    KeyMap_t::iterator it =  Keys.find(aname);
    if(it == Keys.end()) {
      return 0;
    } else {
      return Nodes[(*it).second];
    }  
  }

  /*!\fn void OhmmsElementSet::addNode(OhmmsElementBase* o)
   *\param o an OhmmsElementBase
   *\param id unique id an OhmmsElementBase
   *\brief Add an OhmmsElementBase if it's tag is not yet used.
   */
  void OhmmsElementSet::addNode(OhmmsElementBase* o, int id) {
    KeyMap_t::iterator it =  Keys.find(o->getName());
    if(it == Keys.end()) {
      Keys[o->getName()] = Nodes.size();
      Nodes.push_back(o);
    } 
  }
  
  /*!\fn   bool OhmmsElementSet::put(xmlNodePtr cur) 
   *\param cur the root of the current xmlNode
   *\brief Process the children nodes if their tags are found.
   */
  bool OhmmsElementSet::put(xmlNodePtr cur) {
    cur = cur->xmlChildrenNode;    
    while (cur != NULL) {
      string cname((const char*)(cur->name));
      if(cname != "comment") {
        /**This is to ensure the main tags start with a lower-case char */
        //cname[0] = tolower(cname[0]);
        OhmmsElementBase* anode = getNode(cname);
        if(anode) { 
          DEBUGMSG("XML tag " << cname << " is processed")
          anode->put(cur);
        } else {
          WARNMSG("XML tag " << cname << " is not found")
        }
      }
      cur = cur->next;
    }
    return true;
  }

  ///write to a ostream
  bool OhmmsElementSet::get(std::ostream& ) const {
    return true;
  }
  
  ///read from istream
  bool OhmmsElementSet::put(std::istream& ) {
    return true;
  }

  void OhmmsElementSet::reset() {
  }
}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 354 $   $Date: 2005-07-07 21:20:49 -0500 (Thu, 07 Jul 2005) $
 * $Id: OhmmsElementSet.cpp 354 2005-07-08 02:20:49Z jnkim $ 
 ***************************************************************************/

