      subroutine bzdef(iut,b1,b2,b3,rsdir,ndim,nplane,idir)
      implicit none
c
c --- brillouin-zone definition:
c       finding the vectors whose halves defines the 1st brioullin
c       zone.  
c --- input data:
c       iut --> unit no. that output data to be written on for checking.
c       b1, b2, b3 --> recipical lattice vector.
c       ndim --> variable dimension of the array rsdir
c       rsdir --> vectors normal to the planes delimiting the bz.  the
c                 fourth component of the array define the length.
c                 the brillouin zone surface intersects each vector 
c                 listed in rsdir at the half of its length.
c       nplane --> no. of planes cutting the brillouin-zone surfaces.
c       idir(3,ndim)=integer coordinates of normals to planes
c
      integer iut,ndim,nplane
      integer idir(3,ndim)
      double precision b1(3),b2(3),b3(3),rsdir(4,ndim)

      integer i,j,ix,iy,iz,npl,nb1,nb2,nb3,ip
      double precision b1l,b2l,b3l,bl,eps,proj
      double precision bt(3)
      parameter (eps = 1.0d-6)
c --- method:
c       starting with the parallelopiped spanned by b1, b2, and b3 
c       around the origin, vectors inside a sufficiently large sphere
c       are tested to see whether the planes at 1/2 * b will further 
c       confine the 1bz.
c
      b1l = b1(1)**2 + b1(2)**2 + b1(3)**2
      b2l = b2(1)**2 + b2(2)**2 + b2(3)**2
      b3l = b3(1)**2 + b3(2)**2 + b3(3)**2
c
      bl = b1l + b2l + b3l

c     Set search ranges scaled to overall size of brillouin zone
      nb1 = int(sqrt(bl/b1l) + 1.0d-6)
      nb2 = int(sqrt(bl/b2l) + 1.0d-6)
      nb3 = int(sqrt(bl/b3l) + 1.0d-6)

c
      do 201 i = 1, 4
      do 201 j = 1, ndim
      rsdir(i,j) = 0.0d0
201   continue
      nplane = 0
c
c --- 1bz is certainly confined inside the b1, b2, and b3
c     parallelopiped.
c
      do 101 i = 1, 3
      rsdir(i,1) = b1(i)
      rsdir(i,2) = b2(i)
      rsdir(i,3) = b3(i)
101   continue
      rsdir(4,1) = b1l
      rsdir(4,2) = b2l
      rsdir(4,3) = b3l
      nplane = 3

c     Also define integer coordinates of normals
      do i=1,3
       do j=1,3
        idir(j,i)=0
       enddo
       idir(i,i)=1
      enddo
 
c     Try to get the plane normals to have positive coordinates
      do 102 ix = nb1, -nb1, -1
      do 102 iy = nb2, -nb2, -1
      do 102 iz = nb3, -nb3, -1
c
      if( ix .eq. 0 .and. iy .eq. 0 .and. iz .eq. 0) go to 102
      do 103 i = 1, 3
      bt(i) = dble(ix) * b1(i) + dble(iy) * b2(i) + dble(iz) * b3(i)
103   continue
c
c --- check whether this plane narrows down the 1st bz
c
      do 104 npl = 1, nplane
      proj = 0.5d0 * (rsdir(1,npl) * bt(1) + rsdir(2,npl) * bt(2) 
     &              + rsdir(3,npl) * bt(3)) / rsdir(4,npl)
      if(abs(proj) .gt. 0.5d0 - eps) go to 102
104   continue
c
c --- include this plane into the list
c
      nplane = nplane + 1
      if(nplane .gt. ndim) then
       write(iut,401) nplane, ndim
401    format(/' nplane is larger than ndim in bzdef --> ',2i8)
       stop
      end if
c
      do 105 i = 1, 3
      rsdir(i,nplane) = bt(i)
105   continue
      rsdir(4,nplane) = bt(1)**2 + bt(2)**2 + bt(3)**2
      idir(1,nplane)=ix
      idir(2,nplane)=iy
      idir(3,nplane)=iz
c
102   continue
c
      write(iut,402) 2*nplane
402   format('no. of planes bounding 1st brillouin zone ',i4)
      write(iut,1030)
 1030 format(' normals of bounding planes',
     &  ' (negatives of these not shown)')
      do 106 npl = 1, nplane
      write(iut,403) npl, (rsdir(ip,npl),ip = 1,4)
106   continue
403   format('vector ',i3,' --> ',3f12.6,' length^2 = ',f12.6)
      write(iut,1010)
 1010 format(' Integer coordinates of plane normals',
     &  ' (basis of (b1, b2, b3))')
      do npl=1,nplane
       write(iut,1020) npl,(idir(ip,npl),ip=1,3)
      enddo
 1020 format('vector ',i3,' --> ',3i6)
c
      return
      end
