      subroutine bzred(kpt,b1,b2,b3,rsdir,ndim,nplane)

c This subroutine reduces an input k vector to lie within the 1st bz.
c It has been revised to treat input and output k vectors in a basis
c of reciprocal lattice translations (i.e. fractional coordinates).

      implicit none
      integer ndim,nplane
      double precision kpt(3),b1(3),b2(3),b3(3),rsdir(4,ndim)

      integer nzone, nzone1, nzone3
      parameter (nzone = 2, nzone1 = nzone + 1, nzone3 = 2*nzone + 1)

      integer ik1,ik2,ik3,i1,i2,i3,ix,iy,iz,i
      double precision kptc(3)
      integer inbz
      external inbz

 
c     inbz expects a vector in cartesian coordinates

      kptc(1)=b1(1)*kpt(1)+b2(1)*kpt(2)+b3(1)*kpt(3)
      kptc(2)=b1(2)*kpt(1)+b2(2)*kpt(2)+b3(2)*kpt(3)
      kptc(3)=b1(3)*kpt(1)+b2(3)*kpt(2)+b3(3)*kpt(3)

      if (inbz(kptc,rsdir,ndim,nplane) .eq. 1) then
       return
      endif
 
c     Find nearest integer coordinates of kpt
      ik1 = nint(kpt(1))
      ik2 = nint(kpt(2))
      ik3 = nint(kpt(3))

      do 101 ix = 1, nzone3
      i1 = nzone1 - ix - ik1
      do 101 iy = 1, nzone3
      i2 = nzone1 - iy - ik2
      do 101 iz = 1, nzone3
      i3 = nzone1 - iz - ik3
 
c     Look near the trial vector for one contained within 1st bz.

c     Get cartesian coordinates for a test k vector
      do i = 1, 3
       kptc(i) = (kpt(1)+dble(i1)) * b1(i) + (kpt(2)+dble(i2)) * b2(i) 
     &         + (kpt(3)+dble(i3)) * b3(i)
      enddo
 
c     check whether kptc lies inside all the planes that define the 1bz
 
      if (inbz(kptc,rsdir,ndim,nplane) .eq. 1) then
c      Return the fractional coordinates of the k vector
       kpt(1)=kpt(1)+dble(i1)
       kpt(2)=kpt(2)+dble(i2)
       kpt(3)=kpt(3)+dble(i3)
       return
      endif
 
  101 continue

c     Error condition
      kptc(1)=b1(1)*kpt(1)+b2(1)*kpt(2)+b3(1)*kpt(3)
      kptc(2)=b1(2)*kpt(1)+b2(2)*kpt(2)+b3(2)*kpt(3)
      kptc(3)=b1(3)*kpt(1)+b2(3)*kpt(2)+b3(3)*kpt(3)

      write(*,1010) kpt,kptc,ik1,ik2,ik3
 1010 format(/,'bzred error: vector -----> ',3f15.8,/,
     &  'with cartesian coordinates ',3f15.8,/,
     &  'can not be confined inside the 1st bz in bzred',/,
     &  'integer coords are ',3i5)
      stop
      end
