      function inbz(gr,rsdir,ndim,nplane)
      implicit none
      integer inbz,ndim,nplane
      double precision gr(3),rsdir(4,ndim)
      integer npl
      double precision eps, proj
 
c function to determine whether vector gr is inside the 1st bz
 
c Returns 0 if not inside, 1 if inside

c gr(3)=g space vector in same units as b1, b2, b3
c rsdir(4,ndim)=vectors normal to the planes delimiting the bz.  the
c                fourth component of the array defines the length.
c                the brillouin zone surface intersects each vector
c                listed in rsdir at the half of its length.
c ndim=second dimension of rsdir in calling program (irrelevant)
c nplane=number of planes cutting the BZ surfaces

c     eps used to be 1.d-6
      parameter (eps = 1.0d-9)

      inbz = 0

c     Plane by plane, look at the projection of input vector gr
c     on each normal vector rsdir; if |projection| exceeds 1/2 then
c     gr lies outside 1st brillouin zone.
c     eps is added to allow a vector to lie exactly on the bz boundary.
c     NOTE that this allows vectors on both front and back boundaries,
c     which double counts the allowed vectors since the boundaries are
c     actually equivalent.

c     Modify 18 Jan 1995: disallow back boundary by replacing
c     abs(proj).gt. 0.5+eps with abs(proj-eps).gt. 0.5


      do 101 npl = 1, nplane
       proj = (rsdir(1,npl) * gr(1) + rsdir(2,npl) * gr(2) 
     &        + rsdir(3,npl) * gr(3)) / rsdir(4,npl)
       if (abs(proj-eps) .gt. 0.5d0) then
        return
       endif
101   continue
 
      inbz = 1
      return
      end
