      subroutine pgsymo(pgind,s,ntot,iut)
      implicit none
c   set up the point-group rotation matrices for all 32 point groups
c   for the simple (p) bravais lattices.
c   these matrices all have integer elements, and express the relations
c   between the primitive basis vectors due to each point-group
c   rotation.  the special axis is always the z-axis, one basal-plane
c   vector is always along x, and the other basal-plane vector is at
c   angle beta for monoclinic (beta is not actually used), at 120
c   degrees for trigonal and hexagonal(p) groups, and at 90 degrees for
c   remaining groups.
c   subroutine trgsym takes care of the trigonal(r) groups, subroutine
c   fccsym takes care of the cubic (f) groups, subroutine bccsym takes
c   care of the cubic (i) groups, and subroutine bctsym takes care of
c   the tetragonal (i) groups.  for these latter
c   groups, the crystallographic vectors are chosen
c   differently.  for the triclinic groups, the matrices are
c   simply identity and inversion.
c
c     and 14 for triclinic p (for the last four
c     cases, a wedge equivalent to the irreducible
c     wedge, but simpler in shape, is used.). see table below.
c     the remaining bravais lattices are not programmed - use ibrav=8
c     for body-centered orthorhombic, etc.
c
c    pgsym  takes care of bravais latticec (ibrav=1,4,6,8,12,14)
c    fccsym takes care of bravais latticec (ibrav=2)
c    bccsym takes care of bravais latticec (ibrav=3)
c    trgsym takes care of bravais latticec (ibrav=5)
c    bctsym takes care of bravais latticec (ibrav=7)
c    ibrav = 9,10,11,13 "not programmed yet"
c
c
c     point group/             |ibrav|pgind| celldm(2)|  nc
c            bravais lattice   |     |     |-celldm(6)|
c_____________________________________________________________________
c     432,<4>3m,m3m     sc      |   1|    1|not used  | n(n+1)(n+2)/6
c_______________________________________________________________________
c     23,m3             sc      |   1|    2|    "     | n(n+1)(2n+1)/6
c____________________________________________________________________
c     432,<4>3m,m3m    fcc      |   2|    1|    "     |n(n+2)(2n+5)/24
c_____________________________________________________________________
c     23,m3            fcc      |   2|    2|    "     |n(n+2)(2n+5)/12
c______________________________________________________________________
c     432,<4>3m,m3m    bcc      |   3|    1|    "     |n(n+2)(n+4)/24
c_____________________________________________________________________
c     23,m3            bcc      |   3|    2|    "     |n(n+1)(n+2)/12
c_____________________________________________________________________
c     622,6mm,                  |    |     |celldm(3)=|
c     <6>m2,6/mmm      hex(p)   |   4|    1|  c/a     | n**2(n+1)/2
c_____________________________________________________________________
c     6,<6>,6/m,       hex(p)   |    |     |          |
c     32,3m,<3>m      trig(p)   |   4|    2|    "     | n**3
c_____________________________________________________________________
c     3,<3>           trig(p)   |   4|    3|    "     | 2n**2(n+1)
c_____________________________________________________________________
c     32,3m,<3>m      trig(r)   |   5|    2|celldm(4)=| n**3
c                               |    |     |cos(alpha)|
c_____________________________________________________________________
c     3,<3>           trig(r)   |   5|    3|    "     | 2n**2(n+1)
c_____________________________________________________________________
c     422,4mm,                  |    |     |celldm(3)=|
c     <4>2m,4/mmm     tetr(p)   |   6|    1| c/a      | n**2(n+1)/2
c_____________________________________________________________________
c     4,<4>,4/m       tetr(p)   |   6|    2|    "     |    n**3
c_____________________________________________________________________
c     422,4mm,                  |    |     |          |
c     <4>2m,4/mmm     tetr(i)   |   7|    1|    "     | n**2(n+2)/4
c_____________________________________________________________________
c     4,<4>,4/m       tetr(i)   |   7|    2|    "     | n**2(n+1)/2
c_____________________________________________________________________
c     222,mm2,mmm     orth(p)   |   8|    1|above +   |
c                               |    |     |celldm(2)=|
c                               |    |     |b/a       |    n**3
c____________________________________________________________________
c     2,m,2/m         mcln(p)   |  12|    1|above +   |
c                               |    |     |celldm(4)=|
c                               |    |     |cos(ab)   |   2n**3
c______________________________________________________________________
c     1,<1>           tcln(p)   |  14|    1|celldm(2)=|
c                               |    |     |b/a       |
c                               |    |     |celldm(3)=|
c                               |    |     |c/a       |
c                               |    |     |celldm(4)=|
c                               |    |     |cos(bc)   |   4n**3
c                               |    |     |celldm(5)=|
c                               |    |     |cos(ac)   |
c                               |    |     |celldm(6)=|
c                               |    |     |cos(ab)   |
c_______________________________________________________________________
c   pgind is the point-group index, defined as follows:
c
c   pgind   group     pgind   group     pgind   group     pgind   group
c    1     1 (c1)       9     3m (c3v)    17   4/mmm(d4h)   25   222(d2)
c    2    <1>(ci)      10     <3>m(d3d)   18   6 (c6)       26  mm2(c2v)
c    3     2 (c2)      11     4 (c4)      19   <6>(c3h)     27  mmm(d2h)
c    4     m (c1h)     12     <4>(s4)     20   6/m(c6h)     28  23 (t)
c    5     2/m(c2h)    13     4/m(c4h)    21   622(d6)      29  m3 (th)
c    6     3 (c3)      14     422(d4)     22   6mm(c6v)     30  432 (o)
c    7     <3>(c3i)    15     4mm(c4v)    23   <6>m2(d3h)   31 <4>3m(td)
c    8     32 (d3)     16    <4>2m(d2d)   24    6/mmm(d6h)  32  m3m(oh)
c
      integer ntot,iut
      integer pgind,s(3,3,48),sum
      integer i,j,k,kind,jind

      if(pgind.lt.1) go to 100
      do 1 k=1,48
      do 1 j=1,3
      do 1 i=1,3
    1 s(i,j,k)=0
      do 2 i=1,3
    2 s(i,i,1)=1
      if(pgind.gt.2) go to 5
c   triclinic groups here.
c  point group 1.
      ntot=1
      if(pgind.eq.1) return
c   add inversion for <1>.
      ntot=2
      do 3 i=1,3
    3 s(i,i,2)=-1
      return
c   monoclinic groups here.
    5 if(pgind.gt.5) go to 10
      ntot=2
      do 6 i=1,3
      s(i,i,2)=1
      s(i,i,3)=1
    6 s(i,i,4)=1
      if(pgind.gt.3) go to 7
c  point group 2. or c2
c  july 23 1982 : i have changed for c2 to have y as the 2-fold axis
c  it should only affect ibrav=12, indpg=3
c***  s(1,1,2)=-1
c***  s(2,2,2)=-1
      s(1,1,2)=-1
      s(2,2,2)= 1
      s(3,3,2)=-1
c*****************
      return
    7 if(pgind.gt.4) go to 8
c   point group m.
      s(3,3,2)=-1
      return
    8 ntot=4
c   point group 2/m.
      s(1,1,2)=-1
      s(2,2,2)=-1
      s(3,3,3)=-1
      do 9 i=1,3
    9 s(i,i,4)=-1
      return
c   trigonal groups here.
   10 if(pgind.gt.10) go to 20
c  point group 3.
      ntot=3
      s(1,2,2)=1
      s(2,1,2)=-1
      s(2,2,2)=-1
      s(3,3,2)=1
      s(1,1,3)=-1
      s(1,2,3)=-1
      s(2,1,3)=1
      s(3,3,3)=1
      if(pgind.eq.6) return
      if(pgind.eq.7.or.pgind.eq.10) go to 15
      ntot=6
c   set up c2x.
      s(1,1,4)=1
      s(3,3,4)=-1
      s(2,1,4)=-1
      s(2,2,4)=-1
      if(pgind.eq.8) go to 12
c   set up mx.
      do 11 j=1,3
      do 11 i=1,3
   11 s(i,j,4)=-s(i,j,4)
      s(3,3,4)=1
c  point group 32 =3(x)c2x or 3m = 3(x)mx.
   12 do 14 kind=2,3
      do 14 i=1,3
      do 14 j=1,3
      sum=0
      do 13 k=1,3
   13 sum=sum+s(i,k,4)*s(k,j,kind)
   14 s(i,j,kind+3)=sum
      return
   15 ntot=6
c   point group <3> = 3(x)i
      do 16 kind=1,3
      do 16 j=1,3
      do 16 i=1,3
   16 s(i,j,kind+3)=-s(i,j,kind)
      if(pgind.eq.7) return
c   point group <3>m = <3>(x)mx
      ntot=12
      s(1,1,7)=-1
      s(2,1,7)=1
      s(2,2,7)=1
      s(3,3,7)=1
      do 18 kind=2,6
      do 18 j=1,3
      do 18 i=1,3
      sum=0
      do 17 k=1,3
   17 sum=sum+s(i,k,7)*s(k,j,kind)
   18 s(i,j,kind+6)=sum
      return
c   here for tetragonal groups.
   20 if(pgind.gt.17) go to 40
c   point group 4.
      ntot=4
      s(1,2,2)=1
      s(2,1,2)=-1
      s(3,3,2)=1
      s(1,1,3)=-1
      s(2,2,3)=-1
      s(3,3,3)=1
      s(1,2,4)=-1
      s(2,1,4)=1
      s(3,3,4)=1
      if(pgind.eq.11) return
      if(pgind.ne.12.and.pgind.ne.16) go to 25
c   point group <4>.
      do 21 kind=2,4,2
      do 21 j=1,3
      do 21 i=1,3
   21 s(i,j,kind)=-s(i,j,kind)
      if(pgind.eq.12) return
c   point group <4>2m = <4>(x)c2x.
      ntot=8
      s(1,1,5)=1
      s(2,2,5)=-1
      s(3,3,5)=-1
      do 23 kind=2,4
      do 23 j=1,3
      do 23 i=1,3
      sum=0
      do 22 k=1,3
   22 sum=sum+s(i,k,5)*s(k,j,kind)
   23 s(i,j,kind+4)=sum
      return
   25 ntot=8
      if(pgind.ne.13) go to 26
c   point group 4/m = mz(x)4
      s(1,1,5)=1
      s(2,2,5)=1
      s(3,3,5)=-1
      go to 30
   26 if(pgind.ne.15) go to 27
c   point group 4mm = mx(x)4.
      s(1,1,5)=-1
      s(2,2,5)=1
      s(3,3,5)=1
      go to 30
c   point groups 422 = c2x(x)4 and 4/mmm = i(x)422.
   27 s(1,1,5)=1
      s(2,2,5)=-1
      s(3,3,5)=-1
   30 do 32 kind=2,4
      do 32 j=1,3
      do 32 i=1,3
      sum=0
      do 31 k=1,3
   31 sum=sum+s(i,k,5)*s(k,j,kind)
   32 s(i,j,kind+4)=sum
      if(pgind.ne.17) return
      ntot=16
      do 34 kind=1,8
      do 34 j=1,3
      do 34 i=1,3
   34 s(i,j,kind+8)=-s(i,j,kind)
      return
c   here for hexagonal point groups.
   40 if(pgind.gt.24) go to 60
      ntot=6
c   point group 6.
      s(1,1,2)=1
      s(1,2,2)=1
      s(2,1,2)=-1
      s(1,2,3)=1
      s(2,1,3)=-1
      s(2,2,3)=-1
      s(1,1,4)=-1
      s(2,2,4)=-1
      s(1,1,5)=-1
      s(1,2,5)=-1
      s(2,1,5)=1
      s(1,2,6)=-1
      s(2,1,6)=1
      s(2,2,6)=1
      do 41 i=2,6
   41 s(3,3,i)=1
      if(pgind.eq.18) return
      if(pgind.ne.19.and.pgind.ne.23) go to 45
c   point group <6>.
      do 42 kind=2,6,2
      do 42 j=1,3
      do 42 i=1,3
   42 s(i,j,kind)=-s(i,j,kind)
      if(pgind.eq.19) return
c   point group <6>m2 = c2y(x)<6>.
      ntot=12
      s(1,1,7)=-1
      s(2,2,7)=1
      s(3,3,7)=-1
      do 44 kind=2,6
      do 44 j=1,3
      do 44 i=1,3
      sum=0
      do 43 k=1,3
   43 sum=sum+s(i,k,7)*s(k,j,kind)
   44 s(i,j,kind+6)=sum
      return
   45 ntot=12
      if(pgind.ne.20) go to 47
c   point group 6/m = i(x)6
      do 46 kind=1,6
      do 46 j=1,3
      do 46 i=1,3
   46 s(i,j,kind+6)=-s(i,j,kind)
      return
   47 if(pgind.ne.22) go to 48
c   point group 6mm = mx(x)6
      s(1,1,7)=-1
      s(2,2,7)=1
      s(3,3,7)=1
      go to 50
c   point groups 622 = c2x(x)6 and 6/mmm = i(x)622.
   48 s(1,1,7)=1
      s(2,2,7)=-1
      s(3,3,7)=-1
   50 do 52 kind=2,6
      do 52 j=1,3
      do 52 i=1,3
      sum=0
      do 51 k=1,3
   51 sum=sum+s(i,k,7)*s(k,j,kind)
   52 s(i,j,kind+6)=sum
      if(pgind.ne.24) return
      ntot=24
      do 53 kind=1,12
      do 53 j=1,3
      do 53 i=1,3
   53 s(i,j,kind+12)=-s(i,j,kind)
      return
c   here for orthorhombic point groups.
   60 if(pgind.gt.27) go to 70
      ntot=4
      if(pgind.ne.26) go to 64
c   point group mm2.
      do 61 kind=2,4
      do 61 i=1,3
   61 s(i,i,kind)=1
      s(1,1,2)=-1
      s(2,2,3)=-1
      s(1,1,4)=-1
      s(2,2,4)=-1
      return
c   point group 222.
   64 do 65 kind=2,4
      do 65 i=1,3
   65 s(i,i,kind)=-1
      s(1,1,2)=1
      s(2,2,3)=1
      s(3,3,4)=1
      if(pgind.eq.25) return
c   point group mmm = i(x)222
      ntot=8
      do 66 kind=1,4
      do 66 j=1,3
      do 66 i=1,3
   66 s(i,j,kind+4)=-s(i,j,kind)
      return
c   here for cubic point groups.
   70 if(pgind.gt.32) go to 100
      ntot=12
c   point group 23.
      do 71 kind=2,4
      do 71 i=1,3
   71 s(i,i,kind)=-1
      s(1,1,2)=1
      s(2,2,3)=1
      s(3,3,4)=1
      s(1,2,5)=1
      s(2,3,5)=1
      s(3,1,5)=1
      s(1,3,6)=1
      s(2,1,6)=1
      s(3,2,6)=1
      do 74 kind=5,6
      do 74 jind=2,4
      do 74 j=1,3
      do 74 i=1,3
      sum=0
      do 72 k=1,3
   72 sum=sum+s(i,k,jind)*s(k,j,kind)
   74 s(i,j,kind+2*jind-2)=sum
      if(pgind.eq.28) return
      ntot=24
      if(pgind.ne.29) go to 76
c   point group m3 = i(x)23
      do 75 kind=1,12
      do 75 j=1,3
      do 75 i=1,3
   75 s(i,j,kind+12)=-s(i,j,kind)
      return
   76 if(pgind.eq.31) go to 80
c   point group 432 = c4x(x)23
      s(1,1,13)=1
      s(2,3,13)=1
      s(3,2,13)=-1
      do 78 kind=2,12
      do 78 j=1,3
      do 78 i=1,3
      sum=0
      do 77 k=1,3
   77 sum=sum+s(i,k,13)*s(k,j,kind)
   78 s(i,j,kind+12)=sum
      if(pgind.eq.30) return
c   point group m3m = i(x)432
      ntot=48
      do 79 kind=1,24
      do 79 j=1,3
      do 79 i=1,3
   79 s(i,j,kind+24)=-s(i,j,kind)
      return
c   point <4>3m = <c4x>(x)23
   80 s(1,1,13)=-1
      s(2,3,13)=-1
      s(3,2,13)=1
      do 85 kind=2,12
      do 85 j=1,3
      do 85 i=1,3
      sum=0
      do 84 k=1,3
   84 sum=sum+s(i,k,13)*s(k,j,kind)
   85 s(i,j,kind+12)=sum
      return
c   here if pgind exceeds 32 or is less than 1.
  100 write(iut,110) pgind
  110 format(' point-group index',i4,' is not between 1 and 32')
      ntot = 1
      do 301 i=1,3
301   s(i,i,1) = 1
      return
      end
