//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002, 2003- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
/*! @file NEBPotential.h
 * \author Jeongnim Kim
 * \brief Specialized potential to perform NEB calculations.
 ******************************************************************************/
#ifndef OHMMS_NEBPOTENTIAL_H
#define OHMMS_NEBPOTENTIAL_H
#include "OhmmsData/RecordProperty.h"
#include "OhmmsPETE/OhmmsMatrix.h"
#include "Potentials/PotentialBase.h"
namespace OHMMS {

  class NEBPotential;
  /*!\class RecordNEB
   * \brief Record Engine a NEB
   */
  class RecordNEB: public RecordProperty {

  public:

    typedef ParticleSet Particle_t;

    RecordNEB(const vector<Particle_t*>& pin, NEBPotential& pot): 
      ptcl_ref(pin), pot_ref(pot)
    { }
    ~RecordNEB();

    void reset(const char* fileroot);
    void report(int iter);

    void finalize() { }
    bool put(xmlNodePtr cur);

  private:
    const vector<Particle_t*>& ptcl_ref;
    NEBPotential& pot_ref;
    string FileRoot;
    string LogFile;
    void reportNEB(int iter, int firstchain, int lastchain, int nodeid = 0, int offset=0);
  };

  class NEBPotential: public PotentialBase {

    friend class RecordNEB;
    friend class NEBPropagator;

  public:

    typedef PotentialBase Interaction_t;

    NEBPotential();
    ~NEBPotential();

    void initialize();

    Scalar_t evalEnergy();
    Scalar_t evalForce(ParticlePos_t& f, unsigned stressmode =0);
    bool evalDynamicMatrix(DynMatrix_t&);
    void print(ostream&, int outputlevel = 0) const;

    /// implementing virtual functions for OhmmsElementBase
    bool get(ostream&) const;
    bool put(istream&);
    bool put(xmlNodePtr cur);
    void reset();

    void setParticleSet(Particle_t* ptclset);
    void resetParticleSet(Particle_t* ptclset) {}
    void setCurrentPos(ParticlePos_t& pos){}
    void setCurrentPos(ParticlePos_t& pos, bool wrap, bool updatecell=true){}
    void setCurrentPos(const ParticlePos_t& pos, const ParticlePos_t& dr){}

    void init(Interaction_t* pot, vector<Particle_t*>& Images);
    void getNEBForce(vector<Particle_t*>& );
  protected:

    int WorkingPtcl;
    int neb_mode;
    Scalar_t SpringConst, SpringConst2;

    /// physical potential
    Interaction_t* realPotential;
    /// array of extensive energy
    vector<string> Ename;
    Matrix<Scalar_t> E;
    vector<ParticlePos_t*> F;
    ParticlePos_t Fc, Ft, Lpos, Rpos, Rfix;
  };


}
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 381 $   $Date: 2005-09-05 15:24:10 -0500 (Mon, 05 Sep 2005) $
 * $Id: NEBPotential.h 381 2005-09-05 20:24:10Z jnkim $ 
 ***************************************************************************/

