//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include <string>
#include <vector>
#include <utility>
#include <iostream>
#include <fstream>
using namespace std;

#ifdef HAVE_CONFIG_H
#include "ohmms-config.h"
#include "OhmmsApp/AppConfig.h"
#endif
#include "OhmmsApp/IonicPotential.h"
#if defined(ENABLE_SW)
#include "Potentials/SWPotential.h"
#endif // ENABLE_MEAM
                                                                                            
#if defined(ENABLE_MEAM)
#include "Potentials/MEAMPotential.h"
#endif // ENABLE_MEAM
                                                                                            
#if defined(ENABLE_EAM)
#include "Potentials/EAMPotential.h"
#endif // ENABLE_EAM
                                                                                            
#if defined(ENABLE_KEATING)
#include "Potentials/KeatingPotential.h"
#endif // ENABLE_KEATING
                                                                                            
#if defined(ENABLE_LJ)
#include "ScaleFunctors/LJScale.h"
#include "Potentials/TwoBodyPotentialG.h"
#endif // ENABLE_LJ
                                                                                            
#if defined(ENABLE_TB)
#include "SlaterKosterTB/TBPotentialFactory.h"
#endif // ENABLE_TB
#include "Utilities/OhmmsInfo.h"
#include "Potentials/FastPotential.h"
namespace OHMMS {

  enum {swpot_id = 0, meampot_id, eampot_id, ljpot_id, keating_id, tbpot_id};
  
  int getPotentialID(const string& mdname) {
    if(mdname == "SW") return swpot_id;
    else if(mdname == "MEAM") return meampot_id;
    else if(mdname == "Keating") return keating_id;
    else if(mdname == "EAM") return eampot_id;
    else if(mdname == "TB") return tbpot_id;
    return -1;
  }

  IonicPotential::IonicPotential(const char* aname): 
    OhmmsElementBase(aname), m_pot(0),playout_ptr(0), pot_type(-1)
  { 
  }

  
  IonicPotential::~IonicPotential() {
    map<int,PotentialBase*>::iterator it(pReserve.begin());
    map<int,PotentialBase*>::iterator it_end(pReserve.end());
    while(it != it_end) { delete (*it).second; ++it;}
  }

  /// generic output
  bool IonicPotential::get(std::ostream& os) const { 
    //cout << m_name << " ";
    //return m_pot->get(os);
    return true;
  }
  
  /// generic input
  bool IonicPotential::put(std::istream& is) {
    return true;
  }

  PotentialBase* IonicPotential::get() {
    return m_pot;
  }

  void IonicPotential::reset() {

  }  

  bool 
  IonicPotential::addRecord(RecordPropertyList& reclist, xmlNodePtr cur) {
    if(m_pot) {
      RecordProperty* ar=m_pot->getRecorder();
      if(ar) reclist.addRecord(ar);
    }
    return true; 
  }

  bool IonicPotential:: put(xmlNodePtr cur){

    string ename((const char*)cur->name);
    if(ename == "particlelayout") {
      playout_ptr = cur;
      return true;
    }

    string pname = (const char*)(xmlGetProp(cur, (const xmlChar *) "name"));
    pot_type = getPotentialID(pname);
    if(pot_type<0) return false;
    map<int,PotentialBase*>::iterator it(pReserve.find(pot_type));
    if(it!= pReserve.end()) {
      m_pot = (*it).second;
      return true;
    }

    PotentialBase* anewpot=0;
    if(pot_type ==tbpot_id) {
#if defined(ENABLE_TB)
      anewpot = createTBPotential(cur,playout_ptr);
#else
      ERRORMSG("Tight-binding potentials are disabled")
#endif
    } else {
      ShortRangePotential* srp=0;
      switch(pot_type) {
        case(swpot_id):
#if defined(ENABLE_SW)
          srp=new SWPotential;
#endif
          break;
          case(meampot_id):
#if defined(ENABLE_MEAM)
          srp=new MEAMPotential;
#endif
          break;
          case(eampot_id): 
#if defined(ENABLE_EAM)
          srp=new EAMPotential;
#endif 
          break; 
          case(keating_id): 
#if defined(ENABLE_KEATING)
          srp=new KeatingPotential;
#endif 
          break;
          default:
          break; 
      }
      if(srp) {
        FastPotential *srp_wrap = new FastPotential(cur,playout_ptr);
        srp_wrap->setSRP(srp);
        anewpot=srp_wrap;
      }
    }
    if(anewpot) {
      pReserve[pot_type]=anewpot;
      m_pot = anewpot;
      return true;
    } else {
      return false;
    }
  }
}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 354 $   $Date: 2005-07-07 21:20:49 -0500 (Thu, 07 Jul 2005) $
 * $Id: IonicPotential.cpp 354 2005-07-08 02:20:49Z jnkim $ 
 ***************************************************************************/
