//////////////////////////////////////////////////////////////////
// (c) Copyright 2004-  by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifndef OHMMS_MINIMAHOPPING_H
#define OHHMS_MINIMAHOPPING_H

#include "AppBase/OhmmsElementSet.h"
#include "AppBase/ParticleSets.h"
#include "Potentials/PotentialBase.h"
namespace OHMMS {

  //Forward declaration of PropagatorBase
  class PropagatorBase;
  class RecordPropertyListManager;

  /** Implements MinimaHopping by S. Godecker
   *
   *The algorithm is presented in J. Chem. Phys. V 120, 9911 (2004).
   */
  class MinimaHopping: public OhmmsElementBase {

  public:

    typedef PotentialBase Interaction_t;

    // constructors and destructors
    MinimaHopping(OhmmsElementSet& nodelist, 
		  const char* aname="minima-hopping");

    ~MinimaHopping();

    bool get(std::ostream& os) const;
    bool put(std::istream& is);
    bool put(xmlNodePtr cur);
    void reset();

  private:

    ///boolean to print debug message to LOGMSG
    bool print_debug;

    OhmmsElementSet& NodeList;

    ///The number of target minima to be tabulated
    int NumTargets;

    ///The number of MD steps per block
    int MDSteps;

    ///The number of relaxation steps
    int MinSteps;

    ///The number of relaxation steps
    int SDMinSteps;

    ///Target temperature in Kelvin for a MD
    double KEtarget;

    ///The energy criteria to start minimization
    double Ediff;

    ///The energy criteria to find a matching configuration: 1 meV
    double Etol;


    ///The temperature to stop Steepest-decent minimization and start CG minimization.
    double LowTemperature;

    ///Control parameters as described by  JCP
    double beta1,beta2,beta3;
    double alpha1, alpha2;

    ///Potential energies of the minima
    std::vector<double> PE;

    ///The number of times a minimization goes to the minima
    std::vector<int> Visited;

    xmlNodePtr pe_ptr;
    PropagatorBase* md_engine;
    PropagatorBase* sd_minimizer;
    PropagatorBase* cg_minimizer;
    RecordPropertyListManager* recorders;

    Interaction_t* myPot;
    ParticleSets::Particle_t* myPtcl;
    ParticleSets::Particle_t::ParticlePos_t Rmin;
    ParticleSets::Particle_t::ParticlePos_t Rcur, Vcur;

    void processXmlNode(xmlNodePtr cur);
    double getMinimumConfig(Interaction_t* workingpot);
    int matchingConfig(double pe, int curconf);
    void saveConfiguration(const char* fileroot, int i);
  };
}
#endif
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 317 $   $Date: 2005-03-01 17:13:41 -0600 (Tue, 01 Mar 2005) $
 * $Id: MinimaHopping.h 317 2005-03-01 23:13:41Z jnkim $ 
 ***************************************************************************/
