#include "OhmmsData/OhmmsElementBase.h"
#include "Configuration.h"
#include "ProjectData.h"
#include "MDRunData.h"
#include "ParticleSets.h"
#include "WorkingPropagator.h"
#include "IonicPotential.h"
#include "NEBRun.h"
#if defined(ENABLE_DIMER)
#include "DimerRun.h"
#endif
#if defined(ENABLE_ELPH)
#include "ElectronPhononCoupling.h"
#endif
#include "RandomNumberControl.h"
#include "Message/CommCreate.h"
#include <libxml/xpath.h>
#include <map>
#include <iostream>
using namespace std;
using namespace OHMMS;

inline int getKeyIndex(const char* s) {
  //have to count how many keys it has
  int i=0;
  while(i<Key_quit) {
    if(!strcmp(OhmmsKeys[i],s)) return i;
    i++;
  }
  return -1;
}

xmlDocPtr parseXMLFile(const char* fname) {

  xmlDocPtr doc;
  xmlNsPtr ns;
  xmlNodePtr cur;

  // build an XML tree from a the file;
  doc = xmlParseFile(fname);
  if (doc == NULL) return(NULL);

  // Check the document is of the right kind
  cur = xmlDocGetRootElement(doc);
  if (cur == NULL) {
    fprintf(stderr,"empty document\n");
    xmlFreeDoc(doc);
    return(NULL);
  }

  ns = NULL;

  if (xmlStrcmp(cur->name, (const xmlChar *) "Main")) {
    fprintf(stderr,"document of the wrong type, root node != Main");
    xmlFreeDoc(doc);
    return(NULL);
  }

  // First level we expect just Jobs
  cur = cur->xmlChildrenNode;
  while ( cur && xmlIsBlankNode ( cur ) )
    {
      cur = cur -> next;
    }
  if ( cur == 0 )
    return false;


  ///using XPath instead of recursive search
  xmlXPathContextPtr context;
  xmlXPathObjectPtr result;
  context = xmlXPathNewContext(doc);
  result = xmlXPathEvalExpression((const xmlChar*)"//Jobs",context);

  if(!xmlXPathNodeSetIsEmpty(result->nodesetval)) {

    xmlNodeSetPtr jobs = result->nodesetval;

    if(jobs->nodeNr > 1) {
      LOGMSG("Launching a multiple jobs.")
      if(jobs->nodeNr < CommCreate::get()->getNumNodes()) {
	ERRORMSG("MPI processors > Jobs")
      } else {
	cur = jobs->nodeTab[CommCreate::get()->getNodeID()];
	// build an XML tree from a the file;
	xmlDocPtr doc1 = xmlParseFile((const char*)(xmlGetProp(cur, (const xmlChar *) "file")));
	if (doc1 == NULL) return(NULL);
	xmlXPathContextPtr context1;
	xmlXPathObjectPtr result1;
	context1 = xmlXPathNewContext(doc1);
	result1 = xmlXPathEvalExpression((const xmlChar*)"//Jobs",context1);

	if(!xmlXPathNodeSetIsEmpty(result1->nodesetval)) {
	  cur = result1->nodesetval->nodeTab[0];
	  ///always initialize the random number generator first
	  OhmmsData::Members[Key_random]->put(doc1,ns,cur);

	  // Second level is a list of Job, using OhmmsData member functions
	  cur = cur->xmlChildrenNode;
	  while (cur != NULL) {
	    int ikey = getKeyIndex((const char*)(cur->name));
	    if(ikey >= 0) OhmmsData::Members[ikey]->put(doc1,ns,cur);
	    cur = cur->next;
	  }
	}
	xmlXPathFreeObject(result1);
      }
    } else {
      /// typical xml file
      cur = jobs->nodeTab[0];

      ///always initialize the random number generator first
      OhmmsData::Members[Key_random]->put(doc,ns,cur);

      // Second level is a list of Job, using OhmmsData member functions
      cur = cur->xmlChildrenNode;
      while (cur != NULL) {
	int ikey = getKeyIndex((const char*)(cur->name));
	if(ikey >= 0) OhmmsData::Members[ikey]->put(doc,ns,cur);
	cur = cur->next;
      }
    }
  } else {
    fprintf(stderr,"document of the wrong type, was '%s', Jobs expected",
	    cur->name);
    fprintf(stderr,"xmlDocDump follows\n");
    xmlDocDump ( stderr, doc );
    fprintf(stderr,"xmlDocDump finished\n");
    xmlFreeDoc(doc);
    return(NULL);
  }

  xmlXPathFreeObject(result);
  return doc;
}


int main(int argc, char **argv) {

  /* COMPAT: Do not genrate nodes for formatting spaces */
  LIBXML_TEST_VERSION
  xmlKeepBlanksDefault(0);
  xmlDocPtr doc;

  OhmmsData::Add(Key_project, new ProjectData);
  OhmmsData::Add(Key_ptclin, new ParticleSets);
  OhmmsData::Add(Key_mdtype, new WorkingPropagator);
  OhmmsData::Add(Key_ionpot, new IonicPotential);
  OhmmsData::Add(Key_ionmd, new MDRunData);
  OhmmsData::Add(Key_neb, new NEBRun);

#if defined(ENABLE_DIMER)
  OhmmsData::Add(Key_dimer, new DimerRun);
#endif

#if defined(ENABLE_ELPH)
  OhmmsData::Add(Key_elph, new ElectronPhononCoupling);
#endif

  OhmmsData::Add(Key_random, new RandomNumberControl);

  if(argc == 2) {
    doc = parseXMLFile(argv[1]);
    if(doc == NULL) 
      fprintf( stderr, "Error parsing file '%s'\n", argv[1]);
    xmlSaveFormatFile("newfile.xml",doc,1);  
  } else if(argc > 2) {
    int iargc=0;
    while(iargc<argc) {
      if(!strcmp(argv[iargc],"-f")) {
	doc = parseXMLFile(argv[++iargc]);
	if(doc == NULL) 
	  fprintf( stderr, "Error parsing file '%s'\n", argv[iargc]);
	xmlSaveFormatFile("newfile.xml",doc,1);  
      }
      iargc++;
    }
  } else {
    WARNMSG("No input file is provided.")
  }

//   for (int i = 1; i < argc ; i++) {
//     doc = parseXMLFile(argv[i]);
//     if(doc == NULL) 
//       fprintf( stderr, "Error parsing file '%s'\n", argv[i]);
//     xmlSaveFormatFile("newfile.xml",doc,1);  
//   }

  /* Clean up everything else before quitting. */
  xmlCleanupParser();
  CommCreate::remove();
  return(0);
}
