//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include <libxml/xpath.h>
#include "OhmmsApp/RecordPropertyList.h"
#include "AppBase/ProjectData.h"
#include "AppBase/ParticleSets.h"
#include "OhmmsApp/PropagatorBuilder.h"
#include "OhmmsApp/IonicPotential.h"
#include "Propagators/PropagatorBase.h"
#include "Potentials/RecordPotential.h"
#include "Utilities/OhmmsInfo.h"

namespace OHMMS {

  RecordPropertyListManager::RecordPropertyListManager(OhmmsElementSet& nodelist, 
						       const char* aname):
    OhmmsElementBase(aname),
    NodeList(nodelist)
  {  }

  RecordPropertyListManager::~RecordPropertyListManager(){  
  }

  bool RecordPropertyListManager::get(std::ostream& os) const{
    return true;
  }

  bool RecordPropertyListManager::put(std::istream& is){
    return true;
  }

  void RecordPropertyListManager::reset() {
    CurrentRoot = dynamic_cast<ProjectData*>(NodeList.getNode(Key_project))->CurrentRoot();
    resetRecords(CurrentRoot.c_str());
//     vector<RecordProperty* >::iterator it = Properties.begin();
//     while(it != Properties.end()) {
//       (*it)->reset(CurrentRoot.c_str()); it++;
//     } 
  }

  void RecordPropertyListManager::push_back() {
    NewRecord.push(Properties.size());
  }

  void RecordPropertyListManager::reset_top() {
    for(int i=NewRecord.top(); i<Properties.size(); i++) {
      Properties[i]->reset(CurrentRoot.c_str());
    }
  }

  void RecordPropertyListManager::pop_back() {
    if(NewRecord.size()) {
      int n=NewRecord.top();
      for(int i=n; i<Properties.size(); i++) {
	Properties[i]->finalize();
	delete Properties[i];
      }
      Properties.erase(Properties.begin()+n,Properties.end());
      NewRecord.pop();
    }
  }

  bool RecordPropertyListManager::put(xmlNodePtr cur) {

    bool success = true;
    xmlAttrPtr att = cur->properties;
    while(att != NULL) {
      string aname((const char*)(att->name));
      string pname((const char*)(att->children->content));
      if(aname == "name") {
	RecordProperty *fromPool = addRecord(pname);
	if(fromPool) {
	  success = true;
	  fromPool->put(cur);
	  LOGMSG(pname << " in the Registered Records is used");
	} else {
	  if(pname == "particle" || pname == "Particle") {
	    ParticleSets* ptcl = (dynamic_cast<ParticleSets*>(NodeList.getNode(Key_particle)));
	    vector<RecordProperty*> alist;
	    success = ptcl->createRecordProperty(cur,alist);
	    if(success) {
	      for(int i=0;i<alist.size(); i++) {
		addRecord(alist[i]);
	      }
	    }
	  }  else if(pname == "ensemble" || pname == "Ensemble") {
	    PropagatorBase* mover
	      = (dynamic_cast<PropagatorBuilder*>(NodeList.getNode(Key_ensemble)))->get();
	    RecordProperty *arecord = new RecordPropagator(*mover);
	    addRecord(arecord);
	    success = arecord->put(cur);
	  } else if(pname == "potential" || pname == "Potential") {
	    success 
	      = (dynamic_cast<IonicPotential*>(NodeList.getNode(Key_potential)))->addRecord(*this,cur);
	  } 
	}
      }
      att = att->next;
    }
    return success;
  }

//   bool 
//   RecordPropertyListManager::addRecords(xmlXPathObjectPtr result) {

//     bool success = false;

//     if(!xmlXPathNodeSetIsEmpty(result->nodesetval)) {
//       xmlNodeSetPtr nodeset = result->nodesetval;
//       for(int irecord=0; irecord<nodeset->nodeNr; irecord++) {
// 	xmlNodePtr cur = nodeset->nodeTab[irecord];

// 	xmlAttrPtr att = cur->properties;
// 	while(att != NULL) {
// 	  string aname((const char*)(att->name));
// 	  string pname((const char*)(att->children->content));
// 	  if(aname == "name") {
// 	    if(pname == "particle" || pname == "Particle") {
// 	      ParticleSets* ptcl = (dynamic_cast<ParticleSets*>(NodeList.getNode(Key_particle)));
// 	      vector<RecordProperty*> alist;
// 	      success = ptcl->createRecordProperty(cur,alist);
// 	      if(success) {
// 		for(int i=0;i<alist.size(); i++) addRecord(alist[i]);
// 	      }
// 	    }  else if(pname == "ensemble" || pname == "Ensemble") {
// 	      PropagatorBase* mover
// 		= (dynamic_cast<PropagatorBuilder*>(NodeList.getNode(Key_ensemble)))->get();
// 	      Properties.push_back(new RecordPropagator(*mover));
// 	      success = Properties.back()->put(cur);
// 	    } else if(pname == "potential" || pname == "Potential") {
// 	      PotentialBase* pot
// 		=(dynamic_cast<IonicPotential*>(NodeList.getNode(Key_potential)))->get();
// 	      if(pot) {
// 		Properties.push_back(new RecordPotential(*pot));
// 		success =  Properties.back()->put(cur);
// 	      } else {
// 		success = false;
// 	      }
// 	    }
// 	  }
// 	  att = att->next;
// 	}
//       }
//     }
//     return success;
//   }

}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 254 $   $Date: 2005-01-14 11:40:45 -0600 (Fri, 14 Jan 2005) $
 * $Id: RecordPropertyList.cpp 254 2005-01-14 17:40:45Z jnkim $ 
 ***************************************************************************/
