//////////////////////////////////////////////////////////////////
// (c) Copyright 2003  by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "Utilities/OhmmsInfo.h"
#include "Message/Communicate.h"
#include "OhmmsApp.h"

/**\file main.cpp
 * The main routine that creates OhmmsApp to perform atomistic simulations.
 * For new applications, the easiest way is to create a derived class from OhmmsAppBase
 * and to implement initialize and dump functions.
 */
int main(int argc, char **argv) {

  OHMMS::Controller->initialize(argc,argv);

  OHMMS::OhmmsApp app(argc,argv);

  app.initialize();

  xmlKeepBlanksDefault(0);
  xmlDocPtr doc;
  if(argc == 2) {
    if(!app.parse(argv[1])) {
      ERRORMSG("Error parsing file " << argv[1])
    }
  } else if(argc > 2) {
    int iargc=0;
    while(iargc<argc) {
      if(!strcmp(argv[iargc],"-f")) {
	if(!app.parse(argv[++iargc])) {
          ERRORMSG("Error parsing file " << argv[iargc])
	}
      }
      iargc++;
    }
  } else {
    ERRORMSG("No input file is provided.")
    ERRORMSG("Usage: ohmms.x input-xml")
  }

  /* Clean up everything else before quitting. */
  xmlCleanupParser();
  OHMMS::Controller->finalize();
  return(0);
}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 166 $   $Date: 2004-11-04 17:37:11 -0600 (Thu, 04 Nov 2004) $
 * $Id: main.cpp 166 2004-11-04 23:37:11Z jnkim $ 
 ***************************************************************************/
