//////////////////////////////////////////////////////////////////
// (c) Copyright 2003- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifndef OHMMS_ARPACKSPARSESOLVER_IO_H
#define OHMMS_ARPACKSPARSESOLVER_IO_H

#include "OhmmsNumeric/HDFNumericAttrib.h"
#include "OhmmsNumeric/ArpackSparseSolvers.h"
#include "OhmmsData/ParameterSet.h"

namespace OHMMS {

  /** A set of parameters for multiple spectrum case 
   */
  struct ArpackParam {
    ///the number of Ritz vectors  
    int ncv;
    ///the maximum iterations
    int maxiter;
    ///the constant to shift the diagonal elements H -> H + shfift I
    double shift;
    ///the tolerance for the solver
    double  tol;
    ///the spectrum option accepted by Arpack library (SM|SA|..)
    string which;
  
    ///default constructor
    ArpackParam(): ncv(2), maxiter(1000), shift(0.0), tol(1e-8), which("SM") { } 

    ///copy constructor
    ArpackParam(const ArpackParam& a):
      ncv(a.ncv), maxiter(a.maxiter),shift(a.shift),tol(a.tol),which(a.which) { }
  
    ///initialize the parameters
    bool put(xmlNodePtr cur) {
      ParameterSet m_param;
      m_param.add(which,"spectrum","string");
      m_param.add(ncv,"nstates","int");
      m_param.add(tol,"tolerance","double");
      m_param.add(maxiter,"maxiter","int");
      m_param.add(shift,"eshift","double");
      m_param.put(cur);
      m_param.get(log());

      return true;
    }

    ///copy the parameters to the solver parameters
    void assign(ArpackSolver_t& x) {
      x.M = ncv;
      x.maxiter = maxiter;
      x.sigma = shift;
      x.tol = tol;
      x.which = which;
    }
  };
}
///Not the most elegant way but will work
#if defined(USE_PARPACK)
struct HDFAttribIO<ParpackSymSolver<double,Matrix<double> > >: 
  public HDFAttribIOBase {
  typedef ParpackSymSolver<double,Matrix<double> > data_type;
#else
#if defined(ENABLE_OPENMP)
template<>
struct HDFAttribIO<ArpackSymSolverOMP<double,Matrix<double> > >: 
  public HDFAttribIOBase {
  typedef ArpackSymSolverOMP<double,Matrix<double> > data_type;
#else
template<>
  struct HDFAttribIO<ArpackSymSolver<double,Matrix<double> > >: 
  public HDFAttribIOBase {
  typedef ArpackSymSolver<double,Matrix<double> > data_type;
#endif
#endif
  data_type&  ref;
  double shift;

  HDFAttribIO<data_type>(data_type& a):ref(a) { }

  inline void write(hid_t  hdfFile, const char* name) {

    hsize_t m = ref.M;
    hsize_t n = ref.N;

    string mname(name);
    mname.append("/NumStates");
    HDFAttribIO<hsize_t> m_out(m);
    m_out.write(hdfFile,mname.c_str());

    string nname(name);
    nname.append("/NumBasis");
    HDFAttribIO<hsize_t> n_out(n);
    n_out.write(hdfFile,nname.c_str());

    string ename(name);
    ename.append("/EigValue");
    hid_t dataspace  = H5Screate_simple(1, &m, NULL);
    hid_t dataset =  
      H5Dcreate(hdfFile, ename.c_str(), H5T_NATIVE_DOUBLE, dataspace, H5P_DEFAULT);
    hid_t ret = 
      H5Dwrite(dataset,H5T_NATIVE_DOUBLE,H5S_ALL,H5S_ALL,H5P_DEFAULT,ref.eigVal.data());
    H5Sclose(dataspace);
    H5Dclose(dataset);

    hsize_t dims = n;
    double* ev_ptr = ref.eigVec.data();    
    for(int ieig=0; ieig<m; ieig++, ev_ptr += n) {
      char vname[128];
      sprintf(vname,"%s%s%d03",name,"/EigVector",ieig);
      hid_t dataspacev  = H5Screate_simple(1, &dims, NULL);
      hid_t datasetv =  
        H5Dcreate(hdfFile, vname, H5T_NATIVE_DOUBLE, dataspacev, H5P_DEFAULT);
      hid_t retv = 
        H5Dwrite(datasetv, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT,ev_ptr);
      H5Sclose(dataspacev);
      H5Dclose(datasetv);
    }

//     string gname(name);
//     gname.append("/EigVector");
//     hid_t grp = H5Gcreate(hdfFile, gname.c_str(), 0);
//     for(int i=0; i<m; i++) {
//       char vname[128];
//       sprintf(vname,"%s/%d",gname.c_str(),i);
//       hid_t dataspace1  = H5Screate_simple(1, &n, NULL);
//       hid_t dataset1 =  H5Dcreate(hdfFile, vname, H5T_NATIVE_DOUBLE, dataspace1, H5P_DEFAULT);
//       H5Dwrite(dataset1, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT,
// 	       ref.eigVec.data()+i*n);
//       H5Sclose(dataspace1);
//       H5Dclose(dataset1);
//     }
//     H5Gclose(grp);
  }

  inline void read(hid_t  grp, const char* name) {
//     hid_t h1 = H5Dopen(grp, name);
//     hid_t ret = H5Dread(h1, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT, ref.begin());
//     H5Dclose(h1);
  }

};

#endif
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 210 $   $Date: 2004-12-22 10:45:18 -0600 (Wed, 22 Dec 2004) $
 * $Id: ArpackSolverIO.h 210 2004-12-22 16:45:18Z jnkim $
 ***************************************************************************/
