//////////////////////////////////////////////////////////////////
// (c) Copyright 2003- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "Utilities/OhmmsInfo.h"
#include "OhmmsNumeric/ArpackSparseSolvers.h"
#include "SparseMatrix/SparseMatrix2MatrixCopy.h"
using namespace OHMMS;

ArpackSparseSolver::ArpackSparseSolver(EigenSpaceDimension& space): 
  SymEigenSolver(space), 
  ArpackSolver_t(space), 
  ArpackCheck(false),
  accumShift(0.0)
 {  
   //Psi.resize(M,N);
 }

ArpackSparseSolver::~ArpackSparseSolver() {
  //for(int i=0; i<Spectrum.size(); i++) delete Spectrum[i];
}

void 
ArpackSparseSolver::getEigvectors(SparseMatrix_t& hmat,RealMatrix_t& E){

  //WARNMSG("Incomplete implementation of ArpackSparseSolver::getEigvectors")
  WARNMSG("Using ArpackSparseSolver::getEigvalues")
  getEigvalues(hmat,E);
  //solve(hmat,Psi);
}

void 
ArpackSparseSolver::getEigvalues(SparseMatrix_t& hmat, RealMatrix_t& E){

  //Add the diagonal elements
  real_type d = accumShift-sigma;

  LOGMSG("Checking the diagonal shift " << sigma << " actual shift = " << d)

  if(fabs(d)>1e-10) hmat+= d;
  
  solve(hmat);

  if(ArpackCheck) checkEigvectors(hmat,cout);

  eigVal += sigma;
  accumShift = sigma;

//   ///if spectrum is not specificed. Peform SM with zero shift
//   if(Spectrum.empty()) {
//     Spectrum.push_back(new ArpackParam());
//   }

//   N = States.N;
//   real_type lambda = 0.0;

//   if(States.M == States.N) {
//     M = 0;
//     for(int i=0; i<Spectrum.size(); i++) {
//       M = max(M,Spectrum[i]->ncv);
//     }
//     States.M = M;
//   } else {
//     M = States.M;
//   }
//   Psi.resize(M, N);

//   string fname = FileRoot;

//   fname.append(".h5");
//   hid_t hdfFile;
//   hdfFile =  H5Fopen(fname.c_str(),H5F_ACC_RDWR,H5P_DEFAULT);
//   if(hdfFile<0)
//     hdfFile = H5Fcreate(fname.c_str(), H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);

//   HDFAttribIO<ArpackSolver_t> out(*this);

//   for(int i=0; i<Spectrum.size(); i++) {
//     Spectrum[i]->assign(*this);
//     real_type d = lambda - sigma;
//     if(fabs(d)>1e-10) hmat+= d;
//     solve(hmat);

//     for(int is=0; is<Spectrum[i]->ncv; is++)  E[0][is] = eigval(is)+sigma;

//     if(ArpackCheck) checkEigvectors(hmat,cout);

//     eigVal += sigma;
//     lambda = sigma;
//     char gname[128];
//     sprintf(gname,"/Spectrum%d",i);
//     hid_t grp = H5Gcreate(hdfFile, gname, 0);
//     out.write(hdfFile,gname);
//     H5Gclose(grp);
//   }

//   H5Fclose(hdfFile);
}

void 
ArpackSparseSolver::getEigvectors(SparseMatrix_t& hmat, 
				  SparseMatrix_t& smat, 
				  RealMatrix_t& E) {
  WARNMSG("Incomplete implementation of ArpackSparseSolver::getEigvectors")
  WARNMSG("Using ArpackSparseSolver::getEigvalues")
  getEigvalues(hmat,smat,E);
}

void 
ArpackSparseSolver::getEigvalues(SparseMatrix_t& hmat, 
				 SparseMatrix_t& smat, 
				 RealMatrix_t& E) {
  //Add the diagonal elements
  real_type d = accumShift-sigma;

  LOGMSG("Checking the diagonal shift " << sigma << " actual shift = " << d)

  if(fabs(d)>1e-10) hmat+= d;
  
  solve(hmat,smat);

  if(ArpackCheck) checkEigvectors(hmat,cout);

  eigVal += sigma;
  accumShift = sigma;

//   ///if spectrum is not specificed. Peform SM with zero shift
//   if(Spectrum.empty()) {
//     Spectrum.push_back(new ArpackParam());
//   }

//   N = States.N;
//   real_type lambda = 0.0;

//   HDFAttribIO<ArpackSolver_t> out(*this);

//   N = States.N;
//   if(States.M == States.N) {
//     M = 0;
//     for(int i=0; i<Spectrum.size(); i++) {
//       M = max(M,Spectrum[i]->ncv);
//     }
//   } else {
//     M = States.M;
//   }
//   Psi.resize(M, N);
  
//   string fname = FileRoot;
//   fname.append(".h5");
//   hid_t hdfFile;
//   hdfFile =  H5Fopen(fname.c_str(),H5F_ACC_RDWR,H5P_DEFAULT);

//   if(hdfFile<0)
//     hdfFile = H5Fcreate(fname.c_str(), H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);

//   for(int i=0; i<Spectrum.size(); i++) {
//     Spectrum[i]->assign(*this);
//     real_type d = lambda - sigma;
//     if(fabs(d)>1e-10) hmat.add(smat,d);
//     solve(hmat,smat);
//     for(int is=0; is<Spectrum[i]->ncv; is++)  E[0][is] = eigval(is)+sigma;

//     if(ArpackCheck) checkEigvectors(hmat,smat,cout);

//     eigVal += sigma;
//     lambda = sigma;
//     char gname[128];
//     sprintf(gname,"/Spectrum%d",i);
//     hid_t grp = H5Gcreate(hdfFile, gname, 0);
//     out.write(hdfFile,gname);
//     H5Gclose(grp);
//   }
//   H5Fclose(hdfFile);
}

bool 
ArpackSparseSolver::put(xmlNodePtr cur){

  //Nothing to do
//   int mynode = Controller->mycontext();
//   int pid = mynode;
//   xmlChar *p = xmlGetProp(cur,(const xmlChar*)"node");
//   if(p) {
//     pid = atoi((const char*)p);
//   }

//   if(pid == mynode) {
//     int item = Spectrum.size();
//     Spectrum.push_back(new ArpackParam());
//     Spectrum[item]->put(cur);
//   }
  return true;
}

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 180 $   $Date: 2004-11-12 13:34:26 -0600 (Fri, 12 Nov 2004) $
 * $Id: ArpackSparseSolvers.cpp 180 2004-11-12 19:34:26Z jnkim $
 ***************************************************************************/
