//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2003 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

/***************************************************************************
 *
 ***************************************************************************/
#include "OhmmsNumeric/EsslSymSparseSolvers.h"
#include "SparseMatrix/SparseMatrix2MatrixCopy.h"
using namespace OHMMS;

EsslSym::EsslSym(EigenSpaceDimension& space): 
  SymEigenSolver(space), Solver1(NULL), Solver2(NULL) 
{  }

EsslSym::~EsslSym() {
  if(Solver1) delete Solver1;
  if(Solver2) delete Solver2;
}

void 
EsslSym::getEigvectors(SparseMatrix_t& hmat, RealMatrix_t& E) {

  if(!Solver1) {
    Solver1 = new PackedSolver_t(States,false);
    H1.resize(States.N,States.N);
  }

  if(!Psi.size()) {
    if(States.N/States.M>5) {
      Psi.resize(States.M, States.N);
    } else {
      Psi.resize(States.N, States.N);
    }
  }
  Solver1->iopt = 1; 
  copy(hmat,H1); 
  Solver1->solve(H1.data(), Psi.data()); 
  for(int is=0; is<E.cols(); is++)  E[0][is] = Solver1->eigval(is);
}

void 
EsslSym::getEigvalues(SparseMatrix_t& hmat, RealMatrix_t& E) {
  if(!Solver1) {
    Solver1 = new PackedSolver_t(States,false);
    H1.resize(States.N,States.N);
  }
  Solver1->iopt = 0; 
  copy(hmat,H1); 
  Solver1->solve(H1.data(), H1.data());
  for(int is=0; is<E.cols(); is++)  E[0][is] = Solver1->eigval(is);
}

void 
EsslSym::getEigvectors(SparseMatrix_t& hmat, 
		       SparseMatrix_t& smat, 
		       RealMatrix_t& E) {
  if(!Solver2) {
    Solver2 = new UnpackedSolver_t(States,true);
    H2.resize(States.N,States.N);
    S2.resize(States.N,States.N);
  }
  if(!Psi.size()) {
    Psi.resize(States.N, States.N);
  }
  Solver2->iopt = 1; 
  copy(hmat,H2); 
  copy(smat,S2); 
  Solver2->solve(H2.data(), S2.data(), Psi.data());
  for(int is=0; is<E.cols(); is++)  E[0][is] = Solver2->eigval(is);
}

void 
EsslSym::getEigvalues(SparseMatrix_t& hmat, 
		      SparseMatrix_t& smat, 
		      RealMatrix_t& E) {
  if(!Solver2) {
    Solver2 = new UnpackedSolver_t(States,true);
    H2.resize(States.N,States.N);
    S2.resize(States.N,States.N);
  }
  Solver2->iopt = 0; 
  copy(hmat,H2); 
  copy(smat,S2); 
  Solver2->solve(H2.data(), S2.data(), NULL);
  for(int is=0; is<E.cols(); is++)  E[0][is] = Solver2->eigval(is);
 }


/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: EsslSymSparseSolvers.cpp 151 2004-11-02 23:51:14Z jnkim $
 ***************************************************************************/
