//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifndef OHMMS_HDF_NUMERICATTRIBIO_H
#define OHMMS_HDF_NUMERICATTRIBIO_H
#include "OhmmsData/HDFAttribIO.h"
#include "OhmmsPETE/OhmmsVector.h"
#include "OhmmsPETE/OhmmsMatrix.h"

// specialization for ParticleAttrib<int> type
template<>
struct HDFAttribIO<hsize_t>: public HDFAttribIOBase {

  hsize_t& ref;

  HDFAttribIO<hsize_t>(hsize_t& a):ref(a) { }

  inline void write(hid_t  hdfFile, const char* name) {
    hsize_t dim = 1;
    hid_t dataspace  = H5Screate_simple(1, &dim, NULL);
    hid_t dataset =  
      H5Dcreate(hdfFile, name, H5T_NATIVE_INT, dataspace, H5P_DEFAULT);
    hid_t ret = 
      H5Dwrite(dataset, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT,&ref);
    H5Sclose(dataspace);
    H5Dclose(dataset);
  }

  inline void read(hid_t  grp, const char* name) {
    hid_t h1 = H5Dopen(grp, name);
    hid_t ret = H5Dread(h1, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, &ref);
    H5Dclose(h1);
  }
};

// specialization for ParticleAttrib<int> type
template<>
struct HDFAttribIO<Vector<int> >: public HDFAttribIOBase {

  typedef Vector<int> ArrayType_t;
  ArrayType_t&  ref;

  HDFAttribIO<ArrayType_t>(ArrayType_t& a):ref(a) { }

  inline void write(hid_t  hdfFile, const char* name) {
    hsize_t dim = ref.size();
    hid_t dataspace  = H5Screate_simple(1, &dim, NULL);
    hid_t dataset =  
      H5Dcreate(hdfFile, name, H5T_NATIVE_INT, dataspace, H5P_DEFAULT);
    hid_t ret = 
      H5Dwrite(dataset, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT,ref.begin());
    H5Sclose(dataspace);
    H5Dclose(dataset);
  }

  inline void read(hid_t  grp, const char* name) {
    hid_t h1 = H5Dopen(grp, name);
    hid_t ret = H5Dread(h1, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, ref.begin());
    H5Dclose(h1);
  }
};

// specialization for ParticleAttrib<int> type
template<>
struct HDFAttribIO<vector<int> >: public HDFAttribIOBase {

  typedef vector<int> ArrayType_t;
  ArrayType_t&  ref;

  HDFAttribIO<ArrayType_t>(ArrayType_t& a):ref(a) { }

  inline void write(hid_t  hdfFile, const char* name) {
    hsize_t dim = ref.size();
    hid_t dataspace  = H5Screate_simple(1, &dim, NULL);
    hid_t dataset =  
      H5Dcreate(hdfFile, name, H5T_NATIVE_INT, dataspace, H5P_DEFAULT);
    hid_t ret = 
      H5Dwrite(dataset, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT,&(ref[0]));
    H5Sclose(dataspace);
    H5Dclose(dataset);
  }

  inline void read(hid_t  grp, const char* name) {
    hid_t h1 = H5Dopen(grp, name);
    hid_t ret = H5Dread(h1, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, &(ref[0]));
    H5Dclose(h1);
  }
};

// specialization for double type
template<>
struct HDFAttribIO<Vector<double> >: public HDFAttribIOBase {

  typedef Vector<double> ArrayType_t;
  ArrayType_t&  ref;

  HDFAttribIO<ArrayType_t>(ArrayType_t& a):ref(a) { }

  inline void write(hid_t  hdfFile, const char* name) {

    hsize_t dim = ref.size();
    hid_t dataspace  = H5Screate_simple(1, &dim, NULL);
    hid_t dataset =  
      H5Dcreate(hdfFile, name, H5T_NATIVE_DOUBLE, dataspace, H5P_DEFAULT);
    hid_t ret = 
      H5Dwrite(dataset, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT,ref.begin());
    H5Sclose(dataspace);
    H5Dclose(dataset);
  }

  inline void read(hid_t  grp, const char* name) {
    hid_t h1 = H5Dopen(grp, name);
    hid_t ret = H5Dread(h1, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT, ref.begin());
    H5Dclose(h1);
  }

};

template<>
struct HDFAttribIO<Matrix<double> >: public HDFAttribIOBase {

  bool SplitRows;
  typedef Matrix<double> ArrayType_t;
  ArrayType_t&  ref;
  
  HDFAttribIO<ArrayType_t>(ArrayType_t& a, bool split=false):ref(a),SplitRows(split) { }

  inline void write(hid_t  hdfFile, const char* name) {
    if(SplitRows) {
      hsize_t dims = ref.cols();
      for(int irow=0; irow<ref.rows(); irow++) {
	hid_t dataspace  = H5Screate_simple(1, &dims, NULL);
	char aname[128];
        sprintf(aname,"%s%d03",name,irow);
	hid_t dataset =  
	  H5Dcreate(hdfFile, aname, H5T_NATIVE_DOUBLE, dataspace, H5P_DEFAULT);
	hid_t ret = 
	  H5Dwrite(dataset, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT,ref.data());
	H5Sclose(dataspace);
	H5Dclose(dataset);
      }
    } else {
      hsize_t dims[2];
      dims[0] = ref.rows(); dims[1] = ref.cols();
      hid_t dataspace  = H5Screate_simple(2, dims, NULL);
      hid_t dataset =  
	H5Dcreate(hdfFile, name, H5T_NATIVE_DOUBLE, dataspace, H5P_DEFAULT);
      hid_t ret = 
	H5Dwrite(dataset, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT,ref.data());
      H5Sclose(dataspace);
      H5Dclose(dataset);
    }
  }

  inline void read(hid_t  grp, const char* name) {

    hid_t h1 = H5Dopen(grp, name);
    if(h1<0) return;

    hid_t dataspace = H5Dget_space(h1);
    hsize_t dims_out[2];
    int rank = H5Sget_simple_extent_ndims(dataspace);
    int status_n = H5Sget_simple_extent_dims(dataspace, dims_out, NULL);
    ref.resize(dims_out[0],dims_out[1]);

    //hid_t h1 = H5Dopen(grp, name);
    hid_t ret = H5Dread(h1, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT, ref.data());
    H5Sclose(dataspace);
    H5Dclose(h1);

//     hid_t h1 = H5Dopen(grp, name);
//     hid_t ret = H5Dread(h1, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT, ref.data());
//     H5Dclose(h1);
  }

};



#endif
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 209 $   $Date: 2004-12-22 10:44:15 -0600 (Wed, 22 Dec 2004) $
 * $Id: HDFNumericAttrib.h 209 2004-12-22 16:44:15Z jnkim $ 
 ***************************************************************************/
