//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2003 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

/***************************************************************************
 *
 ***************************************************************************/
#include "OhmmsNumeric/LapackHerSparseSolvers.h"
using namespace OHMMS;

LapackHerA::~LapackHerA() {
  if(thisSolver) delete thisSolver;
}

void 
LapackHerA::getEigvectors(const SparseMatrix_t& hmat, 
			  const ComplexMatrix_t& phi,
			  RealMatrix_t& E, 
			  int first, int last) {
  if(!thisSolver) {
    thisSolver = new Solver_t(States,false);
    H.resize(States.N,States.N);
  }
  if(Psi.empty()) {
    for(int ik=0; ik<last-first; ik++)
      Psi.push_back(new ComplexMatrix_t(States.M, States.N));
  }
  int nk=0;
  thisSolver->Jobz = 'V';
  for(int ik=first; ik<last; ik++, nk++) {
    Sparse2DenseMatrix::copy(hmat,phi[nk],H); 
    thisSolver->solve(H.data(), Psi[nk]->data()); 
    for(int is=0; is<E.cols(); is++)  E[ik][is] = thisSolver->eigval(is);
  }
}

void 
LapackHerA::getEigvalues(const SparseMatrix_t& hmat, 
			 const ComplexMatrix_t& phi,
			 RealMatrix_t& E, 
			 int first, int last) {
  if(!thisSolver) {
    thisSolver = new Solver_t(States,false);
    H.resize(States.N,States.N);
  }

  thisSolver->Jobz = 'N';
  int nk=0;
  for(int ik=first; ik<last; ik++, nk++) {
    Sparse2DenseMatrix::copy(hmat,phi[nk],H); 
    thisSolver->solve(H.data(), H.data());
    for(int is=0; is<E.cols(); is++)  E[ik][is] = thisSolver->eigval(is);
  }
}

void 
LapackHerA::getEigvectors(const SparseMatrix_t& hmat, 
			  const SparseMatrix_t& smat, 
			  const ComplexMatrix_t& phi,
			  RealMatrix_t& E, 
			  int first, int last) {
  if(!thisSolver) {
    thisSolver = new Solver_t(States,true);
    H.resize(States.N,States.N);
    S.resize(States.N,States.N);
  }

  if(Psi.empty()) {
    for(int ik=0; ik<last-first; ik++)
      Psi.push_back(new ComplexMatrix_t(States.M, States.N));
  }
  int nk=0;
  thisSolver->Jobz = 'V';
  for(int ik=first; ik<last; ik++, nk++) {
    Sparse2DenseMatrix::copy(hmat,phi[nk],H); 
    Sparse2DenseMatrix::copy(smat,phi[nk],S); 
    thisSolver->solve(H.data(), S.data(), Psi[nk]->data()); 
    for(int is=0; is<E.cols(); is++)  E[ik][is] = thisSolver->eigval(is);
  }
}

void 
LapackHerA::getEigvalues(const SparseMatrix_t& hmat, 
			 const SparseMatrix_t& smat, 
			 const ComplexMatrix_t& phi,
			 RealMatrix_t& E, 
			 int first, int last) {
  if(!thisSolver) {
    thisSolver = new Solver_t(States,true);
    H.resize(States.N,States.N);
    S.resize(States.N,States.N);
  }
  int nk=0;
  thisSolver->Jobz = 'N';
  for(int ik=first; ik<last; ik++, nk++) {
    Sparse2DenseMatrix::copy(hmat,phi[nk],H); 
    Sparse2DenseMatrix::copy(smat,phi[nk],S); 
    thisSolver->solve(H.data(), S.data(), NULL);
    for(int is=0; is<E.cols(); is++)  {
      E[ik][is] = thisSolver->eigval(is);
    }
  }
}


/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 378 $   $Date: 2005-08-28 18:40:33 -0500 (Sun, 28 Aug 2005) $
 * $Id: LapackHerSparseSolvers.cpp 378 2005-08-28 23:40:33Z jnkim $
 ***************************************************************************/
