//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2003 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

/***************************************************************************
 *
 ***************************************************************************/
#include "OhmmsNumeric/SparseEigenSolverBuilder.h"
#include "OhmmsNumeric/LapackSymSparseSolvers.h"
#include "OhmmsNumeric/LapackHerSparseSolvers.h"
#if defined(USE_ARPACK) || defined(USE_PARPACK)
#include "OhmmsNumeric/ArpackSparseSolvers.h"
#endif
#if defined(USE_ESSL)
#include "OhmmsNumeric/EsslSymSparseSolvers.h"
#endif

OHMMS::SymEigenSolver* 
OHMMS::createSymSolver(EigenSpaceDimension& space, xmlNodePtr cur) {

#if defined(USE_ARPACK) || defined(USE_PARPACK)
 xmlNodePtr tcur = cur->xmlChildrenNode;

 while(tcur != NULL) {
   if ((!xmlStrcmp(tcur->name, (const xmlChar *)"Solver"))) {
     string aname = (const char*)(xmlGetProp(tcur, (const xmlChar *) "name"));
     if(aname == "arpack") {

       LOGMSG("Creating Arpack Sparse Solver")
       ArpackSparseSolver* asolver = new ArpackSparseSolver(space);
       asolver->put(cur->doc,cur->ns,cur);           
       return asolver;
     }
   }
   tcur = tcur->next;
 }
#endif

#if defined(USE_ESSL)
  return  new EsslSym(space);
#else
  return  new LapackSymA(space);
#endif
}
OHMMS::HerEigenSolver* 
OHMMS::createHerSolver(EigenSpaceDimension& space, xmlNodePtr cur) {
  return  new LapackHerA(space);
}


/*
void 
OHMMS::HerEigenSolver::partitionKmesh(KPointBase* KSet, xmlDocPtr doc) {

//   if(!ptclin->KSet) {
//     DEBUGMSG("\tCreating a kpoint set for a particle ")
//     xmlNodePtr cur = m_cur->xmlChildrenNode;
//     bool gamma = true;
//     while (cur != NULL && gamma) {
//       ///search for k-point
//       if ((!xmlStrcmp(cur->name, (const xmlChar *)"kpoint"))) {
// 	///file is given, use the file
// 	if(xmlHasProp(cur,(const xmlChar*)("file"))) {
// 	  std::ifstream 
// 	    kin((const char*)(xmlGetProp(cur, (const xmlChar *) "file")));
// 	  ptclin->KSet = createKPointMesh(*ptclin, kin);
// 	} else {
// 	  ///grid are given explicitly
// 	  xmlNodePtr subcur = cur->xmlChildrenNode;
// 	  SingleParticleIndex_t kgrid(1);
// 	  SingleParticlePos_t korigin;
// 	  while(subcur != NULL) {
// 	    if ((!xmlStrcmp(subcur->name, (const xmlChar *)"grid"))) {
// 	      putXMLNodeContent<SingleParticleIndex_t>::put(kgrid,m_doc,subcur);
// 	    }
// 	    if ((!xmlStrcmp(subcur->name, (const xmlChar *)"origin"))) {
// 	      putXMLNodeContent<SingleParticlePos_t>::put(korigin,m_doc,subcur);
// 	    }
// 	    subcur = subcur->next;
// 	  }
// 	  ptclin->KSet = new KPointBase;
// 	  ptclin->KSet->findSpaceGroup(*ptclin);
// 	  ptclin->KSet->buildKpoint(&kgrid[0],korigin);
// 	}
// 	gamma = false;
//  	if(EvaluateDensityMatrix)  ptclin->KSet->extend2FBZ();
//       }
//       cur = cur->next;
//     }
//     if(gamma) {
//       ptclin->KSet = new KPointBase;
//       ptclin->KSet->add(SingleParticlePos_t(0,0,0),1);
//     }
//     ///overwrite Gamma with the input data
//     Gamma = gamma;
//   }

  ///using XPath instead of recursive search
  xmlXPathContextPtr context;
  xmlXPathObjectPtr result;
  context = xmlXPathNewContext(doc);
  result = xmlXPathEvalExpression((const xmlChar*)"//kpoint",context);
  if(!xmlXPathNodeSetIsEmpty(result->nodesetval)) {
    xmlNodePtr cur = result->nodesetval->nodeTab[0];
    if(xmlHasProp(cur,(const xmlChar*)("distributed"))) {
      int nproc = Controller->ncontexts();
      if(nproc > 1) {
	Kgid  = Controller->mycontext();
	nK.resize(nproc);
	dnK.resize(nproc);
      }
    }
    if(!nK.empty() && !KSet->distributed()) {
      int nproc = nK.size();
      KSet->partition(nproc);
      for(int ip=0; ip<nproc; ip++) {
	nK[ip] = (KSet->end(ip)- KSet->begin(ip))*NumStates;
      }
      dnK[0] = 0;
      for(int ip=0; ip<nproc-1; ip++) {
	dnK[ip+1] = dnK[ip]+nK[ip];
      }
      Kstart = KSet->begin(Kgid);    
      Kend = KSet->end(Kgid);    
    } else {
      Kstart = 0;
      Kend = KSet->size();
    }
  }
}
*/

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: SparseEigenSolverBuilder.cpp 151 2004-11-02 23:51:14Z jnkim $
 ***************************************************************************/
