      SUBROUTINE abcdq(asym,a,bsym,b,q,n,m)
      integer n,m
      double precision asym,a,bsym,b,q
      DIMENSION asym(n,n), a(m,n), bsym(n,n), b(m,n)
      DIMENSION q(m,n)

      integer i,j,k
      double precision tmp1, tmp2

      Do i=1,n
         Do k=1,m
            q(k,i) = 0d0
         enddo
         Do j=1,n
            if(i .le. j) then
               tmp1 = asym(j,i)
               tmp2 = -bsym(j,i)
            else
               tmp1 = asym(i,j)
               tmp2 = -bsym(i,j)
            endif
            Do k=1,m
               q(k,i) = q(k,i) + tmp1*a(k,j) + tmp2*b(k,j)
            enddo
         enddo
      enddo

c$$$      do i=1,n
c$$$         do j=1,m
c$$$            tmp1 = asym(i,i)*a(j,i) - bsym(i,i)*b(j,i)
c$$$            do k=1,i-1
c$$$               tmp1 = tmp1 + asym(i,k)*a(j,k)-bsym(i,k)*b(j,k)
c$$$            enddo
c$$$            do k=i+1,n
c$$$               tmp1 = tmp1 + asym(k,i)*a(j,k)-bsym(k,i)*b(j,k)
c$$$            enddo
c$$$            q(j,i) = tmp1
c$$$         enddo
c$$$      enddo

      RETURN
      END


      SUBROUTINE atbsym(a,b,q,n,m)
      integer n,m
      double precision a,b,q
      DIMENSION a(m,n), b(m,n), q(n,n)
      integer ilow,i,j,k,nh
      double precision sum

      do i=1,n
         do j=i,n
            sum = 0d0
            do k=1,m
               sum = sum + a(k,i)*b(k,j)
            enddo
            q(j,i) = sum
         enddo
         do j=i+1,n
            q(i,j) = q(j,i)
         enddo
      enddo
      return
      end

      SUBROUTINE twoims(a,q,n,m)
      integer n,m
      double precision a,q
      DIMENSION a(m,n), q(n,n)
      integer i,j,k
      double precision sum

      do i=1,n
         sum = 0d0
         do k=1,m
            sum = sum + a(k,i)*a(k,i)
         enddo
         q(i,i) = 2d0-sum
         do j=i+1,n
            sum = 0d0
            do k=1,m
               sum = sum + a(k,i)*a(k,j)
            enddo
            q(j,i) = -sum
         enddo
      enddo
      return
      end


      function dotrace(a,b,n)
      double precision dotrace
      integer n
      double precision a,b
      dimension a(n,n), b(n,n)
      integer i,j,k
      double precision sum
      sum = 0.0

!$omp parallel do private(i,j,k) reduction(+:sum)
      do i=1,n
         sum = sum + 0.5*a(i,i)*b(i,i)
         do j=i+1,n
            sum = sum + a(j,i)*b(j,i)
         enddo
      enddo
      dotrace = 2.0d0*sum
      return 
      end

      subroutine xasymbc(asym, b, alpha,c,n,m)
      integer n,m
      double precision asym,b,c,alpha
      dimension asym(n,n), b(m,n), c(m,n)
      integer i,j,k
      double precision sum,tmp1

      do i=1,n
         Do k=1,m
            c(k,i) = 0d0
         enddo
         do j=1,n
            if(i .le. j) then
               tmp1 = asym(j,i)*alpha
            else
               tmp1 = asym(i,j)*alpha
            endif
            do k=1,m
               c(k,i) = c(k,i) + tmp1*b(k,j)
            enddo
         enddo
      enddo
c$$$      do i=1,n
c$$$         do j=1,m
c$$$            sum = 0d0
c$$$            do k=i,n
c$$$               sum = sum + asym(k,i)*b(j,k)
c$$$            enddo
c$$$            do k=1,i-1
c$$$               sum = sum + asym(i,k)*b(j,k)
c$$$            enddo
c$$$            c(j,i) = alpha*sum
c$$$         enddo
c$$$      enddo
      return 
      end

      subroutine matmult44(h, a, i0, b, j0, n, m)
      integer i0,j0,n,m
      double precision h, a, b
      dimension h(16), a(m,n), b(m,n)
      integer i1,i2,i3,j1,j2,j3
      i1=i0+1
      i2=i0+2
      i3=i0+3
      j1=j0+1
      j2=j0+2
      j3=j0+3
      do is=1,n
         b(j0,is) = b(j0,is)
     &             +h(1)*a(i0,is)+h(2)*a(i1,is)
     &             +h(3)*a(i2,is)+h(4)*a(i3,is)
         b(j1,is) = b(j1,is)
     &             +h(5)*a(i0,is)+h(6)*a(i1,is)
     &             +h(7)*a(i2,is)+h(8)*a(i3,is)
         b(j2,is) = b(j2,is)
     &             +h(9)*a(i0,is)+h(10)*a(i1,is)
     &             +h(11)*a(i2,is)+h(12)*a(i3,is)
         b(j3,is) = b(j3,is)
     &             +h(13)*a(i0,is)+h(14)*a(i1,is)
     &             +h(15)*a(i2,is)+h(16)*a(i3,is)
      enddo
      return
      end

