      SUBROUTINE abcdq(asym,a,bsym,b,q,n,m)
      integer n,m
      double precision asym,a,bsym,b,q
      DIMENSION asym(n,n), a(m,n), bsym(n,n), b(m,n)
      DIMENSION q(m,n)

      integer i,j,k
      double precision tmp1, tmp2

!$omp parallel do private(i,j,k,tmp1,tmp2)
      Do i=1,n
         Do k=1,m
            q(k,i) = 0d0
         enddo
         Do j=1,n
            if(i .le. j) then
               tmp1 = asym(j,i)
               tmp2 = -bsym(j,i)
            else
               tmp1 = asym(i,j)
               tmp2 = -bsym(i,j)
            endif
            Do k=1,m
               q(k,i) = q(k,i) + tmp1*a(k,j) + tmp2*b(k,j)
            enddo
         enddo
c$$$         do j=1,m
c$$$            tmp1 = asym(i,i)*a(j,i) - bsym(i,i)*b(j,i)
c$$$            do k=1,i-1
c$$$               tmp1 = tmp1 + asym(i,k)*a(j,k)-bsym(i,k)*b(j,k)
c$$$            enddo
c$$$            do k=i+1,n
c$$$               tmp1 = tmp1 + asym(k,i)*a(j,k)-bsym(k,i)*b(j,k)
c$$$            enddo
c$$$            q(j,i) = tmp1
c$$$         enddo
      enddo

      RETURN
      END


      SUBROUTINE atbsym(a,b,q,n,m)
      integer n,m
      double precision a,b,q
      DIMENSION a(m,n), b(m,n), q(n,n)
      integer ilow,i,j,k,nh
      double precision sum

      if(mod(n,2) .ne. 0) then
         i = n/2+1
         do j=i,n
            sum = 0d0
            do k=1,m
               sum = sum + a(k,i)*b(k,j)
            enddo
            q(j,i) = sum
         enddo
      endif
!$omp parallel do private(i,ilow,j,k,sum)
      do i=1,n/2
         do j=i,n
            sum = 0d0
            do k=1,m
               sum = sum + a(k,i)*b(k,j)
            enddo
            q(j,i) = sum
         enddo
         ilow = n+1-i
         do j=ilow,n
            sum = 0d0
            do k=1,m
               sum = sum + a(k,ilow)*b(k,j)
            enddo
            q(j,ilow) = sum
         enddo
      enddo
      return
      end

      SUBROUTINE twoims(a,q,n,m)
      integer n,m
      double precision a,q
      DIMENSION a(m,n), q(n,n)
      integer i,j,k
      double precision sum

!$omp parallel do private(i,j,k,sum)
      do i=1,n
         sum = 0d0
         do k=1,m
            sum = sum + a(k,i)*a(k,i)
         enddo
         q(i,i) = 2d0-sum
         do j=i+1,n
            sum = 0d0
            do k=1,m
               sum = sum + a(k,i)*a(k,j)
            enddo
            q(j,i) = -sum
         enddo
      enddo
      return
      end


      function dotrace(a,b,n)
      double precision dotrace
      integer n
      double precision a,b
      dimension a(n,n), b(n,n)
      integer i,j,k
      double precision sum
      sum = 0.0

!$omp parallel do private(i,j,k) reduction(+:sum)
      do i=1,n
         sum = sum + 0.5*a(i,i)*b(i,i)
         do j=i+1,n
            sum = sum + a(j,i)*b(j,i)
         enddo
      enddo
      dotrace = sum
      return 
      end

      subroutine xabc(a, b, alpha,c,n,m)
      integer n,m
      double precision a,b,c,alpha
      dimension a(n,n), b(m,n), c(m,n)
      integer i,j,k
      double precision sum

!$omp parallel do private(i,j,k,sum)
      do i=1,n
         do j=1,m
            sum = 0d0
            do k=i,n
               sum = sum + a(k,i)*b(j,k)
            enddo
            do k=1,i-1
               sum = sum + a(i,k)*b(j,k)
            enddo
            c(j,i) = alpha*sum
         enddo
      enddo
      return 
      end
