//////////////////////////////////////////////////////////////////
// (c) Copyright 2003  by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "OhmmsElementSet.h"
#include "ProjectData.h"
#include "ParticleSets.h"
#include "ParticleLayoutManager.h"
#include <qlistview.h>

namespace OHMMS {

  OhmmsElementSet::OhmmsElementSet()
 {

    ///Default nodes: the order is related to the  MainKeys
    ///\warning When this order is changed, the enumeration should be also changed.
    addNode(new ProjectData);

    ParticleSets* pset = new ParticleSets;
    addNode(pset);
    addNode(new ParticleLayoutManager(*pset));
  }

  OhmmsElementSet::~OhmmsElementSet() {
    for(int i=0; i<Nodes.size(); i++) delete Nodes[i];
  }

  OhmmsElementBase* OhmmsElementSet::getNode(const std::string& aname){
    std::map<std::string,int>::iterator it =  Keys.find(aname);
    if(it == Keys.end()) {
      return NULL;
    } else {
      int inode = (*it).second;
      return Nodes[inode];
    }  
  }

  /*!\fn void OhmmsElementSet::addNode(OhmmsElementBase* o)
   *\param o an OhmmsElementBase
   *\brief Add an OhmmsElementBase if it's tag is not yet used.
   */
  void OhmmsElementSet::addNode(OhmmsElementBase* o) {
    map<string,int>::iterator it =  Keys.find(o->getName());
    if(it == Keys.end()) {
      Keys[o->getName()] = Nodes.size();
      Nodes.push_back(o);
    }
  }
  
  /*!\fn   bool OhmmsElementSet::put(xmlNodePtr cur) 
   *\param cur the root of the current xmlNode
   *\brief Process the children nodes if their tags are found.
   */
  bool OhmmsElementSet::put(xmlNodePtr cur) {

    cur = cur->xmlChildrenNode;    
    while (cur != NULL) {
      QString cname((const char*)(cur->name));
      if(cname != "comment") {
        /**This is to ensure the main tags start with a upper-case char */
        //cname[0] = toupper(cname[0]);
        OhmmsElementBase* anode = getNode(cname.ascii());
        if(anode) { 
          DEBUGMSG("XML tag " << cname << " is processed")
          anode->put(cur);
        }
      }
      cur = cur->next;
    }
    return true;
  }

  void 
  OhmmsElementSet::buildTree(QListViewItem *parentItem, xmlNodePtr parentNode) {

    QListViewItem *thisItem = 0;
    xmlNodePtr cur=parentNode->children;
    while(cur != NULL) {
      QString cname((const char*)(cur->name));
      if(cname != "comment") {
        OhmmsElementBase* anode = getNode(cname.ascii());
        if(anode) { 
          anode->put(cur);
	}
	if(parentItem != 0) {
	  if(cname == "text") {
	    parentItem->setText(1,(const char*)(cur->content));
	  } else if(cname == "parameter"){
	    thisItem = new QListViewItem(parentItem,thisItem);
	    ///process attributes: type or format
	    xmlAttrPtr att = cur->properties;
	    while(att != NULL) {
	      QString aname((const char*)(att->name));
	      if(aname == "role") {
		thisItem->setText(0,(const char*)(att->children->content));
	      } else if(aname == "condition") {
		thisItem->setText(2,(const char*)(att->children->content));
	      }
	      att = att->next;
	    }
	    thisItem->setText(1,(const char*)(cur->children->content));
	  } else {

	    thisItem = new QListViewItem(parentItem,thisItem);
	    thisItem->setText(0,cname);

	    buildTree(thisItem,cur);

	    ///process attributes: type or format
	    xmlAttrPtr att = cur->properties;
	    while(att != NULL) {
	      QString aname((const char*)(att->name));
	      if(aname == "name") {
		thisItem->setText(1,(const char*)(att->children->content));
	      } else {
		new QListViewItem(thisItem,aname,(const char*)(att->children->content));
	      }
	      att = att->next;
	    }
	    
	  }
	}
      }
      cur = cur->next;
    }
  }

  ///write to a ostream
  bool OhmmsElementSet::get(std::ostream& ) const {
    return true;
  }
  
  ///read from istream
  bool OhmmsElementSet::put(std::istream& ) {
    return true;
  }

  void OhmmsElementSet::reset() {
  }
}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: OhmmsElementSet.cpp 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/

