#include "OhmmsQt/ParticleScene.h"
#include "Particle/NNEngine.h"
#include <iostream>
using namespace qglviewer;
using namespace std;

ParticleScene::ParticleScene():mySys(NULL) {

  Lattice.resize(8);
  UnitLattice.resize(8);

  Lattice[0] = PointType(-0.5,-0.5,-0.5);
  Lattice[1] = PointType( 0.5,-0.5,-0.5);
  Lattice[2] = PointType( 0.5, 0.5,-0.5);
  Lattice[3] = PointType(-0.5, 0.5,-0.5);
  Lattice[4] = PointType(-0.5,-0.5,0.5);
  Lattice[5] = PointType( 0.5,-0.5,0.5);
  Lattice[6] = PointType( 0.5, 0.5,0.5);
  Lattice[7] = PointType(-0.5, 0.5,0.5);

  UnitLattice[0] = SingleParticlePos_t(0.0,0.0,0.0);
  UnitLattice[1] = SingleParticlePos_t(1.0,0.0,0.0);
  UnitLattice[2] = SingleParticlePos_t(1.0,1.0,0.0);
  UnitLattice[3] = SingleParticlePos_t(0.0,1.0,0.0);

  UnitLattice[4] = SingleParticlePos_t(0.0,0.0,1.0);
  UnitLattice[5] = SingleParticlePos_t(1.0,0.0,1.0);
  UnitLattice[6] = SingleParticlePos_t(1.0,1.0,1.0);
  UnitLattice[7] = SingleParticlePos_t(0.0,1.0,1.0);

}


void ParticleScene::connectParticle(Particle_t* p) {

  mySys = p;

  SingleParticlePos_t pos, c(0.5);
  //   c = p->Lattice.a(0)+p->Lattice.a(1)+p->Lattice.a(2);
  //   c /= 3.0;
  //   Center = PointType(c[0],c[1],c[2]);
  pos = p->Lattice.toCart(c);
  Center = PointType(static_cast<float>(pos[0]),
		     static_cast<float>(pos[1]),
		     static_cast<float>(pos[2]));

  Scale = 0.0;
  for(int i=0; i<8; i++) {
    pos = p->Lattice.toCart(UnitLattice[i]);
    Lattice[i] = PointType(static_cast<float>(pos[0])-Center[0],
			   static_cast<float>(pos[1])-Center[1],
			   static_cast<float>(pos[2])-Center[2]);

    Scale = std::max(fabs(Lattice[i][0]),Scale);
    Scale = std::max(fabs(Lattice[i][1]),Scale);
    Scale = std::max(fabs(Lattice[i][2]),Scale);
  }

  invScale = 0.5/Scale;
  for(int i=0; i<8; i++) Lattice[i]*=invScale;
  Center *= invScale;

  if(R.size() != mySys->getTotalNum()) R.resize(mySys->getTotalNum());

  mySys->Lattice.convert2Cart(mySys->R);
  for(int i=0; i<mySys->getTotalNum(); i++) {
    R[i] = invScale*PointType(static_cast<float>(mySys->R[i][0]),
			      static_cast<float>(mySys->R[i][1]),
			      static_cast<float>(mySys->R[i][2]))-Center;

  }
}

void ParticleScene::drawBox()
{

  glColor3f(1,1,1);
  glLineWidth( 2.0 );
  glBegin( GL_LINE_LOOP );
  for(int i=0; i<4; i++)  
    glVertex3fv(Lattice[i].address()); //glVertex3f(  1.0,  0.5, -0.4 );
  glEnd();

  glBegin( GL_LINE_LOOP );
  for(int i=4; i<8; i++)  
    glVertex3fv(Lattice[i].address()); //glVertex3f(  1.0,  0.5, -0.4 );
  glEnd();
    
  glBegin( GL_LINES );
  for(int i=0; i<4; i++) {
    glVertex3fv(Lattice[i].address());
    glVertex3fv(Lattice[i+4].address());
  }
  glEnd();
}


void ParticleScene::drawParticles() {

  if(mySys) {
    if(showR.size() < mySys->getTotalNum()) {
      int iat=showR.size();
      //showR.resize(mySys->getTotalNum());
      for(int i=iat; i<mySys->getTotalNum(); i++)
	showR.push_back(gluNewQuadric());
	//showR.insert(i,gluNewQuadric());
    }

    glColor3f(0.0,0.0,1.0);
    for(int i=0; i<R.size(); i++) {
      glTranslatef(R[i][0],R[i][1],R[i][2]);
      gluSphere(showR[i],0.01,20,20);
      glTranslatef(-R[i][0],-R[i][1],-R[i][2]);
    }

    mySys->getPairlist(2.0);
    const Particle_t::PtclPairList_t& nnlist = mySys->getNNEngine()->getPairAttrib(0);

    glColor3f(1,1,0);
    glLineWidth( 2.0 );
    glBegin( GL_LINES );
    int nn=0;
    for(int i=0; i<nnlist.getLocalNum(); i++) {
      const float* rptr = R[i].address();
      //!< Returns a number of neighbors of the i-th ptcl.
      for(int nn=0; nn<nnlist.nadj(i); nn++) {
	int j = nnlist.iadj(i,nn);
	if(j>i) {
	  glVertex3fv(rptr);
	  glVertex3fv(R[j].address());
	}
      }
    }
    glEnd();
  }
}

// Draws a spiral
void ParticleScene::draw()
{

  //  drawBox();
  drawParticles();
/*
  const float nbSteps = 500.0;
  glBegin(GL_QUAD_STRIP);
  for (float i=0; i<nbSteps; ++i)
    {
      float ratio = i/nbSteps;
      float angle = 21.0*ratio;
      float c = cos(angle);
      float s = sin(angle);
      float r1 = 1.0 - 0.8*ratio;
      float r2 = 0.8 - 0.8*ratio;
      float alt = ratio - 0.5;
      const float nor = .5;
      const float up = sqrt(1.0-nor*nor);
      glColor3f(1-ratio, .2 , ratio);
      glNormal3f(nor*c, up, nor*s);
      glVertex3f(r1*c, alt, r1*s);
      glVertex3f(r2*c, alt+0.05, r2*s);
    }
  glEnd();
*/
}
