#include <iostream>
#include <qapplication.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qlabel.h>
#include <qstatusbar.h>
#include <qwhatsthis.h>
#include <qfiledialog.h>
#include <qtabwidget.h>
#include <qmessagebox.h>
#include <qpushbutton.h>
#include <qtoolbutton.h>
#include <qtooltip.h>
#include <qlineedit.h>
#include <qtextedit.h>
#include <qhbox.h>
#include <qradiobutton.h>
#include <qbuttongroup.h>
#include <qsplitter.h>
#include "OhmmsQt/ParticleSetWindow.h"
#include "OhmmsQt/fileopen.xpm"
#include "OhmmsQt/clear.xpm"
#include "OhmmsQt/plot32.xpm"
#include "OhmmsQt/OhmmsToolBar.h"
#include "OhmmsQt/ParticleSetEditor.h"
#include "ParticleIO/XMLParticleIO.h"
//#include "OhmmsQt/interface.h"
//#include "OhmmsQt/glbox.h"
//#include "OhmmsQt/globjwin.h"
#include "OhmmsQt/ParticleScene.h"
#include "OhmmsQt/multiView.h"

ParticleSetWindow::ParticleSetWindow()
{
  setDockEnabled(TornOff, TRUE);
  setRightJustification(TRUE);

  (void)initMenuBar();
  (void)statusBar();

  QHBox *tb = new QHBox(this);
  setCentralWidget(tb);
  tb->setSpacing(5);
  tb->setMargin(5);

  editSystem = new ParticleSetEditor(tb);

  // Create Splitters
  QSplitter *hSplit  = new QSplitter(Qt::Vertical,tb);
  //QSplitter *vSplit1 = new QSplitter(hSplit);
  //QSplitter *vSplit2 = new QSplitter(hSplit);
  
  // Create the scene
  //ParticleScene* s = new ParticleScene();
  viewSystem =  new ParticleScene();

  // Instantiate the viewers.
  //viewers.push_back(new Viewer(viewSystem,0,vSplit1));
  //viewers.push_back(new Viewer(viewSystem,1,vSplit1));
  //viewers.push_back(new Viewer(viewSystem,2,vSplit2));
  viewers.push_back(new Viewer(viewSystem,3,hSplit));
  //viewers.push_back(new Viewer(viewSystem,0,vSplit2));
  //viewers.push_back(new Viewer(viewSystem,1,vSplit2));
  //viewers.push_back(new Viewer(viewSystem,2,vSplit2));

  tb->setMinimumSize(2*editSystem->width(),editSystem->height());

  ///connect signal load with slotAddPlot  
  connect(this, SIGNAL(load(const QString&)), 
	  this, SLOT(slotLoad(const QString&)));

  ///connect signal load with slotAddPlot  
  connect(this, SIGNAL(save(const QString&)), 
	  this, SLOT(slotSave(const QString&)));

  initWhatsThis();
}

/*!
 *\brief File::Open menu pops up a FileDialog. With a valid file, emitting a signal to slotLoad(const QString&)
 */
void ParticleSetWindow::slotLoad() {

  QFileDialog fd;
  if(fd.exec() == QDialog::Accepted) {
    QString filename = fd.selectedFile();
    if(!filename.isEmpty()) 
      emit load(filename);
  }
}

/*!
 *\brief signal load(const QString&) is emitted. Read in using a parser.
 */
void 
ParticleSetWindow::slotLoad(const QString& fname) {

  theSystem.clear();

  XMLParticleParser ptcl_parser(theSystem);
  ptcl_parser.put(fname.ascii());

  editSystem->setConfiguration(&theSystem);
  viewSystem->connectParticle(&theSystem);
  for(int i=0; i<viewers.size(); i++) viewers[i]->updateGL();
}

/*!
 *\brief button save is pushed.
 */
void 
ParticleSetWindow::slotSave() {
  QString fname("generic");
  slotSave(fname);
}


/*!
 *\brief saveAs button is pushed.
 */
void ParticleSetWindow::slotSaveAs() {

  QString filename = QFileDialog::getSaveFileName(".",QString::null,this);
  if(!filename.isEmpty())  
    emit save(filename);
}

/*!
 *\brief save to a file.
 * 
 * - slotSave provide a generic file 
 * - signal save(const QString&) is emitted by slotSaveAs
 */
void 
ParticleSetWindow::slotSave(const QString& fname) {
  XMLSaveParticle ptclsave(theSystem);
  ptclsave.reset(fname.ascii());
  ptclsave.report(0);
}

/*!
 *\brief initialize the menubar of the mainwindow
 */
QMenuBar* ParticleSetWindow::initMenuBar(){

  QMenuBar *_menubar = new QMenuBar(this);

  ///file-menu
  _filemenu = new QPopupMenu;
  _filemenu->insertItem("&Open",this,SLOT( slotLoad() ));
  _filemenu->insertItem("&Save",this,SLOT( slotSave() ));
  _filemenu->insertItem("Save &As",this, SLOT( slotSaveAs() ));
  _filemenu->insertItem("&Quit",qApp,SLOT( quit() ));

  ///add File and Dataset to the menu
  _menubar->insertItem("&File", _filemenu);


  ///file-menu
  _setmenu = new QPopupMenu;
  _setmenu->insertItem("&Open",this,SLOT( slotLoad() ));

  ///add File and Dataset to the menu
  _menubar->insertItem("&Set", _setmenu);

  return _menubar;
}


/**
 *\brief not used anymore but is kept as an example
 */
QToolBar *ParticleSetWindow::initToolBar()
{

  OhmmsToolBar *toolBar = new OhmmsToolBar(this);
  return toolBar;
}

/*!
 *\brief Add a whatsthis message for the buttons and menu items
 */
void ParticleSetWindow::initWhatsThis()
{
//   QWhatsThis::add(_plotBtn,
// 		  "Plot the selected dataset.\n"
// 		  "Load a data file and highlight the dataset to be plotted "
// 		  "on the left panel of a project page.");
}

