//////////////////////////////////////////////////////////////////
// (c) Copyright 2003  by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifdef HAVE_CONFIG_H
#include "ohmms-config.h"
#endif
#include "QXmlTreeBuilder.h"
#include <qimage.h>
#include <qpixmap.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qtextedit.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qprinter.h>
#include <qapplication.h>
#include <qaccel.h>
#include <qtextstream.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qwhatsthis.h>
#include <qsimplerichtext.h>
#include <qsplitter.h>
#include <qlistview.h>
#include <qlabel.h>
#include <qvbox.h>
#include "filesave.xpm"
#include "fileopen.xpm"
#include "fileprint.xpm"

/*! \brief Constructor 
 * Register OhmmsElementBase objects corresponding to the major tags of input xml files.
 * OhmmsElementSet contains default objects \ref ohmmsdefaults and the constructor
 * adds OhmmsElementBase subclasses that depend on the default objects.
 *
 */
XmlTreeBuilder::XmlTreeBuilder() 
  : QMainWindow( 0, "ohmms application main window", WDestructiveClose | WGroupLeader )
{
  QPixmap openIcon, saveIcon, printIcon;

  QToolBar * fileTools = new QToolBar( this, "file operations" );
  fileTools->setLabel( "File Operations" );

  openIcon = QPixmap( fileopen_xpm );
  QToolButton * fileOpen
    = new QToolButton( openIcon, "Open File", QString::null,
		       this, SLOT(choose()), fileTools, "open file" );

  saveIcon = QPixmap( filesave );
  QToolButton * fileSave
    = new QToolButton( saveIcon, "Save File", QString::null,
		       this, SLOT(save()), fileTools, "save file" );


  (void)QWhatsThis::whatsThisButton( fileTools );

  const char * fileOpenText = "<p><img source=\"fileopen\"> "
    "Click this button to open a <em>new file</em>.<br>"
    "You can also select the <b>Open</b> command "
    "from the <b>File</b> menu.</p>";

  QWhatsThis::add( fileOpen, fileOpenText );

  QMimeSourceFactory::defaultFactory()->setPixmap( "fileopen", openIcon );

  const char * fileSaveText = "<p>Click this button to save the file you "
    "are editing. You will be prompted for a file name.\n"
    "You can also select the <b>Save</b> command "
    "from the <b>File</b> menu.</p>";

  QWhatsThis::add( fileSave, fileSaveText );

  const char * filePrintText = "Click this button to print the file you "
    "are editing.\n"
    "You can also select the Print command "
    "from the File menu.";

  QPopupMenu * file = new QPopupMenu( this );
  menuBar()->insertItem( "&File", file );

  int id;
  id = file->insertItem( openIcon, "&Open...",
			 this, SLOT(choose()), CTRL+Key_O );
  file->setWhatsThis( id, fileOpenText );

  id = file->insertItem( saveIcon, "&Save",
			 this, SLOT(save()), CTRL+Key_S );
  file->setWhatsThis( id, fileSaveText );

  id = file->insertItem( "Save &As...", this, SLOT(saveAs()) );
  file->setWhatsThis( id, fileSaveText );

  file->insertSeparator();

  file->insertItem( "&Close", this, SLOT(close()), CTRL+Key_W );

  file->insertItem( "&Quit", qApp, SLOT( closeAllWindows() ), CTRL+Key_Q );

  menuBar()->insertSeparator();

  QPopupMenu * help = new QPopupMenu( this );
  menuBar()->insertItem( "&Help", help );

  help->insertItem( "&About", this, SLOT(about()), Key_F1 );
  help->insertSeparator();
  help->insertItem( "What's &This", this, SLOT(whatsThis()), SHIFT+Key_F1 );

  //d_view = new QSplitter( Qt::Vertical, this);
  //d_view = new QScrollView(this);
  d_view = new QVBox(this);
  setCentralWidget(d_view);

  nodeView = new QListView(d_view,"whatever");
  nodeView->addColumn("Node Name");
  nodeView->addColumn("Value");
  nodeView->addColumn("Unit");
  nodeView->setSorting(-1);
  nodeView->setRootIsDecorated(TRUE);

  resize( 450, 600 );
}

XmlTreeBuilder::~XmlTreeBuilder() {
}

void XmlTreeBuilder::closeEvent( QCloseEvent* ce )
{
    
  switch( QMessageBox::information( this, "Ohmms Session",
				    "Do you want to save the changes"
				    " to the document?",
				    "Yes", "No", "Cancel",
				    0, 1 ) ) {
  case 0:
    save();
    ce->accept();
    break;
  case 1:
    ce->accept();
    break;
  case 2:
  default: // just for sanity
    ce->ignore();
    break;
  }
}

void XmlTreeBuilder::resizeEvent(QResizeEvent *e) {
  d_view->resize(e->size());
}

void XmlTreeBuilder::choose()
{
  QString fn = QFileDialog::getOpenFileName( QString::null, QString::null,
					     this);
  if ( !fn.isEmpty() )
    load( fn );
  else
    statusBar()->message( "Loading aborted", 2000 );
}

void XmlTreeBuilder::load( const QString &fileName )
{
  parse(fileName.ascii());    
  setCaption( fileName );
  statusBar()->message( "Loaded document " + fileName, 2000 );
}


void XmlTreeBuilder::save() {

  QFile fout(d_outfile);
  fout.open(IO_WriteOnly);
  QTextStream str(&fout);
  m_dom.save(str,2);
  fout.close();
}

void XmlTreeBuilder::saveAs()
{
  QString fn = QFileDialog::getSaveFileName( QString::null, QString::null,
					     this );
  if ( !fn.isEmpty() ) {
    d_outfile = fn;
    save();
  } else {
    statusBar()->message( "Saving aborted", 2000 );
  }
}

void XmlTreeBuilder::about()
{
  QMessageBox::about( this, "Ohmms Session",
		      "This example demonstrates simple use of "
		      "QMainWindow,\nQMenuBar and QToolBar.");
}

/*!\fn bool XmlTreeBuilder::parse(const char* fname) {
 *\param fname an input xml file to be parsed.
 *\return true if successful.
 */
bool XmlTreeBuilder::parse(const char* fname) {

  d_infile=fname;d_outfile=fname;

  QFile xmlFile(fname);
  if(!xmlFile.open(IO_ReadOnly)) {
    QMessageBox::critical( 0,
			   tr( "Critical Error" ),
			   tr( "Cannot open file %1" ).arg( fname ) );
    return false;
  } 

  if ( !m_dom.setContent( &xmlFile ) ) {
    QMessageBox::critical( 0,
			   tr( "Critical Error" ),
			   tr( "Parsing error for file %1" ).arg( fname ) );
    xmlFile.close();
    return false;
  }

  xmlFile.close();

  QDomElement root = m_dom.documentElement();
  
  QFileInfo fi(fname);
  QListViewItem *a=new QListViewItem(nodeView,fi.fileName());

  //SimpleTreeBuilder(a,root);
  buildTree(a,root);
  nodeView->setOpen(a,true);
  nodeView->show();
  return true;
}

void 
XmlTreeBuilder::buildTree(QListViewItem *parentItem, const QDomElement& parentNode) {

  QListViewItem *thisItem = 0;
  QDomNode node = parentNode.firstChild();

  while ( !node.isNull() ) {
    if(node.isText()) {
      parentItem->setText(1,node.nodeValue());
    } else if(node.isElement()) {
      if(parentItem != 0) {
	if(node.nodeName() == "parameter"){
	  thisItem = new QListViewItem(parentItem,thisItem);
	  QDomNamedNodeMap att_map=node.attributes();
	  for(int i=0; i<att_map.count(); i++) {
	    QDomNode att=att_map.item(i);
	    if(att.nodeName() == "role") 
	      thisItem->setText(0,att.nodeValue());
	    else if(att.nodeName() == "condition") 
	      thisItem->setText(2,att.nodeValue());
	  }
	  if(!node.firstChild().isNull())
	    thisItem->setText(1,node.firstChild().nodeValue());
	} else {
	  thisItem = new QListViewItem(parentItem,thisItem);
	  thisItem->setText(0,node.nodeName());
	  buildTree(thisItem,node.toElement());
	  QDomNamedNodeMap att_map=node.attributes();
	  for(int i=0; i<att_map.count(); i++) {
	    QDomAttr att=att_map.item(i).toAttr(); //QDomNode att=att_map.item(i);
	    thisItem->setText(1,att.nodeValue());
	    
	  }
	}
      }
    }
    node = node.nextSibling();
  }
}


/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: QXmlTreeBuilder.cpp 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
