//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "Message/Communicate.h"
#include "TransformParticle.h"
#include "ParticleBase/ParticleFunctions.h"
#include "Message/OpenMP.h"

namespace OHMMS {
  bool TransformParticleFactory(vector<Particle_t*>& pin,
				Particle_t& pout, 
				xmlNodePtr cur) {

    string whattodo = (const char*)(xmlGetProp(cur, (const xmlChar *) "action"));
    int i=0;
    if(whattodo == "expand") {
      ExpandParticle expandit(*(pin.front()),pout);
      return expandit.put(cur);
    } 
    if(whattodo == "shape") {
      ShapeParticle shapeit(*(pin.front()),pout);
      return shapeit.put(cur);
    } 
    if(whattodo == "translate") {
      if(pout.getTotalNum()== 0) {
        pout.assign(*pin.front());
      }
      TranslateParticle translateit(pout,pout);
      return translateit.put(cur);
    }
    ///no transformation is done
    return false;
  }
}

using namespace OHMMS;

  bool TranslateParticle::put(xmlNodePtr cur) {

    Particle_t::SingleParticlePos_t tvec,displacement;
    string selectedset; 
    bool inunit = false;    

    if(xmlHasProp(cur, (const xmlChar *) "name")) {
      selectedset = (const char*)(xmlGetProp(cur, (const xmlChar *) "name"));
    }
    cur = cur->xmlChildrenNode;
    while (cur != NULL) {
      if(!xmlStrcmp(cur->name, (const xmlChar *)"parameter")) {
        string aname 
	  = (const char*)(xmlGetProp(cur, (const xmlChar *) "role"));
	if(aname == "displacement") {
	  putContent(tvec,cur);
	  if(xmlHasProp(cur, (const xmlChar*)"condition")) {
	    string u((const char*)(xmlGetProp(cur, (const xmlChar *) "condition")));
	    if(u == "lattice") {
	      inunit = true;
	    }
	  }
	} 
      }
      cur=cur->next;
    }
    displacement = tvec;
    if(out_.R.InUnit) {
      if(!inunit) displacement = out_.Lattice.toUnit(tvec);
      LOGMSG("Translating positions by " << displacement <<" (Unit-Cell unit)")
    } else {
      if(inunit) displacement = out_.Lattice.toCart(tvec);
      LOGMSG("Translating positions by " << displacement <<" (Cartesian Unit in AA)")
    }

    out_.R += displacement;
    out_.NeedPartition = false;
   
    return true;
  }


  bool ExpandParticle::put(xmlNodePtr cur) {

    typedef Particle_t::ParticleIndex_t       ParticleIndex_t;
    typedef Particle_t::ParticlePos_t         ParticlePos_t;
    typedef Particle_t::SingleParticleIndex_t SingleParticleIndex_t;
    typedef Particle_t::SingleParticlePos_t   SingleParticlePos_t;
    SingleParticleIndex_t uc_grid(1);

    const int mpigrid  = ParticleLayout_t::MPI_GRID;
    const int ompgrid  = ParticleLayout_t::OMP_GRID;
    const int finegrid = ParticleLayout_t::SPATIAL_GRID;
    const int gl       = ParticleLayout_t::GridLevel;
    bool grid_defined = false;
    bool distribute  = false;
    vector<SingleParticleIndex_t> grid(gl,SingleParticleIndex_t(1));
    string selectedset; 

    xmlAttrPtr att = cur->properties;
    while(att != NULL) {
      string aname((const char*)(att->name));
      string vname((const char*)(att->children->content));
      if(aname == "name") {
	selectedset = vname;
      } else if(aname == "distribute") {
	if(vname == "true" || vname == "1") distribute =true;
      }
      att = att->next;
    }
    cur = cur->xmlChildrenNode;
    while (cur != NULL) {
      if(!xmlStrcmp(cur->name, (const xmlChar *)"parameter")) {
        string aname = (const char*)(xmlGetProp(cur, (const xmlChar *) "role"));
	if(aname == "unitcell_grid") {
	  putContent(uc_grid,cur);
	} else if(aname == "grid") {
	  putContent(grid[finegrid],cur);
	  grid_defined = true;
	} else if(aname == "ompgrid") {
	  putContent(grid[ompgrid],cur);
	} else if(aname == "mppgrid") {
	  putContent(grid[mpigrid],cur);
	}
      }
      cur=cur->next;
    }

    ///\todogoing to use selectedset
    if(uc_grid[0] == 1 && uc_grid[1] == 1 && uc_grid[2] == 1) {
      WARNMSG("Expand function is requested with 1x1x1")
      return false;
    }

    WARNMSG("Expand function initialize R and ID only.")

      //     if(!grid_defined) {
      //       for(int idim=0; idim<OHMMS_DIM; idim++) 
      // 	grid[finegrid][idim] = uc_grid[idim]*in_.Lattice.ngrid(idim);
      //     }

    if(distribute) {
      ///Prepare lattice
      out_.Lattice.set(in_.Lattice,&uc_grid[0]);
      ParticleLayout_t& lattice = out_.Lattice;

      lattice.makeGrid(grid);

      UniformCartesianGrid<Scalar_t,OHMMS_DIM> SuperGrid;
      SuperGrid.setGrid(grid[mpigrid]);

      vector<int> PID(lattice.getTotalNum());
      int ng = 0;
      for(int ig=0; ig<lattice.ngrid(finegrid,0); ig++) {
	for(int jg=0; jg<lattice.ngrid(finegrid,1); jg++) {
	  for(int kg=0; kg<lattice.ngrid(finegrid,2); kg++) {
	    PID[ng++] = SuperGrid.loc(lattice.center(ig,jg,kg));
	  }
	}
      }

      in_.Lattice.convert2Unit(in_.R);

      int mycontext = Controller->mycontext();
      SingleParticlePos_t del;
      const ParticlePos_t& R = in_.R;
      const ParticleIndex_t& ionid = in_.GroupID;
      vector<int> myID;
      vector<SingleParticlePos_t> myR;
      vector<int> nsub_m(SuperGrid.getTotalNum(),0);
      for(int ic=0; ic<uc_grid[0]; ic++) {
	del[0] = static_cast<Scalar_t>(ic);
	for(int jc=0; jc<uc_grid[1]; jc++) {
	  del[1] = static_cast<Scalar_t>(jc);
	  for(int kc=0; kc<uc_grid[2]; kc++) {
	    del[2] = static_cast<Scalar_t>(kc);
	    for(int nb=0; nb<in_.getLocalNum(); nb++) {
	      SingleParticlePos_t pos 
		= lattice.toUnit(in_.Lattice.toCart(R[nb]+del));
	      int pid = PID[lattice.loc(pos)];
	      if(pid == mycontext) {
		myID.push_back(ionid[nb]);
		myR.push_back(pos);
	      }
	      nsub_m[pid]++;
	    }
	  }
	}
      }

      out_.clear();
      out_.create(myID.size());
      int nacc = 0;
      out_.R.InUnit = true;

      int offset = 0;
      for(int i=0; i<mycontext; i++) offset+=nsub_m[i];
      for(int iat=0; iat<myID.size(); iat++) {
        out_.GroupID[nacc] = myID[iat];
	out_.R[nacc] = myR[iat];
	out_.ID[nacc] = offset;
	++nacc; ++offset;
      }
      //out_.Distributed = true;
      out_.setMode(Particle_t::Distributed);
    } else {
      ExpandSuperCell(in_,out_,uc_grid,grid);
    }
    return true;
  }

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: TransformParticle.cpp 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
