//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
#include <stdio.h>

#include <math.h>
#include <iomanip>
#include <iostream>
#include <complex>

#if (__GNUC__ == 2)
#include <algo.h>
#else
#include <algorithm>
#endif
//using namespace std;

template<class T>
void shift (T& a, T& b, T& c, T d) {
  a = b; b= c; c = d;
}

double sign(double a, double b) {
  return (b > 0.0)? fabs(a): -fabs(a); 
}

//#define SHFT(a,b,c,d) (a)=(b);(b)=(c);(c)=(d);
//#define MAX(a,b) ((a) > (b) ? (a) : (b))

//#define SIGN(a,b) ((b) > 0.0 ? fabs(a) : -fabs(a))

template<class AT>
inline 
typename CGOptimization<AT>::Return_t 
CGOptimization<AT>::brentNRC(typename CGOptimization<AT>::Return_t ax, 
			     typename CGOptimization<AT>::Return_t bx,  
			     typename CGOptimization<AT>::Return_t cx, 
			     typename CGOptimization<AT>::Return_t& xmin) {

}


template<class AT>
inline void 
CGOptimization<AT>::mnbrakNRC(typename CGOptimization<AT>::Return_t& ax, 
			      typename CGOptimization<AT>::Return_t& bx, 
			      typename CGOptimization<AT>::Return_t& cx,
			      typename CGOptimization<AT>::Return_t& fa, 
			      typename CGOptimization<AT>::Return_t& fb, 
			      typename CGOptimization<AT>::Return_t& fc) {

  Return_t ulim,u,r,q,fu,dum = 0.0e0;

  fa = Func(ax); // *fa=(*func)(*ax);
  fb = Func(bx); // *fb=(*func)(*bx);

  if (fb > fa) {
    shift(dum,ax,bx,dum);
    shift(dum,fb,fa,dum);
  }

  cx=bx+GOLD*(bx-ax);
  fc = Func(cx); // *fc=(*func)(*cx);
  while (fb > fc) {
    r=(bx-ax)*(fb-fc);
    q=(bx-cx)*(fb-fa);
    u=(bx)-((bx-cx)*q-(bx-ax)*r)/(2.0*sign(max(fabs(q-r),TINY),q-r));
    ulim=(bx)+GLIMIT*(cx-bx);
    if ((bx-u)*(u-cx) > 0.0) {
      fu = Func(u); // fu=(*func)(u);
      if (fu < fc) {
	ax=bx;
        bx=u;
	fa=fb;
	fb=fu;
	return;
      } else if (fu > fb) {
	cx=u;
	fc=fu;
	return;
      }
      u=cx+GOLD*(cx-bx);
      fu = Func(u);//fu=(*func)(u);
    } else if ((cx-u)*(u-ulim) > 0.0) {
      fu = Func(u);//fu=(*func)(u);
      if (fu < fc) {

        shift(bx,cx,u,cx + GOLD*(cx-bx));
        shift(fb,fc,fu,Func(u)) ;
      }
    } else if ((u-ulim)*(ulim-cx) >= 0.0) {
      u=ulim;
      fu = Func(u);//fu=(*func)(u);
    } else {
      u=cx+GOLD*(cx-bx);
      fu = Func(u);//fu=(*func)(u);
    }
    shift(ax,bx,cx,u);
    shift(fa,fb,fc,fu);
  }
}


// ///////////////////////////////////////////////////////////
// // \todo not so pretty
// ///////////////////////////////////////////////////////////
// extern 
// void laguer(vector<complex<double> >& a,int m,
// 	    complex<double>& x,double eps,bool polish);

// extern 
// void zroots(vector<complex<double> >& a,
// 	    int m,vector<complex<double> >& roots,bool polish);

// template<class AT>
// typename CGOptimization<AT>::Return_t
// CGOptimization<AT>::lineoptimization(const vector<typename CGOptimization<AT>::Return_t>& e, 
// 				     bool getMinimum){

//   const Return_t small = 1.0e-6;
//   int m = e.size() - 1;  // order of polynomial [0....m]
  
//   typedef complex<double> dcomplex;
//   vector<dcomplex> a(m+1);
//   vector<dcomplex> roots(m+1);

//   a[0] = dcomplex(0.0e0, 0.0e0);
//   for(int i=1; i <= m; i++) {
//     a[i] = dcomplex(static_cast<Return_t>(i)*e[i], 0.0e0);
//   }

//   Return_t amaxinv = 1.0e0/a[m].real();


//   for(int i=1; i<= m; i++) {
//     a[i] *= amaxinv;
//   }
  

//   zroots(a, m, roots, true);

//   if(getMinimum) {

//     Return_t xmin = 1.0e10;  
//     Return_t z0 = 0.0e0; 
//     for(int i=0; i <= m; i++) {
//       if(fabs(roots[i].imag()) < small) {
//         Return_t x = 0.0e0;
//         Return_t z = roots[i].real();

// // will be substituted by more efficient routines
// //        for(int j=1; j<= m; j++) x +=pow(z,j)*e[j];
//         Return_t zp = 1;
//         for(int j=1; j<= m; j++) { zp *= z; x += zp*e[j];} 
//         if(x < xmin) { xmin = x; z0 = z;}
//      }
//     }

//     return z0;
//   } else {

//     Return_t xmax = -1.e10;
//     Return_t z0 = 0.0e0;

//     for(int i=0; i <= m; i++) {
//       if(abs(roots[i].imag()) < small) {
//         Return_t x = 0.0e0;
//         Return_t z = roots[i].real();

// // will be substituted by more efficient routines
// //       for(int j=1; j<= m; j++) x += pow(z,j)*e[j];
//         Return_t zp = 1;
//         for(int j=1; j<= m; j++) { zp *= z; x += zp*e[j];} 
//         if(x > xmax) { xmax = x; z0 = z;}
//       }
//     }
//     return z0;
//   }
// }


//#undef SIGN
//#undef SHFT
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 258 $   $Date: 2005-01-14 11:44:55 -0600 (Fri, 14 Jan 2005) $
 * $Id: CGOptimization.cpp 258 2005-01-14 17:44:55Z jnkim $ 
 ***************************************************************************/
