//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
#include <stdio.h>

#include <math.h>
#include <iomanip>
#include <iostream>
#include <complex>

#if (__GNUC__ == 2)
#include <algo.h>
#else
#include <algorithm>
#endif
using namespace std;
#include "Optimize/OptimizationBase.h"

template<class T>
void shift (T& a, T& b, T& c, T d) {
  a = b; b= c; c = d;
}

double sign(double a, double b) {
  return (b > 0.0)? fabs(a): -fabs(a); 
}

//#define SHFT(a,b,c,d) (a)=(b);(b)=(c);(c)=(d);
//#define MAX(a,b) ((a) > (b) ? (a) : (b))
//#define SIGN(a,b) ((b) > 0.0 ? fabs(a) : -fabs(a))

OptimizationBase::OptimizationBase() {
  ITMAX = 100;
  ZEPS = 1.0e-10;
  CGOLD = 0.3819660e0;
  GOLD = 1.618034e0;
  TOL = 2.0e-4;
  GLIMIT = 100.0;
  TINY = 1.0e-20;
  LambdaMax = 0.2;
}

bool 
OptimizationBase::lineoptimization()  {

  Return_t ax = 0;
  Return_t bx, fa, fx, fb;
  Return_t xx = LambdaMax;

  mnbrakNRC(ax,xx,bx,fa,fx,fb);

  //cout << " after bracking " << endl;
  //cout << "  f(ax = " << ax  << ") = " << fa << endl;
  //cout << "  f(xx = " << xx  << ") = " << fx << endl;
  //cout << "  f(bx = " << bx  << ") = " << fb << endl;

  Lambda = 0.0e0;
  Return_t ep = brentNRC(ax,xx,bx,Lambda);

  if(Lambda <= 1.0e-5) return false;
  else {
    Return_t e0 = Func(Lambda);
    //cout << " lambda along CG " << Lambda  << endl;
    //cout << " new energy " << ep << " " << e0 << endl;
    return true;
  }
}


OptimizationBase::Return_t 
OptimizationBase::brentNRC(Return_t ax, Return_t bx,  Return_t cx, Return_t& xmin) {

  Return_t a,b,d,etemp,fu,fv,fw,fx,p,q,r,tol1,tol2,u,v,w,x,xm;
  Return_t e=0.0e0;

  a=((ax < cx) ? ax : cx);
  b=((ax > cx) ? ax : cx);

  x=w=v=bx;

  fw=fv=fx=Func(x);

  for(int iter=1;iter<=ITMAX;iter++) {
    xm=0.5*(a+b);
    tol2=2.0*(tol1=TOL*fabs(x)+ZEPS);
    if (fabs(x-xm) <= (tol2-0.5*(b-a))) {
      xmin=x;
      return fx;
    }
    if (fabs(e) > tol1) {
      r=(x-w)*(fx-fv);
      q=(x-v)*(fx-fw);
      p=(x-v)*q-(x-w)*r;
      q=2.0*(q-r);
      if (q > 0.0) p = -p;
      q=fabs(q);
      etemp=e;
      e=d;
      if (fabs(p) >= fabs(0.5*q*etemp) || p <= q*(a-x) || p >= q*(b-x))
        d=CGOLD*(e=(x >= xm ? a-x : b-x));
      else {
	d=p/q;
	u=x+d;
	if (u-a < tol2 || b-u < tol2) d=sign(tol1,xm-x);
      }
    } else {
      d=CGOLD*(e=(x >= xm ? a-x : b-x));
    }
    u=(fabs(d) >= tol1 ? x+d : x+sign(tol1,d));
      fu = Func(u); // fu=(*f)(u);
      if (fu <= fx) {
        if (u >= x) a=x; else b=x;
   	shift(v,w,x,u);
	shift(fv,fw,fx,fu);
      } else {
	if (u < x) a=u; else b=u;
	if (fu <= fw || w == x) {
  	  v=w;
	  w=u;
	  fv=fw;
	  fw=fu;
	} else if (fu <= fv || v == x || v == w) {
  	  v=u;
 	  fv=fu;
	}
      }
  }

  xmin=x;
  return fx;
}

void 
OptimizationBase::mnbrakNRC(Return_t& ax, Return_t& bx, Return_t& cx,
			    Return_t& fa, Return_t& fb, Return_t& fc) {

  Return_t ulim,u,r,q,fu,dum = 0.0e0;

  fa = Func(ax); // *fa=(*func)(*ax);
  fb = Func(bx); // *fb=(*func)(*bx);

  if (fb > fa) {
    shift(dum,ax,bx,dum);
    shift(dum,fb,fa,dum);
  }

  cx=bx+GOLD*(bx-ax);
  fc = Func(cx); // *fc=(*func)(*cx);
  while (fb > fc) {
    r=(bx-ax)*(fb-fc);
    q=(bx-cx)*(fb-fa);
    u=(bx)-((bx-cx)*q-(bx-ax)*r)/(2.0*sign(max(fabs(q-r),TINY),q-r));
    ulim=(bx)+GLIMIT*(cx-bx);
    if ((bx-u)*(u-cx) > 0.0) {
      fu = Func(u); // fu=(*func)(u);
      if (fu < fc) {
	ax=bx;
        bx=u;
	fa=fb;
	fb=fu;
	return;
      } else if (fu > fb) {
	cx=u;
	fc=fu;
	return;
      }
      u=cx+GOLD*(cx-bx);
      fu = Func(u);//fu=(*func)(u);
    } else if ((cx-u)*(u-ulim) > 0.0) {
      fu = Func(u);//fu=(*func)(u);
      if (fu < fc) {

        shift(bx,cx,u,cx + GOLD*(cx-bx));
        shift(fb,fc,fu,Func(u)) ;
      }
    } else if ((u-ulim)*(ulim-cx) >= 0.0) {
      u=ulim;
      fu = Func(u);//fu=(*func)(u);
    } else {
      u=cx+GOLD*(cx-bx);
      fu = Func(u);//fu=(*func)(u);
    }
    shift(ax,bx,cx,u);
    shift(fa,fb,fc,fu);
  }
}


///////////////////////////////////////////////////////////
// \todo not so pretty
///////////////////////////////////////////////////////////
extern 
void laguer(vector<complex<double> >& a,int m,
	    complex<double>& x,double eps,bool polish);

extern 
void zroots(vector<complex<double> >& a,
	    int m,vector<complex<double> >& roots,bool polish);

OptimizationBase::Return_t 
OptimizationBase::lineoptimization(const vector<Return_t>& e, 
				   bool getMinimum){

  const Return_t small = 1.0e-6;
  int m = e.size() - 1;  // order of polynomial [0....m]
  
  typedef complex<double> dcomplex;
  vector<dcomplex> a(m+1);
  vector<dcomplex> roots(m+1);

  a[0] = dcomplex(0.0e0, 0.0e0);
  for(int i=1; i <= m; i++) {
    a[i] = dcomplex(static_cast<Return_t>(i)*e[i], 0.0e0);
  }

  Return_t amaxinv = 1.0e0/a[m].real();


  for(int i=1; i<= m; i++) {
    a[i] *= amaxinv;
  }
  

  zroots(a, m, roots, true);

  if(getMinimum) {

    Return_t xmin = 1.0e10;  
    Return_t z0 = 0.0e0; 
    for(int i=0; i <= m; i++) {
      if(fabs(roots[i].imag()) < small) {
        Return_t x = 0.0e0;
        Return_t z = roots[i].real();

// will be substituted by more efficient routines
//        for(int j=1; j<= m; j++) x +=pow(z,j)*e[j];
        Return_t zp = 1;
        for(int j=1; j<= m; j++) { zp *= z; x += zp*e[j];} 
        if(x < xmin) { xmin = x; z0 = z;}
     }
    }

    return z0;
  } else {

    Return_t xmax = -1.e10;
    Return_t z0 = 0.0e0;

    for(int i=0; i <= m; i++) {
      if(abs(roots[i].imag()) < small) {
        Return_t x = 0.0e0;
        Return_t z = roots[i].real();

// will be substituted by more efficient routines
//       for(int j=1; j<= m; j++) x += pow(z,j)*e[j];
        Return_t zp = 1;
        for(int j=1; j<= m; j++) { zp *= z; x += zp*e[j];} 
        if(x > xmax) { xmax = x; z0 = z;}
      }
    }
    return z0;
  }
}


//#undef SIGN
//#undef SHFT
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 260 $   $Date: 2005-01-14 13:14:41 -0600 (Fri, 14 Jan 2005) $
 * $Id: OptimizationBase.cpp 260 2005-01-14 19:14:41Z jnkim $ 
 ***************************************************************************/
