//////////////////////////////////////////////////////////////////
// (c) Copyright 2005- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
/**@file MetisConnection.h
 * @brief Declaration of MetisConnection class that reflects the storage requirement of Metis library
 */
#ifndef OHMMS_METISCONNECTION_H
#define OHMMS_METISCONNECTION_H

#include <vector>
/** Connecton data
 */
struct MetisConnection {

  typedef std::size_t                size_type;
  typedef std::vector<size_type>::iterator iterator;
  typedef std::vector<size_type>::const_iterator const_iterator;
  std::vector<size_type> M, J;

  size_type NumData;

  ///default constructor
  MetisConnection(): NumData(0) {}

  ///copy constructor
  MetisConnection(const MetisConnection& a):
    NumData(a.NumData),M(a.M), J(a.J){}

  ///assignment operator
  MetisConnection& operator=(const MetisConnection& a){
    NumData=a.NumData;
    M=a.M; J=a.J;
    return *this;
  }

  void resize(size_type n, size_type p){NumData=n; M.resize(n+1); J.resize(p);}
  inline size_type size() const { return NumData;}
  inline size_type getTotNadj() const { return J.size();}
  inline size_type nadj(size_type i) const { return M[i-1]-M[i];}
  inline size_type iadj(size_type i, size_type nn ) const { return J[M[i]+nn];}
  inline size_type first(size_type i) const { return M[i];}
  inline size_type last(size_type i) const { return M[i+1];}
  
  inline const_iterator begin(size_type i) const { return J.begin()+M[i];}
  inline const_iterator end(size_type i) const { return J.begin()+M[i+1];}
  inline iterator begin(size_type i) { return J.begin()+M[i];}
  inline iterator end(size_type i) { return J.begin()+M[i+1];}

  template <class PDT>
  void refresh(const PDT& conn) {
    std::cerr << "MetisConnection::refresh is broken. Fix it" <<std::endl;
    //resize(m.size()-1,conn.size()); 
    //std::copy(m.begin(), m.end(), M.begin());
    //typename PDT::const_iterator it(conn.begin()); 
    //typename PDT::const_iterator it_end(conn.end()); 
    //std::vector<size_type>::iterator j(J.begin());
    //while(it != it_end) {
    //  *j = (*it).J; ++j; ++it;
    //}
  }

  void update(const MetisConnection& conn, bool unique);

};
#endif
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 354 $   $Date: 2005-07-07 21:20:49 -0500 (Thu, 07 Jul 2005) $
 * $Id: MetisConnection.h 354 2005-07-08 02:20:49Z jnkim $ 
 ***************************************************************************/
