//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "Particle/NNMetisEngine.h"
#include "Utilities/OhmmsInfo.h"
#include "Particle/MetisGraphPartition.h"
#include "ParticleIO/XMLParticleIO.h"

namespace OHMMS {

  NNMetisEngine::NNMetisEngine(){
    LOGMSG("NNMetisEngine::NNMetisEngine is created.")
      setID(ThisMethod);
  }

  NNMetisEngine::~NNMetisEngine() { 
  }

  void 
  NNMetisEngine::updateR(Particle_t& p, 
			 const ParticlePos_t& pos, 
			 bool useupdate) {

//     if(!MaxPtclsPerCell) {
//       LOGMSG("\tLinked cell data")
//       LOGMSG("\t  Max connected cells " << MaxConnectedCells)
//       LOGMSG("\t  Max numer of particles per cell " << MaxPtclsPerCell)
//       LOGMSG("\t  Max numer of pairs per particle " << MaxPairsPerPtcl)
//     }

    if(Stride>0) NeedPartition = ((Counter == Stride) && useupdate);
    
    if(p.NeedPartition || NeedPartition) {
      p.curR.setUnit(PosUnit::LatticeUnit);
      p.Lattice.applyBC(pos,p.curR);
      p.updateLayout();
      
      makeNewPairs(p);
      p.NeedPartition=false;
      NewConfiguration=false;
      Counter=0;
    } else {
      NewConfiguration = true;
      Counter++;
    }
  }

  void 
  NNMetisEngine::partition(Particle_t& p) {

    //    if(Distributed[enable_mpi] && p.NeedPartition) {

      //p.partition(false);
//       const ParticleLayout_t::PtclGrid_t& bg = *(p.Lattice.dGrid[2]);
//       int ng = bg.getTotalNum();
//       //weight propotional to the number of particles per grid
//       vector<int> w(ng);
//       for(int ig=0; ig<ng; ig++) {
// 	w[ig] = bg.PtclDist[ig+1]-bg.PtclDist[ig];
//       }
//       partTool->partition(NextCell.M, NextCell.ID, w);
//       //reset the map(logical,physical)
//       int iL = 0;
//       for(int ipart=0; ipart<partTool->NumPart; ipart++) {
// 	for(int k=0;k<partTool->gSet[ipart].size(); k++) {
// 	  int ic = partTool->gSet[ipart][k];
// 	  for(int ig=bg.PtclDist[ic];ig<bg.PtclDist[ic+1]; ig++) {
// 	    p.ID[iL++] = bg.I[ig];
// 	  }
// 	}
//       }
//    }
  }
  
  void 
  NNMetisEngine::evaluate(const Particle_t& p, 
			  PtclPairList_t& myppl, 
			  int myID) {

    myppl.reset(); // reset the pair lists

    //aliasing 
    const ParticleLayout_t& lat = p.Lattice;
    const ParticlePos_t& R = p.curR;
    int maxnn = 0;
    SingleParticlePos_t dr;
    Scalar_t rsq;    
    int iL = 0;

    for(int iat=PtclDist[myID]; iat<PtclDist[myID+1]; iat++, iL++) {
      int nn = 0;// the number of neighbor particles
      int ic = p.Cell[iat]; //get the cell index
      myppl.I[iL] = iat;    //save the local index
      ParticleLayout_t::const_iterator it = p.Lattice.beginData(ic);
      ParticleLayout_t::const_iterator it_end = p.Lattice.endData(ic);
      while(it != it_end) { ///check the particles in the same grid
	int jat = *it++;
	if(iat == jat) continue;
	dr = lat.toCart(R[jat]-R[iat]);
	rsq = dot(dr,dr);
	if(rsq< RmaxSq && rsq > RminSq) {
	  rsq = sqrt(rsq);
	  Scalar_t rinv = 1.0e0/rsq;
	  myppl.add(iL,jat,rsq,rinv*dr);
	  nn++;
	}          
      }//it != it_end
      
      //==================================================
      // iterate over the neighboring cells
      //==================================================
      for(int nc=lat.first_connected(ic); nc<lat.last_connected(ic);nc++) {
	int jc = lat.id_connected(nc);//the cell index of the neighbor cell
	SingleParticlePos_t del = lat.bc(nc);
	ParticleLayout_t::const_iterator jt = p.Lattice.beginData(jc);
	ParticleLayout_t::const_iterator jt_end = p.Lattice.endData(jc);
	while(jt != jt_end) {//check the particles of the current neighbor cell
	  int jat = *jt;
	  dr = lat.toCart(R[jat]-R[iat]) + del;
	  rsq = dot(dr,dr);
	  if(rsq< RmaxSq && rsq > RminSq) {
	    rsq = sqrt(rsq);
	    Scalar_t rinv = 1.0e0/rsq;
	    myppl.add(iL,jat,rsq,rinv*dr); //  logical index jg is used
	    nn++;
	  }
	  jt++;
	}//jt!=jt_end
      }//for(int nc;;)
      
      maxnn = (maxnn>nn)? maxnn:nn;
    }//go thru all the particles for this thread
    
    myppl.add(iL,-1,rsq,dr);
    myppl.setMaxNadj(maxnn);
  }

  bool
  NNMetisEngine::initialize(Particle_t& p, Scalar_t rmax, 
			    bool needvisitingforce) {
    
    Rmax = rmax;
    RmaxSq = rmax*rmax*Rscale*Rscale;
    UseVisitingForce=needvisitingforce;

    //going to allocate storage for linked cells
    if(!MaxConnectedCells) { 

      MaxConnectedCells = p.Lattice.connectGrid(rmax);
      p.Lattice.createData();

      int npart = omp_get_max_threads();
      CellID.resize(p.Lattice.ncontexts(),0);

      //use metis to order the particle position
      if(npart >1) {
	WARNMSG("NNMetisEngine::initialize. The particles are reorded.")
	typedef std::vector<int> IndexList_t;
	IndexList_t wgt(p.Lattice.ncontexts());
	//IndexList_t cindex(p.Lattice.ncontexts());
	for(int ig=0; ig<p.Lattice.ncontexts(); ig++)
	  wgt[ig]=p.Lattice.last_connected(ig)-p.Lattice.first_connected(ig);
	
	MetisGraphPartition<IndexList_t> graph(npart);
	graph.partition(p.Lattice.c_offset,
			p.Lattice.c_id,
			wgt,CellID);
	
	int r_unit = p.R.getUnit();
	
	p.curR.setUnit(PosUnit::LatticeUnit);
	p.Lattice.applyBC(p.R,p.curR);
	MaxPtclsPerCell = p.updateLayout();
	
	ParticleIndex_t GroupIDsave(p.GroupID);
	p.R.setUnit(PosUnit::LatticeUnit);
	int nloc=0;
	int cell_offset=0;
	  
	//count the number of particles per thread
	vector<int> mPtclDist(npart,0);
	  
	//CellID has the map to the grid-based index to the physical index
	for(int ipart=0; ipart<npart; ipart++) {
	  for(int k=0;k<graph.gSet[ipart].size(); k++,cell_offset++) {
	    int ig= graph.gSet[ipart][k];
	    CellID[cell_offset]=ig;
	    ParticleLayout_t::const_iterator jt = p.Lattice.beginData(ig);
	    ParticleLayout_t::const_iterator jt_end = p.Lattice.endData(ig);
	    while(jt != jt_end) {
	      int jat = *jt;
	      p.R[nloc]=p.curR[jat];
	      p.GroupID[nloc]=GroupIDsave[jat];
	      p.ID[nloc]=nloc;
	      p.Cell[nloc]=ig;
	      jt++; nloc++;
	      mPtclDist[ipart]++;
	    }
	  }
	}
	
	p.Lattice.clearData();
	for(int i=0;i<p.getLocalNum(); i++)  p.Lattice.addData(p.Cell[i],i);
	
	//update curR with the reorded position 
	p.curR = p.R;
	//restore R to the original unit
	if(r_unit == PosUnit::CartesianUnit) {
	  p.Lattice.convert2Cart(p.curR,p.R);
	} 
	
	PtclDist.resize(npart+1);
	PtclDist[0]=0;
	for(int ipart=0; ipart<npart; ipart++) {
	  PtclDist[ipart+1]=PtclDist[ipart]+mPtclDist[ipart];
	}
	MaxPairsPerPtcl=(MaxConnectedCells+1)*(MaxPtclsPerCell+1);
	allocateAttributes(UseVisitingForce);
	makeNewPairs(p);
	p.NeedPartition=false;
	NewConfiguration=false;
	Counter=0;
      } else {
	LOGMSG("NNMetisEngine::initialize. Identical to NNRegGridEngine.")
	makeFairPartition(p);
	p.curR.setUnit(PosUnit::LatticeUnit);
	p.Lattice.applyBC(p.R,p.curR);
	MaxPtclsPerCell = p.updateLayout();
	MaxPairsPerPtcl=(MaxConnectedCells+1)*(MaxPtclsPerCell+1);
	allocateAttributes(UseVisitingForce);
	makeNewPairs(p);
	p.NeedPartition=false;
	NewConfiguration=false;
	Counter=0;
      }
      LOGMSG("\tLinked cell data")
      LOGMSG("\t  Max connected cells " << MaxConnectedCells)
      LOGMSG("\t  Max numer of particles per cell " << MaxPtclsPerCell)
      LOGMSG("\t  Max numer of pairs per particle " << MaxPairsPerPtcl)
    }
    return true;
  }

  bool NNMetisEngine::putExtra(xmlNodePtr cur){
    return true;
  }
}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: NNMetisEngine.cpp 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/

