//////////////////////////////////////////////////////////////////
// (c) Copyright 2003- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifndef OHMMS_NEARESTNEIGHBOR_SPATIALENGINE_SAVE_H
#define OHMMS_NEARESTNEIGHBOR_SPATIALENGINE_SAVE_H

#include "Particle/NNEngine.h"

/*\class NNSpatialMethod
 *\brief Using grid partition
 */
namespace OHMMS {

  class NNSpatialEngine: public NNEngine {
    
  public:
    
    enum {ThisMethod = 3};

    NNSpatialEngine();
    ~NNSpatialEngine();
    
    //@{initiate and finalize collecting visiting forces from other processors
    void startCollect(ParticlePos_t& fv);
    void endCollect(ParticlePos_t& fv);
    //@}

    bool initialize(Particle_t& p, bool needvisitingforce);
    Index_t updateGridData(const ParticlePos_t& pos, ParticlePos_t& pu, ParticleLayout_t& lattice);
    void collectForce(ParticlePos_t& f, omp_int_t ip) ;

    void getOptions(int argc, char** argv);
    
  protected:
    
    int myNode;
    int NumSendMsg, CurSendMsg;
    int NumRecvMsg, CurRecvMsg;
    int NumLocalPtcls,NumGhosts;

    /** pointer to ParticleSet on which this engine operates
     * 
     * horrible but necessary to handle ghost data
     */
    Particle_t* myPtcl;

    ///pointer to Grid_t at MPI level
    ParticleLayout_t::Grid_t* mppGrid;

    ///pointer to ParticleIndex with the name "mppid"
    ParticleIndex_t* mppAttrib;

    typedef vector<Index_t> IndexList_t;

    ///the number of particles per context
    IndexList_t PtclPerNode;

    ///the starting index of the particles on each context
    IndexList_t PtclOffset;

    vector<IndexList_t*> ghostcell;
    vector<IndexList_t*> SendList;    
    IndexList_t RecvCounter;
    IndexList_t GhostOffset;

    struct GhostDataType {
      int ID;
      int GroupID;
      SingleParticlePos_t R;
    };

    vector<GhostDataType> GhostList;

    ///virtual function
    void evalPairsPerThread(const ParticleLayout_t& lattice, 
                            const ParticlePos_t& pos, 
                            const ParticleIndex_t& grID,
                            PtclPairList_t& myppl);

    void updatePairsPerThread(const ParticlePos_t& del, PtclPairList_t& myppl);
    bool putExtra(xmlNodePtr cur);

    /** distribute the particle among the threads 
     * @param p ParticleSet to be partitioned.
     */
    void makePartitions(Particle_t& p);
    void setRemoteConnections(ParticleLayout_t& Lattice, Scalar_t rmax);
    void setMessagePatterns(const ParticleLayout_t::Grid_t* restrict sg);
    void sendrecvGhostParticles(const ParticlePos_t& pos);

    int swap(Particle_t& p, int mynode);
    void debugmsg(int i);

    //struct nnMessageItem {
    //  int id;
    //  int cell;
    //  int species;
    //  SingleParticlePos_t pos;
    //  nnMessageItem(int i,
    //    	    int ig, 
    //    	    int ispecies, 
    //    	    const SingleParticlePos_t& r): id(i), cell(ig), species(ispecies), pos(r){}  
    //};
    //
    //vector<list<nnMessageItem>*> remotePtcl;
    //void sendGhostParticles(Particle_t&);
    //int recvGhostParticles(Particle_t&);
    //int scatter(Particle_t& p, int mynode);
  };
}
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 354 $   $Date: 2005-07-07 21:20:49 -0500 (Thu, 07 Jul 2005) $
 * $Id: NNSpatialEngine.h 354 2005-07-08 02:20:49Z jnkim $ 
 ***************************************************************************/
