//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002,2003- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
/** @file NNSpatialOMPEngine.h
 * @brief Definition of a NN Engine using spatial locaity for OpenMP parallelization
 */
#ifndef OHMMS_NEARESTNEIGHBOR_REGULARGRIDENGINE_OMP_H
#define OHMMS_NEARESTNEIGHBOR_REGULARGRIDENGINE_OMP_H

#include "Particle/NNEngine.h"
#include <set>

/** The main engine to build the nearest-neighbor list
 */
namespace OHMMS {

  class NNSpatialOMPEngine: public NNEngine {
    public:

    enum {ThisMethod = 4};

    typedef Particle_t::ParticlePos_t       ParticlePos_t;
    typedef Particle_t::SingleParticlePos_t SingleParticlePos_t;
    typedef Grid_t::DistributedIndex_t      OMPIndex_t;

    NNSpatialOMPEngine();
    ~NNSpatialOMPEngine();
   
    bool initialize(Particle_t& p, bool needvisitingforce);
    Index_t updateGridData(const ParticlePos_t& pos, ParticlePos_t& pu, ParticleLayout_t& lattice);
    void collectForce(ParticlePos_t& f, omp_int_t ip) ;

  protected:

    void evalPairsPerThread(const ParticleLayout_t& lattice, 
                            const ParticlePos_t& pos, 
                            const ParticleIndex_t& grID,
                            PtclPairList_t& myppl);
    
    bool putExtra(xmlNodePtr cur);

    /** distribute the particle among the threads 
     * @param lattice the layout object
     * @param pin input position vector
     * @param pu output position vector 
     *
     * The input position is wrapped into the unit cell to evaluate the
     * grid index and distribute the particles over the threads.
     */
    Index_t createGridData(const ParticleLayout_t& lattice,
                       const ParticlePos_t& pin,
                       ParticlePos_t& pu);

    void metisGridPartition(ParticleLayout_t& lattice);

    bool CopyA;
    bool FirstTouch;

    std::vector<int> NumLocalPtcls;
    std::vector<int> Grid2Node;
    std::vector<OMPIndex_t*> OmpPartsA;
    std::vector<OMPIndex_t*> OmpPartsB;
    std::vector<OMPIndex_t*> CurrentList;

    typedef map<int,vector<int>* > GhostListType;
    std::vector<GhostListType* > GhostList;
    std::vector<set<int>* > ConnectedNode;
  };
}
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 354 $   $Date: 2005-07-07 21:20:49 -0500 (Thu, 07 Jul 2005) $
 * $Id: NNSpatialOMPEngine.h 354 2005-07-08 02:20:49Z jnkim $ 
 ***************************************************************************/
