//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
/**@file ParticleSet.1.cpp
 * @brief contains member functions using layout class of ParticleSet.h
 */
#include "Utilities/OhmmsInfo.h"
#include "Particle/ParticleSet.h"
#include "ParticleBase/ParticleUtility.h"
#include "Particle/NNEngine.h"
#include "Message/OpenMP.h"
#include <bitset>
#include <limits>

namespace OHMMS {

  /** implementatio of the virutal function
   *@param imode, what kind of update is done
   *\warning very unstable and not used by any object
   */
  void 
  ParticleSet::update(int imode) {
    if(imode == -1) {
      updateLayout(Lattice.InteractionRadius,Lattice.ConnectionRadius);
//      curR.setUnit(PosUnit::CartesianUnit);//temporarily change curR to the Cartesian unit
//      //Lattice.convert2Cart(R); //make sure R is in Cartesian
//      convert2Cart(R); //make sure R is in Cartesian
//      wrapAroundBox(Lattice,R,curR);//apply BC
//      if(hasAttrib("sumbc")) {// translate exists, accumulate it
//	ParticlePos_t& del = *(getVectorAttrib("sumbc"));
//	del += R - curR;
//      } else { // first time translation is calculated
//	ParticlePos_t& del = *(getVectorAttrib("sumbc"));
//	del = R - curR;
//      }
//
//      R = curR; //assign the position adjusted by boundary conditions
//      curR.setUnit(PosUnit::LatticeUnit);
//      Lattice.distribute(getTotalNum());
    }
  }


  /** Apply boundary conditions on ConfigList
   *
   * Propagators use addPositionAttrib to register particle attribute 
   * which are subject to boundary conditions. 
   * applyBConds translate those attributes and accumulate the translations
   * to an attribute called sumbc. 
   * In order to calculate diffusion constant, it is necessary to add sumbc to R.
   * ParticleSet::curR is the position in the LatticeUnit and all the components
   * belong to [0,1).
   */
  void ParticleSet::applyBConds(bool updateCurR, bool latticeunit) {

#pragma omp parallel
    {
      omp_int_t myid = omp_get_thread_num();
      size_type first = SubPtcl[myid];
      size_type last = SubPtcl[myid+1];
      bool inunit = (R.getUnit() ==  latticeunit);
      size_type nconf(ConfigList.size());
      if(inunit) {
        ParticlePos_t::const_iterator cit(curR.begin()+first);
        ParticlePos_t::iterator pit(dR.begin()+first);
        ParticlePos_t::iterator rit(R.begin()+first);
        size_type iat=first;
        while(iat<last) {
          *pit = *cit-*rit;
          *rit = *cit;
          ++iat;++cit;++rit;++pit;
        }
        for(size_type ic=1; ic<ConfigList.size(); ic++) {
          ParticlePos_t::const_iterator oit(dR.begin()+first);
          ParticlePos_t::iterator nit(ConfigList[ic]->begin()+first);
          iat=first;
          while(iat<last) {
            *nit+=*oit; ++iat;++oit;++nit;
          }
        }
      } else {
        ParticlePos_t::const_iterator cit(curR.begin()+first);
        ParticlePos_t::iterator pit(dR.begin()+first);
        ParticlePos_t::iterator rit(R.begin()+first);
        size_type iat=first;
        while(iat<last) {
          SingleParticlePos_t del(*rit);
          *rit=Lattice.toCart(*cit);
          *pit=*rit-del;
          ++iat;++cit;++rit;++pit;
        }
        for(size_type ic=1; ic<ConfigList.size(); ic++) {
          ParticlePos_t::const_iterator oit(dR.begin()+first);
          ParticlePos_t::iterator nit(ConfigList[ic]->begin()+first);
          iat=first;
          while(iat<last) {
            *nit += *oit;
            ++iat; ++oit; ++nit;
          }
        }
      }
      ParticlePos_t::iterator pit(dR.begin()+first);
      size_type iat=first;
      while(iat!=last) {*pit=0.0; ++pit; ++iat;}
    }
//    if(ConfigList.size()) {
//
//      //For non-periodic system, check the boundary
//      if(PeriodicSystem) {
//
//	ParticlePos_t rbc(LocalNum);
//	rbc.setUnit(PosUnit::LatticeUnit);
//
//        /*
//        //curR now are inside the unit cell
//        for(size_type idim=0; idim<OHMMS_DIM; idim++) {
//          for(size_type iat=0; iat<LocalNum; iat++) { rbc[iat][idim]=floor(curR[iat][idim]);}
//        }
//        */
//	//Lattice.convert2Unit(R,curR); 
//	convert2Unit(R,curR); 
//        const double epsilon =  -numeric_limits<double>::epsilon();
//        const double pone =  1.0e0;
//        const double mone =  -1.0e0;
//        for(size_type iat=0; iat<LocalNum; iat++) {
//          for(size_type idim=0; idim<OHMMS_DIM; idim++) {
//            if(curR[iat][idim]< epsilon) {
//              rbc[iat][idim]=pone; curR[iat][idim] +=pone;
//            } else if(curR[iat][idim]>=pone) {
//              rbc[iat][idim]=mone; curR[iat][idim] +=mone;
//            } 
//          }
//        }
//
//	bool need2convert = false;
//	size_type ic = 0;
//	while(!need2convert && ic<ConfigList.size()) {
//	  if(ConfigList[ic]->getUnit() == PosUnit::CartesianUnit) need2convert=true;
//	  ic++;
//	}
//
//	//Lattice.convert(curR,R); 
//	//convert(curR,R); 
//	if(need2convert) {
//	  for(ic=0; ic<ConfigList.size(); ic++) {
//	    if(ConfigList[ic]->getUnit() == PosUnit::LatticeUnit) (*ConfigList[ic]) += rbc;
//          }
//	  //Lattice.convert2Cart(rbc);
//	  convert2Cart(rbc);
//	  for(ic=0; ic<ConfigList.size(); ic++) {
//	    if(ConfigList[ic]->getUnit() != PosUnit::LatticeUnit) (*ConfigList[ic]) += rbc;
//          }
//	} else {
//	  for(ic=0; ic<ConfigList.size(); ic++) (*ConfigList[ic]) += rbc;
//	}	
//
//	if(KeepSumBC) {
//          //if(!need2convert) Lattice.convert2Cart(rbc);
//          if(!need2convert) convert2Cart(rbc);
//	  ParticlePos_t& del = *(getVectorAttrib("sumbc"));
//	  del += rbc;
//	}
//      }
//      NeedPartition=true;
//    }
  }

  /** Initialize/Reset the grid
   *@param int_rad the interaction radius
   *@param con_rad the connection radius
   *
   */
  void ParticleSet::updateLayout(Scalar_t int_rad, Scalar_t con_rad) {

    DEBUGMSG("\tParticleSet::updateLayout")
    std::bitset<OHMMS_DIM> checkbound;
    for(size_type idim=0; idim<OHMMS_DIM; idim++) {
      if(!Lattice.BoxBConds[idim]) checkbound.set(idim);
    }

    PeriodicSystem = !checkbound.any();
    
    //Translate position attributes if any direction is not periodic
    if(checkbound.any()) {
      SingleParticlePos_t lower(1e12,1e12,1e12);
      SingleParticlePos_t upper(-1e12,-1e12,-1e12);
      for(size_type i=0; i<getTotalNum(); i++) {
	for(size_type idim=0; idim<OHMMS_DIM; idim++) {
	  lower[idim] = min(lower[idim],R[i][idim]);
	  upper[idim] = max(upper[idim],R[i][idim]);
	}
      }
      if(R.getUnit() == PosUnit::CartesianUnit) {
	lower = Lattice.toUnit(lower);
	upper = Lattice.toUnit(upper);
      }
      
      bool outofbound = false;
      SingleParticlePos_t shift_unit;
      for(size_type idim=0; idim<OHMMS_DIM; idim++) {
	if(checkbound[idim]) {
	  if(lower[idim]<0.0 || upper[idim] >= 1.0) {
	    outofbound = true;
	    shift_unit[idim] = 0.5-0.5*(lower[idim]+upper[idim]);
	  }
	}
      }
      
      if(outofbound) {
	
	WARNMSG("lower bound [0,1) " << lower)
	WARNMSG("upper bound (0,1) " << upper)
	WARNMSG("Shifting all positions by (lattice unit) " << shift_unit)
	SingleParticlePos_t shift_cart = Lattice.toCart(shift_unit);
	
	if(R.getUnit() == PosUnit::LatticeUnit) 
	  R += shift_unit;
	else 
	  R += shift_cart;
	for(size_type it=1; it<ConfigList.size(); it++) {
	  if(ConfigList[it]->getUnit() == PosUnit::LatticeUnit)
	    *(ConfigList[it]) += shift_unit;
	  else                       
	    *(ConfigList[it]) += shift_cart;
	}
      }
    }

    //Check if the grid layout has been modified
    if(nnListEngine) nnListEngine->checkGrid(Lattice);

    //Adjust the grid size based on the input and interaction radius
    Lattice.checkGrid(int_rad);

    //Connect spatial grid
    Lattice.connectGrid(int_rad,con_rad);

    SubPtcl.resize(omp_get_max_threads()+1);
    FairDivide(LocalNum,omp_get_max_threads(),SubPtcl);
  }

  void ParticleSet::makeMove(size_type i, const SingleParticlePos_t& d) {
    nnListEngine->makeMove(i,d);
  }

  void ParticleSet::restoreMove(size_type i) {
    nnListEngine->restoreMove(i);
  }

}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 354 $   $Date: 2005-07-07 21:20:49 -0500 (Thu, 07 Jul 2005) $
 * $Id: ParticleSet.1.cpp 354 2005-07-08 02:20:49Z jnkim $ 
 ***************************************************************************/

