//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "Message/OpenMP.h"
#include <string>
#include <vector>
#include <list>
#include <bitset>
#include <numeric>
#include <algorithm>
#include "Message/CommOperators.h"
using namespace std;

#include "Particle/ParticleSet.h"
#include "Particle/NNEngine.h"
using namespace OHMMS;
#include "ParticleBase/ParticleUtility.h"

/** remove everything else but selected particles
 * @param selected indices of the selected particles
 */
void ParticleSet::select(const vector<Index_t>& selected) {

  int nloc(selected.size());

  //copy ParticleIndex_t
  ParticleIndex_t indCopy(LocalNum);
  vector<ParticleIndex_t*>::iterator iit(INDEX.begin()), iit_end(INDEX.end());
  while(iit != iit_end) {
    ParticleIndex_t &ind(**iit);
    std::copy(ind.begin(),ind.end(),indCopy.begin());
    ind.resize(nloc);
    vector<Index_t>::const_iterator it(selected.begin());
    for(int i=0; i<nloc; i++) ind[i]=indCopy[*it++];
    ++iit;
  }

  //copy ParticleScalar_t
  ParticleScalar_t valCopy(LocalNum);
  vector<ParticleScalar_t*>::iterator vit(VAL.begin()), vit_end(VAL.end());
  while(vit != vit_end) {
    ParticleScalar_t &v(**vit);
    std::copy(v.begin(),v.end(), valCopy.begin());
    v.resize(nloc);
    vector<Index_t>::const_iterator it(selected.begin());
    for(int i=0; i<nloc; i++) v[i]=valCopy[*it++];
    ++vit;
  }

  //copy ParticlePos_t
  ParticlePos_t posCopy(LocalNum);
  vector<ParticlePos_t*>::iterator pit(POS.begin()), pit_end(POS.end());
  while(pit != pit_end) {
    ParticlePos_t &pos(**pit);
    std::copy(pos.begin(),pos.end(), posCopy.begin());
    pos.resize(nloc);
    vector<Index_t>::const_iterator it(selected.begin());
    for(int i=0; i<nloc; i++) pos[i]=posCopy[*it++];
    ++pit;
  }

  //copy ParticleTensor_t
  ParticleTensor_t tCopy(LocalNum);
  vector<ParticleTensor_t*>::iterator tit(TENZOR.begin()), tit_end(TENZOR.end());
  while(tit != tit_end) {
    ParticleTensor_t &t(**tit);
    std::copy(t.begin(),t.end(), tCopy.begin());
    t.resize(nloc);
    vector<Index_t>::const_iterator it(selected.begin());
    for(int i=0; i<nloc; i++) t[i]=tCopy[*it++];
    ++pit;
  }

  LocalNum=nloc;
  GlobalNum=LocalNum;
  if(Mode != Serial) {
    MPI_Allreduce(&LocalNum,&GlobalNum, 1, MPI_INT, MPI_SUM, MPI_COMM_WORLD);
  }

  //GlobalNum= collect(LocalNum);
  SubPtcl[0]=0;
  SubPtcl[1]=LocalNum;
}
/**@return the maximum number of particles per grid
int 
ParticleSet::partition(ParticleLayout_t::Grid_t *sgrid, 
	 	       int mynode) {
  int maxptclpercell = 0;
  if(LocalNum == GlobalNum) { 
    ///convert to a unit vector
    Lattice.applyBC(R);

    ///create a temporary data
    int ncontexts = sgrid->getTotalNum();
    vector<int> nsub_m(ncontexts,0);
    vector<int> nsub_g(Lattice.getTotalNum(),0);

    vector<int> selected;
    vector<int> newid, gid;
    vector<SingleParticlePos_t> pos;

    for(int iG=0; iG<getTotalNum(); iG++) {
      int thisgrid = Lattice.loc(R[iG]);
      int pid = sgrid->group(thisgrid);
      if(pid == mynode) {
	newid.push_back(GroupID[iG]);
	pos.push_back(R[iG]);
	gid.push_back(thisgrid);
	selected.push_back(iG);
      }
      nsub_g[thisgrid]++;
      nsub_m[pid]++;
    }
    
    ///convert nsub_m -> SuperGrid[0]->PtclDist
    sgrid->distribute(nsub_m.begin(), nsub_m.end());

    ///resize local data: note that R, ID and GroupID cannot  be deleted.
    LocalNum = nsub_m[mynode];
    Cell.resize(LocalNum);
    R.resize(LocalNum);
    GroupID.resize(LocalNum);
    ID.resize(LocalNum);

    Lattice.createData();
    int first = sgrid->firstData(mynode);
    for(int nloc=0; nloc<LocalNum; nloc++,first++){
      ID[nloc] = first;
      GroupID[nloc] = newid[nloc];
      R[nloc] = pos[nloc];
      Cell[nloc] = gid[nloc];
      Lattice.addData(gid[nloc],nloc);
    }

    maxptclpercell = *max_element(nsub_g.begin(), nsub_g.end());

    ///excluding ID and GroupID
    for(int i=2; i<INDEX.size(); i++) {
      ParticleIndex_t t(*INDEX[i]);
      INDEX[i]->resize(LocalNum);
      for(int iat=0; iat< LocalNum; iat++)
        (*INDEX[i])[iat] = t[selected[iat]];
    }

    for(int i=0; i<VAL.size(); i++) {
      ParticleScalar_t t(*VAL[i]);
      VAL[i]->resize(LocalNum);
      for(int iat=0; iat< LocalNum; iat++)
        (*VAL[i])[iat] = t[selected[iat]];
    }

    for(int i=1; i<POS.size(); i++) {
      ParticlePos_t t(*POS[i]);
      POS[i]->resize(LocalNum);
      for(int iat=0; iat< LocalNum; iat++)
        (*POS[i])[iat] = t[selected[iat]];
    }
     Lattice.getDataSets().distribute(nsub_g.begin(), nsub_g.end());
     Mode = Distributed;
  } else {
    ///time to repartition with metis
  }
  return maxptclpercell;
}
 */


void ParticleSet::startCollect(ParticlePos_t& fv) {
  if(nnListEngine) nnListEngine->startCollect(fv);
}

void ParticleSet::endCollect(ParticlePos_t& fv) {
  if(nnListEngine) nnListEngine->endCollect(fv);
}

ParticleSet::Scalar_t 
ParticleSet::collect(Scalar_t x) {
  Scalar_t xL(x);
  if(Mode != Serial)  {
    MPI_Allreduce(&xL,&x, 1, MPI_DOUBLE, MPI_SUM, MPI_COMM_WORLD);
  }
  return x;
}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 354 $   $Date: 2005-07-07 21:20:49 -0500 (Thu, 07 Jul 2005) $
 * $Id: ParticleSet.MPI.cpp 354 2005-07-08 02:20:49Z jnkim $ 
 ***************************************************************************/

