//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include <bitset>
#include "Message/OpenMP.h"
#include "Particle/NNEngine.h"
#include "Particle/N2Method.h"
#include "Particle/NNRegGridMethod.h"
#include "Particle/NNMetisMethod.h"
using namespace OHMMS;

NNEngine::NNEngine(int id) { 
  UnInitialized=true;
  InBox = true;
  Reordered=false;
  IndexOnly=false;
  Symmetric=false;
  Unique=true; 
  Counter=-1;
  TotalNum=0;
  TotalNumPairs=0;
  MethodID=id;
  Rscale=1.05;
  RmaxSq =1.0;
  RminSq=1e-6;
  
  PtclDist.reserve(omp_get_max_threads()+1);
}

NNEngine::~NNEngine(){ 
  if(myMethod) delete myMethod;
}

bool NNEngine::put(istream& is) {
  return true;
}

bool NNEngine::put(xmlNodePtr cur){
 
#if defined(ENABLE_LIBXML2)
 ///overwrite Rscale
  if(xmlHasProp(cur, (const xmlChar*) "scale")) {
    Rscale = atof((const char*)(xmlGetProp(cur, (const xmlChar *) "scale")));
  }
#endif
  ///pass it to the derived class to check its input parameters
  return putExtra(cur);
//   int method_old = -1;

//   if(myMethod) {
//     method_old = myMethod->getID();
//   }

//   ///default method = NNRegGridMethod
//   MethodID = NNRegGridMethod::ThisMethod;

//   ///overwrite Rscale
//   if(xmlHasProp(cur, (const xmlChar*) "scale")) {
//     Rscale = atof((const char*)(xmlGetProp(cur, (const xmlChar *) "scale")));
//   }
//   if(xmlHasProp(cur, (const xmlChar*) "method")) {
//     string what = (const char*)(xmlGetProp(cur, (const xmlChar *) "method"));
//     if(what == "grid") 
//       MethodID = NNRegGridMethod::ThisMethod;
//     else if(what == "N2") 
//       MethodID = N2Method::ThisMethod;
//     else if(what == "metis") 
//       MethodID = NNMetisMethod::ThisMethod;
//     else if(what == "optimize") 
//       MethodID = 3;
//   }

//   if(!myMethod) {   // algorithm specific parition
//     switch(MethodID) {
//     case(0):
//       LOGMSG("\tCreating NNRegGridMethod for a nearest-neighbor engine")
//       myMethod = new NNRegGridMethod();
//       break;
//     case(1):
//       LOGMSG("\tCreating N2Method for a nearest-neighbor engine")
//       myMethod = new N2Method();
//       break;
//     case(2):
//       LOGMSG("\tCreating NNMetisMethod for a nearest-neighbor engine")
//       myMethod = new NNMetisMethod();
//       break;
//     case(3):
//       LOGMSG("\tCreating NNRegGridMethod for a nearest-neighbor engine")
//       myMethod = new NNRegGridMethod();
//       break;
//     }
//   }

//   myMethod->setID(MethodID);
//   return myMethod->put(cur);
}

// Scalar_t NNEngine::initMethod(Particle_t& p, Scalar_t rmax) {

//   if(!myMethod) {
//     LOGMSG("\tCreating NNRegGridMethod for a nearest-neighbor engine")
//     myMethod = new NNRegGridMethod();
//     MethodID = myMethod->getID();;
// #if defined(USE_OPENMP)
//     myMethod->dstribute(enable_omp,true);
// #endif
//   }
//   ///dangerous: assume that MethodID == 1 is N2
//   return (MethodID == N2Method::ThisMethod)? rmax : rmax*Rscale;
// }

bool NNEngine::get(ostream& os) const{

  for(int myID=0; myID< size(); myID++) {
    int iL = 0;
    int nnloc = 0;
    const PtclPairList_t& nnlist = PairAttrib[myID];
    os << "My node = " << myID << " starting with atom " << PtclDist[myID] << endl;
    for(int iG=PtclDist[myID]; iG<PtclDist[myID+1]; iL++, iG++) {
      //os << iG << " has " << nnlist.nadj(iL) << endl;
      os << nnlist.I[iL] << " has " << nnlist.nadj(iL) << endl;
      for(int nni=0; nni<nnlist.nadj(iL); nni++, nnloc++) {
	os << nnlist.iadj(iL,nni) 
	   << " " << nnlist.R[nnloc] << " " << nnlist.dR[nnloc] << endl;
      }//nni
    } // iG,iL

  }//myID

  return true;
}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: NNEngine.IO.cpp 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/

