//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include <bitset>
#include <set>
using namespace std;
#include <math.h>

#include "Message/Communicate.h"
#include "Message/OpenMP.h"
#include "Particle/NNMethodBase.h"
using namespace OHMMS;
#include "ParticleBase/ParticleUtility.h"

/**constructor
 * A inherited class can provide
 * enum {ThisMethod = integer};
 * unique to each class and passy the enumeration to the base class.
 *
 * As Jan/2/2004, three methods are implmeneted:
 * NNRegGridMethod
 * N2Method
 * N2MetisMethod
 */
NNMethodBase::NNMethodBase(int id):ID(id), nGhosts(0) {
  RmaxSq = 1.0; RminSq = 1e-6;
}

/**@fn void NNMethodBase::simplePartition(Particle_t& p)
 * @param p Particletype
 * @brief Using a global communicator OHMMS::Controller in Message/Communicate.h
 * @todo Each particle can have a different communicator
 */
void NNMethodBase::simplePartition(Particle_t& p) {

  ///mpi overwrites openmp: single mpi partition is assumed.
  //\todo include using groups of mpi nodes
  if(Distributed[enable_mpi]) {
    int mgrid[] = {1,1,1};

    //========================================
    mgrid[0] = Controller->ncontexts();
    //========================================
    p.Lattice.resetGrid(0,mgrid);
    p.Lattice.dGrid[0]->distribute(p.getTotalNum());
    
    ///supports only one thread per MPI: will supprt multiple threads
    //========================================
    int nodeid = Controller->mycontext();
    //========================================
    int first = p.Lattice.dGrid[0]->PtclDist[nodeid];
    int last = p.Lattice.dGrid[0]->PtclDist[nodeid+1];
    p.Lattice.dGrid[1]->distribute(last-first);
    p.Lattice.dGrid[1]->PtclDist[0] = first;
    p.Lattice.dGrid[1]->PtclDist[1] = last;
    
    //p.Lattice.printGrid(cout);
  } else {
    int ogrid[] = {1,1,1};
    if(Distributed[enable_omp]) ogrid[0] = omp_get_max_threads();
    p.Lattice.resetGrid(1,ogrid);
    p.Lattice.dGrid[1]->distribute(p.getLocalNum());
    p.Lattice.dGrid[0]->distribute(p.getLocalNum());
  }
}

void NNMethodBase::updateR(Particle_t& p, const ParticlePos_t& pos) {
  convert(p.Lattice,pos,p.R);
}


bool NNMethodBase::put(xmlNodePtr cur){

  Distributed.reset();

#if defined(USE_OPENMP)
  Distributed[enable_omp] = true;
#endif

  xmlAttrPtr att = cur->properties;
  while(att != NULL) {
    string aname((const char*)(att->name));
    string vname((const char*)(att->children->content));
    if(aname == "omp") {
      Distributed[enable_omp] = (vname == "on"|vname == "true");
    } else if(aname == "mpi"){
      Distributed[enable_mpi] = (vname == "on"|vname == "true");
    }
    att = att->next;
  }

  return putExtra(cur);
}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: NNMethodBase.cpp 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/

