//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifndef OHMMS_NEARESTNEIGHBOR_METHODBASE_H
#define OHMMS_NEARESTNEIGHBOR_METHODBASE_H

#include <bitset>
using std::bitset;
#include "Particle/OhmmsParticle.h"
#include "OhmmsData/OhmmsElementBase.h"

namespace OHMMS {

  /*\class NNMethodBase
   *\brief Base class to evaluate nearest-neighbor lists
   */

  class NNMethodBase {

  public:

    enum {enable_omp = 0, enable_mpi};

    typedef Particle_t::ParticlePos_t       ParticlePos_t;
    typedef Particle_t::SingleParticlePos_t SingleParticlePos_t;

    NNMethodBase(int id=0);
    virtual ~NNMethodBase() { }

    inline int getID() const { return ID;}
    inline void setID(int id) { ID = id;}

    inline bool distributed(int i = enable_mpi) const { 
      return Distributed[i];
    }

    inline void distribute(int i, bool doit) {
      Distributed[i] = doit;
    }

    //virtual method to evaluate nearest-neighbor lists, indices
    virtual void get(const Particle_t& p, PtclPairList_t& myppl, int myID) = 0;
    virtual int init(Particle_t&,Scalar_t rmax) = 0;
    virtual void partition(Particle_t& ) = 0;
    virtual void updateR(Particle_t& p, const ParticlePos_t& pos);
    inline int getNumGhosts() const { return nGhosts;}

    void simplePartition(Particle_t& );
    bool put(xmlNodePtr cur);
  protected:

    bitset<2> Distributed;
    int ID;
    int nGhosts;
    double RmaxSq;
    double RminSq;
    virtual bool putExtra(xmlNodePtr cur) = 0;

  };

}
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: NNMethodBase.h 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
