//////////////////////////////////////////////////////////////////
// (c) Copyright 2005- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifndef OHMMS_BOOST_PARTICLEUTILITY_H
#define OHMMS_BOOST_PARTICLEUTILITY_H
//////////////////////////////////////////////////////////
// functors to generate vectors with each element [-0.5,0.5)
//////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////
// specialized of RandomVector in Utilities/RandomGenerator.h for TinyVector<T,D>
//////////////////////////////////////////////////////////

#include "Utilities/BoostRandom.h"

template<class VT, class RNG> struct RandomGaussianVector{};

template<class T, unsigned D, class RNG>
struct 
RandomGaussianVector<TinyVector<T,D>, RNG> {

  typedef TinyVector<T,D> Return_t;

  RandomGaussianVector(RNG& gen): GRG(gen){ }

  inline Return_t operator()() {
    Return_t res;
    for(int i=0; i<D; i++) res[i] = GRG();
    return res;
  }

  boost::normal_distribution<T> GRG;
};


template<class T, class RNG>
struct 
RandomGaussianVector<TinyVector<T,3>, RNG> {

  typedef TinyVector<T,3> Return_t;

  RandomGaussianVector(RNG& gen):uni(gen.getGenerator()){ }

  inline Return_t operator()() {
    return Return_t(g(uni),g(uni),g(uni));
  }

  template<class IT>
  inline void generate(IT first, IT last) {
    assignGaussRand(&((*first)[0]),(last-first)*3);
    //while(first != last) {
    //  *first= Return_t(g(uni),g(uni),g(uni));
    //  ++first;
    //}
  }

  inline void assignGaussRand(T* restrict a, unsigned n) {
    for (int i=0; i+1<n; i+=2) {
      T temp1=1-0.9999999999*uni(), temp2=uni();
      a[i]  =sqrt(-2.0*log(temp1))*cos(6.283185306*temp2);
      a[i+1]=sqrt(-2.0*log(temp1))*sin(6.283185306*temp2);
    }
    if (n%2==1) {
      T temp1=1-0.9999999999*uni(), temp2=uni();
      a[n-1]=sqrt(-2.0*log(temp1))*cos(6.283185306*temp2);
    }
  }


  typename RNG::uniform_generator_type&  uni;
  boost::normal_distribution<T> g;
};
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 300 $   $Date: 2005-02-16 09:38:15 -0600 (Wed, 16 Feb 2005) $
 * $Id: ParticleBoostUtility.h 300 2005-02-16 15:38:15Z jnkim $ 
 ***************************************************************************/
