//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifndef OHMMS_PARTICLEIOBASE_H
#define OHMMS_PARTICLEIOBASE_H

#if defined(__GNUC__)
#include "Utilities/OhmmsBitset.h"
#else
#include <bitset>
using std::bitset;
#endif
#ifndef OHMMS_CRYSTALLATTICE_H
#include "Lattice/CrystalLattice.h"
#endif
#ifndef OHMMS_PARTICLEBASE_H
#include "ParticleBase/ParticleBase.h"
#endif
/*! \class ParticleIO
 *  \author Jeongnim Kim
 *  \brief A base class for IO Engines for a ParticleBase object.
 *  
 */

template<class T, unsigned D>
class  ParticleIOBase {
public:

  enum {INPUT, OUTPUT, PROGRESS};
  enum {UNITCELLGRID, MPPGRID, OMPGRID, SHAPE};

  typedef CrystalLattice<T,D>                          ThisParticleLayout_t;
  typedef ParticleBase<ThisParticleLayout_t>           ThisParticle_t;
  typedef typename ThisParticle_t::ParticlePos_t       ParticlePos_t;
  typedef typename ThisParticle_t::ParticleScalar_t    ParticleScalar_t;
  typedef typename ThisParticle_t::ParticleIndex_t     ParticleIndex_t;
  typedef typename ThisParticle_t::SingleParticlePos_t SingleParticlePos_t;
  typedef bitset<8>                                    PosInputFlags_t;    

  PosInputFlags_t InputFlag;

  ParticleIOBase(ThisParticle_t& p, int i, const char* name = NULL)
  : FileRoot(name), MyPtcl(p), initialized(false), ioMode(i) { }

  virtual ~ParticleIOBase() { 
    DEBUGMSG("calling ParticleIOBase::~ParticleIOBase() for " << FileRoot)
  }//!< Destructor

  void set(int argc, char** argv) {
    for(int i=0; i<argc; i++) options.push_back(argv[i]);
  }//!< Add options.

  void set(vector<string>& argv) {
    for(int i=0; i<argv.size(); i++) options.push_back(argv[i]);
  }//!< Add options.

  template<class ITER>
  void set(ITER start, ITER final) {
    ITER it = start;
    while(it != final) {options.push_back(*it); it++;}
  }

  void reset(const char* name = NULL) { 
    initialized = false;
    if(name) FileRoot = name;
  }
  //!< reset internal Counter

  virtual void read(istream&)=0; //!< read a configuration
  virtual void write() = 0; //!< write a configuration for re-runs
  virtual void record(int)=0;//!< record a configuration (time-stepping)
  virtual void finalize() = 0;

protected:

  int ioMode;
  bool initialized;

  string FileRoot;
  vector<string> options;
  ThisParticle_t& MyPtcl;

private:
  ParticleIOBase() { }//!< Constructor
};

#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: ParticleIOBase.h 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
