//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002,2003- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "ParticleIO/HDFLatticeIO.h"
namespace OHMMS {

#if defined(HAVE_LIBHDF5)

  bool HDFLatticeIO::read(hid_t gid) {
    
    hid_t dataset = H5Dopen(gid, "lattice");
    hid_t ret = H5Dread(dataset, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT, ref_.R.data());
    H5Dclose(dataset);
    
    int spatialgrid=ParticleLayout_t::SPATIAL_GRID;
    int latticegrid[]={1,1,1}, bconds[]={1,1,1};
    typedef PtclOnLatticeTraits::SingleParticleIndex_t SingleParticleIndex_t;
    vector<SingleParticleIndex_t> grid(3,SingleParticleIndex_t(1));
    
    dataset = H5Dopen(gid, "bconds");
    ret = H5Dread(dataset, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, bconds);
    H5Dclose(dataset);
    for(int idir=0; idir<OHMMS_DIM; idir++)
      ref_.BoxBConds[idir]=bconds[idir];
    
    dataset = H5Dopen(gid, "grid");
    ret = H5Dread(dataset, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, latticegrid);
    H5Dclose(dataset);
    for(int idir=0; idir<OHMMS_DIM; idir++) grid[spatialgrid][idir]=latticegrid[idir];
    
    ref_.Grid[spatialgrid]=grid[spatialgrid];
    ref_.reset();
    //ref_.makeGrid(grid);
    return true;
  }

  bool HDFLatticeIO::write(hid_t gid) {

    hsize_t dim2 = OHMMS_DIM*OHMMS_DIM;
    hid_t dataspace  = H5Screate_simple(1, &dim2, NULL);
    hid_t dataset =  H5Dcreate(gid, "lattice", H5T_NATIVE_DOUBLE, dataspace, H5P_DEFAULT);
    hid_t ret = H5Dwrite(dataset, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT,ref_.R.data());
    H5Sclose(dataspace);
    H5Dclose(dataset);
    
    hsize_t dim = OHMMS_DIM;
    int spatialgrid=ParticleLayout_t::SPATIAL_GRID;
    int grid[OHMMS_DIM];
    for(int idir=0; idir<OHMMS_DIM; idir++) grid[idir]=ref_.BoxBConds[idir];
    dataspace  = H5Screate_simple(1, &dim, NULL);
    dataset =  H5Dcreate(gid, "bconds", H5T_NATIVE_INT, dataspace, H5P_DEFAULT);
    ret = H5Dwrite(dataset, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT,grid);
    H5Sclose(dataspace);
    H5Dclose(dataset);
    
    for(int idir=0; idir<OHMMS_DIM; idir++) grid[idir]=ref_.Grid[spatialgrid][idir];
    dataspace  = H5Screate_simple(1, &dim, NULL);
    dataset =  H5Dcreate(gid, "grid", H5T_NATIVE_INT, dataspace, H5P_DEFAULT);
    ret = H5Dwrite(dataset, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT,grid);
    H5Sclose(dataspace);
    H5Dclose(dataset); 
   return true;
  }
#else
  bool HDFLatticeIO::read(hid_t gid) {
    return true;
  }
  
  bool HDFLatticeIO::write(hid_t gid) {
    return true;
  }
#endif
}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 354 $   $Date: 2005-07-07 21:20:49 -0500 (Thu, 07 Jul 2005) $
 * $Id: HDFLatticeIO.cpp 354 2005-07-08 02:20:49Z jnkim $ 
 ***************************************************************************/
