//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include <string>
#include <vector>
#include <utility>
#include <iostream>
#include <fstream>
using namespace std;
#include "Utilities/OhmmsInfo.h"
#include "ParticleIO/ParticleLayoutIO.h"
#include "ParticleIO/HDFParticleIO.h"
using namespace OHMMS;

#if defined(HAVE_LIBHDF5)
#include "ParticleIO/HDFParticleAttrib.h"
#include "ParticleIO/HDFLatticeIO.h"

bool HDFParticleParser::put(const char* hfile, xmlNodePtr cur) {

  cur = cur->children;
  while(cur != NULL) {
    std::string cname((const char*)cur->name);
    if(cname == "unitcell") {
      LatticeParser areader(ref_.Lattice);
      areader.put(cur);
    }
    cur = cur->next;
  }


  hid_t hdfFile = H5Fopen(hfile, H5F_ACC_RDWR, H5P_DEFAULT);

  //get /particleset group.
  hid_t pgroup = H5Gopen(hdfFile, "/particleset");

  bool success=false;
  if(pgroup<0) {
    ERRORMSG("Cannot find the group /particleset")
    success =  false;
  }
  else
    success = read_v03_format(pgroup);

  H5Gclose(pgroup);
  H5Fclose(hdfFile);
  
  if(success) ref_.update(-1);

  return success;

//   xmlDocPtr doc;
//   xmlNsPtr ns;
//   xmlNodePtr cur;
    
//   // build an XML tree from a the file;
//   doc = xmlParseFile(fname);
//   if (doc == NULL) {
//     ERRORMSG(fname << " does not exist")
//       return false;
//   }

//   ///using XPath instead of recursive search
//   xmlXPathContextPtr context;
//   xmlXPathObjectPtr result;
//   context = xmlXPathNewContext(doc);
//   result = xmlXPathEvalExpression((const xmlChar*)"//particleset",context);

//   if(!xmlXPathNodeSetIsEmpty(result->nodesetval)) {
//     cur = result->nodesetval->nodeTab[0];
//     put(cur);
//   }

//   xmlXPathFreeObject(result);
//   return true;

}

bool 
HDFParticleParser::put(xmlNodePtr cur) {

  string hfile;
  xmlChar *h_ptr = xmlGetProp(cur, (const xmlChar *) "href");
  if(h_ptr) {
    hfile = (const char*)h_ptr;
  } else {
    xmlChar *a= xmlGetProp(cur, (const xmlChar *) "src");
    if(a) hfile = (const char*)a;
  }

  if(hfile.empty()) {
    ERRORMSG("No h5 file is provided. Exit");
    return false;
  }

  return put(hfile.c_str(),cur);

  hid_t hdfFile = H5Fopen(hfile.c_str(), H5F_ACC_RDWR, H5P_DEFAULT);
  
  //get /particleset group.
  hid_t pgroup = H5Gopen(hdfFile, "/particleset");

  bool success=false;
  if(pgroup<0) 
    success = read_v02_format(hdfFile,cur);
  else
    success = read_v03_format(pgroup);

  H5Gclose(pgroup);
  H5Fclose(hdfFile);
  
  if(success) ref_.update(-1);

  return success;
}


bool 
HDFParticleParser::read_v03_format(hid_t pgroup) {

  //process group unitcell
  hid_t lat_id = H5Gopen(pgroup,"unitcell");
  if(lat_id>=0) {
    HDFLatticeIO latticein(ref_.Lattice);
    latticein.read(lat_id);
  }
  H5Gclose(lat_id);

  hid_t agroup = H5Gopen(pgroup,"attributes");
  if(agroup>=0) {
    HDFParticleIO pin(ref_);
    pin.read(agroup);
  }
  H5Gclose(agroup);
  return true;
}

bool 
HDFParticleParser::read_v02_format(hid_t h, xmlNodePtr cur) {

  typedef Particle_t::ParticleIndex_t ParticleIndex_t;
  typedef Particle_t::ParticleScalar_t ParticleScalar_t;
  typedef Particle_t::ParticlePos_t ParticlePos_t;

  int natom = 0;
  hid_t dataset = H5Dopen(h, "atnum");
  hid_t ret = H5Dread(dataset, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, &natom);
  H5Dclose(dataset);

  if(natom <= 0) return false;

  cur = cur->xmlChildrenNode;
  while(cur!=NULL) {
    string cname((const char*)(cur->name));
    /**@warning transition from UnitCell -> unitcell */
    if (cname == "UnitCell" || cname == "unitcell") {
      LatticeParser lat(ref_.Lattice);
      lat.put(cur);
    }
    else if (cname == "attrib"){
      string oname(null_tag), otype(null_tag);
      int utype = 0;
      ///process attributes: type or format
      xmlAttrPtr att = cur->properties;
      while(att != NULL) {
	string aname((const char*)(att->name));
	if(aname == "name") {
	  oname = (const char*)(att->children->content);
	} else if(aname == datatype_tag) {
	  otype = (const char*)(att->children->content);
	} else if(aname == condition_tag) {
	  utype = atoi( (const char*)(att->children->content));
	}
	att = att->next;
      }
      if(oname == null_tag || otype == null_tag) break;
      if(otype == scalartype_tag) { 
        ParticleScalar_t* stmp = 0;
        if(oname == energy_tag) 
          stmp = &(ref_.E);
        else 
          stmp = ref_.getScalarAttrib(oname);
        if(stmp->size()<ref_.getLocalNum()) stmp->resize(ref_.getLocalNum());
	stmp->setUnit(utype);
	HDFAttribIO<ParticleScalar_t> hdf_s(*stmp);
	hdf_s.read(h,oname.c_str());
      } else if(otype == postype_tag) {
        if(oname == displacement_tag) continue;
        ParticlePos_t* ptmp =0;
        if(oname == force_tag) 
          ptmp = &(ref_.F);
        else if(oname == velocity_tag) 
          ptmp = &(ref_.V);
        else 
          ptmp = ref_.getVectorAttrib(oname);
        if(ptmp->size()<ref_.getLocalNum()) ptmp->resize(ref_.getLocalNum());
        ptmp->setUnit(utype);
	HDFAttribIO<ParticlePos_t> hdf_p(*ptmp);
	hdf_p.read(h,oname.c_str());
      } else if(otype == indextype_tag) { 
	ParticleIndex_t& itmp = *(ref_.getIndexAttrib(oname.c_str()));
	itmp.setUnit(utype);
	HDFAttribIO<ParticleIndex_t> hdf_i(itmp);
	hdf_i.read(h,oname.c_str());
      }
    }
    cur = cur->next;
  }
  return true;
}

HDFSaveParticle::~HDFSaveParticle() { 
}

void HDFSaveParticle::reset(const char* fileroot){
  FileRoot = fileroot;
}


void HDFSaveParticle::report(int iter) {

  typedef Particle_t::ParticleIndex_t ParticleIndex_t;
  typedef Particle_t::ParticleScalar_t ParticleScalar_t;
  typedef Particle_t::ParticlePos_t ParticlePos_t;

//   string metafile = FileRoot;
//   metafile.append(".xml");
//   // writing a meta file
//   ofstream ftbmd(metafile.c_str()); // always overwrite
//   ftbmd << "<?xml version=\"1.0\"?>" << endl;
//   ftbmd << "<particleset src=\"" << hfile << "\" size=\"" << ref_.getTotalNum()
// 	<<"\" format=\"hdf5\">" << endl;
//   ftbmd.setf(ios_base::scientific);
//   ftbmd.precision(15);
//   LatticeXMLWriter latticeout(ref_.Lattice);
//   latticeout.get(ftbmd);
//   Particle_t::PAListIterator it = ref_.first_attrib();
//   while(it != ref_.last_attrib()) {
//     OhmmsObject& ooref= *((*it).second);
//     ooref.begin_node(ftbmd);
//     ooref.end_node(ftbmd);
//     it++;
//   }
//   ftbmd << "</particleset>" << endl;
//   ftbmd.close();

  string hfile = FileRoot;
  hfile.append(".h5");

  // creat HDF file handler
  hid_t hdfFile =  H5Fcreate(hfile.c_str(), H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
  hid_t pgroup = H5Gcreate(hdfFile, "/particleset",0);

  /////////////////////////////////////////////
  //add the group attributes
  /////////////////////////////////////////////
  hid_t agroup = H5Gcreate(pgroup,"attributes",0);
  HDFParticleIO pout(ref_);
  pout.write(agroup);
  H5Gclose(agroup);

  //////////////////////////////////////////////
  //add of the group unitcell
  //////////////////////////////////////////////
  hid_t lat_id = H5Gcreate(pgroup, "unitcell",0);
  HDFLatticeIO latticeout(ref_.Lattice);
  latticeout.write(lat_id);
  H5Gclose(lat_id);

  H5Gclose(pgroup);
  H5Fclose(hdfFile);
}

bool HDFSaveParticle::put(xmlNodePtr cur) {
  return true;
}

#else
bool HDFParticleParser::put(const char* hfile, xmlNodePtr cur) {
  return false;
}
bool 
HDFParticleParser::put(xmlNodePtr cur) {
  return false;
}

HDFSaveParticle::~HDFSaveParticle() { 
}
void HDFSaveParticle::reset(const char* fileroot){
}
void HDFSaveParticle::report(int iter) {
}
bool HDFSaveParticle::put(xmlNodePtr cur) {
  return true;
}

bool HDFParticleParser::read_v02_format(hid_t h, xmlNodePtr cur) {
  return false;
}
bool HDFParticleParser::read_v03_format(hid_t pgroup) {
  return false;
}

#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 399 $   $Date: 2006-05-04 20:28:01 -0500 (Thu, 04 May 2006) $
 * $Id: HDFParticleIO.cpp 399 2006-05-05 01:28:01Z jnkim $ 
 ***************************************************************************/
