//////////////////////////////////////////////////////////////////
// (c) Copyright 2005- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "Particle/ParticleSet.h"
#include "ParticleIO/HDFRecordParticle.h"
using namespace OHMMS;
void HDFRecordParticle::reset(const char* fileroot) {
  FileName = fileroot;
  FileName.append(".h5");
  ConnectedFile = H5Fcreate(FileName.c_str(), H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
  RootID = H5Gcreate(ConnectedFile, "frames", 0);
}

void HDFRecordParticle::report(int iter) {
  char fname[64];
  sprintf(fname,"f%08d",iter);
  typedef vector<HDFAttribIOBase*>::size_type size_type;
  //RootID = H5Gcreate(ConnectedFile, fname, 0);
  size_type n=Records.size();
  hid_t gid = H5Gcreate(RootID, fname, n);
  for(size_type i=0; i<n; ++i){
    Records[i]->write(gid,RecordNames[i].c_str());
  }
  H5Gclose(gid);
}

void HDFRecordParticle::finalize() {
  H5Gclose(RootID);
  H5Fclose(ConnectedFile);
}

bool HDFRecordParticle::put(xmlNodePtr cur) {

  cur = cur->xmlChildrenNode;
  while (cur != NULL) {
    string cname((const char*)cur->name);
    if (cname == ParticleTags::attrib_tag) { 
      const xmlChar* optr=xmlGetProp(cur,(const xmlChar*)"name");
      const xmlChar* tptr=xmlGetProp(cur,(const xmlChar*)"datatype");
      if(optr == NULL || tptr == NULL) return false;
      string tname((const char*)tptr),oname((const char*)optr);
      if(tname == ParticleTags::scalartype_tag) {
        RecordNames.push_back(oname);
        Records.push_back(new HDFAttribIO<Particle_t::ParticleScalar_t>(*ref_.getScalarAttrib(oname)));
      } else if(tname == ParticleTags::postype_tag) {
        RecordNames.push_back(oname);
        Records.push_back(new HDFAttribIO<Particle_t::ParticlePos_t>(*ref_.getVectorAttrib(oname)));
      }
    }
    cur = cur->next;
  }
  return true;
}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 354 $   $Date: 2005-07-07 21:20:49 -0500 (Thu, 07 Jul 2005) $
 * $Id: HDFRecordParticle.cpp 354 2005-07-08 02:20:49Z jnkim $ 
 ***************************************************************************/
