//////////////////////////////////////////////////////////////////
// (c) Copyright 2005- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#ifndef OHMMS_HDF_RECORDPARTICLEATTRIB_H__
#define OHMMS_HDF_RECORDPARTICLEATTRIB_H__

#include "Particle/ParticleTraits.h"
#include "OhmmsData/RecordProperty.h"
#include "ParticleIO/HDFParticleAttrib.h"

namespace OHMMS {

  /** Record Engine a ParticleAttrib<T> */
  template<class PAT>
  struct HDFRecordAttrib:public RecordProperty {

    typedef PAT Data_t;

    Data_t& ref_;
    string FileName;
    string RootName;
    hid_t ConnectedFile;
    hid_t RootID;

    HDFRecordAttrib(Data_t& a):ref_(a) { }    
    void reset(const char* fileroot) {
      FileName = fileroot;
      FileName.append(".");
      FileName.append(ref_.objName());
      FileName.append(".h5");
      ConnectedFile = H5Fcreate(FileName.c_str(), H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
      RootName="/";
      RootName.append(ref_.objName());
      RootID = H5Gcreate(ConnectedFile, RootName.c_str(),0);
    }

    void report(int iter) {
      //ConnectedFile = H5Fopen(FileName.c_str(), H5F_ACC_RDWR, H5P_DEFAULT);
      //RootID = H5Gopen(ConnectedFile, RootName.c_str());
      HDFAttribIO<Data_t> hdf_p(ref_);
      char newframe[64];
      sprintf(newframe,"f%06d",iter);
      hdf_p.write(RootID,newframe);
    }

    void finalize() {
      H5Gclose(RootID);
      H5Fclose(ConnectedFile);
    }
    bool put(xmlNodePtr cur) {
      return true;
    }
  };

  /** Specialization for TinyVector<real,3> **/
  /*
  template<class T>
  struct HDFRecordAttrib<ParticleAttrib<TinyVector<T,3> > >:public RecordProperty {

    typedef ParticleAttrib<TinyVector<T,3> > Data_t;
    typedef ParticleAttrib<int> ConvertedData_t;

    Data_t& ref_;
    int FirstIndex;
    int LastIndex;
    hid_t ConnectedFile;
    hid_t RootID;
    ConvertedData_t iData;
    string FileName;

    HDFRecordAttrib(Data_t& a, int first = 0, int last = -1):ref_(a),FirstIndex(first),LastIndex(last) { }    
    void reset(const char* fileroot) {
      if(LastIndex<0) { LastIndex = ref_.size();}
      FileName = fileroot;
      FileName.append(".pos100.h5");
      ConnectedFile = H5Fcreate(FileName.c_str(), H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
      char rname[64];
      sprintf(rname,"/%s",ref_.objName().c_str());
      RootID = H5Gcreate(ConnectedFile, rname,0);
      iData.resize((LastIndex-FirstIndex)*3);
    }

    void report(int iter) {
      int iii=0;
      for(int i=FirstIndex; i< LastIndex; i++) {
        iData[iii++] = static_cast<int>(100.*ref_[i][0]);
        iData[iii++] = static_cast<int>(100.*ref_[i][1]);
        iData[iii++] = static_cast<int>(100.*ref_[i][2]);
      }
      HDFAttribIO<ConvertedData_t> hdf_p(iData);
      char newframe[64];
      sprintf(newframe,"f%06d",iter);
      hdf_p.write(RootID,newframe);
    }

    void finalize() {
      H5Gclose(RootID);
      H5Fclose(ConnectedFile);
    }
    bool put(xmlNodePtr cur) {
      return true;
    }
  };
  */
}
#endif
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 299 $   $Date: 2005-02-16 09:37:33 -0600 (Wed, 16 Feb 2005) $
 * $Id: HDFRecordParticleAttrib.h 299 2005-02-16 15:37:33Z jnkim $ 
 ***************************************************************************/
