//////////////////////////////////////////////////////////////////
//                        COPYRIGHT NOTICE
//   (c) Copyright 1998-2002,2005- David A. Richie and Jeongnim Kim.
//   All rights reserved.
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   David A. Richie
//   Department of Physics, Ohio State University
//   174 W 18th Avenue
//   Columbus, OH 43210
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
/**@file MCFRecordParticleAttrib.cpp
 * @brief Definition of MCFRecordScalarAttrib and MCFRecordPosAttrib
 * Records using MCF format in Progress mode.  Relies on 
 * components from the STORM Scientific Wavelet Package (v3.1.0)
 * which is (c) Copyright 1995-2002 David A. Richie.
 */
#include "Utilities/SimpleParser.h"
#include "ParticleIO/MCFRecordParticleAttrib.h"
#include "Utilities/OhmmsInfo.h"
#include <stdio.h>
using namespace OHMMS;

MCFRecordScalarAttrib::MCFRecordScalarAttrib(const InData_t& a): 
  MCFRecordAttrib<OHMMS_PRECISION,double>(0,a.size(),1), ref_(a) {
}

MCFRecordScalarAttrib::MCFRecordScalarAttrib(const InData_t& a, int first, int last):
  MCFRecordAttrib<OHMMS_PRECISION,double>(first,last,3), ref_(a) {
}

MCFRecordScalarAttrib::~MCFRecordScalarAttrib(){ }

void MCFRecordScalarAttrib::reset(const char* fileroot) {
  string aname(fileroot);
  aname.append(".").append(ref_.objName());
  resetRecord(aname);
}

void MCFRecordScalarAttrib::finalize() {
  finalizeRecord();
}

bool MCFRecordScalarAttrib::put(xmlNodePtr cur) {
  return putRecord(cur);
}

void MCFRecordScalarAttrib::report(int iter) {
  for(int i=FirstIndex,ii=0;i<LastIndex;++i,++ii) { 
    m_rtmra[ii]=static_cast<value_type>(m_mag*ref_[i]);
  }
  m_rtmra++;
  *m_mcfofs << m_rtmra;
}

MCFRecordPosAttrib::MCFRecordPosAttrib(const InData_t& a): 
  MCFRecordAttrib<OHMMS_PRECISION,double>(0,a.size(),3), ref_(a) {
}
MCFRecordPosAttrib::MCFRecordPosAttrib(const InData_t& a, int first, int last): 
  MCFRecordAttrib<OHMMS_PRECISION,double>(first,last,3), ref_(a) {
}

MCFRecordPosAttrib::~MCFRecordPosAttrib(){ }

void MCFRecordPosAttrib::reset(const char* fileroot) {
  string aname(fileroot);
  aname.append(".").append(ref_.objName());
  //mra_mmax=12;
  //dcf_thresh = 100000;
  //m_mag = 1.0e6;
  resetRecord(aname);
}

void MCFRecordPosAttrib::finalize() {
  finalizeRecord();
}

bool MCFRecordPosAttrib::put(xmlNodePtr cur) {
  return putRecord(cur);
}

void MCFRecordPosAttrib::report(int iter) {
  for(int i=FirstIndex,ii=0;i<LastIndex;++i) { 
    //m_rtmra[ii++]=static_cast<value_type>(m_mag*ref_[i][0]);
    //m_rtmra[ii++]=static_cast<value_type>(m_mag*ref_[i][1]);
    //m_rtmra[ii++]=static_cast<value_type>(m_mag*ref_[i][2]);
    m_rtmra[ii++]=ref_[i][0];
    m_rtmra[ii++]=ref_[i][1];
    m_rtmra[ii++]=ref_[i][2];
  }
  m_rtmra++;
  *m_mcfofs << m_rtmra;
}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 371 $   $Date: 2005-07-28 10:07:15 -0500 (Thu, 28 Jul 2005) $
 * $Id: MCFRecordParticleAttrib.cpp 371 2005-07-28 15:07:15Z jnkim $ 
 ***************************************************************************/

