//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include <string>
#include <vector>
#include <utility>
#include <iostream>
#include <fstream>
#include "OhmmsData/FileUtility.h"
using namespace std;
#include "Utilities/OhmmsInfo.h"
#include "ParticleIO/ParticleParser.h"
#include "ParticleBase/ParticleFunctions.h"
#if defined(HAVE_LIBHDF5)
#include "ParticleIO/HDFParticleIO.h"
#endif
#include "ParticleIO/XMLParticleIO.h"
#include "ParticleIO/VaspParticleIO.h"
#include "ParticleIO/TBMDParticleParser.h"
#include "ParticleIO/XYZParticleParser.h"

namespace OHMMS {

  bool ParticleParser::put(xmlNodePtr cur){

    const char* fname = NULL;

    if(xmlHasProp(cur, (const xmlChar *) "file")) {
      fname =(const char*)(xmlGetProp(cur, (const xmlChar *) "file"));
    } else if(xmlHasProp(cur, (const xmlChar *) "src")) {
      fname =(const char*)(xmlGetProp(cur, (const xmlChar *) "src"));
    }

    if(fname != NULL) {
      LOGMSG("\tReding from an external source " << fname)
      string pformat = getExtension(fname);
      if(pformat == "xml") {
	XMLParticleParser aHandle(ref_);
	return  aHandle.put(fname,pformat);
      } else if(pformat == "vasp") {
	VaspParticleParser aHandle(ref_);
	return  aHandle.put(fname);
      } else if(pformat == "xyz") {
	WARNMSG("Using xyz format: check <unitcell/> is defined")
	XYZParticleParser aHandle(ref_);
	return aHandle.put(fname,cur);
      } else if(pformat == "tbmd") { 
	WARNMSG("Using tbmd format: please use xml format")
	TBMDParticleParser aHandle(ref_);
	return aHandle.put(fname);
      } 
#if defined(HAVE_LIBHDF5)
      else if(pformat == "h5") {
	WARNMSG("Using hdf5 format")
	HDFParticleParser aHandle(ref_);
	return aHandle.put(fname,cur);
      }
#endif
    }

    XMLParticleParser aHandle(ref_);
    return aHandle.put(cur);
  }
}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 323 $   $Date: 2005-03-01 17:30:32 -0600 (Tue, 01 Mar 2005) $
 * $Id: ParticleParser.cpp 323 2005-03-01 23:30:32Z jnkim $ 
 ***************************************************************************/
