//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#include "Utilities/OhmmsInfo.h"
#include "ParticleIO/RecordParticleAttrib.h"
#include "ParticleIO/RecordParticle.h"
#ifdef HAVE_LIBHDF5
#include "ParticleIO/HDFParticleIO.h"
//#include "ParticleIO/HDFRecordParticleAttrib.h"
#include "ParticleIO/HDFRecordParticle.h"
#endif
#include "ParticleIO/XMLParticleIO.h"
#include "ParticleIO/VaspParticleIO.h"
#ifdef HAVE_LIBMCF
#include "ParticleIO/MCFRecordParticle.h"
#endif
using namespace OHMMS;

RecordParticle::~RecordParticle(){

  for(int i=0; i<Recorders.size(); i++) delete Recorders[i];

}

void RecordParticle::reset(const char* fileroot){

  char fname[128];
  if(Recorders.size() == 1) {
      sprintf(fname,"%s",fileroot);
      Recorders[0]->reset(fname);
  } else {
    for(int i=0; i<Recorders.size(); i++) {
      sprintf(fname,"%s.%d",fileroot,i);
      Recorders[i]->reset(fname);
    }
  }
}

void RecordParticle::report(int iter) {

  //prevent duplicated record
  if(iter<0) return;  

  if(stride && iter%stride == 0) {
    for(int i=0; i<Recorders.size(); i++) {
      Recorders[i]->report(iter);
    }
  }
}

void RecordParticle::finalize() {
  for(int i=0; i<Recorders.size(); i++) {
    Recorders[i]->finalize();
  }
}

bool RecordParticle::put(xmlNodePtr cur) {
  
  ///default format is ascii
  string aformat("ascii");
  xmlAttrPtr att = cur->properties;
  while(att != NULL) {
    string aname((const char*)(att->name));
    if(aname == "stride") {
      stride = atoi((const char*)(att->children->content));
    } else if(aname == format_tag) {
      aformat =(const char*)(att->children->content);
    }
    att = att->next;
  }

#ifdef HAVE_LIBMCF
  if(aformat == "mcf") {
    int k=Recorders.size();
    Recorders.push_back(new MCFRecordParticle(ref_));
    return Recorders[k]->put(cur); 
    //cur = cur->children;
    //while(cur != NULL) {
    //  string cname((const char*)cur->name);
    //  if (cname == attrib_tag) { 
    //    const xmlChar* nptr = xmlGetProp(cur,(const xmlChar*)"name");
    //    if(nptr) {
    //      string aname((const char*)nptr);
    //      RecordProperty* arecord = 0;
    //      if(aname == position_tag) {
    //        arecord = new MCFRecordPosAttrib(ref_.R);
    //      } else if(aname == energy_tag) {
    //        arecord = new MCFRecordScalarAttrib(ref_.E);
    //      }
    //      if(arecord) {
    //        LOGMSG("\tCreating mcf record engine for " << aname)
    //        arecord->put(cur);
    //        Recorders.push_back(arecord);
    //      }
    //    }
    //  }
    //  cur = cur->next;
    //}
    //return true;
  }
#endif

#ifdef HAVE_LIBHDF5
  if(aformat == "hdf5") {
    vector<RecordProperty*>::size_type k=Recorders.size();
    Recorders.push_back(new HDFRecordParticle(ref_));
    return Recorders[k]->put(cur); 
  }
#endif

  cur = cur->xmlChildrenNode;
  while (cur != NULL) {

    string cname((const char*)cur->name);

    if (cname == attrib_tag) { 
      string ttype(null_tag), otype(null_tag);
      bool accumulative = false;
      att = cur->properties;
      while(att != NULL) {
	string aname((const char*)(att->name));
	if(aname == name_tag) {
	  otype=(const char*)(att->children->content);
	} else if(aname == datatype_tag) {
	  ttype = (const char*)(att->children->content);
	} else if(aname == role_tag) {
	  string role((const char*)(att->children->content));
	  if(role == "accum") accumulative = true;
	}
	att = att->next;
      }

      bool hasattrib = ref_.hasAttrib(otype);
      bool istrajectory = (otype == trajectory_tag);
      //if(!(istrajectory || hasattrib)) {
      //   WARNMSG("\tMissing attribute for a record " << ttype << " " << otype)
      //   break; 
      //}
      if(ttype == indextype_tag) {
	Recorders.push_back(new RecordIndexAttrib(*ref_.getIndexAttrib(otype)));
      } else if(ttype == scalartype_tag) {
        //if(aformat == "hdf5") 
	//  Recorders.push_back(new HDFRecordAttrib<Particle_t::ParticleScalar_t>(*ref_.getScalarAttrib(otype)));
        //else
	  Recorders.push_back(new RecordScalarAttrib(*ref_.getScalarAttrib(otype)));
      } else if(ttype == postype_tag) {
        //if(aformat == "hdf5") {
	//  Recorders.push_back(new HDFRecordAttrib<Particle_t::ParticlePos_t>(*ref_.getVectorAttrib(otype)));
        //} else {
	  if(istrajectory) {
            //treat trajectory differently from other ParticlePos_t
	    if(hasattrib) {
	      Recorders.push_back(new RecordTrajectory(*ref_.getVectorAttrib(otype),ref_.Lattice));
	    } else {
	      Recorders.push_back(new RecordTrajectory(ref_.R,ref_.Lattice));
	    }
	  } else {
	    if(accumulative) {//add accumulated boundary crossings
	      RecordAddAttrib* t
	        = new RecordAddAttrib(*(ref_.getVectorAttrib(otype)), 
	  			    *(ref_.getVectorAttrib(sumbc_tag)),
                                      ref_.Lattice);
	      Recorders.push_back(t);
	      //t->Lattice = ref_.Lattice;
	    } else {
	      Recorders.push_back(new RecordVectorAttrib(*ref_.getVectorAttrib(otype)));
	    }
	  }
        //}
      } else if(ttype == xmoltype_tag) {
	RecordComboAttrib* t
	  = new RecordComboAttrib(ref_.GroupID, *(ref_.getVectorAttrib(otype)),ref_.Lattice);
	Recorders.push_back(t);
	//t->Lattice = ref_.Lattice;
	t->Species = ref_.getSpeciesSet().Name;
      } 
    }
    cur=cur->next;
  }
  return true;
}

void RecordParticle::setAnimationRecord(int freq) {
  RecordComboAttrib* t = new RecordComboAttrib(ref_.GroupID, ref_.R,ref_.Lattice);
  Recorders.push_back(t);
  t->stride= freq;
  t->Species = ref_.getSpeciesSet().Name;
}

/////////////////////////////////////
//functions ot SaveParticle
/////////////////////////////////////
SaveParticle::~SaveParticle(){

  if(Dump) delete Dump; 
}

void SaveParticle::reset(const char* fileroot){

  char fname[128];
  sprintf(fname,"%s.restart",fileroot);
  if(!Dump) {
    switch (DumpType) {
    case 0:
      Dump = new XMLSaveParticle(ref_); break;
    case 1:
      Dump = new VaspSaveParticle(ref_); break;
    default:
#if defined(HAVE_LIBHDF5)
      Dump = new HDFSaveParticle(ref_);
#else
      Dump = new XMLSaveParticle(ref_);
#endif
      break;
    }
  }

  Dump->reset(fname);
  FileName = Dump->FileName;
}

void SaveParticle::report(int iter) {

  if(iter<0) {
    Dump->report(-1);
    return;
  }

  if(stride && iter%stride == 0) {
    Dump->report(iter);
  }
}

bool SaveParticle::put(xmlNodePtr cur) {
  
  ///default format is ascii
  string aformat("xml");
  xmlAttrPtr att = cur->properties;
  while(att != NULL) {
    string aname((const char*)(att->name));
    if(aname == "stride") {
      stride = atoi((const char*)(att->children->content));
    } else if(aname == format_tag) {
      aformat =(const char*)(att->children->content);
    }
    att = att->next;
  }

  if(aformat == "tbmd")   
    DumpType = 0;
  else if(aformat == "xml") 
    DumpType = 0;
  else if(aformat == "vasp") 
    DumpType = 1;
  else
    DumpType = 2;

  stride = atoi((const char*)(xmlGetProp(cur, (const xmlChar *) "stride")));
  //   const char* FileFormat 
  //     = (const char*)(xmlGetProp(cur, (const xmlChar *) "format"));
  
  return true;
}

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 354 $   $Date: 2005-07-07 21:20:49 -0500 (Thu, 07 Jul 2005) $
 * $Id: RecordParticle.cpp 354 2005-07-08 02:20:49Z jnkim $ 
 ***************************************************************************/
