//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#ifndef OHMMS_RECORDPARTICLEATTRIB_H__
#define OHMMS_RECORDPARTICLEATTRIB_H__

#include "Particle/ParticleTraits.h"
#include "ParticleIO/ParticleLayoutIO.h"
#include "OhmmsData/RecordProperty.h"

namespace OHMMS {

  /*!\class RecordIndexAttrib
   * \brief Record Engine a ParticleAttrib<int>
   */
  struct RecordIndexAttrib:public RecordProperty {

    typedef PtclAttribTraits::ParticleIndex_t Data_t;
    typedef Data_t::Type_t                    value_type;

    Data_t& ref_;
    string FileName;

    RecordIndexAttrib(Data_t& a):ref_(a){ } 
    void reset(const char* fileroot);
    void report(int iter);
    void finalize() { }
    bool put(xmlNodePtr cur);
  };

  /*!\class RecordScalarAttrib
   * \brief Record Engine a ParticleAttrib<Scalar_t>
   */
  struct RecordScalarAttrib:public RecordProperty {

    typedef PtclAttribTraits::ParticleScalar_t Data_t;
    typedef Data_t::Type_t                     value_type;

    Data_t& ref_;
    string FileName;

    RecordScalarAttrib(Data_t& a):ref_(a) { }    
    void reset(const char* fileroot);
    void report(int iter);
    void finalize() { }
    bool put(xmlNodePtr cur);
  };

  /*!\class RecordVectorAttrib
   * \brief Record Engine for a ParticleAttrib<SingleParticlePos_t>
   */
  struct RecordVectorAttrib:public RecordProperty {
   
    typedef PtclAttribTraits::ParticlePos_t Data_t;
    typedef Data_t::Type_t                value_type;

    Data_t& ref_;
    string FileName;

    RecordVectorAttrib(Data_t& a):ref_(a){ }    
    void reset(const char* fileroot);
    void report(int iter);
    void finalize() { }
    bool put(xmlNodePtr cur);
  };


  /*!\class RecordTrajectory
   * \brief Record Engine for a ParticleAttrib<SingleParticlePos_t> representing Trajectory
   */
  struct RecordTrajectory:public RecordProperty {

    typedef PtclOnLatticeTraits::ParticleLayout_t ParticleLayout_t;
    typedef PtclOnLatticeTraits::ParticlePos_t    Data_t;
    typedef Data_t::Type_t                        value_type;

    Data_t& ref_;
    LatticeXMLWriter lattice_out;
    std::ostream* _file;

    RecordTrajectory(Data_t& a, ParticleLayout_t& lat):
      ref_(a),
      lattice_out(lat),
      _file(NULL){ }    
    void reset(const char* fileroot);
    void report(int iter);
    void finalize();
    bool put(xmlNodePtr cur);
  };

  /*!\class RecordComboAttrib
   * \brief Record Engine a ParticleAttrib<SingleParticlePos_t> to confirm xyz file
   */
  struct RecordComboAttrib:public RecordProperty {

    typedef PtclOnLatticeTraits::ParticleLayout_t ParticleLayout_t;
    typedef PtclOnLatticeTraits::ParticlePos_t    ParticlePos_t;
    typedef PtclOnLatticeTraits::ParticleIndex_t  ParticleIndex_t;
    typedef ParticlePos_t::Type_t            value_type;

    ParticleIndex_t& ID;
    ParticlePos_t& R;

    string FileName;
    ParticleLayout_t& Lattice;
    vector<string> Species;

    RecordComboAttrib(ParticleIndex_t& id, 
		      ParticlePos_t& r,
                      ParticleLayout_t& lat): ID(id), R(r), Lattice(lat){}

    void reset(const char* fileroot);
    void report(int iter);
    void finalize(){}
    bool put(xmlNodePtr cur);
  };

  /*!\class RecordAddAttrib
   * \brief Record Engine a ParticleAttrib<SingleParticlePos_t> 
   */
  struct RecordAddAttrib:public RecordProperty {

    typedef PtclOnLatticeTraits::ParticleLayout_t ParticleLayout_t;
    typedef PtclOnLatticeTraits::ParticlePos_t    ParticlePos_t;

    ParticlePos_t& R1;
    ParticlePos_t& R2;

    string FileName;
    ParticleLayout_t& Lattice;

    RecordAddAttrib(ParticlePos_t& a,ParticlePos_t& b, ParticleLayout_t& lat): 
      R1(a), R2(b), Lattice(lat){}

    void reset(const char* fileroot);
    void report(int iter);
    void finalize(){}
    bool put(xmlNodePtr cur);
  };

}

#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: RecordParticleAttrib.h 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
