//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include <string>
#include <vector>
#include <utility>
#include <iostream>
#include <fstream>
using namespace std;
#include "Utilities/OhmmsInfo.h"
#include "Utilities/SimpleParser.h"
#include "ParticleIO/TBMDParticleParser.h"
using namespace OHMMS;

bool TBMDParticleParser::put(const char* fname) {

  ifstream fpos(fname);
  if(fpos) {

    DEBUGMSG("Performing TBMDParticleParser::put(fname)");
    ParticleLayout_t::SingleParticleIndex_t uc_grid;

#if (__GNUC__ == 2)
    streampos cur = fpos.tellg();
#else
    istream::pos_type cur = fpos.tellg();
#endif
    read_lattice(fpos,ref_.Lattice, uc_grid);
    fpos.seekg(cur);

    vector<string> slist, vlist;
    bool tofindpos = true;
    do {
      Particle_t::size_type nw = getXwords(slist,fpos);//remove xml related stuff
      if(nw == 0) continue;
      if(slist[0] == "ptclset"){
	tofindpos = false;
	int isub=1;
	while(isub<slist.size()) {
	  if(slist[isub] == "oname") {
	    ref_.setName(slist[++isub].c_str());
	  }
	  isub++;
	}
      }
    } while(tofindpos);

    Particle_t::size_type num_index=0, num_val=0, num_pos=0, num_tensor=0;
    Particle_t::size_type natom = 0;
    do 
    {
      Particle_t::size_type argc = getwords(slist, fpos);
      if(argc == 0) continue;
      if(slist[0] == "number_of_ions") {
        for(int i=1; i<slist.size(); i++)  natom += atoi(slist[i].c_str());
      } else if(slist[0] == position_tag || slist[0] == "positions") {
        if(slist[1][0] == 'u' || slist[1][0] == 'U')  
          ref_.R.setUnit(PosUnit::LatticeUnit);
        else                                          
          ref_.R.setUnit(PosUnit::CartesianUnit);
        break;
      }
    } while(!fpos.eof());

    if(natom) {
      Particle_t::size_type loc = ref_.getLocalNum();
      ref_.create(natom);
      Particle_t::SingleParticlePos_t pos;
      SpeciesSet& species = ref_.getSpeciesSet();
      for(int i=0; i<natom; i++) {
        Particle_t::size_type argc =  getwords(slist, fpos);
        ref_.ID[loc] = i;
	vector<string>::iterator it(slist.begin());
	if(isdigit((*it)[0])) {
	  ref_.GroupID[loc] = atoi((*it).c_str())-1;
	} else {
	  ref_.GroupID[loc] = species.getSpeciesID((*it));
	}
	++it;
	for(int idim=0; idim<OHMMS_DIM; idim++,it++)       
	  ref_.R[loc][idim] = atof((*it).c_str());
	if(it != slist.end()) {
	  ref_.ID[loc] = atoi((*it).c_str());
	}
        loc++;
      }

      while(!fpos.eof()) { // reading extra attributes
        Particle_t::size_type argc = getwords(slist,fpos);
        if(argc == 0) continue;
        if(slist[0] == position_tag) // reading a position type
	  { 
	    ref_.addAttribute(postype_tag, slist[1].c_str());
	    ParticlePos_t& pref = *(ref_.getVectorAttrib(slist[1].c_str()));
            pref.setUnit(atoi(slist[2].c_str()));
	    for(int i=0; i<natom; i++) 
	      {
		argc = getwords(vlist,fpos);
		for(int idim=0; idim<OHMMS_DIM; idim++)       
		  pref[i][idim] = atof(vlist[idim].c_str());
	      }
	  }
        else if(slist[0] == scalartype_tag) 
	  {
	    ref_.addAttribute(scalartype_tag, slist[1].c_str());

            ParticleScalar_t& sref = *(ref_.getScalarAttrib(slist[1].c_str()));
	    int i=0;
	    while(i<natom) {
	      argc = getwords(vlist,fpos);
	      for(int iargc=0; iargc<argc; iargc++) {
		sref[i++] = atof(vlist[iargc].c_str());
	      }
	    }
	  }
	else if(slist[0] == indextype_tag)
	  {
            ref_.addAttribute(indextype_tag, slist[1].c_str());
	    
            ParticleIndex_t& sref = *(ref_.getIndexAttrib(slist[1].c_str()));
	    int i=0;
	    while(i<natom) {
	      argc = getwords(vlist,fpos);
	      for(int iargc=0; iargc<argc; iargc++) {
		sref[i++] = atoi(vlist[iargc].c_str());
	      }
	    }
	  }
	else if(slist[0] == "</TBMD>") break;
      } 

      ref_.update(-1);
    }
    return true;
  } else {
    ERRORMSG("\tFile "<<fname << " is not found")
    return false;
  }
}

void 
TBMDParticleParser::read_lattice(std::istream& fpos, 
                                 ParticleLayout_t& lat, 
  	                         SingleParticleIndex_t& uc_grid) {

  enum {SPATIAL_GRID=ParticleLayout_t::SPATIAL_GRID,
	OMP_GRID,
        MPI_GRID,
	GridLevel};

  typedef ParticleLayout_t::SingleParticleIndex_t SingleParticleIndex_t;
  vector<SingleParticleIndex_t> mgrid(GridLevel,SingleParticleIndex_t(1));
  vector<string> vlist,slist;
  int ic=0;

  typedef Particle_t::Scalar_t Scalar_t;
  Scalar_t a0tmp=1.0;
  Scalar_t vtmp[] = {1.0e0, 0.0e0, 0.0e0, 
		     0.0e0, 1.0e0, 0.0e0, 
		     0.0e0, 0.0e0, 1.0e0};

  bool tofindlattice = true;
  while(!fpos.eof() && tofindlattice) {
    Particle_t::size_type argc = getXwords(slist,fpos);
    if(argc == 0) continue;
    if(slist[0] == "unitcell") tofindlattice = false;
  }

  tofindlattice = true;
  do {
    Particle_t::size_type argc = getXwords(slist,fpos);
    if(argc == 0) continue;
    vlist = slist;
    if(slist[0] == "a0") {
      getXwords(vlist, fpos, "/a0");
      a0tmp = atof(vlist[1].c_str());
    } else if(slist[0] == "lattice") {
      getXwords(vlist, fpos, "/lattice");
      for(int ij=0; ij<9; ij++) vtmp[ij] = atof(vlist[ij+1].c_str());
    }else if(slist[0] == "mppgrid") {
      getXwords(vlist, fpos, "/mppgrid");
      mgrid[MPI_GRID][0] = atoi(vlist[1].c_str());
      mgrid[MPI_GRID][1] = atoi(vlist[2].c_str());
      mgrid[MPI_GRID][2] = atoi(vlist[3].c_str());
    } else if(slist[0] == "ompgrid") {
      getXwords(vlist, fpos, "/ompgrid");
      mgrid[OMP_GRID][0] = atoi(vlist[1].c_str());
      mgrid[OMP_GRID][1] = atoi(vlist[2].c_str());
      mgrid[OMP_GRID][2] = atoi(vlist[3].c_str());
    } else if(slist[0] == "grid") {
      getXwords(vlist, fpos, "/grid");
      mgrid[SPATIAL_GRID][0] = atoi(vlist[1].c_str());
      mgrid[SPATIAL_GRID][1] = atoi(vlist[2].c_str());
      mgrid[SPATIAL_GRID][2] = atoi(vlist[3].c_str());
    } else if(slist[0] == "unit_cell_grid") {
      getXwords(vlist, fpos, "/unit_cell_grid");
      uc_grid[0] = atoi(vlist[1].c_str());
      uc_grid[1] = atoi(vlist[2].c_str());
      uc_grid[2] = atoi(vlist[3].c_str());
    } else if(slist[0] == "bconds") { // boundary conditions
      getXwords(vlist, fpos, "/bconds");
        if(vlist[1][0] == 'n' || vlist[1][0] == 'N') {
          lat.BoxBConds[0] = false;
        }
        if(vlist[2][0] == 'n' || vlist[2][0] == 'N') {
          lat.BoxBConds[1] = false;
        }
        if(vlist[3][0] == 'n' || vlist[3][0] == 'N') {
          lat.BoxBConds[2] = false;
        }
    } else if(slist[0] == "/unitcell") {
      tofindlattice = false;
    }
  } while(!fpos.eof() && tofindlattice);


  ///reset the lattice and grid partitions
  lat.set(a0tmp,vtmp);
  lat.makeGrid(mgrid);
}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 354 $   $Date: 2005-07-07 21:20:49 -0500 (Thu, 07 Jul 2005) $
 * $Id: TBMDParticleParser.cpp 354 2005-07-08 02:20:49Z jnkim $ 
 ***************************************************************************/
