//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include <string>
#include <vector>
#include <utility>
#include <iostream>
#include <fstream>
using namespace std;
#include "OhmmsUnits.h"
#include "OhmmsData/FileUtility.h"
#include "Utilities/OhmmsInfo.h"
#include "ParticleIO/VaspParticleIO.h"
#include "ParticleBase/ParticleFunctions.h"
using namespace OHMMS;

///reading from a file
bool VaspParticleParser::put(const char* fname) {

  typedef Particle_t::Scalar_t Scalar_t;
  typedef Particle_t::size_type size_type;

  ifstream vasp(fname);
  if(vasp) {
    string textline;
    size_type linenumber = 0, nspecies = 0, nat_tot = 0;
    Scalar_t a0 = 1.0;
    Tensor<Scalar_t,3> lat;
    vector<size_type> nat;
    bool inunit = 0;

    while(linenumber<8) {
      getline(vasp,textline,'\n');
      std::istringstream  stream(textline);
      switch (linenumber) {
      case 0:
	m_title = textline; break;
      case 1:
	stream >> a0; a0 *= bohr; break;
      case 2:
	stream >> lat(0,0) >> lat(0,1) >> lat(0,2); break;
      case 3:
	stream >> lat(1,0) >> lat(1,1) >> lat(1,2); break;
      case 4:
	stream >> lat(2,0) >> lat(2,1) >> lat(2,2); 
        lat *= a0;
	ref_.Lattice.set(lat);
	break;
      case 5:
	while(stream) {
	  nat.push_back(0); stream >> nat[nspecies];
	  nat_tot += nat[nspecies++];
	}
	break;
      case 6: break;
      case 7:
	string atype;
	stream >> atype;
	if(atype[0] == 'D' || atype[0] == 'd') {
	  inunit = 1;
	}
	break;
      }
      linenumber++;
    }

    // ref_.create(nat_tot);
    if(!nat.back()) nat.pop_back();
    for(size_type ispecies=0; ispecies < nat.size(); ispecies++) {
      cout << nat[ispecies] << endl;
    }

    ref_.create(nat);
    ref_.R.setUnit(inunit);
    for(size_type iat=0; iat<nat_tot; iat++) {
      getline(vasp,textline,'\n');
      std::istringstream  stream(textline);
      stream >> ref_.R[iat][0] >> ref_.R[iat][1] >> ref_.R[iat][2];
      ref_.ID[iat] = iat;
    }

    //need to convert Ry to AA
    if(!inunit) {
      WARNMSG("The unit of the input configuration is in Ry. Converting to AA")
      ref_.R *= bohr;
    }

    ///barebone initialization
    ref_.update(-1);
    return true;
  } else {
    ERRORMSG("File " << fname << " does not exist")
    return false;
  }
}

VaspSaveParticle::~VaspSaveParticle() { 
}

void VaspSaveParticle::reset(const char* fileroot){
  FileRoot = fileroot;
}

void VaspSaveParticle::report(int iter) {

  string metafile = FileRoot;
  metafile.append(".vasp");

  typedef Particle_t::Scalar_t Scalar_t;
  const Scalar_t bohrinv = 1.0/bohr;
  // writing a meta file
  ofstream vasp(metafile.c_str()); // always overwrite
  vasp << ref_.getName() << endl;
  vasp << 1.0 << endl;
  Tensor<Scalar_t,3> lat = ref_.Lattice.R*bohrinv;
  vasp << lat;
  for(int ig=0; ig<ref_.groups(); ig++) {
    vasp << " " << ref_.last(ig)-ref_.first(ig);
  }
  vasp << endl;
  vasp << "Selective dynamics" << endl;
  if(ref_.R.getUnit() == PosUnit::LatticeUnit) {
    vasp << "Direct" << endl;
  } else {
    vasp << "Cartesian" << endl;
  }

  for(int iat=0; iat<ref_.getTotalNum(); iat++) {
    vasp << ref_.R[iat] << endl;
  }
}

bool VaspSaveParticle::put(xmlNodePtr cur) {
  
  return true;
}


/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 354 $   $Date: 2005-07-07 21:20:49 -0500 (Thu, 07 Jul 2005) $
 * $Id: VaspParticleIO.cpp 354 2005-07-08 02:20:49Z jnkim $ 
 ***************************************************************************/
