//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "Utilities/OhmmsInfo.h"
#include "Utilities/SimpleParser.h"
#include "ParticleIO/XYZParticleParser.h"
#include "ParticleIO/ParticleLayoutIO.h"
using namespace OHMMS;

bool XYZParticleParser::put(const char* fname, xmlNodePtr cur) {

  cur = cur->children;
  while(cur != NULL) {
    string cname((const char*)(cur->name));
    if(cname == "unitcell") {
      LatticeParser lat(ref_.Lattice);
      lat.put(cur);
    }
    cur = cur->next;
  }

  ifstream fxmol(fname);
  if(fxmol) {
    DEBUGMSG("Performing XYZParticleParser::put(fname)");

    SpeciesSet& species = ref_.getSpeciesSet();
    vector<string> slist;

    //get the number of particles
    getwords(slist,fxmol);
    unsigned natom = atoi(slist[0].c_str());

    //skip the next line
    getwords(slist,fxmol);

    ref_.R.setUnit(PosUnit::CartesianUnit);
    if(natom) {
      Particle_t::size_type loc(ref_.getLocalNum());
      ref_.create(natom);
      Particle_t::SingleParticlePos_t pos;
      int i=0;
      while(i<natom && !fxmol.eof()) {
        getwords(slist, fxmol);
        ref_.ID[loc] = i;
	int ipos = 0;
        ref_.GroupID[loc] = species.getSpeciesID(slist[ipos++]);
	///xyz is always 3
	for(int idim=0; idim<3; idim++) 
	  ref_.R[loc][idim] = atof(slist[ipos++].c_str());
        loc++; i++;
      }
    }
    return true;
  } else {
    ERRORMSG("\tFile "<<fname << " is not found")
    return false;
  }
}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 354 $   $Date: 2005-07-07 21:20:49 -0500 (Thu, 07 Jul 2005) $
 * $Id: XYZParticleParser.cpp 354 2005-07-08 02:20:49Z jnkim $ 
 ***************************************************************************/
