//////////////////////////////////////////////////////////////////
//                        COPYRIGHT NOTICE
//   (c) Copyright 1998-2002 David A. Richie and Jeongnim Kim.
//   All rights reserved.
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   David A. Richie
//   Department of Physics, Ohio State University
//   174 W 18th Avenue
//   Columbus, OH 43210
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//
// class DCFParticleIO
//
// Records using DCF format in Progress mode.  Relies on 
// components from the STORM Scientific Wavelet Package (v3.1.0)
// which is (c) Copyright 1995-2002 David A. Richie.
//
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifndef OHMMS_DCFPARTICLEIO_H
#define OHMMS_DCFPARTICLEIO_H

#include "Particle/OhmmsParticle.h"
using namespace OHMMS;

#include "ParticleIO/ParticleIOBase.h"
#include "Utilities/SpeciesCollection.h"
#include "Utilities/SimpleParser.h"


//BEGIN ADDED BY DR
#include "RtmraTL.h"
#include "DCFsbofstream.h"
//END ADDED BY DR

/*! \class DCFParticleIO
 *  \author Jeongnim Kim & David A. Richie
 *  \brief A dervied class from ParticleIOBase
 *  
 *  Record in DCF format.
 *  PROGRESS mode: --E/--V/--F --stride 
 *                 --mmax/--POSmmax/--Emmax/--Fmmax
 *                 --thresh/--POSthresh/--Ethresh/--Fthresh
 *                 --descr
 */
class  DCFParticleIO: public ParticleIOBase<Scalar_t,OHMMS_DIM> {

public:
  enum DCFATTRIB {E, Q, F, V};
  bitset<4> attribMode;

  typedef  Scalar_t T;
  typedef  Particle_t::ParticlePos_t       ParticlePos_t;
  typedef  Particle_t::ParticleScalar_t    ParticleScalar_t;
  typedef  Particle_t::ParticleIndex_t     ParticleIndex_t;
  typedef  Particle_t::SingleParticlePos_t SingleParticlePos_t;
 

  DCFParticleIO(ThisParticle_t& p, int i, const char* name = NULL)
  :ParticleIOBase<Scalar_t,OHMMS_DIM>(p,i,name) { }

  ~DCFParticleIO() {  }

  void read(istream&) { } // read a configuration
  void write(){ } // write a configuration for re-runs
  void record(int);// record a configuration (time-stepping)
  void finalize();

private:
  int DCFMode, stride;
  vector<string> Species;
  ParticlePos_t* f_ref;  
  ParticlePos_t* p_ref;  
  ParticleScalar_t* e_ref;  

  Rtmra<T,array> posrtmra, ertmra, frtmra;
  DCFsbofstream< Rtmra<T,array> > posdcfofs, edcfofs, fdcfofs;
  int posrtmra_mmax, ertmra_mmax, frtmra_mmax;
  char posdcfofs_descript[256],  edcfofs_descript[256],  fdcfofs_descript[256];
  float posdcfofs_thresh, edcfofs_thresh, fdcfofs_thresh;

  void setup();
  void recordDCF(int);
};

#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: DCFParticleIO.h 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
