//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifndef OHMMS_METIS_PARTICLEIO_H
#define OHMMS_METIS_PARTICLEIO_H

#include "ParticleBase/ParticleIOBase.h"
#include "Utilities/SpeciesCollection.h"
#include "Utilities/SimpleParser.h"

/*! \class MetisParticleIO: specialized for 3D, using metis to partition
 *  the particle system
 *  \author Jeongnim Kim
 *  \brief A dervied class from ParticleIOBase
 *  
 */
class
MetisParticleIO: public ParticleIOBase<double,3> {

public:

  typedef double T;
  enum {D = 3};

  typedef ParticleIOBase<double,3>::ThisParticle_t ThisParticle_t;
  typedef ThisParticle_t::ParticlePos_t            ParticlePos_t;
  typedef ThisParticle_t::ParticleScalar_t         ParticleScalar_t;
  typedef ThisParticle_t::ParticleIndex_t          ParticleIndex_t;
  typedef ThisParticle_t::SingleParticlePos_t      SingleParticlePos_t;

  MetisParticleIO(ThisParticle_t& p, int i, const char* name = NULL)
  :ParticleIOBase<T,D>(p,i,name) { }

  ~MetisParticleIO();

  void read(); // read a configuration
  void write(); // write a configuration for re-runs
  void record(int);// record a configuration (time-stepping)
  void finalize();

private:
  void setup();
};
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: MetisParticleIO.h 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
