//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include <iostream>
#include <fstream>
#include <map>
using namespace std;

#include "Utilities/SpeciesCollection.h"
#include "ParticleIO/PtclGeometryBase.h"
#include "ParticleIO/SphericalQD.h"
#include "ParticleIO/PyramidalQD.h"

PtclGeometryBase* 
createPtclGeometry(const vector<string>& argv,  const ParticleLayout_t& pl) {

  PtclGeometryBase* Shape = NULL;

  if(!argv.empty()) {
    int i=0, ish=-1;
    while(i < argv.size()) {
      if(argv[i] == "<shape>" ) {
       ish = i+1;
       i = argv.size();
      }
      i++;
    }
    if(ish > 0) {
      i = ish;  
      if(i < argv.size()) {
	if(argv[i] == "sphere") {
	  Shape = new SphericalQD;
	} else if(argv[i] == "pyramid") {
	  Shape = new PyramidalQD;
	}
      }
    }
  }

  if(!Shape) Shape = new PtclGeometryBase;

  Shape->set(argv);
  Shape->set(pl);
  return Shape;
}

bool
changePtclNumbers(const vector<string>& argv){
  vector<string>::const_iterator it = argv.begin();
  while(it != argv.end()) {
    if((*it) == "sphere") {
      while(it != argv.end()) {
        if((*it) == "exchange") return false;
        it++;
      }
      return true;
    }
    it++;
  }

  return false;
}

///////////////////////////////////////////////////////////////////////////
//Tried SingletonPattern but not very useful
///////////////////////////////////////////////////////////////////////////
//  PtclGeometryBase* ParticleBuilder::Shape = NULL;

//  ////////////////////////////////////////////////////////////////////////
//  PtclGeometryBase* 
//  ParticleBuilder::getPtclGeometry() {
//    if(!Shape) Shape = new PtclGeometryBase;
//    return Shape;
//  }

//  void 
//  ParticleBuilder::set(istream& is) {
  
//    vector<string> slist,argv;
//    if(!Shape) {
//      getwords(slist,is);
//      if(slist[0] == "sphere") {
//        Shape = new SphericalQD;
//        cout << "creating a spherical QD" << endl;
//      } else if(slist[0] == "pyramid") {
//        Shape = new PyramidalQD;
//      }
//    }
//  //    else if(slist[0] == "embbeded_sphere") {
//  //      Shape = new EmbbededSphericalQD;
//  //    }
//    if(Shape) {
//      do {
//        getwords(slist,is);
//        argv.insert(argv.end(),slist.begin(), slist.end());
//      } while(slist[0] != "</shape>");
//      Shape->set(argv);
//    } else {
//      ERRORMSG(slist[0] << " is not defined. Will use default: bulk")
//    }
//  }


//  void 
//  ParticleBuilder::set(const vector<string>& argv) {
  
//    if(!Shape) {
//      if(argv[0] == "sphere") {
//        Shape = new SphericalQD;
//        cout << "creating a spherical QD" << endl;
//      } else if(argv[0] == "pyramid") {
//        Shape = new PyramidalQD;
//      }
//    }
//  //    else if(slist[0] == "embbeded_sphere") {
//  //      Shape = new EmbbededSphericalQD;
//  //    }

//    if(Shape) {
//      Shape->set(argv);
//    } else {
//      ERRORMSG(slist[0] << " is not defined. Will use default: bulk")
//    }
//  }
//  void 
//  ParticleBuilder::create(const Particle_t& ptcl_i, 
//  			Particle_t& ptcl_o, const int* uc_grid) {

//    if(!Shape) // no shape is defined, use a default mode for bulk
//      Shape = new PtclGeometryBase;

//    Shape->set(ptcl_i.Lattice);// prepare for geometry-dependepdent query

//    typedef ParticleLayout_t::PtclGrid_t PtclGrid_t;
//    typedef Particle_t::SingleParticlePos_t SingleParticlePos_t;

//    PtclGrid_t& mppgrid = *ptcl_o.Lattice.dGrid[0];//mpi level
//    PtclGrid_t& ompgrid = *ptcl_o.Lattice.dGrid[1];//omp level
//    PtclGrid_t& grid = *ptcl_o.Lattice.dGrid[2];//grid level

//    vector<vector<SingleParticlePos_t>* >  ptmp;
//    vector<vector<int>* >                  idtmp;

//    //force nodeid = 0 when mppgrid has only one mesh element
//    int nodeid = 0;// get MPI node

//    for(int i=ompgrid.NodeDist[nodeid]; i<ompgrid.NodeDist[nodeid+1]; i++) {
//      ptmp.push_back(new vector<SingleParticlePos_t>);
//    }
//    for(int i=ompgrid.NodeDist[nodeid]; i<ompgrid.NodeDist[nodeid+1]; i++) {
//      idtmp.push_back(new vector<int>);
//    }

//    SingleParticlePos_t dx(1.0/static_cast<Scalar_t>(uc_grid[0]),
//  			 1.0/static_cast<Scalar_t>(uc_grid[1]),
//  			 1.0/static_cast<Scalar_t>(uc_grid[2]));

//    SingleParticlePos_t del,pos;
//    double x,y,z;
//    int newid;
//    for(int ic=0; ic<uc_grid[0]; ic++) {
//      del[0] = static_cast<Scalar_t>(ic);
//      x = del[0]*dx[0];
//      for(int jc=0; jc<uc_grid[1]; jc++) {
//        del[1] = static_cast<Scalar_t>(jc);
//        y = del[1]*dx[1];
//        for(int kc=0; kc<uc_grid[2]; kc++) {
//          del[2] = static_cast<Scalar_t>(kc);
//          z = del[2]*dx[2];

//  	//this grid is not on this local MPI node, skip the grid
//          if(mppgrid.loc(x,y,z) != nodeid) continue;               

//  	//find a location on ompgrid
//          int ompid = ompgrid.loc(x,y,z);
//          for(int nb=0; nb<ptcl_i.getLocalNum(); nb++) {
//            pos = ptcl_i.Lattice.toCart(ptcl_i.R[nb]+del);
//            newid = ptcl_i.IonID[nb];
//            if(Shape->keepit(pos,newid)) {
//              ptmp[ompid]->push_back(pos);       
//  	    idtmp[ompid]->push_back(newid);
//  	  }
//  	}
//        }
//      }
//    }
//  //    ////////////////////////////////////////////////////////
//  //    //Checking the distribution
//  //    ////////////////////////////////////////////////////////
//  //    char fname[128];
//  //    ofstream fout("pos.xyz");
//  //    fout << ptcl_o.getLocalNum() << endl << endl;
//  //    for(int iat=0; iat<ptcl_o.getLocalNum(); iat++) {
//  //      fout << "Si"<<ptcl_o.ID[iat] << " "<< ptcl_o.R[iat] << endl;
//  //    }
//  //    //////////////////////////////////////////////////////////

//    //////////////////////////////////////////////////////////////////////
//    //clean up the temporary position and id
//    //////////////////////////////////////////////////////////////////////
//    for(int i=ompgrid.NodeDist[nodeid]; i<ompgrid.NodeDist[nodeid+1]; i++) {
//      delete ptmp[i];      
//      delete idtmp[i];     
//    }

//  }  

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: PtclGeometryBase.cpp 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
