//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifndef OHMMS_REGUARGRID_PARTICLEIO_H
#define OHMMS_REGUARGRID_PARTICLEIO_H

#include "ParticleBase/ParticleIOBase.h"
#include "Utilities/SpeciesCollection.h"
#include "Utilities/SimpleParser.h"

/*! \class RegGridParticleIO: specialized for 3D, using a regular
 *  grid partition even distribution
 *  \author Jeongnim Kim
 *  \brief A dervied class from ParticleIOBase
 */
class
RegGridParticleIO: public ParticleIOBase<double,3> {

public:

  typedef double T;
  enum {D = 3};

  typedef ParticleIOBase<double,3>::ThisParticle_t ThisParticle_t;
  typedef ThisParticle_t::ParticlePos_t            ParticlePos_t;
  typedef ThisParticle_t::ParticleScalar_t         ParticleScalar_t;
  typedef ThisParticle_t::ParticleIndex_t          ParticleIndex_t;
  typedef ThisParticle_t::SingleParticlePos_t      SingleParticlePos_t;

  RegGridParticleIO(ThisParticle_t& p, int i, const char* name = NULL)
  :ParticleIOBase<T,D>(p,i,name) { }

  ~RegGridParticleIO();

  void read(istream&, int *); // read a configuration
  void write(); // write a configuration for re-runs
  void record(int);// record a configuration (time-stepping)
  void finalize();

private:
  void setup();
  void buildBasicPtcl(istream&,int,int);
  void buildOMPDistPtcl(istream&,int,int);
  void rebuildOMPDistPtcl(istream&,int,int);
};
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: RegGridParticleIO.h 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
