//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include <stdio.h>
#include "ParticleIO/TbmdParticleIO.old.h"
#include "ParticleIO/ParticleInputTemplate.h"
#include "ParticleBase/ParticleUtility.h"

void TbmdParticleIO::read(istream& fpos) {

  if(ioMode == INPUT) {

    DEBUGMSG("Performing TbmdParticleIO::read()");

    vector<string> slist, vlist;
    int  natom = 0;
    do 
    {
      int argc = getwords(slist, fpos);
      if(argc == 0) continue;
      if(slist[0] == "number_of_ions") {
        for(int i=1; i<slist.size(); i++)  natom += atoi(slist[i].c_str());
      } else if(slist[0] == "positions") {
        if(slist[1][0] == 'u' || slist[1][0] == 'U')  MyPtcl.R.InUnit = true; 
        else                                          MyPtcl.R.InUnit = false;
        break;
      }
    } while(!fpos.eof());

    if(natom) {
      int loc = MyPtcl.getLocalNum();
      MyPtcl.create(natom);
      SingleParticlePos_t pos;
      SpeciesBase* species = SpeciesCollection::getSpecies();

      for(int i=0; i<natom; i++) {
      int argc =  getwords(slist, fpos);
        MyPtcl.ID[loc] = i;
	if(isdigit(slist[0][0])) {
	  MyPtcl.IonID[loc] = atoi(slist[0].c_str())-1;
	} else {
	  MyPtcl.IonID[loc] = species->getSpeciesID(slist[0]);
	}

        getValue<SingleParticlePos_t>::get(pos, slist.begin()+1);
	MyPtcl.R[loc] = pos;
        if(slist.size() > 4) MyPtcl.ID[loc] = atoi(slist[4].c_str());
        loc++;
      }

      //MyPtcl.R.InUnit = true; // converted into unit matrices

      while(!fpos.eof()) { // reading extra attributes
        int argc = getwords(slist,fpos);
        if(argc == 0) continue;
        if(slist[0] == POSSTR) // reading a position type
	  { 
	    MyPtcl.addAttribute(POSSTR, slist[1].c_str());
	    ParticlePos_t& pref = *(MyPtcl.getVectorAttrib(slist[1].c_str()));
	    if(slist[2]=="0") 
	      pref.InUnit = false;
	    else
	      pref.InUnit = true;
	    for(int i=0; i<natom; i++) 
	      {
		argc = getwords(vlist,fpos);
		getValue<SingleParticlePos_t>::get(pref[i], vlist.begin());
	      }
	  }
        else if(slist[0] == SCALARSTR) 
	  {
	    MyPtcl.addAttribute(SCALARSTR, slist[1].c_str());

            ParticleScalar_t& sref = *(MyPtcl.getScalarAttrib(slist[1].c_str()));
	    int i=0;
	    while(i<natom) {
	      argc = getwords(vlist,fpos);
	      for(int iargc=0; iargc<argc; iargc++) {
		sref[i++] = atof(vlist[iargc].c_str());
	      }
	    }
	  }
	else if(slist[0] == INDEXSTR)
	  {
            MyPtcl.addAttribute(INDEXSTR, slist[1].c_str());
	    
            ParticleIndex_t& sref = *(MyPtcl.getIndexAttrib(slist[1].c_str()));
	    int i=0;
	    while(i<natom) {
	      argc = getwords(vlist,fpos);
	      for(int iargc=0; iargc<argc; iargc++) {
		sref[i++] = atoi(vlist[iargc].c_str());
	      }
	    }
	  }
	else if(slist[0] == "</TBMD>") break;
      } 

      MyPtcl.update(-1);

      //expandLattice<ThisParticleLayout_t>::apply(MyPtcl,uc_grid);
      //MyPtcl.update(1); // + ID re-assignmenet
    } else {
      ERRORMSG("Input file does not confirm tbmd format. No Action is taken.")
    }
  } else {
    ERRORMSG("read for OUPUT/PROGRESS mode is not allowed.\n")
  }
}

void TbmdParticleIO::record(int iter) {
  if(ioMode == PROGRESS) {
    DEBUGMSG("Performing TbmdParticleIO::record()")
    if(!initialized) { setup(); initialized = true;}
    recordTbmd(iter); 
  } else {
    ERRORMSG("record for INPUT/OUPUT mode is not allowed.\n")
  }
}

void TbmdParticleIO::write() {

  if(ioMode == OUTPUT) {
    DEBUGMSG("Performing TbmdParticleIO::write()")
    if(!initialized) {
      sprintf(tbmdfile,"%s.meta",FileRoot.c_str());
      initialized = true;
    }

    ofstream ftbmd(tbmdfile); // always overwrite
    ftbmd << "<unitcell>" << endl;
    ftbmd << MyPtcl.Lattice.R;
    ftbmd << "</unitcell>" << endl;
    ftbmd <<"<ptclset oname=\"" << MyPtcl.getName() << "\">" << endl;
    ftbmd << "\t<format> tbmd </format> " << endl;
    ftbmd << "\t<name> none  </name>" <<  endl;
    ftbmd <<"<TBMD>" << endl;
    ftbmd << "number_of_ions = " << MyPtcl.getLocalNum()<< endl;
    ftbmd << "positions = " ;
    if(MyPtcl.R.InUnit) ftbmd << " unit\n";
    else                ftbmd << " cartesian\n";
    for(int i=0; i<MyPtcl.getLocalNum(); i++)
      ftbmd << MyPtcl.IonID[i]+1 << " " << MyPtcl.R[i]
	    << " " << MyPtcl.ID[i] << endl;

    for(int i=3; i<MyPtcl.getNumAttrib(); i++) {
      OhmmsObject& ooref=MyPtcl.getAttrib(i);
      ftbmd << ooref.typeName() << " " << ooref.objName() << " ";
      ooref.get(ftbmd);
      ftbmd << endl;
      if(!strcmp(ooref.typeName(), POSSTR)) {
	const ParticlePos_t& rtmp =*(MyPtcl.getVectorAttrib(i));
	for(int iat=0; iat<MyPtcl.getLocalNum(); iat++) {
	  ftbmd << rtmp[iat] << endl;
	}
      } else if(!strcmp(ooref.typeName(), SCALARSTR)) {
	const ParticleScalar_t& stmp =*(MyPtcl.getScalarAttrib(i));
	for(int iat=0; iat<MyPtcl.getLocalNum(); iat++) {
	  ftbmd << stmp[iat] << " ";
	  if(iat%5 == 4) ftbmd << endl;
	}
	if(MyPtcl.getLocalNum()%5 != 0) ftbmd<< endl;
      } else if(!strcmp(ooref.typeName(), INDEXSTR)){
	const ParticleIndex_t& itmp =*(MyPtcl.getIndexAttrib(i));
	for(int iat=0; iat<MyPtcl.getLocalNum(); iat++) {
	  ftbmd << itmp[iat] << " ";
	  if(iat%40 == 39) ftbmd << endl;
	}
	if(MyPtcl.getLocalNum()%40 != 0) ftbmd<< endl;
      }
    }
    ftbmd <<"</TBMD>" << endl;
    ftbmd << "</ptclset>" << endl;
    ftbmd.close();

  } else {
    ERRORMSG("write for INPUT/PROGRESS  mode is not allowed.\n")
  }
}

void TbmdParticleIO::setup() {

  sprintf(tbmdfile,"%s.tbmd",FileRoot.c_str());
  int i=0;
  while(i<options.size()) {
    if(options[i] == "--E") {
      if(MyPtcl.hasAttrib(ENERGY_TAG)) {
        val_list.push_back(MyPtcl.getScalarAttrib(ENERGY_TAG));
      } else 
        ERRORMSG("Energy is not found among the Particle Attributes.\n");
    } else if(options[i] == "--Q") {
      if(MyPtcl.hasAttrib(VAL_CHARGE_TAG)) {
        val_list.push_back(MyPtcl.getScalarAttrib(VAL_CHARGE_TAG));
      } else 
        ERRORMSG("Energy is not found among the Particle Attributes.\n");
    } else if(options[i] == "--V") {
      if(MyPtcl.hasAttrib(VELOCITY_TAG)) {
        pos_list.push_back(MyPtcl.getVectorAttrib(VELOCITY_TAG));
      } else 
        ERRORMSG("Velocity is not found among the Particle Attributes.\n");
    } else if(options[i] == "--F") {
      if(MyPtcl.hasAttrib(POSITION_TAG)) {
        pos_list.push_back(MyPtcl.getVectorAttrib(POSITION_TAG));
      } else 
        ERRORMSG("Force or Acceleration is not found among the Particle Attributes.\n");
    }
      i++;
  }
  ofstream ftbmd(tbmdfile);
  ftbmd <<"<TBMD>" << endl;
}


void TbmdParticleIO::recordTbmd(int iter) {

  DEBUGMSG("TbmdParticleOut::init() is completed.\n");
  int nat = MyPtcl.getLocalNum();
  ofstream ftbmd(tbmdfile, ios::app);
  ftbmd << "begin-mdframe  " << iter << "\n";
  //MyPtcl.Lattice.print(ftbmd,0); // only lattice
  ftbmd << MyPtcl.Lattice.R;
  ftbmd << "number_of_ions = " << nat << endl;
  ftbmd << "positions " ;
  if(MyPtcl.R.InUnit) ftbmd << " unit" << endl;
  else                ftbmd << " cartesian" << endl;
  for(int i=0; i<MyPtcl.getLocalNum(); i++)
    ftbmd << MyPtcl.IonID[i]+1 << " " << MyPtcl.R[i] << endl;

  // Added by DRT

  for(int i=3; i<MyPtcl.getNumAttrib(); i++) {
    OhmmsObject& ooref=MyPtcl.getAttrib(i);
    ftbmd << ooref.typeName() << " " << ooref.objName() << " ";
    ooref.get(ftbmd);
    ftbmd << endl;
    if(!strcmp(ooref.typeName(), POSSTR)) {
      const ParticlePos_t& rtmp =*(MyPtcl.getVectorAttrib(i));
      for(int iat=0; iat<MyPtcl.getLocalNum(); iat++) {
        ftbmd << rtmp[iat] << endl;
      }
    } else if(!strcmp(ooref.typeName(), SCALARSTR)) {
      const ParticleScalar_t& stmp =*(MyPtcl.getScalarAttrib(i));
      for(int iat=0; iat<MyPtcl.getLocalNum(); iat++) {
        ftbmd << stmp[iat] << " ";
        if(iat%5 == 4) ftbmd << endl;
      }
      if(MyPtcl.getLocalNum()%5 != 0) ftbmd<< endl;
    } else if(!strcmp(ooref.typeName(), INDEXSTR)){
      const ParticleIndex_t& itmp =*(MyPtcl.getIndexAttrib(i));
      for(int iat=0; iat<MyPtcl.getLocalNum(); iat++) {
        ftbmd << itmp[iat] << " ";
        if(iat%40 == 39) ftbmd << endl;
      }
      if(MyPtcl.getLocalNum()%40 != 0) ftbmd<< endl;
    }
  }
  
//  for(int i=0; i<pos_list.size(); i++) {
//    ftbmd << pos_list[i]->typeName() << " " << pos_list[i]->objName() << endl;
//    pos_list[i]->write_data(ftbmd);
//  }
//  for(int i=0; i<val_list.size(); i++) {
//    ftbmd << val_list[i]->typeName() << " " << val_list[i]->objName() << endl;
//    val_list[i]->write_data(ftbmd);
//  }

  ftbmd << "end-mdframe\n";
}

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: TbmdParticleIO.old.cpp 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/

