//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifndef OHMMS_TBMDPARTICLEIO_H
#define OHMMS_TBMDPARTICLEIO_H

#include "Particle/OhmmsParticle.h"
using namespace OHMMS;

#include "ParticleIO/ParticleIOBase.h"
#include "Utilities/SpeciesCollection.h"
#include "Utilities/SimpleParser.h"

/*! \class TbmDParticleIO
 *  \author Jeongnim Kim
 *  \brief A dervied class from ParticleIOBase
 *  
 *  Read/Write/Record in a format xmol can process.
 *  INPUT mode: -f filename 
 *  OUTPUT mode: --stride
 *  PROGRESS mode: --E/--V/--F --stride 
 */

class  TbmdParticleIO: public ParticleIOBase<Scalar_t,OHMMS_DIM> {

public:

 
  typedef ParticleIOBase<Scalar_t,OHMMS_DIM>::ThisParticle_t ThisParticle_t;
  typedef ThisParticle_t::ParticlePos_t       ParticlePos_t;
  typedef ThisParticle_t::ParticleScalar_t    ParticleScalar_t;
  typedef ThisParticle_t::ParticleIndex_t     ParticleIndex_t;
  typedef ThisParticle_t::SingleParticlePos_t SingleParticlePos_t;

  TbmdParticleIO(ThisParticle_t& p, int i, const char* name = NULL)
  :ParticleIOBase<Scalar_t,OHMMS_DIM>(p,i,name) { }

  ~TbmdParticleIO() {
    DEBUGMSG("calling TbmdParticleIO::~TbmdParticleIO() for " << FileRoot)
  }

  void read(istream&); // read a configuration
  void write(); // write a configuration for re-runs
  void record(int);// record a configuration (time-stepping)
  void finalize() { }

private:

  char tbmdfile[128];
  vector<ParticlePos_t*>    pos_list;
  vector<ParticleScalar_t*> val_list;

  void setup();
  void recordTbmd(int);
};
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: TbmdParticleIO.old.h 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
