//////////////////////////////////////////////////////////////////
// (c) Copyright 2003  by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "Utilities/OhmmsInfo.h"
#include "Pooma/Communicate.h"
#include "Pooma/CommCreator.h"
#include "Pooma/CommMPI.h"
#include "Pooma/Message.h"
//using namespace OHMMS;
/**\file main.cpp
 * The main routine that creates OhmmsApp to perform atomistic simulations.
 */

void report(const char *str, bool result) {
  cout  << "Test " << str << ": ";
  cout << (result ? "PASSED" : "FAILED") << endl;
}

int main(int argc, char **argv) {


  int cm = CommCreator::MPI;
  Communicate* comm = CommCreator::create(cm,argc,argv);

  OhmmsInfo ohmms(argc,argv,comm->myNode());
  LOGMSG("Has Created OhmmsInfo")

  Message *msg;
  int mynode = comm->myNode();
  char recbuf[128];
  double whatever = 0.12;
  int recint;
  int sendtag = 80;
  int npt = 10;
  double dn = 1.0/static_cast<double>(npt);

  vector<double> X;

  if (mynode == 0) {
    X.resize(npt);
    for(int i=0; i<npt; i++) 
      X[i] = dn*static_cast<double>(i+mynode+1);
    // first have the master node send a message to the other nodes
    msg = new Message();
    msg->put("This is a message from the master node: ");
    msg->put(mynode);
    msg->put(npt);
    putMessage(*msg,X.begin(), X.end());
    comm->broadcast_all(msg, sendtag);

    // now have the master receive back all the replies and print them
    int notReceived = comm->getNodes();

    while (notReceived > 0) {
      int node = COMM_ANY_NODE;
      int tag = sendtag;
      msg = comm->receive_block(node, tag);
      if (msg == 0) {
	report("Master receive", false);
	ERRORMSG("Could not receive from client nodes in main." << endl);
	return 1;
      }
      msg->get(recbuf).get(recint);
      delete msg;
      if (recint != node) {
	report("Master receive", false);
	ERRORMSG("Incorrect message from node " << node << endl);
	return 1;
      }
      notReceived--;
    }
    report("Master receive", true);
  } else {
    // the client nodes just receive the message and send one back
    int node = 0;
    int tag = sendtag;
    int m = 0;
    msg = comm->receive_block(node, tag);
    if (msg == 0) {
      report("Client receive", false);
      ERRORMSG("Could not receive from master node in main." << endl);
      return 1;
    }
    //msg->get(recbuf).get(recint).get(x);
    getMessage(*msg,recbuf);
    getMessage(*msg,recint);
    getMessage(*msg,m);
    X.resize(m);
    getMessage(*msg,X.begin(),X.end());
    double sum = 0.0;
    for(int i=0; i<npt; i++) sum += X[i];
    cout << "Sum of the vector (" << mynode << ")= "  << sum << endl;
    delete msg;
    if (recint != node) {
      report("Client receive", false);
      ERRORMSG("Incorrect message from node " << node << endl);
      return 1;
    }
    report("Client receive", true);
    msg = new Message;
    msg->put("This is a reply message from the client node: ");
    msg->put(mynode);
    comm->send(msg, 0, sendtag);
  }

  delete comm;
  return(0);
}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: message.cpp 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
