//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
///////////////////////////////////////////////////////////////////////
// * This class implements the orthogonal tight-binding model
// * of Harrison scale, i.e., (r0/r)^2 
/////////////////////////////////////////////////////////////////////
#ifndef OHMMS_POTENTIALCOMPOSITE_H
#define OHMMS_POTENTIALCOMPOSITE_H

class CompositePotential: public PotentialBase {
public:

  CompositePotential();
  CompositePotential(ThisParticle_t* ptclset);
  CompositePotential(int argc, char **argv); 

  ~CompositePotential();

  inline void add(PotentialBase* pot) { PotList.push_back(pot);}

  void set(ThisParticle_t* ptclset) { 
    myPtcl = ptclset;
    for(int i=0; i<PotList.size(); i++) PotList[i]->set(ptclset);
  }

  void initialize();
  Scalar_t evalEnergy();
  Scalar_t evalForce(ParticlePos_t& f, ParticleScalar_t& e, Tensor_t& stress, 
                     unsigned stressmode = 0);
  bool evalDynamicMatrix(DynMatrix_t&);
  Scalar_t e(const ParticlePos_t&, int i);  // returns a local energy

private:
  vector<PotentialBase* > PotList;
};

#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: CompositePotential.h 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
