//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "Potentials/CoulombPotential.h"
#include "Particle/NNInfiniteEngine.h"
#include "Utilities/SpeciesCollection.h"
#include "Utilities/Clock.h"
#include "Message/OpenMP.h"
namespace OHMMS {

  CoulombPotential::CoulombPotential():ewald_engine(0),NotInitialized(true){

  }

  CoulombPotential::~CoulombPotential(){

    if(ewald_engine) delete ewald_engine;
  }

  void CoulombPotential::setParticleSet(Particle_t* ptclset){
    myPtcl = ptclset;
    Rcart.resize(myPtcl->getLocalNum());
    Rcart.setUnit(PosUnit::CartesianUnit);
    bool hascharge = myPtcl->hasAttrib("charge");
    Charge = myPtcl->getScalarAttrib("charge");
    if(!hascharge) {
      SpeciesBase* tspecies = SpeciesCollection::getSpecies();
      int species_attrib = tspecies->addAttrib("charge");
      for(int i=0; i<myPtcl->getLocalNum(); i++) {       
	(*Charge)[i] = (*tspecies)(species_attrib,myPtcl->GroupID[i]);
      }
    }

    if(!ewald_engine && myPtcl) {
      ewald_engine = new EwaldEngineType;
      ewald_engine->init(myPtcl->Lattice, *Charge);
      //myPtcl->createPairlist(5,false);
      NotInitialized = false;
    }
  }

  CoulombPotential::Scalar_t 
  CoulombPotential::evalEnergy(){

    cout << "Maximum radius " << ewald_engine->Rmax << endl;
    return 0;
    myPtcl->applyBC(myPtcl->R,Rcart);
    //Scalar_t e =ewald_engine->energy(myPtcl->Lattice,Rcart,*Charge);
    
    Pooma::Clock timer;
    timer.start();
    Scalar_t e =ewald_engine->energyG(myPtcl->Lattice,Rcart,*Charge);
    timer.stop();

    //    e +=ewald_engine->energyR(myPtcl->Lattice,Rcart,*Charge);

    cout << "G-sum " << timer.cpu_time() << endl;
    timer.start();
#pragma omp parallel reduction(+:e)
    {
      const NNEngine *nnListEngine = myPtcl->nnListEngine;
      int myID = omp_get_thread_num();
      e+= ewald_engine->energyR(nnListEngine->getPairAttrib(myID),
				nnListEngine->first_id(myID),
				nnListEngine->last_id(myID),*Charge);
    }//end-of-omp
    timer.stop();

    cout << "R-sum " << timer.cpu_time() << endl;
    e += ewald_engine->E0;

    ewald_engine->print(cout);
    return e;
  }

  CoulombPotential::Scalar_t 
  CoulombPotential::evalForce(ParticlePos_t& f, 
			      unsigned stressmode){

    myPtcl->applyBC(myPtcl->R,Rcart);
    return 
      ewald_engine->forcestress(myPtcl->Lattice,Rcart,*Charge,f,Stress);
  }

  bool 
  CoulombPotential::evalDynamicMatrix(DynMatrix_t& dmat){
    return false;
  }

  bool CoulombPotential::get(ostream&) const {
    return true;
  }

  bool CoulombPotential::put(istream&) {
    return true;
  }

  bool CoulombPotential::put(xmlNodePtr cur) {
    return true;
  }
}

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 308 $   $Date: 2005-02-21 09:21:44 -0600 (Mon, 21 Feb 2005) $
 * $Id: CoulombPotential.cpp 308 2005-02-21 15:21:44Z jnkim $ 
 ***************************************************************************/
