//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim and Thomas Lenosky
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "Potentials/MEAMParam.h"
namespace OHMMS {

/////////////////////////////
#define MEAMRHO 
#define EDIPG FALSE
#define SQUARED TRUE
#define KAXIRASH TRUE
#define KAXIRASQ TRUE
#define KAXIRASL TRUE
#define MEPSMAX 10
#define MHMAX 11
#define MRHOMAX 2
#define MRHOMAX2 2

MEAMParam::MEAMParam(const char *fname) { 
  clsgnecut=3.5;  
  clsrhonecut=clstricut1 = 3.5;
  eps.reserve(MEPSMAX);
  u.reserve(MRHOMAX);
  rho.reserve(MRHOMAX);
  h.reserve(MHMAX);
  xf.reserve(MRHOMAX2);
  xg.reserve(MRHOMAX2);
  xu.reserve(MRHOMAX2);
  xrho.reserve(MRHOMAX2);
  xh.reserve(MRHOMAX2);
  xl.reserve(MRHOMAX2);
  xq.reserve(MRHOMAX2);
  readMEAMParam(fname);
}


void MEAMParam::readMEAMParam(const char *fname) {
  //void prmst38c() /*Read in a potential from the file "coeff.cls"*/
  ifstream fin(fname);
  if(!fin) { 
    cerr << fname << " is not found in this directory.\n";
    cerr << "Exiting very ungracefully.\n";
    exit(1);
  }

  vector<string> slist;
  do {
    getwords(slist,fin);
  } while(slist[0] != "<spline>"); 

  //fin >> clsusephi >> clsusetri >> clsmrho >> clsmhtb >> clsmepstb >> clsmrho2;
  
  std::size_t nw = getwords(slist,fin);
  clsusephi = atoi(slist[0].c_str());
  if(nw < 6) {
    getwords(slist,fin);
    clsusetri = atoi(slist[0].c_str());
    clsmrho   = atoi(slist[1].c_str());
    clsmhtb   = atoi(slist[2].c_str());
    clsmepstb = atoi(slist[3].c_str());
    clsmrho2  = atoi(slist[4].c_str());
  } else {
    clsusetri = atoi(slist[1].c_str());
    clsmrho   = atoi(slist[2].c_str());
    clsmhtb   = atoi(slist[3].c_str());
    clsmepstb = atoi(slist[4].c_str());
    clsmrho2  = atoi(slist[5].c_str());
  }

  if (clsusephi) {
    phi.readSpline(fin);
  }
    
  for(int mm=0;mm<clsmrho;mm++) {
    rho.push_back(new SplineType);  rho[mm]->readSpline(fin);
    u.push_back(new SplineType);    u[mm]->readSpline(fin);
  }  
  if (clsusetri) {
    f.readSpline(fin);
    g.readSpline(fin);
  } 

  for(int mm=0;mm<clsmhtb;mm++) {
    h.push_back(new SplineType); h[mm]->readSpline(fin);
  }

  value_type epstmp;
  for(int mm=0;mm<clsmepstb;mm++) {
    fin >> epstmp;
    eps.push_back(epstmp);
  }

  for(int mm=0;mm<clsmrho2;mm++) {
#ifdef MEAMRHO
    xrho.push_back(new SplineType); xrho[mm]->readSpline(fin);
#endif 
    xu.push_back(new SplineType); xu[mm]->readSpline(fin);
    xf.push_back(new SplineType); xf[mm]->readSpline(fin);
    xg.push_back(new SplineType); xg[mm]->readSpline(fin);
#if (EDIPG)
#if (KAXIRASQ == FALSE)
    xq.push_back(new SplineType); xq[mm]->readSpline(fin);
#endif
#if (KAXIRASL == FALSE)
    xl.push_back(new SplineType); xl[mm]->readSpline(fin);
#endif
#if (KAXIRASH == FALSE)
    xh.push_back(new SplineType); xh[mm]->readSpline(fin);
#endif
#endif
  }

////////////////////////////////////////////////////////
// spline is done upon reading spline points
////////////////////////////////////////////////////////
//   //Reading is done, get spline them
//   if (clsusephi) phi.getSpline();
    
//   for(int mm=0;mm<clsmrho;mm++) {
//     rho[mm]->getSpline();
//     u[mm]->getSpline();
//   }  
//   if (clsusetri) {
//     f.getSpline();
//     g.getSpline();
//   } 

//   for(int mm=0;mm<clsmhtb;mm++) {
//     h[mm]->getSpline();
//   }

//   for(int mm=0;mm<clsmrho2;mm++) {
// #ifdef MEAMRHO
//     xrho[mm]->getSpline();
// #endif 
//     xu[mm]->getSpline();
//     xf[mm]->getSpline();
//     xg[mm]->getSpline();
// #if (EDIPG)
// #if (KAXIRASQ == FALSE)
//     xq[mm]->getSpline();
// #endif
// #if (KAXIRASL == FALSE)
//     xl[mm]->getSpline();
// #endif
// #if (KAXIRASH == FALSE)
//     xh[mm]->getSpline();
// #endif
// #endif
//   }

/*
  if(clsusephi)  clsgnecut = phi.X[phi.Npt] ;
  if(clsmrho!=0)  clsrhonecut = rho[0]->X[rho[0]->Npt] ;
  if(clsusetri)  clstricut1 = f.X[f.Npt] ;
  else if(clsmrho2!=0) clstricut1 = xf[0]->X[xf[0]->Npt] ;
*/
  clsgnecut = phi.X[phi.Npt];
  clsrhonecut = xrho[0]->X[xrho[0]->Npt];
  clstricut1 = xf[0]->X[xf[0]->Npt];

  R2sq = clsgnecut*clsgnecut;
  Rrhosq = clsrhonecut*clsrhonecut;
  R3sq = clstricut1*clstricut1;

  //setuptb(&clsp,clsmepstb,clsmhtb); // 
  //setuptb2(clsgnecut,clsusephi); //

}
}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 354 $   $Date: 2005-07-07 21:20:49 -0500 (Thu, 07 Jul 2005) $
 * $Id: MEAMParam.cpp 354 2005-07-08 02:20:49Z jnkim $ 
 ***************************************************************************/

