//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim and Thomas Lenosky
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
/**@file MEAMPotential.h
 * @brief Declaration of MEAMPotential using spline functions.
 */
#ifndef OHMMS_MEAMPOTENTIAL_H
#define OHMMS_MEAMPOTENTIAL_H

#include "Potentials/ShortRangePotential.h"

class SpeciesSet;

namespace OHMMS {

  class MEAMParam;

  /** Implements MEAM potential for a single species.
   *
   * Original code is written by Thomas Lenosky, LLNL, 12/8/1999
   * Modified for OHMMS by Jeongnim Kim 12/8/99, 09/27/2001
   */
  class MEAMPotential : public ShortRangePotential{

  public:

    MEAMPotential();
    MEAMPotential(int argc, char** argv);
    ~MEAMPotential();

    bool get(ostream&) const;
    bool put(istream&);
    void reset();
    bool put(xmlNodePtr cur);

    void resetParticleSet(Particle_t* ptclset) {
      myPtcl=ptclset;
    }
 
    Scalar_t evalForceNN(PtclPairList_t& nnlist, ParticlePos_t& f0, unsigned stressmode) const;

    Scalar_t evalEnergyNN(const PtclPairList_t& nnlist, Scalar_t rscale=1.0) const;

  protected:
  
    int SpeciesID;
    string PotFile;
    vector<MEAMParam*> MEAMSet;
    int MEAMSpecies, MEAMBond;
    SpeciesSet* mySpecies;

    void readMEAMPOT(const string& fname);

    template<class T, unsigned D>
    struct MEAM2Body {
      int Loc;
      T f,df,r,rinv;
      TinyVector<T,D> dcos;
      inline MEAM2Body(){}
    };
    
    template<class T>
    struct MEAM3Body {
      T lcos,g,dg;
      inline MEAM3Body(){}
    };

  };
}
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 363 $   $Date: 2005-07-13 13:33:18 -0500 (Wed, 13 Jul 2005) $
 * $Id: MEAMPotential.h 363 2005-07-13 18:33:18Z jnkim $ 
 ***************************************************************************/
