//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#include "Potentials/RecordPotential.h"

namespace OHMMS {

  RecordPotential::RecordPotential(PotentialBase& pot): 
    ref_(pot), _recorder(NULL)
  { 
  }

  RecordPotential::~RecordPotential(){ }

  void RecordPotential::reset(const char* fileroot){
    ///get the recorder
    if(stride == -1) 
      _recorder = NULL;
    else 
      _recorder = ref_.getRecorder();

    if(_recorder) {
      _recorder->reset(fileroot);
    }
  }
  
  void RecordPotential::finalize() {
    if(_recorder) {
      _recorder->finalize();
    }
  }

  void RecordPotential::report(int iter) {
    if(iter<0) return;
    if(_recorder && iter%stride == 0) {
      _recorder->report(iter);
    }
  }

  bool RecordPotential::put(xmlNodePtr cur) {
    stride = atoi((const char*)(xmlGetProp(cur, (const xmlChar *) "stride")));
    if(!_recorder) {
      _recorder = ref_.getRecorder();
    }

    if(_recorder) {
      return _recorder->put(cur);
    } else 
      return false;
  }

  /////////////////////////////////////
  //functions ot SavePotential
  /////////////////////////////////////
  SavePotential::~SavePotential(){
    
  }

  void SavePotential::reset(const char* fileroot){

  }

  void SavePotential::report(int iter) {
    //nothing is done at this point
  }

  bool SavePotential::put(xmlNodePtr cur) {
    
    return true;
  }
}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 198 $   $Date: 2004-12-22 10:27:32 -0600 (Wed, 22 Dec 2004) $
 * $Id: RecordPotential.cpp 198 2004-12-22 16:27:32Z jnkim $ 
 ***************************************************************************/
