//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
/*! \class SWPotential
 * \author Jeongnim Kim
 * \brief Derived class from PotentialBase.  Implements SW potential for single
 *  species.
 * \todo Multi-species SW if parameters exist.
 */
#ifndef OHMMS_SWPOTENTIAL_H
#define OHMMS_SWPOTENTIAL_H

#include "Potentials/ShortRangePotential.h"

namespace OHMMS {


  //forward declaration of SWParameter
  template<class T> class SWParam;

  class SWPotential:public ShortRangePotential {

  public:

    SWPotential();
    SWPotential(int argc, char **argv); 
    ~SWPotential();

    void initialize();
    bool get(ostream&) const;
    bool put(istream&);
    void reset();
    bool put(xmlNodePtr cur);

    void resetParticleSet(Particle_t* ptclset) {
      myPtcl=ptclset;
    }

    Scalar_t evalForceNN(PtclPairList_t& nnlist, 
			 ParticlePos_t& f0, 
			 unsigned stressmode) const;

    Scalar_t evalEnergyNN(const PtclPairList_t& nnlist, Scalar_t rscale=1.0) const;

    void analize(const PtclPairList_t& nnlist,size_type first, size_type last); 

  protected:

    Scalar_t Lambda,   Theta0;
    size_type          NumSpecies;
    string             InFileName;
    vector<SWParam<Scalar_t>*> SWSet;

    bool putSpecial(xmlNodePtr cur);

  };
}
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 363 $   $Date: 2005-07-13 13:33:18 -0500 (Wed, 13 Jul 2005) $
 * $Id: SWPotential.h 363 2005-07-13 18:33:18Z jnkim $ 
 ***************************************************************************/

