//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Richard Hennig and Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
/*!\class AlloyEAMPotential
 * \author Richard Hennig
 * \author Jeongnim Kim
 * \brief Derived class from PotentialBase.  Implements AlloyEAM potential for single
 *  species.
 * \note  
 * Implemented by Richard Hennig 6/2001
 *
 * Based on MAlloyEAMPotential originally written by Thomas Lenosky, LLNL, 12/8/99
 *
 * Modified to confirm new interfaces of PotentialBase Jeongnim Kim 9/27/2001
 ************************************************************************/
#ifndef AlloyEAMPOTENTIAL_H
#define AlloyEAMPOTENTIAL_H

#include "Potentials/PotentialBase.h"

class SpeciesBase;

namespace OHMMS {
  class AlloyEAMParam;

  class AlloyEAMPotential: public PotentialBase{

  public:

    AlloyEAMPotential();
    AlloyEAMPotential(int argc, char** argv);
    ~AlloyEAMPotential();

    void initialize();
    Scalar_t evalEnergy();
    Scalar_t evalForce(ParticlePos_t& f, ParticleScalar_t& e, Tensor_t&, 
		       unsigned stressmode = 0);
    bool evalDynamicMatrix(DynMatrix_t&);
    Scalar_t e(const ParticlePos_t&, int i, int myid=0);  
    // returns a local energy

    /// implementing virtual functions for OhmmsElementBase
    bool get(ostream&) const;
    bool put(istream&);
    void reset();
    bool put(xmlNodePtr cur);

  protected:

    struct SaveRho{
      unsigned id;
      Scalar_t rho, drho, r;
      SingleParticlePos_t dr;
      SaveRho() { } // do nothing
      ~SaveRho() { }
    };

    struct SaveF{
      unsigned id;
      Scalar_t r, rinv, f, df;
      SingleParticlePos_t  dr;
      SaveF() { }
      ~SaveF() { }
    };

    struct SaveG{
      Scalar_t lcos, g, dg;
      SaveG() {}
      ~SaveG() {}
    };

    int SpeciesID;
    vector<AlloyEAMParam*> AlloyEAMSet;
    int AlloyEAMSpecies, AlloyEAMBond;
    SpeciesBase* mySpecies;
//      //removed from the data members.
//      vector<SaveRho> srho;
//      vector<SaveF>   sf;
//      vector<SaveG>   sg;
//      void resize(int);

    Scalar_t evalForceNN(const PtclPairList_t& nnlist, 
			 int myID,
			 ParticlePos_t& f0, ParticleScalar_t& q, 
			 Tensor_t& stressloc, unsigned stressmode);
  };
}
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: AlloyEAMPotential.h 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
