//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Richard Hennig and Jeongnim Kim
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

/************************************************************************
 Modified for MgEAM by Richard Hennig 6/2001
 Modified by Jeongnim Kim 9/27/2001
 ************************************************************************/
#include <math.h>
#include "Utilities/SimpleParser.h"
#include "Potentials/MgEAMParam.h"

void MgEAMParam::readMgEAMParam(const char *fname) {
  //void prmst38c() /* Read potential parameters from file "coeff.cls"*/
  ifstream fin(fname);
    if(!fin) { 
    cerr << fname << " is not found in this directory.\n";
    cerr << "Exiting very ungracefully.\n";
    exit(1);
  }

  vector<string> slist;
  do {
    getwords(slist,fin);
  } while(slist[0] != "<spline>");

  cerr << "Read parameters from file "<< fname << "\n";

  phi.readSpline(fin);
  cout << "Read phi\n";
  rho.readSpline(fin);
  cout << "Read rho\n";
  f.readSpline(fin);
  cout << "Read f\n";

}

void SplineTypeMgEAM::readSpline(istream& is){
/*Read s from f, using format common with fwrite_spline(f,s) */
  is >> Npt;
  X = vector<Scalar_t>(Npt+1);
  Y = vector<Scalar_t>(Npt+1);
  b = vector<Scalar_t>(Npt+1);
  c = vector<Scalar_t>(Npt+1);
  d = vector<Scalar_t>(Npt+1);

  X[0] = 0.0; Y[0] = 0.0; b[0] = 0.0; c[0] = 0.0; d[0] = 0.0;
  //double t1, t2, t3, t4, t5;
  for (int i=1; i<=Npt;i++) {
    is >> X[i] >> Y[i] >> b[i]>> c[i] >> d[i];
  }
}

SplineTypeMgEAM::Scalar_t SplineTypeMgEAM::splint(SplineTypeMgEAM::Scalar_t r) {

  int klo, khi, k;
  Scalar_t h;
  if(r>X[1] && r<X[Npt]) { //NR part
    klo=1;
    khi=Npt;
    while (khi-klo > 1) {
      k=(khi+klo) >> 1;
      if (X[k] > r) khi=k;
      else klo=k;
    }
    h=X[khi]-X[klo];
    if (h == 0.0) {
      // ERRORMSG("splint called with x = %lf\n",x);
    };
    h=r-X[klo];
    return Y[klo]+(b[klo]+(c[klo]+d[klo]*h)*h)*h;

  } else { // Extrapolation past endpoints
    if(r<=X[1]) {
      h=r-X[1];
      return Y[1] + (b[1]+(c[1]+d[1]*h)*h)*h;
    }
    else{
      h=r-X[Npt];
      return Y[Npt] + (b[Npt]+(c[Npt]+d[Npt]*h)*h)*h;
    }
  }
}

// return val = f(r), deriv = df(r)/dr
void SplineTypeMgEAM::splint(SplineTypeMgEAM::Scalar_t r, SplineTypeMgEAM::Scalar_t& val,
			SplineTypeMgEAM::Scalar_t& deriv) {
  int klo,khi,k;
  Scalar_t h;
  if(r>X[1] && r<X[Npt]) { //NR part
    klo=1;
    khi=Npt;
    while (khi-klo > 1) {
      k=(khi+klo) >> 1;
      if (X[k] > r) khi=k;
      else klo=k;
    }
    h=X[khi]-X[klo];
    if (h == 0.0) {
      //ERRORMSG("splint called with r = " << r << endl);
    };
    h=r-X[klo];
    val=Y[klo]+(b[klo]+(c[klo]+d[klo]*h)*h)*h;
    deriv = b[klo]+(2.0*c[klo]+3*d[klo]*h)*h;

  } else { // Extrapolation past endpoints
    if(r<=X[1]) {
      h=r-X[1];
      val=Y[1]+(b[1]+(c[1]+d[1]*h)*h)*h;
      deriv = b[1]+(2.0*c[1]+3*d[1]*h)*h;
    } else {
      h=r-X[Npt];
      val=Y[Npt]+(b[Npt]+(c[Npt]+d[Npt]*h)*h)*h;
      deriv = b[Npt]+(2.0*c[Npt]+3*d[Npt]*h)*h;
    }
  }
}

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: MgEAMParam.cpp 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/

