//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#include "Propagators/APropagator.h"
#include "Potentials/PotentialBase.h"
using namespace OHMMS;

/** default constructor
 *
 * Initialize the basic properties of a propagator. Here is where one can
 * add <parameter name="parameter-name" unit="unit-name"> value </parameter>
 * of any intrinsic type or any class with overriding operators  << and >>
 * to m_param (ParameterSet).
 * PropagatorBase::put(xmlNodePtr) function will read the values from the
 * input xml file.
 */
APropagator::APropagator(){ 
  //m_param.add(GetForce,"force","bool");
}

/** initialize a propagator
 * @param ptclin the ParticleSet on which a propagator operate.
 * @param potin the PotentialBase with which a propagator move the ParticleSet
 * @param makecopy if true, a propagator creates a copy of the input ParticeSet.
 */
void 
APropagator::initialize(Particle_t* ptclin, PotentialBase* potin, bool makecopy) {

  Counter = -1;
  if(!Initialized) { 
    //Always need to call PropagatorBase::initializeBase
    initializeBase(ptclin,potin,makecopy);

    //Tag the particle attributes that are associated with the particle position.
    //The attributes added to ParticleSet::ConfigList are adjusted by the boundary
    //conditions when ParticleSet::applyBConds is called.
    myPtcl->addPositionAttrib(myPtcl->R);

    //Add anything needed to initialize the propagator.
    
    //Set it true, so that the initialization is done only once.
    Initialized = true; 
  }
}

/** Set the temperature of the myPtcl to a target temperature
 * @param rescale if true, scale the current velocities
 * @param tref the target temperature
 *
 * Leave it empty if APropagator never needs the velocity.
 */
void APropagator::setTemperature(bool rescale, Scalar_t tref) {
 
  //If APropagator needs the velocites, include the two lines below.
  ParticlePos_t& V = *myPtcl->getVectorAttrib(velocity_tag);
  scaleTemperature(rescale,T_ref);

} 

/** Update the particle attributes.
 *@param pot the PotentialBase (legacy)
 *@return true, if the stopping condition is not met.
 */
bool APropagator::propagate(PotentialBase* pot) {

  if(!Initialized) {initialize(myPtcl,pot,false);}

  //This is where APropagator update the particle configuration.
 
  Counter++;
  return false; // do not propagate
}

/** Apply boundary conditions to the particle attributes affected by the boundary crossings.
 *@param wrapping if true, move the position-dependent attributes back in a cell.
 *
 * Any particle attribute added to ParticleSet::ConfigList in APropagator::initialize
 * will be adjusted by the boundary crossing of the ParticleSet::R.
 */
void APropagator::applyBConds(bool wrapping) 
{
}

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 228 $   $Date: 2005-01-03 12:22:16 -0600 (Mon, 03 Jan 2005) $
 * $Id: APropagator.cpp 228 2005-01-03 18:22:16Z jnkim $ 
 ***************************************************************************/
