//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim and Nichols A. Romero
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// Implemented by Jeongnim Kim, May 2002
// Modifided by Jeongnim Kim and Nichols A. Romero, May/30 2002
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include <complex>
using namespace std;

#include "Propagators/ForceConstant.h"
#include "Potentials/PotentialBase.h"
#include "EigSolver/SymMatrixSolver.h"
#include "Message/Communicate.h"
#ifdef HAVE_MPI
#include <mpi.h>
#endif
using namespace OHMMS;
#include "ParticleBase/ParticleUtility.h"
#include "CrystalSymmetry/KPointBase.h"
#include "Utilities/GaussianDOS.h"

ForceConstant::ForceConstant(): Displacement(0.001), GetMode(true), GetDOS(true)
{ 
  /*! add EVNVerlet specific <parameter/>  */
  m_param.add(Displacement,"displacement","AA");
  m_param.add(GetMode,"mode","bool");
  m_param.add(GetDOS,"dos","bool");
  myName="phonon";
}
ForceConstant::~ForceConstant()
{ }

void 
ForceConstant::initialize(Particle_t* ptclin, Interaction_t* potin, bool makecopy) {

  //friction set to zero for EVN simulation. sdmin should be used instead.
  // reset, time coefficients with  new or old MD parameters
  Counter = -1;

  initializeBase(ptclin,potin,makecopy);

  if(!Initialized) { 
    myPot->setCurrentPos(myPtcl->R, true);
    myPtcl->convert2Cart(myPtcl->R);
    Initialized = true;
  }


}

bool ForceConstant::propagate(PotentialBase* pot) {


  int ntot = myPtcl->getLocalNum()*OHMMS_DIM;

  if(GetMode) {
    FCMat.resize(ntot,ntot);
    FCMat = 0.0;
  }


  Particle_t::ParticlePos_t fp(myPtcl->getLocalNum());

  ParticlePos_t& F = myPtcl->F;
  ParticlePos_t& dR = myPtcl->dR;

  Counter = 0;
  myPot->setCurrentPos(myPtcl->R);
  myPtcl->dR=0.0;

  // to partition the calculation of force-constant matrix
  int nproc = Controller->ncontexts();
  int id = Controller->mycontext();
  LocPtcl.resize(nproc+1);
  vector<int>  num(nproc), nloct(nproc+1);

  Tensor_t stress;

  ParticlePos_t resF(myPtcl->getLocalNum());

  // assign the total potential energy
  E[POT_ENERGY] = myPot->evalForce(resF, 0);
  cout << "Starting energy " << E[POT_ENERGY] << endl;

  if(PrimaryCell.getTotalNum()) {
    LocPtcl[0] = 0; LocPtcl[1] = PrimaryCell.getTotalNum();
    num[0] = LocPtcl[1];
  } else {
    if(nproc > 1) {
      int np = myPtcl->getTotalNum();
      int bat= np/nproc;
      int residue = np%nproc;
      LocPtcl[0] = 0;
      for(int ip=0; ip<nproc; ip++) {
	if(ip<residue) 
	  num[ip] = bat+1;
	else    
	  num[ip] = bat;
	LocPtcl[ip+1] = LocPtcl[ip] + num[ip];
      }
    } else {
      LocPtcl[0] = 0; LocPtcl[1] = myPtcl->getLocalNum();
      num[0] = myPtcl->getLocalNum();
    }
  }

  Scalar_t invdelta = -1.0/Displacement;
  Scalar_t inv2delta = 0.5/Displacement;
  vector<SingleParticlePos_t> disp(OHMMS_DIM); // 3D displacement vectors
  disp[0] = SingleParticlePos_t(Displacement,0,0); 
  disp[1] = SingleParticlePos_t(0,Displacement,0);
  disp[2] = SingleParticlePos_t(0,0,Displacement);

  bool PLUS = true;
  bool MINUS = false;

  cout.precision(12);

  FCMat=0.0;

  int ii=0;
  for(int i=LocPtcl[id]; i< LocPtcl[id+1]; i++) { // for each atom
    for(int j=0; j<OHMMS_DIM; j++) { // apply +/- displacement for each atom

      myPtcl->makeMove(i,disp[j]);
      E[POT_ENERGY]=pot->evalForce(F,0);

      myPtcl->makeMove(i,-2.0*disp[j]);
      E[POT_ENERGY]=pot->evalForce(fp,0);

      if(GetMode) {
        int jj=0;
        for(int k=0; k<myPtcl->getLocalNum(); k++) {
          for(int jk=0; jk<OHMMS_DIM; jk++) {
            FCMat(ii,jj++) = (F[k][jk]-fp[k][jk])*invdelta;
          }
        }
      }

      /*
      myPtcl->makeMove(i,disp[j]);
      E[POT_ENERGY]=pot->evalForce(F,0);
      cout << " plus  " << i << F[i] <<  " " << resF[i] << endl;
      F = (F - resF)*invdelta;      //F *= invdelta;
      if(GetMode) {
        int jj=0;
        for(int k=0; k<myPtcl->getLocalNum(); k++) {
          for(int jk=0; jk<OHMMS_DIM; jk++) {
            FCMat(ii,jj++) += F[k][jk];
          }
        }
      } 
      printForceConstant(F,i,j,PLUS); 
      myPtcl->makeMove(i,-2.0*disp[j]);
      E[POT_ENERGY]=pot->evalForce(F,0);
      cout << " minus " << i << F[i] <<  " " << resF[i] << endl;
      F = (F - resF)*invdelta;     //F *= invdelta;
      if(GetMode) {
        int jj=0;
        for(int k=0; k<myPtcl->getLocalNum(); k++) {
          for(int jk=0; jk<OHMMS_DIM; jk++) {
            FCMat(ii,jj++) -= F[k][jk];
          }
        }
      }
      printForceConstant(F,i,j,MINUS); 
      */

      myPtcl->makeMove(i,disp[j]);
      ii++;
    }
  }


  if(GetMode) {
#ifdef HAVE_MPI
    if(nproc > 1) {
      // collect force constants calculated on each MPI node
      for(int ip=0; ip<nproc; ip++) {
	num[ip] *= OHMMS_DIM*ntot;
	nloct[ip] = LocPtcl[ip]*OHMMS_DIM*ntot;
      }
      nloct[nproc] = LocPtcl[nproc]*OHMMS_DIM*ntot;

      MPI_Barrier(MPI_COMM_WORLD);
      MPI_Allgatherv(FCMat.data(), num[id], MPI_DOUBLE, 
		  FCMat.data(), &num[0], &nloct[0], MPI_DOUBLE,  
		  MPI_COMM_WORLD);
    }
#endif

    // only single node
    // if(id == 0) {
    // symmetrize the Hessian 1/2 * 1/2 (definition)
    FCMat *= 0.5;

    if(PrimaryCell.getTotalNum()) {
      getPhononDispersion();
    } else {
      SymMatrixSolver<Scalar_t> DMSolver; // solver for the dynamic matrix
      DMSolver.solve(FCMat.data(), ntot);
      printFreq(DMSolver.Lambda,id);
      printMode(FCMat,LocPtcl[id]*OHMMS_DIM, LocPtcl[id+1]*OHMMS_DIM, id);
    }
  }

  return false; // do not propagate
}


void ForceConstant::applyBConds(bool wrapping)
{

}

void ForceConstant::setTemperature(bool rescale, Scalar_t tref) {
}

void ForceConstant::getPhononDispersion() {

  int nr = PrimaryCell.getTotalNum();
  int nc = myPtcl->getTotalNum()/PrimaryCell.getTotalNum();            
  int nd = nr*OHMMS_DIM;
  
  LOGMSG("Number of primary cells = " << nc)
  LOGMSG("Dimension of a primary cell = " << nd)
  ComplexMatrix_t  FCsym(nd,nd);

  //make sure that everything is in unit
  myPtcl->convert2Unit(myPtcl->R);

  char fcname[128];
  sprintf(fcname,"%s.pdis", myPtcl->getName().c_str());
  ofstream fcout(fcname);
  fcout.setf(ios::scientific, ios::floatfield);
  fcout.setf(ios::right,      ios::adjustfield);

  const Scalar_t c_cm  = 2.99792458e10; // [cm/s]
  const Scalar_t amu   = 1.0e-30;// [eV/(Ang/s)^2]
  const Scalar_t pi    = 3.14159265358979; // pi
  const Scalar_t massconv = 0.02998/(2*pi)/c_cm/sqrt(amu);

  const KPointBase* kprimary(PrimaryCell.getKSet());
  RealMatrix_t freqs(kprimary->size(),nd);

  Scalar_t max_freq = 0.0;

  for(int ik=0;ik<kprimary->size(); ik++) {
    SingleParticlePos_t kpt = kprimary->k_cart(ik);
    FCsym = 0.0;
    int i = 0; 
    int iat = 0;
    int iii = 0;
    for(int ir=0; ir<nr; ir++, iat++) {
      int itype = myPtcl->GroupID[iat];
      for(int idim=0; idim<OHMMS_DIM; idim++, i++, iii++) {
	int j=0; int jat = 0;
	for(int jc=0; jc<nc; jc++) {
	  int jjj = 0;
	  for(int jr=0; jr<nr; jr++, jat++) {
	    Scalar_t massfac = 1.0/sqrt(mass[itype]*mass[myPtcl->GroupID[jat]]);
	    SingleParticlePos_t dR 
	      = myPtcl->Lattice.toCart(myPtcl->Lattice.BConds.apply(myPtcl->R[jat]-myPtcl->R[iat]));
	    Scalar_t kr = dot(dR,kpt);
	    complex<Scalar_t> phi(cos(kr),sin(kr));
	    for(int jdim=0; jdim<OHMMS_DIM; jdim++, j++, jjj++) {
	      FCsym(iii,jjj) += FCMat(i,j)*phi*massfac;
	    }
	  }
	}
      }
    }

    SymMatrixSolver<complex<Scalar_t> > DMSolver; // solver for the dynamic matrix
    DMSolver.solve(FCsym.data(), nd);
    fcout << ik << " ";
    for(int i=0; i<nd; i++) {
      complex<Scalar_t> comega = sqrt(complex<Scalar_t>(DMSolver.Lambda[i]));          
      fcout << comega.real()*massconv << " ";
      freqs(ik,i) = comega.real()*massconv;
    }
    fcout << endl;
    max_freq = max(max_freq,freqs(ik,nd-1));
  }
  fcout.close();

  if(GetDOS) {
    sprintf(fcname,"%s.vdos",myPtcl->getName().c_str());
    Scalar_t gwidth = 0.1;
    max_freq += 5*gwidth;
    int nbin = static_cast<int>(max_freq/gwidth)*4;
    GaussianDOS<Scalar_t> vdos(0.0,max_freq,nbin);
    for(int ik=0; ik<freqs.rows(); ik++) {
      Scalar_t wkpt = kprimary->W[ik];
      for(int i=0; i<nd; i++) {
	vdos.accumulate(freqs(ik,i),gwidth,wkpt);
      }
    }
    fcout.open(fcname);
    vdos.print(fcout);
  }

//   ParticlePos_t R(myPtcl->getTotalNum());
//   R.InUnit = true;
//   if(myPtcl->R.InUnit) {
//     for(int iat=0; iat<myPtcl->getTotalNum(); iat++) {
//       R[iat] = PrimaryCell.Lattice.toUnit(myPtcl->Lattice.toCart(myPtcl->R[iat]));
//     }
//   } else {
//     for(int iat=0; iat<myPtcl->getTotalNum(); iat++) {
//       R[iat] = PrimaryCell.Lattice.toUnit(myPtcl->R[iat]);
//     }
//   }

//   //make sure that everything is in unit
//   convert2Unit(PrimaryCell.Lattice,PrimaryCell.R);
//   int iat = 0;
//   for(int ic=0; ic<nc; ic++) {
//     for(int ir=0; ir<nr; ir++, iat++) {
//       R[iat] -= PrimaryCell.R[ir];
//     }
//   }
//   convert2Cart(PrimaryCell.Lattice,R);
//   Scalar_t f = 1.0/static_cast<Scalar_t>(nc);
//   RealMatrix_t Omega(PrimaryCell.KSet->size(),nd);
//   for(int ik=0;ik<PrimaryCell.KSet->size(); ik++) {
//     SingleParticlePos_t kpt = PrimaryCell.KSet->k_cart(ik);
//     cout << "#q " << kpt << endl;
//     FCsym = 0.0;
//     int i = 0; 
//     iat = 0;
//     for(int ic=0; ic<nc; ic++) {
//       int iii = 0;
//       for(int ir=0; ir<nr; ir++, iat++) {
// 	int itype = myPtcl->GroupID[iat];
// 	for(int idim=0; idim<OHMMS_DIM; idim++, i++, iii++) {
// 	  int j=0; int jat = 0;
// 	  for(int jc=0; jc<nc; jc++) {
// 	    int jjj = 0;
// 	    for(int jr=0; jr<nr; jr++, jat++) {
// 	      Scalar_t massfac = 1.0/sqrt(mass[itype]*mass[myPtcl->GroupID[jat]]);
// 	      Scalar_t kr = dot(R[iat]-R[jat],kpt);
// 	      complex<Scalar_t> phi(cos(kr),sin(kr));
// 	      for(int jdim=0; jdim<OHMMS_DIM; jdim++, j++, jjj++) {
// 		FCsym(iii,jjj) += FCMat(i,j)*phi*massfac;
// 	      }
// 	    }
// 	  }
// 	}
//       }
//     }
//     FCsym *= f;
//     SymMatrixSolver<complex<Scalar_t> > DMSolver; // solver for the dynamic matrix
//     DMSolver.solve(FCsym.data(), nd);
//    for(int i=0; i<nd; i++) {
//      Omega(ik,i) = DMSolver.Lambda[i];
//     }
//   }


}


/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 378 $   $Date: 2005-08-28 18:40:33 -0500 (Sun, 28 Aug 2005) $
 * $Id: ForceConstant.cpp 378 2005-08-28 23:40:33Z jnkim $ 
 ***************************************************************************/
