//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002,2004- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "OhmmsUnits.h"
#include "Particle/ParticleSet.h"
#include "OhmmsData/RecordProperty.h"
#include "OhmmsData/ParameterSet.h"
#include "Propagators/TPLangevinBase.h"
#include <limits>
using namespace OHMMS;

TPLangevinBase::TPLangevinBase(): 
  T_ref(0.0), P_ref(0.0), Dt(1.0), Friction(0.0), VolMass(1.0),
  randomSource(Random), AddNoise(false), VolFreq(1.0e-5)
{

}

TPLangevinBase::~TPLangevinBase() {
} 

void 
TPLangevinBase::initParameters(RecordNamedProperty<T>& properties,
				ParameterSet& params) {

  pressure_ind = properties.add("Pressure");
  pv_ind = properties.add("P*V");
  pk_ind = properties.add("KE-pressure");
  volume_ind = properties.add("Volume");
  
  params.add(T_ref, "temperature","K");
  params.add(Dt,"timestep","fsec");
  params.add(Friction,"friction","fsec-1");
  params.add(VolFriction,"volfriction","fsec-1");
  params.add(P_ref_kB,"pressure","kB");
  params.add(VolFreq,"volfreq","rad/fsec");
  params.add(AddNoise,"noise","bool");
}


void 
TPLangevinBase::checkParameters(const std::string& name) {

  LOGMSG("\tTPLangevin is used for " << name)
  if(name == "epn") {
    AddNoise=false; Friction=0.0; VolFriction=0.0;
  } else if(name == "sdmin") {
    AddNoise=false; 
    if(Friction<numeric_limits<T>::epsilon()) Friction=0.05; 
    if(VolFriction<numeric_limits<T>::epsilon()) VolFriction=0.05;
  } else if(name == "tpn") {
    AddNoise = true;
    if(Friction<zeroKelvin) Friction=0.01/Dt;
    if(VolFriction<zeroKelvin) VolFriction=0.01/Dt;
  }

  dtover2 = Dt/2.0;
  VolMassInv = 1.0/(VolMass+numeric_limits<T>::epsilon());

  dtover2W = dtover2*VolMassInv;
  dtoverW = Dt*VolMassInv;

  P_ref = P_ref_kB/pressure_to_kbar;
  LOGMSG("External Pressure " << P_ref_kB << " kB " << P_ref << " eV/AA^3 ")
  LOGMSG("Volume Mass " << VolMass)
}

void
TPLangevinBase::setSigma(T Temp, const vector<T>& mi) {

  chi_vel = Friction*dtover2;
  cf_vel=1.0-chi_vel;

  chi_vol = VolFriction*dtover2;
  cf_vol=1.0-chi_vol;

  SigmaV.resize(mi.size(),0);
  SigmaVol=0.0;

  if(AddNoise) {
    //Using Dt/2: LeapFrog algorithm to update the momentum(velocity)
    T c0 = exp(-chi_vel);
    T x = Temp*Kelvin_to_eV;
    T sigv = sqrt(x*(1.0e0-c0*c0));
    for(int i=0; i<mi.size(); i++) SigmaV[i] = sigv*sqrt(mi[i]);
    SigmaVol = sqrt(x*VolMass*chi_vol);
  }
}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 354 $   $Date: 2005-07-07 21:20:49 -0500 (Thu, 07 Jul 2005) $
 * $Id: TPLangevinBase.cpp 354 2005-07-08 02:20:49Z jnkim $ 
 ***************************************************************************/
