//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

/**@file TVNLangevin.h
 * @brief Definition of TVN-Langevin propagator.
 */
#ifndef OHMMS_TVN_LANGEVINVELOCITYVERLETPROPAGATOR_H
#define OHMMS_TVN_LANGEVINVELOCITYVERLETPROPAGATOR_H

#include "Propagators/PropagatorBase.h"

namespace OHMMS {
  /** TVN propagator using Langevin dynamics.
   * 
   * Langevin equations are solved by Velocity-Verlet algorithm.
   */
  class TVNLangevin: public PropagatorBase {

  public:

    TVNLangevin();
    ~TVNLangevin() { }
 
    // implementing virutal functions
    void initialize(Particle_t* ptclin, Interaction_t* potin, bool makecopy = false);
    void setTemperature(bool rescale, Scalar_t tref); 
    bool propagate(Interaction_t*);
    void applyBConds(bool wrapping);

  private:

    void setSigma(Scalar_t Temp, Scalar_t dt, Scalar_t friction);

    ///reference temperature
    Scalar_t T_ref;
    Scalar_t Dt;
    Scalar_t Friction;
    Scalar_t c0, c1, c2, c12, c1dt, d2dt, c2dt2, c12dt;
    Scalar_t SigmaR, SigmaV, CorrRVx, CorrRVy;
    std::vector<Scalar_t> OneOverSqrtM, c2dtoverm;
    //std::vector<Scalar_t> SigmaR, SigmaV, CorrRVx, CorrRVy;
    //std::vector<Scalar_t> c2dtoverm, c2dt2overm, c12dtoverm;
  };
}
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 306 $   $Date: 2005-02-17 11:45:40 -0600 (Thu, 17 Feb 2005) $
 * $Id: TVNLangevin.h 306 2005-02-17 17:45:40Z jnkim $ 
 ***************************************************************************/
