//////////////////////////////////////////////////////////////////
// (c) Copyright 2004- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "QuantumBase/ElectronStructureIO.h"
#include "QuantumBase/ElectronStructure.h"
#include "CrystalSymmetry/KPointBase.h"
#include "ParticleIO/XMLParticleIO.h"

namespace OHMMS {

  ESOutput::ESOutput(ElectronStructure& es): 
    ref_(es), 
    FirstTime(true),
    OutputFormat(1)
  {
  }

  void ESOutput::reset(const char* fileroot) {
    FirstTime = true;
    FileName = fileroot;
    FileName.append(".es");
  }

  void ESOutput::report(int iter) {

    const KPointBase& kpts = *(ref_.getKPoint());
    ofstream fout(FileName.c_str());
    if(FirstTime && kpts.size()) {
      string kfile=FileName;
      kfile.append(".kpt.xml");
      ofstream kout(kfile.c_str());
      kout << "<?xml version=\"1.0\"?>" << endl;
      kout << "<CrystalStructure>" << endl;
      XMLSaveParticle pout(*(ref_.RefPtclSet));
      pout.get(kout,0);
      kout.precision(10);
      kpts.print(kout,1); //printKpoints(kpts,kout);
      kout << "</CrystalStructure>" << endl;
      FirstTime = false;
    }

    if(iter<0) return;

    fout.precision(6);
    fout.setf(ios::scientific, ios::floatfield);
    if(OutputFormat) {//simple ascii format nxy: just for convenience
      if(kpts.size() == 1) {
	for(int i=0; i<ref_.NumStates; i++) {
	  fout << setw(5) << i << setw(15) << ref_.E(0,i) << endl;
	}
      } else {
      for(int ik=0; ik<kpts.size(); ik++) {
	fout << ik << " ";
	for(int i=0; i<ref_.NumStates; i++) {
	  fout << setw(15) << ref_.E(ik,i);
	}
	fout << endl;
      }
      }
    } else {
      fout << "<?xml version=\"1.0\"?>" << endl;
      fout << "<ES>" << endl;
      map<string,OhmmsElementBase*>::const_iterator it = ref_.m_param.begin();
      while(it != ref_.m_param.end()) {
	(*it).second->get(fout); it++;
      }
      int nkpt = kpts.size();
      int ns = ref_.NumStates;
      fout << "<eigvalues nkpts=\""<< nkpt << "\" nstates=\"" << ns << "\">" << endl;
      fout << ref_.E << "</eigvalues>" << endl;
      fout << "<occupation nkpts=\""<< nkpt << "\" nstates=\"" << ns << "\">"<< endl;
      fout << ref_.Occ << "</occupation>" << endl;
      fout << "</ES>" << endl;
    }
  }

  void ESOutput::finalize() {}

  bool ESOutput::put(xmlNodePtr cur) {
    if(xmlHasProp(cur,(const xmlChar*)"format")) {
      string outform((const char*)(xmlGetProp(cur, (const xmlChar *) "format")));
      if(outform == "ascii") OutputFormat = 1;
    }
    return true;
  }
}

//   void printKpoints(const KPointBase& kset, ostream& fout) {
//     int NumKpt = kset.size();
//     fout << "<KPOINT nkpts=\"" << kset.size() << "\"";
//     if(kset.Reduced) 
// 	fout << " sym=\"yes\">" << endl; 
//     else             
// 	fout << " sym=\"no\">" << endl;
//     for(int ikpt=0; ikpt<NumKpt; ikpt++) {
//       fout << kset.k_unit(ikpt) 
// 	   << setw(20) << setprecision(10) << kset.W[ikpt] << endl;
//     }
//     fout << "</KPOINT>" << endl;

//     fout << "<Symmetry size=\""<< kset.NumSym << "\">" << endl;
//     for(int isym=0; isym<kset.NumSym; isym++) {
//       for(int i=0; i<3; i++) {
// 	for(int j=0; j<3; j++) {
// 	  fout << setw(3) << kset.is[isym][j][i];
// 	}
//       }
//       fout << endl;
//     }
//     fout <<"</Symmetry>" << endl;
//   }

// void 
// SlaterKosterTBKPT::printExtraProperties(ostream& os,int outputlevel) const {
// }
// void 
// SlaterKosterTBGamma::printExtraProperties(ostream& os,int outputlevel) const {

//   os << "<eigvalues  nkpts=\"1\" nstates=\"" << NumStates << "\">" << endl;
//   int n = NumStates/10;
//   int ii=0;
//   for(int i=0; i<10*n; i+=10) {
//     for(int j=0; j<10; j++, ii++) {
//        os << setw(15) << setprecision(5) << E(0,ii);
//     }
//     os << endl;
//   }
//   for(; ii<NumStates; ii++) {
//     os << setw(15) << setprecision(5) << E(0,ii);
//   }
//   os << endl << "</eigvalues>" << endl;
		  
//   os << "<occupation nkpts=\"1\" nstates=\"" << NumStates << "\">"<< endl;
//   ii=0;
//   for(int i=0; i<10*n; i+=10) {
//     for(int j=0; j<10; j++, ii++) {
//        os << Occ(0,ii) << " ";
//     }
//     os << endl;
//   }
//   for(; ii<NumStates; ii++) {
//     os <<  Occ(0,ii) << " ";
//   }
//   os << endl << "</occupation>" << endl;


//   if(thisSolver->Psi.size()) {
//     int first = Homo-10;
//     int last = Homo+10;
//     last = min(last,NumStates);
//     first = max(first,0);
//     RealMatrix_t q(last-first, myPtcl->getTotalNum());
//     //#pragma omp parallel
//     {
//       const RealMatrix_t& Psi = thisSolver->Psi;
//       int myID = omp_get_thread_num();
//       const SparseIndex_t& hindex = *(Hmat.index());
//       // sparse indices and storage always work with logical 
//       // indecies similarly to the pairlist
//       // physical indecies are required to check the species-related properties
      
//       //Hamiltonian and Overlap matrices are based on the pair list
//       const PtclPairList_t& nnlist = myPtcl->nnListEngine->getPairAttrib(myID);
//       int FirstAtom = hindex.first_diag(myID);
//       int LastAtom = hindex.last_diag(myID);
//       for(int istate=first, i=0; istate<last; i++, istate++) {
// 	int ibtot = hindex.row_loc(FirstAtom);
// 	for(int ig=FirstAtom; ig<LastAtom;ig++) {
// 	  Scalar_t t=0.0;
// 	  for(int ib=0; ib<hindex.row_block(ig); ib++, ibtot++){
// 	    t+= Psi(istate,ibtot)*Psi(istate,ibtot);
// 	  }
// 	  q(i,myPtcl->ID[ig]) = t;
// 	}
//       }
//     }
//     os << "<LDOS lower=\"" << first << "\" upper=\"" << last <<"\">" << endl;
//     for(int istate=first, i=0; istate<last; i++, istate++) {
//       os << istate << " ";
//       for(int iat=0; iat<myPtcl->getTotalNum(); iat++) 
// 	os << q(i,iat) << " ";
//       os << endl;
//     }
//     os << "</LDOS>" << endl;
//   }
// }

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 365 $   $Date: 2005-07-14 09:03:43 -0500 (Thu, 14 Jul 2005) $
 * $Id: ElectronStructureIO.cpp 365 2005-07-14 14:03:43Z jnkim $ 
 ***************************************************************************/
