/////////////////////////////////////////////////////////////////////
// \author Jeongnim Kim
//
//   Polak-Ribiere factor (not used)
//      double x1 = dot(g,g)-dot(gprev,g);
//      double x2 = dot(gprev,gprev);
//      gamma = -x1/(x2+1e-40);
//   
//      gamma = dot(g,g)/(dot(gprev,gprev)+1e-40); // <g|g>/<gp|gp>
//////////////////////////////////////////////////////////////////////
#ifndef OHMMS_UNCONSTRAINT_CONJUGATEGRADIENT_H
#define OHMMS_UNCONSTRAINT_CONJUGATEGRADIENT_H
#include "Utilities/OhmmsInfo.h"
#include "OhmmsPETE/OhmmsVector.h"

// Ham_t should provide apply(const T*, T*)
template<class Ham_t, class Wfs_t>
struct UnconstraintConjGradient {

  UnconstraintConjGradient(){ }
  void solve(const Ham_t& ham, Wfs_t& states, T* eigval);
};


template<class Ham_t, class Wfs_t>
void 
UnconstraintConjGradient<Ham_t,Wfs_t>::solve(const Ham_t& ham, Wfs_t& psi, 
                                             T* eigval) {
  typedef Ham_t::Type_t Type_t;
  typedef Matrix<Type_t, vector<Type_t> > Matrix_t;

  int nband = states.nrow();
  int dim = states.ncol();
  Wfs_t hpsi(dim);
  Matrix_t dm(nband,nband), hdm(nband,nband);  

  do {

    for(int iband=0; iband<states.nrow(); iband++) 
      ham.apply(psi[iband], hpsi[iband]);

    for(int iband=0; iband<nband; iband++)
      for(int jband=iband; jband<nband; jband++) {
	dm(iband,jband) = OhmmsBlas::dot(dim,psi[iband],psi[iband]); 
	if(iband == jband) dm(iband,jband) -= 2.0;
	else               dm(jband,iband) = dm(iband,jband);
      }

    for(int iband=0; iband<nband; iband++)
      for(int jband=iband; jband<nband; jband++) {
	hdm(iband,jband) = OhmmsBlas::dot(dim,psi[iband],hpsi[iband]); 
	if(iband != jband) dm(jband,iband) = dm(iband,jband);
      }

    for(int iband=0; iband<nband; iband++)
      for(int k=0; k<dim; k++) {
        Type_t tmp = 0.0;
        for(int jband=0; jband<nband; jband++) hdm(iband,jband)*psi

  g = sum(hdm(i,k)*psi(k,j) + dm(i,k)*hpsi(k,j),k);

  return -TBPot->SpinDeg*sum(sum(hdm(i,j)*dm(j,i),j));

    Ec[0] = getGradient(h); // getting an energy

    if(iter%Stride == 0) { // calculation of cg
      cg = g;
      gnorm = sum(sum(g(i,j)*g(i,j),j));
    } else {
      gnorm0  = gnorm;
      gnorm   = sum(sum(g(i,j)*g(i,j),j));
      gortho  = sum(sum(cg(i,j)*g(i,j),j));
      gamma   = (gnorm-gortho)/gnorm0;

      cg *= gamma;
      cg += g;
    }

    T qtot = -2.0e0*sum(sum(dm(i,j)*dm(j,i),j));
    for(int is=0; is<TBPot->NumStates; ++is) qtot -= 4.0e0*dm(is,is);

#ifdef TBCGDEBUG
    cout << " Energy(" << iter << ")= " << Ec[0] << " qtot = " << qtot << " Norm(g) = "  << gnorm ;
#endif

    getPolyExpansion(h); // poly expansion of TB energy
    lambda = lineoptimization(Ec); 
    psi += lambda*cg;
    deabs = fabs((e0-Ec[0])/Ec[0]);

#ifdef TBCGDEBUG
    cout << " dE/E = " << deabs << endl;
#endif

    if(iter > MinIter && deabs < Etol) ToMinimize = false;
    e0 = Ec[0];
    
    iter++;

    if(iter > MaxIter) ToMinimize = false;

  } while(ToMinimize);
  
  // allocate the data
  int nband = states.nrow();
  if(BasisDim < states.ncol()) {
    BasisDim = states.ncol();
    psi.resize(BasisDim);
    hpsi.resize(BasisDim);
    g.resize(BasisDim);
    cg.resize(BasisDim);
    hcg.resize(BasisDim);
    gprev.resize(BasisDim);
  }

  T gamma, residue;
  const T etol = 1e-12;
  const T restol = 1e-12;
  T lambda, e1, e2, theta, costheta, sintheta;
  int maxiter = 1000;

  for(int iband=0; iband < nband; iband++) {

    states.orthonormalize(iband);
    states.copy(iband, psi.begin());// copy iband to psi    

    ham.apply(psi.begin(), hpsi.begin());
    lambda = dot(psi,hpsi); 
    g = lambda*psi - hpsi;

    /////////////////////////////////////////////
    //if no precondition is used, Ham_t::precondition should include
    // states.orthogonalize(g.begin(), 0, iband);
    //
    //For precondition,
    //  states.orthogonalize(g.begin(), 0, iband-1);
    //    hamiltonian specific precondition 
    //  states.orthogonalize(g.begin(), 0, iband);
    ////////////////////////////////////////////
    ham.precondition(g.begin(),states,iband);

    residue = dot(g,g);
    gprev = g;
    cg = g;
    normalize(cg);

    int iter = 0;
    do {
      ///////////////////////////////////////////////////////
      // Lineminimization using cosine and sine expansion
      // |psi> = cos(theta)*|psi> + sin(theta)*|cg>
      ///////////////////////////////////////////////////////
      e1 = dot(hpsi,cg);
      ham.apply(cg.begin(), hcg.begin());

      e2 = dot(hcg,cg);
      theta = 0.5*atan(2.0*e1/(lambda - e2));
      costheta = cos(theta);
      sintheta = sin(theta);
      T eproj = costheta*(costheta*lambda+2.0*sintheta*e1)
	+sintheta*sintheta*e2;

      psi = costheta*psi + sintheta*cg;
      hpsi = costheta*hpsi + sintheta*hcg;
      /////////////////////////////////////////////////////////

      if(fabs((lambda-eproj)/lambda) < etol || residue < restol) { 
	lambda = eproj; break;
      }

      lambda = eproj;
      g = lambda*psi - hpsi;
      residue  = dot(g,g);

      states.assign(iband, psi.begin());// copy psi to iband

      /////////////////////////////////////////////
      //no precondition is used
      //states.orthogonalize(g.begin(), 0, iband);
      ////////////////////////////////////////////
      ham.precondition(g.begin(),states, iband);

      double x1 = dot(g,g) - dot(gprev,g);
      double x2 = dot(g,cg) - dot(gprev,cg);
      gamma = -x1/(x2+1e-40);
      DEBUGMSG(iter << " " << eproj << " " << residue)

      cg = g + gamma*cg;
      gprev = g;
      states.orthogonalize(cg.begin(), iband, iband);
      normalize(cg);
      iter++;
    } while(iter < maxiter);
    cout << "Iteration = " << iter << endl;
    eigval[iband] = lambda;

  }
}
#endif
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: UncontraintConjGradient.h 151 2004-11-02 23:51:14Z jnkim $
 ************************************************************************/
