//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#ifndef OHMMS_SLATERKOSTERMAKE_H
#define OHMMS_SLATERKOSTERMAKE_H
#include "SlaterKoster/SlaterKosterBase.h"
#include "SlaterKoster/SlaterKosterSP.h"
#include "SlaterKoster/SlaterKosterSD.h"
#include "SlaterKoster/SlaterKosterPD.h"
#include "SlaterKoster/SlaterKosterDD.h"

template<class T, class SF>
struct SKBlock {

  static SlaterKosterBase<T>* make(int L1, int L2, int i, int j, 
                                   int loc1, int loc2,  T eps,
	 	                   vector<SF>* v){
    int M = (L1<=L2)? L1 : L2;
    bool samespecies = (i == j);

    // create M+1 scaling functions for L1-L2 pair
    v->reserve(M+1);
    for(int mm=0; mm<M+1; mm++) v->push_back(SF());

    typedef SlaterKoster<T,0,0,SF> SK_ss;
    typedef SlaterKoster<T,0,1,SF> SK_sp;
    typedef SlaterKoster<T,1,1,SF> SK_pp;
    typedef SlaterKoster<T,0,2,SF> SK_sd;
    typedef SlaterKoster<T,1,2,SF> SK_pd;
    typedef SlaterKoster<T,2,2,SF> SK_dd;

    SF* vin = &(v->operator[](0));

    if(L1 == 0) {
      if(L2 == 0)  {return new SK_ss(loc1, loc2, vin, eps, samespecies);}
      else if(L2 == 1) {		      
        if(i == j) return new SK_sp(loc1, loc2, vin, true);
        else       return new SK_sp(loc1, loc2, vin, false);
      }					      
      else if(L2 == 2) {		      
        if(i == j) return new SK_sd(loc1, loc2, vin, true);
        else       return new SK_sd(loc1, loc2, vin, false);
      }					      
      else return NULL;			      
    }					      
    if(L1 == 1) {			      
      if(L2 == 0)  {			      
        if(i == j) return new SK_sp(loc2, loc1, vin, true, true);
        else       return new SK_sp(loc2, loc1, vin, false, true);
      }
      else if(L2 == 1) {
                   return new SK_pp(loc1, loc2, vin, eps, samespecies);
      } 
      else if(L2 == 2) {
        if(i == j) return new SK_pd(loc1, loc2, vin, true);
        else       return new SK_pd(loc1, loc2, vin, false);
      }
      else return NULL;
    }

    if(L1 == 2) {
      if(L2 == 0)  {
        if(i == j) return new SK_sd(loc2, loc1, vin, true, true);
        else       return new SK_sd(loc2, loc1, vin, false, true);
      }
      else if(L2 == 1) {
        if(i == j) return new SK_pd(loc2, loc1, vin, true, true);
        else       return new SK_pd(loc2, loc1, vin, false, true);
      }
      else if(L2 == 2) {
	return new SK_dd(loc1, loc2, vin, eps, samespecies);
      }
      else return NULL;
    }
    return NULL;


//     if(L1 == 0) {
//       if(L2 == 0)  {return new SK_ss(loc1, loc2, v->begin(), eps, samespecies);}
//       else if(L2 == 1) {		      
//         if(i == j) return new SK_sp(loc1, loc2, v->begin(), true);
//         else       return new SK_sp(loc1, loc2, v->begin(), false);
//       }					      
//       else if(L2 == 2) {		      
//         if(i == j) return new SK_sd(loc1, loc2, v->begin(), true);
//         else       return new SK_sd(loc1, loc2, v->begin(), false);
//       }					      
//       else return NULL;			      
//     }					      
//     if(L1 == 1) {			      
//       if(L2 == 0)  {			      
//         if(i == j) return new SK_sp(loc2, loc1, v->begin(), true, true);
//         else       return new SK_sp(loc2, loc1, v->begin(), false, true);
//       }
//       else if(L2 == 1) {
//                    return new SK_pp(loc1, loc2, v->begin(), eps, samespecies);
//       } 
//       else if(L2 == 2) {
//         if(i == j) return new SK_pd(loc1, loc2, v->begin(), true);
//         else       return new SK_pd(loc1, loc2, v->begin(), false);
//       }
//       else return NULL;
//     }

//     if(L1 == 2) {
//       if(L2 == 0)  {
//         if(i == j) return new SK_sd(loc2, loc1, v->begin(), true, true);
//         else       return new SK_sd(loc2, loc1, v->begin(), false, true);
//       }
//       else if(L2 == 1) {
//         if(i == j) return new SK_pd(loc2, loc1, v->begin(), true, true);
//         else       return new SK_pd(loc2, loc1, v->begin(), false, true);
//       }
//       else if(L2 == 2) {
// 	return new SK_dd(loc1, loc2, v->begin(), eps, samespecies);
//       }
//       else return NULL;
//     }
//     return NULL;
  }
};

#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 378 $   $Date: 2005-08-28 18:40:33 -0500 (Sun, 28 Aug 2005) $
 * $Id: SlaterKosterMake.h 378 2005-08-28 23:40:33Z jnkim $ 
 ***************************************************************************/

