//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
/***************************************************************************
 *
 ***************************************************************************/
#ifndef SLATERKOSTERARRAY_SP_H
#define SLATERKOSTERARRAY_SP_H
// #ifndef SLATERKOSTERARRAYBASE_H
// #include "SlaterKoster/SlaterKosterBase.h"
// #endif

template<class T, class SF>
class SlaterKoster<T,0,0,SF>: public SlaterKosterBase<T> {
private:

  using SlaterKosterBase<T>::I0;
  using SlaterKosterBase<T>::J0;

  T Eps;
  int myI, myJ;
  bool NeedToSymm;
  SF* V;

public:

  SlaterKoster(int i0, int j0, SF* v, T eps=0, bool samespecies=true) :V(v){
    Eps = eps;
    I0 = i0;  J0 = j0; 
    if(samespecies && I0 != J0)  NeedToSymm = true; 
    else                         NeedToSymm = false;
  }

  void reset(int nrow, int ncol) {
    myI = J0+I0*ncol;
    myJ = I0+J0*ncol;
  }

  void getOnsite(T* b);
  void getOffsite(T r, const T* p,  T* b);
  void getDeriv(T r, const T* , const T [][3], T* bx, T* by, T* bz);
  void getDeriv2(T r, const T*, const T [][3], const T [][3][3], 
       T* bx,int i, int j);
  void assign(T* b);
};

template<class T, class SF>
class SlaterKoster<T,0,1,SF>: public SlaterKosterBase<T> {
private:

  using SlaterKosterBase<T>::I0;
  using SlaterKosterBase<T>::J0;

  int ix,iy,iz,jx,jy,jz;
  bool Symm, Swap;
  SF* V;
public:

  SlaterKoster(int i0, int j0, SF* v, bool symm=true, bool swap=false) : 
    V(v), Symm(symm), Swap(swap){ I0 = i0; J0 = j0;
  }

  void reset(int nrow, int ncol) {
    if(Swap) {
      ix = J0+I0*ncol;  iy = ix+1;    iz = iy+1;
      jx = I0+J0*ncol;  jy = jx+ncol; jz = jy+ncol;
    } else { 
      jx = J0+I0*ncol;  jy = jx+1;    jz = jy+1;
      ix = I0+J0*ncol;  iy = ix+ncol; iz = iy+ncol;
    }
  }

  void getOffsite(T r, const T* p,  T* b);
  void getDeriv(T r, const T* , const T [][3], T* bx, T* by, T* bz);
  void getDeriv2(T r, const T*, const T [][3], const T [][3][3], 
       T* bx,int i, int j);
  void assign(T* b);
};

template<class T, class SF>
class SlaterKoster<T,1,1,SF>: public SlaterKosterBase<T> {
private:

  using SlaterKosterBase<T>::I0;
  using SlaterKosterBase<T>::J0;

  int I[3][3];
  int J[3][3];
  SF* V;
  T Eps;
  bool NeedToSymm;

public:

  SlaterKoster(int i0, int j0, SF* v, T eps=0, bool samespecies=true) : V(v), Eps(eps){
    I0 = i0; J0 = j0;
    if(samespecies && I0 != J0)  NeedToSymm = true; 
    else                         NeedToSymm = false; 
  }

  void reset(int nrow, int ncol){
    for(int i=0; i<3; i++)
      for(int j=0; j<3; j++)
        I[i][j] = J0+j + (I0+i)*ncol;

    if(NeedToSymm) {
      for(int i=0; i<3; i++)
        for(int j=0; j<3; j++)
          J[i][j] = I0+i + (J0+j)*ncol;
    }
  }

  void getOnsite(T* b);
  void getOffsite(T r, const T* p,  T* b);
  void getDeriv(T r, const T* , const T [][3], T* bx, T* by, T* bz);
  void getDeriv2(T r, const T*, const T [][3], const T [][3][3], 
       T* bx,int i, int j);
  void assign(T* b);
};

///////////////////////////////////////////////////////////////////////////////
// specialized for L1=0, L2=0 S-S block
///////////////////////////////////////////////////////////////////////////////
//template<class T, class SF>
//SlaterKoster<T,0,0,SF>::SlaterKoster(int i0, int j0, SF* v, T eps,
//				     bool samespecies):V(v){
//  Eps = eps;
//  I0 = i0;  J0 = j0; 
//  if(samespecies && I0 != J0)  NeedToSymm = true; 
//  else                         NeedToSymm = false;
//}   

//template<class T, class SF>
//void
//SlaterKoster<T,0,0,SF>::reset(int nrow, int ncol) {
//  myI = J0+I0*ncol;
//  myJ = I0+J0*ncol;
//}

template<class T, class SF>
void 
SlaterKoster<T,0,0,SF>::assign(T* b) {
  b[myI] = static_cast<T>(myI);
  if(NeedToSymm) b[myJ] = static_cast<T>(myJ);
}


template<class T, class SF>
void 
SlaterKoster<T,0,0,SF>::getOnsite(T* b) {
  b[myI] += Eps;
}

template<class T, class SF>
void 
SlaterKoster<T,0,0,SF>::getOffsite(T r, const T* p, T* b) {
  T xx = V[0](r);
  b[myI] += xx;
  if(NeedToSymm) b[myJ] +=xx;
}

template<class T, class SF>
void 
SlaterKoster<T,0,0,SF>::getDeriv(T r, const T* p, 
                                  const T pd[][3], T* bx, T* by, T* bz) {
  T y;  
  T dy = V[0](r,y);   //T dy = Vnorm*V->df(r,y);

  bx[myI] += p[SK_Px]*dy;
  by[myI] += p[SK_Py]*dy;
  bz[myI] += p[SK_Pz]*dy;

  if(NeedToSymm) {
    bx[myJ] = -bx[myI];    
    by[myJ] = -by[myI];    
    bz[myJ] = -bz[myI];
  }
}

template<class T, class SF>
void 
SlaterKoster<T,0,0,SF>::getDeriv2(T r, const T* p, const T pd[][3], 
                                  const T pdd[][3][3],  T* b, int d0, int d1) {
  T v, dv;  
  T ddv = V[0](r,v,dv);
  //T ddv = Vnorm*V->d2f(r,v,dv);
  //v *= Vnorm;
  //dv *= Vnorm;

  b[myI] += pd[d0][d1]*dv + p[d0]*p[d1]*ddv;  
  if(NeedToSymm) {
    b[myJ] = b[myI];
  }
}

///////////////////////////////////////////////////////////////////////////////
//specialized fro L1=0, L2=1 :  S-P block
///////////////////////////////////////////////////////////////////////////////
//template<class T, class SF>
//SlaterKoster<T,0,1,SF>::SlaterKoster(int i, int j, SF* v, bool symm, bool swap)
//: V(v), Symm(symm), Swap(swap){
//  I0 = i; J0 = j;
//}
//template<class T, class SF>
//void 
//SlaterKoster<T,0,1,SF>::reset(int nrow, int ncol) {
//  if(Swap) {
//    ix = J0+I0*ncol;  iy = ix+1;    iz = iy+1;
//    jx = I0+J0*ncol;  jy = jx+ncol; jz = jy+ncol;
//  } else { 
//    jx = J0+I0*ncol;  jy = jx+1;    jz = jy+1;
//    ix = I0+J0*ncol;  iy = ix+ncol; iz = iy+ncol;
//  }
//}

template<class T, class SF>
void 
SlaterKoster<T,0,1,SF>::assign(T* b) {
  b[jx] =static_cast<T>(jx);
  b[jy] =static_cast<T>(jy);
  b[jz] =static_cast<T>(jz);
  if(Symm) {
    b[ix] =static_cast<T>(ix);
    b[iy] =static_cast<T>(iy);
    b[iz] =static_cast<T>(iz);
  }
}

template<class T, class SF>
void 
SlaterKoster<T,0,1,SF>::getOffsite(T r, const T* p,  T* b) {

  T vsp = V[0](r);  //T vsp = Vnorm*V->f(r);

  T x = vsp*p[0], y = vsp*p[1], z = vsp*p[2];
  b[jx] +=x ;  b[jy] += y; b[jz] +=  z;
  if(Symm) {
    b[ix]-= x; b[iy] -= y; b[iz]-= z;
  }
}

template<class T, class SF>
void 
SlaterKoster<T,0,1,SF>::getDeriv(T r, const T* p, const T pd[][3], 
                                  T* bx, T* by, T* bz) {

  T v;
  //T dv = Vnorm*V->df(r,v); // df returns a derivative and f(r)/r
  T dv = V[0](r,v);
  //v *= Vnorm;

  // p =[l, m, n] [lm, mn, mn, l2m2, n2r2]
  //         0  1   2   3   4   5   6   7  8
  // pderiv lx, mx, nx, ly, my, ny, lz, mz, nz ....
  T v00 = dv*p[SK_Px]*p[0] + v*pd[SK_Px][0];
  T v01 = dv*p[SK_Py]*p[0] + v*pd[SK_Py][0];
  T v02 = dv*p[SK_Pz]*p[0] + v*pd[SK_Pz][0];

  T v11 = dv*p[SK_Py]*p[1] + v*pd[SK_Py][1];
  T v12 = dv*p[SK_Pz]*p[1] + v*pd[SK_Pz][1];
  T v22 = dv*p[SK_Pz]*p[2] + v*pd[SK_Pz][2];

  bx[jx] += v00; bx[jy] += v01; bx[jz] += v02;
  by[jx] += v01; by[jy] += v11; by[jz] += v12;
  bz[jx] += v02; bz[jy] += v12; bz[jz] += v22;

  if(Symm) {
    bx[ix] -= v00; bx[iy] -= v01; bx[iz] -= v02;
    by[ix] -= v01; by[iy] -= v11; by[iz] -= v12;
    bz[ix] -= v02; bz[iy] -= v12; bz[iz] -= v22;
  }
}

template<class T, class SF>
void
SlaterKoster<T,0,1,SF>::getDeriv2(T r, const T* p, const T pd[][3], 
 		                   const T pdd[][3][3], T* b, int d0, int d1) {
  T v, dv;  
  T ddv = V[0](r,v,dv);//T ddv = Vnorm*V->d2f(r,v,dv);
  //v *= Vnorm;
  //dv *= Vnorm;

  T dsps0 = p[d0]*dv;
  T dsps1 = p[d1]*dv;
  T d2sps = pd[d0][d1]*dv + p[d0]*p[d1]*ddv;
  b[jx]+= v*pdd[SK_Px][d0][d1]+dsps0*pd[SK_Px][d1]+dsps1*pd[SK_Px][d0]+d2sps*p[SK_Px];
  b[jy]+= v*pdd[SK_Py][d0][d1]+dsps0*pd[SK_Py][d1]+dsps1*pd[SK_Py][d0]+d2sps*p[SK_Py];
  b[jz]+= v*pdd[SK_Pz][d0][d1]+dsps0*pd[SK_Pz][d1]+dsps1*pd[SK_Pz][d0]+d2sps*p[SK_Pz];

  if(Symm) {
    b[ix] = -b[jx]; b[iy] = -b[jy]; b[iz] = -b[jz];
  }
}



///////////////////////////////////////////////////////////////////////////////
// specialized for L1=1, L2=1, e.g., P-P block
///////////////////////////////////////////////////////////////////////////////
//template<class T, class SF>
//SlaterKoster<T,1,1,SF>::SlaterKoster(int i0, int j0, SF* v, T eps, 
//				     bool samespecies): V(v), Eps(eps){
//  I0 = i0; J0 = j0;
//  if(samespecies && I0 != J0)  NeedToSymm = true; 
//  else                         NeedToSymm = false; 
//}
//
//template<class T, class SF>
//void 
//SlaterKoster<T,1,1,SF>::reset(int nrow, int ncol) {
//  for(int i=0; i<3; i++)
//    for(int j=0; j<3; j++)
//      I[i][j] = J0+j + (I0+i)*ncol;
//
//  if(NeedToSymm) {
//    for(int i=0; i<3; i++)
//      for(int j=0; j<3; j++)
//	J[i][j] = I0+i + (J0+j)*ncol;
//  }
//}

template<class T, class SF>
void 
SlaterKoster<T,1,1,SF>::assign(T* b) {
  for(int i=0; i<3; i++)
    for(int j=0; j<3; j++)
      b[I[i][j]] = static_cast<T>(I[i][j]);

  if(NeedToSymm) {
    for(int i=0; i<3; i++)
      for(int j=0; j<3; j++)
	b[J[i][j]] = static_cast<T>(J[i][j]);
  }
}

template<class T, class SF>
void
SlaterKoster<T,1,1,SF>::getOnsite(T* b){
  b[I[0][0]] += Eps; b[I[1][1]] += Eps; b[I[2][2]] += Eps;

}

template<class T, class SF>
void
SlaterKoster<T,1,1,SF>::getOffsite(T r, const T* p, T* b) {
  // p =[l, m, n] [lm, mn, mn, l2m2, n2r2]
  //         0  1   2   3   4   5   6   7  8
  // pderiv lx, mx, nx, ly, my, ny, lz, mz, nz ....
  //T vpi = Vpnorm*Vpi->f(r);
  //T dv =  Vsnorm*Vsig->f(r)-vpi;
  T vpi = V[1](r);
  T dv  = V[0](r)-vpi;
  T c00 = vpi+ dv*p[0]*p[0];  
  T c01 =      dv*p[0]*p[1];
  T c02 =      dv*p[0]*p[2];
  T c11 = vpi+ dv*p[1]*p[1];  
  T c12 =      dv*p[1]*p[2];
  T c22 = vpi+ dv*p[2]*p[2];
  b[I[0][0]] += c00; b[I[0][1]] += c01; b[I[0][2]] += c02;
  b[I[1][0]] += c01; b[I[1][1]] += c11; b[I[1][2]] += c12;
  b[I[2][0]] += c02; b[I[2][1]] += c12; b[I[2][2]] += c22;

  if(NeedToSymm) {
    for(int i=0; i<3; i++)
      for(int j=0; j<3; j++)
	b[J[i][j]] = b[I[j][i]];
  }
}

template<class T, class SF>
void 
SlaterKoster<T,1,1,SF>::getDeriv(T r, const  T* p, 
                                  const T pd[][3], T* bx, T* by, T* bz) {
  T vsig, vpi;
  T dv1 = V[1](r,vpi);  //T dv1 = Vpnorm*Vpi->df(r,vpi);   
  T dv2 = V[0](r,vsig); //T dv2 = Vsnorm*Vsig->df(r,vsig); 

  dv2 -= dv1; // d(Vpp_pi - Vpp_sig)
  //T v1 = vpi;//T v1 = vpi*Vpnorm;      // Vpp_pi
  T v2 = vsig-vpi;   //T v2 = vsig*Vsnorm -v1; // Vpp_pi - Vpp_sig  
  
  T c00 = dv1 + p[0]*p[0]*dv2;
  T c01 =       p[0]*p[1]*dv2;
  T c02 =       p[0]*p[2]*dv2;
  T c11 = dv1 + p[1]*p[1]*dv2;
  T c12 =       p[1]*p[2]*dv2;
  T c22 = dv1 + p[2]*p[2]*dv2;

  // x-derivate
  T cp00 = c00*p[SK_Px] + v2*(pd[SK_Px][0]*p[SK_Px]+p[SK_Px]*pd[SK_Px][0]);
  T cp01 = c01*p[SK_Px] + v2*(pd[SK_Px][0]*p[SK_Py]+p[SK_Px]*pd[SK_Py][0]);
  T cp02 = c02*p[SK_Px] + v2*(pd[SK_Px][0]*p[SK_Pz]+p[SK_Px]*pd[SK_Pz][0]);
  T cp11 = c11*p[SK_Px] + v2*(pd[SK_Py][0]*p[SK_Py]+p[SK_Py]*pd[SK_Py][0]);
  T cp12 = c12*p[SK_Px] + v2*(pd[SK_Py][0]*p[SK_Pz]+p[SK_Py]*pd[SK_Pz][0]);
  T cp22 = c22*p[SK_Px] + v2*(pd[SK_Pz][0]*p[SK_Pz]+p[SK_Pz]*pd[SK_Pz][0]);


  bx[I[0][0]] += cp00; bx[I[0][1]] += cp01; bx[I[0][2]]+=cp02;
  bx[I[1][0]] += cp01; bx[I[1][1]] += cp11; bx[I[1][2]]+=cp12;
  bx[I[2][0]] += cp02; bx[I[2][1]] += cp12; bx[I[2][2]]+=cp22;

  // y-derivate
  cp00 = c00*p[SK_Py] + v2*(pd[SK_Px][1]*p[SK_Px]+p[SK_Px]*pd[SK_Px][1]);
  cp01 = c01*p[SK_Py] + v2*(pd[SK_Px][1]*p[SK_Py]+p[SK_Px]*pd[SK_Py][1]);
  cp02 = c02*p[SK_Py] + v2*(pd[SK_Px][1]*p[SK_Pz]+p[SK_Px]*pd[SK_Pz][1]);
  cp11 = c11*p[SK_Py] + v2*(pd[SK_Py][1]*p[SK_Py]+p[SK_Py]*pd[SK_Py][1]);
  cp12 = c12*p[SK_Py] + v2*(pd[SK_Py][1]*p[SK_Pz]+p[SK_Py]*pd[SK_Pz][1]);
  cp22 = c22*p[SK_Py] + v2*(pd[SK_Pz][1]*p[SK_Pz]+p[SK_Pz]*pd[SK_Pz][1]);

  by[I[0][0]] += cp00; by[I[0][1]] += cp01; by[I[0][2]]+=cp02;
  by[I[1][0]] += cp01; by[I[1][1]] += cp11; by[I[1][2]]+=cp12;
  by[I[2][0]] += cp02; by[I[2][1]] += cp12; by[I[2][2]]+=cp22;


  // z-derivate
  cp00 = c00*p[SK_Pz] + v2*(pd[SK_Px][2]*p[SK_Px]+p[SK_Px]*pd[SK_Px][2]);
  cp01 = c01*p[SK_Pz] + v2*(pd[SK_Px][2]*p[SK_Py]+p[SK_Px]*pd[SK_Py][2]);
  cp02 = c02*p[SK_Pz] + v2*(pd[SK_Px][2]*p[SK_Pz]+p[SK_Px]*pd[SK_Pz][2]);
  cp11 = c11*p[SK_Pz] + v2*(pd[SK_Py][2]*p[SK_Py]+p[SK_Py]*pd[SK_Py][2]);
  cp12 = c12*p[SK_Pz] + v2*(pd[SK_Py][2]*p[SK_Pz]+p[SK_Py]*pd[SK_Pz][2]);
  cp22 = c22*p[SK_Pz] + v2*(pd[SK_Pz][2]*p[SK_Pz]+p[SK_Pz]*pd[SK_Pz][2]);

  bz[I[0][0]] += cp00; bz[I[0][1]] += cp01; bz[I[0][2]]+=cp02;
  bz[I[1][0]] += cp01; bz[I[1][1]] += cp11; bz[I[1][2]]+=cp12;
  bz[I[2][0]] += cp02; bz[I[2][1]] += cp12; bz[I[2][2]]+=cp22;

//    if(NeedToSymm) {
//      //\todo add symmetrization
//    }
}


template<class T, class SF>
void 
SlaterKoster<T,1,1,SF>::getDeriv2(T r, const T* p, const T pd[][3], 
                                   const T pdd[][3][3], T* b, int d0, int d1) {
  T v1, v2, dv1, dv2;
  //T d2v1 = Vpnorm*Vpi->d2f(r,vpi,dv1);
  //T d2v2 = Vsnorm*Vsig->d2f(r,vsig,dv2);
  T d2v1 = V[1](r,v1,dv1);
  T d2v2 = V[0](r,v2,dv2);
  //T v1 = vpi*Vpnorm;
  //T v2 = vsig*Vsnorm;
  //dv1 *= Vpnorm;
  //dv2 *= Vsnorm;

  d2v2 -= d2v1;
  v2 -= v1;
  dv2 -= dv1;

  for(int k=0; k<3; k++)
    for(int kp=k; kp<3; kp++) 
      b[I[k][kp]] += v2*(pdd[k][d0][d1]*p[kp] + p[k]*pdd[kp][d0][d1] 
                       + pd[k][d0]*pd[kp][d1] + pd[k][d1]*pd[kp][d0])
                   +dv2*(p[k]*p[kp]*pd[d0][d1] 
                       + p[d0]*(pd[k][d1]*p[kp]+p[k]*pd[kp][d1])  
                       + p[d1]*(pd[k][d0]*p[kp]+p[k]*pd[kp][d0]))
    	           +d2v2*p[k]*p[kp]*p[d0]*p[d1];

  T diag= (d2v1*p[d0]*p[d1]+dv1*pd[d0][d1]);
  b[I[0][0]] +=diag;
  b[I[1][1]] +=diag;
  b[I[2][2]] +=diag;
  
  b[I[1][0]] = b[I[0][1]];
  b[I[2][0]] = b[I[0][2]];
  b[I[2][1]] = b[I[1][2]];
}


#endif


/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 378 $   $Date: 2005-08-28 18:40:33 -0500 (Sun, 28 Aug 2005) $
 * $Id: SlaterKosterSP.h 378 2005-08-28 23:40:33Z jnkim $ 
 ***************************************************************************/
