//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002,2003- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#include "Message/OpenMP.h"
#include "SlaterKosterTB/Models/LenoskyTB.h"
#include "Potentials/GenericTwoBodySRP.h"
#include "SlaterKoster/SlaterKosterMake.h"
#include "Utilities/SimpleParser.h"
using namespace OHMMS;

LenoskyTB::LenoskyTB(){}

LenoskyTB::~LenoskyTB() {
  // remove scaling functions
  for(int i=0; i<V.size(); i++) delete V[i];
}

ShortRangePotential*
LenoskyTB::init(SKList_t& SK_h, SKList_t& SK_s, 
		Matrix<int>& bond, istream& fin){

  SpeciesSet& mySpecies(myPtcl->getSpeciesSet());

  // make private references to Species
  int ib = -1, ia, ja, L1, L2, vloc;
  int l1, l2, lt1, lt2, m;
  Scalar_t eps;
  vector<Scalar_t> x(100), y(100);
  int npts;
  Scalar_t ypstart, ypend;
  
  vector<string> slist, vlist;
  bool tofindend = true;
  while(tofindend) {
    int nw = getwords(slist, fin);
    if(nw == 0) continue;
    if(slist[0] == "bond") {

      ia = mySpecies.addSpecies(slist[1].c_str());
      ja = mySpecies.addSpecies(slist[2].c_str());

      // create a SK block for AB bond
      ib = SK_h.size();
      SK_h.push_back(new SlaterKosterComp<Scalar_t>);

      // assign unphysical L1 and L2
      L1 = -1; L2 = -1;
      bond(ia,ja) = ib;
    } else if(isdigit(slist[0][0])) { 
      l1 = atoi(slist[0].c_str());
      l2 = atoi(slist[1].c_str());
      m  = atoi(slist[2].c_str());
      lt1 = atoi(slist[3].c_str());
      lt2 = atoi(slist[4].c_str());

      if(m < 0) {
	eps = atof(slist[5].c_str());
      } else {
	eps = 0.0;
	npts = atoi(slist[5].c_str());
	ypstart = atof(slist[6].c_str());
	ypend = atof(slist[7].c_str());
      }

      if(l1 != L1 || l2 != L2) {// new L1-L2 pair is added
	vloc = V.size();
	V.push_back(new vector<ScaleFunction_t>);
	SK_h[ib]->add(
		      SKBlock<Scalar_t,ScaleFunction_t>
		      ::make(l1,l2,ia,ja,lt1,lt2,eps,V[vloc]));
      }

      L1 = l1; L2 = l2;
      if(m>=0) {
	for(int ipts=0; ipts<npts; ipts++)  {
	  getwords(vlist,fin);
	  x[ipts] = atof(vlist[0].c_str());
	  y[ipts] = atof(vlist[1].c_str());
	}
	SK_h[ib]->setRcut(x[npts-1]);
	//V[vloc]->operator[](m).spline(npts,x.begin(),y.begin(),ypstart,ypend);
	V[vloc]->operator[](m).spline(npts,&(x[0]),&(y[0]),ypstart,ypend);
      }
    } else if(slist[0] == "</TBPARAM>")
      {
	tofindend = false; 
      }
  } while(tofindend);

  ///////////////////////////////////
  // repulsive
  ///////////////////////////////////
  do {
   getwords(slist, fin);
  } while(!slist[0].find("rep"));

  //////////////////////////////////////
  //create repulsive potential
  //////////////////////////////////////
  typedef CubicSpline<Scalar_t> RepFuncType;

  RepFuncType *phi = new RepFuncType;
  getwords(slist, fin);
  npts = atoi(slist[0].c_str());
  ypstart = atof(slist[1].c_str());
  ypend =   atof(slist[2].c_str());

  for(int ipts=0; ipts<npts; ipts++)  {
    getwords(slist,fin);
    x[ipts] = atof(slist[0].c_str());
    y[ipts] = atof(slist[1].c_str());
  }
  phi->spline(npts,&(x[0]),&(y[0]),ypstart,ypend);

  GenericTwoBodySRP<RepFuncType> *repPot = new GenericTwoBodySRP<RepFuncType>;
  repPot->setParticleSet(myPtcl);
  repPot->addPairPotential(0,0,phi);
  repPot->Rmax=x[npts-1];

  return repPot;
}
  
// Scalar_t 
// LenoskyTB::e(const ParticlePos_t& R, int i, int myID) {

//   const PtclPairList_t& nnlist = myPtcl->nnListEngine->getPairAttrib(myID);

//   Scalar_t sum = 0.0e0;
//   for(int jL=0; jL< nnlist.nadj(i); ++jL) {
//     int j = nnlist.iadj(i,jL);  //atomic index of jL-th nn of i
//     int loc = nnlist.loc(i,jL); //location of R, dR of the pair
//     if(nnlist.R[loc] < Rmax) {
//       sum += Phi(nnlist.R[loc]);// 0.5 factory is not used here
//     }
//   }
//   return sum;
// }

// Scalar_t 
// LenoskyTB::evalEnergy() {
//   Scalar_t esum = 0.0;
// #pragma omp parallel reduction(+:esum)
//   {
//     int myID = omp_get_thread_num();
//     esum += evalEnergyNN(myPtcl->nnListEngine->getPairAttrib(myID));
//   }
//   return esum;
// }

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 330 $   $Date: 2005-05-26 17:44:25 -0500 (Thu, 26 May 2005) $
 * $Id: LenoskyTB.cpp 330 2005-05-26 22:44:25Z jnkim $ 
 ***************************************************************************/
