//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
///////////////////////////////////////////////////////////////////////
// * This class implements the orthogonal and non-orthogonal 
// tight-binding model using NRL scale functions.
/////////////////////////////////////////////////////////////////////
#ifndef OHMMS_SLATERKOSTERTB_NRL_H
#define OHMMS_SLATERKOSTERTB_NRL_H

#include "SlaterKosterTB/TBModelBase.h"
#include "ScaleFunctors/NRLScaleFunction.h"
#include "SlaterKoster/SlaterKosterOnsite.h"

namespace OHMMS {

  class NRLTB: public TBModelBase {
    
  public:

    NRLTB();
    ~NRLTB();
    
    //////////////////////////////////////////////
    // virtual function of TBModelBase, will return NULL
    //////////////////////////////////////////////
    ShortRangePotential* 
    init(SKList_t& SK_h, SKList_t& SK_s, Matrix<int>& bond, istream& fin);
    
    void printSpline(ostream&);
    
  private:
    
    typedef NRLScaleFunction<Scalar_t>  ScaleFunction_t;
    typedef NRLScaleFunction2<Scalar_t> ScaleFunction2_t;
    typedef NRLSplineScale<Scalar_t>    TiScaleFunction_t;
    typedef NRLSplineScale2<Scalar_t>   TiScaleFunction2_t;


    vector<vector<ScaleFunction_t>* >  V;
    vector<vector<ScaleFunction_t>* >  Vs_old;
    vector<vector<ScaleFunction2_t>* > Vs_new;
    vector<vector<TiScaleFunction_t>* > Vh,Vs;
    vector<vector<TiScaleFunction2_t>* > Vs2;
  };
  
  
  class NRLTBOnsite: public TBOnsiteBase {

  public:

    NRLTBOnsite();
    ~NRLTBOnsite();

    bool get(ostream&) const;
    bool put(istream&);
    void reset();
    bool put(xmlNodePtr cur);

    Scalar_t evalForceNN(PtclPairList_t& nnlist, ParticlePos_t& f0, 
        unsigned stressmode) const;

    Scalar_t evalEnergyNN(const PtclPairList_t& nnlist,Scalar_t rscale=1.0) const{
      return 0.0;
    }

    void diagonal(SparseMatrix_t& hmat, const PtclPairList_t& nnlist);

  private:

    static const int MAX_NRLSPECIES=2;

    int NumSpecies;
    Matrix<int> BondMap;
    vector<NRLDensity<Scalar_t> > Fc;    // cutoff function
    typedef NRLOnsite<Scalar_t>  OnSiteFunction_t;
    typedef SlaterKosterOnsiteComp<Scalar_t,OnSiteFunction_t> SKOnSite_t;
    vector<SKOnSite_t* > OnSite;
  };
}
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 330 $   $Date: 2005-05-26 17:44:25 -0500 (Thu, 26 May 2005) $
 * $Id: NRLTB.h 330 2005-05-26 22:44:25Z jnkim $ 
 ***************************************************************************/
