//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
////////////////////////////////////////////////////////////////////////////
// NRL potential for Transition Metals
// Mike Mehl and Dimitri Papaconstantopoulus
// PRB xx
//
// Modified form of hopping terms, to use a smooth spline to a constant
//   value (see SplineScaleFunction.h for details and implementation).
//
// Implemented by D. Trinkle, April 10, 2001.
// Modified by Jeongnim Kim, May 24, 2002
// Modified by D. Trinkle January 19, 2003 -- added quadratic to 
//   SplineScaleFunction, and automagically detect according to rmin value
// Modified by D. Trinkle April 7, 2003 -- added spline functionality
//   to work with new overlap form, and automagically detect.
//   Definition of inmode values:
//   0: Hamil. and Overlap are quadratic * exponential
//   1: Hamil. is same as 0, Overlap is (delta_ll' + r*quadratic)*exponential
//   2: Same as 0 down to Rmin, then cubic spline takes over
//   3: Same as 1 down to Rmin, then cubic spline takes over
//
// For binary potentials, using identical r0, sigma and lambda does not make sense.
////////////////////////////////////////////////////////////////////////////

#include "SlaterKosterTB/Models/NRLTB.h"
#include "SlaterKoster/SlaterKosterMake.h"
#include "Utilities/SpeciesCollection.h"
#include "Utilities/SimpleParser.h"
PotentialBase* 
NRLTB::put(SKArray_t& SK, SKArray_t& SK_O, Matrix<int>& bond, xmlNodePtr cur) {

  SpeciesBase* mySpecies = SpeciesCollection::getSpecies();
  int irmax = SpeciesCollection::addAttrib("rmax");
  int Lmax = 0;

  bool hasoverlap = true;
  string nrl; 
  if(xmlHasProp(cur,(const xmlChar *)"format")) {
    nrl = (const char*)(xmlGetProp(cur,(const xmlChar *)"name"));
  }
  if(nrl[5]== '1') hasoverlap = false;
  int inmode = atoi(nrl[6]);

  int ib, ia, ja, L1, L2;
  int l1, l2, lt1, lt2, m;
  T rcut, screen, eps = 0.0;
  T h_sigma = 1.0, h_rmin = 0.0, s_sigma = 1.0, s_rmin = 0.0;
  T h0, h1, h2, h3;
  T s0, s1, s2, s3;

  /////////////////////////////////////////////////
  // until NRL potential is extended to more than 2 species problem
  // Other possibility is that when NRLTB region is embbeded within 
  // other Potential region
  /////////////////////////////////////////////////
  const int maxspecies = bond.rows();
  bool hasoverlap = true;
  int inmode;

  map<string,double> param;
  param["rcut"]   = 0.0;
  param["screen"] = 0.0;
  param["h_rmin"] = 0.0;
  param["h_sigma"] = 0.0; 
  param["s_rmin"] = 0.0;
  param["s_sigma"] = 0.0; 

  cur = cur->xmlChildrenNode;
  while(cur != NULL) {
    if ((!xmlStrcmp(cur->name, (const xmlChar *)"Bond"))) { 
      int ib = SK.size();
      int ia 
	= SpeciesCollection::addSpecies((const char*)(xmlGetProp(cur,(consc xmlChar*)"A")));
      int ja 
	= SpeciesCollection::addSpecies((const char*)(xmlGetProp(cur,(consc xmlChar*)"B")));
      bond(ia,ja) = ib;

      SK.push_back(new SlaterKosterComp<T>);
      if(hasoverlap) SK_O.push_back(new SlaterKosterComp<T>);

      xmlNodePtr bcur = cur->xmlChildrenNode; 
      while(bcur != NULL) {
	///initialize parameter
	if ((!xmlStrcmp(bcur->name, (const xmlChar *)"parameter"))) {
	  string aname = (const char*)xmlGetProp(bcur,(const xmlChar *)"name");
	  putContent(param[aname],bcur);
	}
	if ((!xmlStrcmp(bcur->name, (const xmlChar *)"SK"))) {

	}
      }
    }
    cur = cur->next;
  }
  vector<string> slist, vlist;
  getwords(slist,fin);
  if(slist[0][5]== '1') hasoverlap = false;
  inmode = atoi(slist[0].c_str()+6);
  WARNMSG("\tNRLTB mode = " << inmode)

 Bool tofindend = true;
  int vloc = -1, sloc;
  do {
    int nw =getwords(slist, fin);
    if(nw == 0) continue;
    if(slist[0] == "bond") {
      ib = SK.size();
      ia = SpeciesCollection::addSpecies(slist[1].c_str());
      ja = SpeciesCollection::addSpecies(slist[2].c_str());
      bond(ia,ja) = ib;

      // create SK blocks for this bond
      SK.push_back(new SlaterKosterComp<T>);
      if(hasoverlap) 
	SK_O.push_back(new SlaterKosterComp<T>);

      // assign unphysical L1 and L2
      L1 = -1; L2 = -1;

    } else if(slist[0] == "rcut") {
      rcut = atof(slist[1].c_str());
    } else if(slist[0] == "screenl") {
      screen = atof(slist[1].c_str());
    } else if(slist[0] == "h_rmin") {
      h_rmin = atof(slist[1].c_str());
    } else if(slist[0] == "h_sigma") {
      h_sigma = atof(slist[1].c_str());
    } else if(slist[0] == "s_rmin") {
      s_rmin = atof(slist[1].c_str());
    } else if(slist[0] == "s_sigma") {
      S_sigma = atof(slist[1].c_str());
    } else if(isdigit(slist[0][0])) { 
      l1 = atoi(slist[0].c_str());
      l2 = atoi(slist[1].c_str());
      m  = atoi(slist[2].c_str());
      lt1 = atoi(slist[3].c_str());
      lt2 = atoi(slist[4].c_str());
      if(m<0) { // onsite matrix
        eps = atof(slist[5].c_str());
      } else {
        eps = 0.0;
	getwords(vlist,fin);
	h0  = atof(vlist[0].c_str());
	h1  = atof(vlist[1].c_str());
	h2  = atof(vlist[2].c_str());
	h3  = atof(vlist[3].c_str());
	if(hasoverlap) {
	  getwords(vlist,fin);
	  s0 = atof(vlist[0].c_str());
	  s1 = atof(vlist[1].c_str());
	  s2 = atof(vlist[2].c_str());
	  s3 = atof(vlist[3].c_str());
	}
      }
      // Automagically reset the type of scaling function if Rmin is
      // set for *both* hamiltonian and overlap (or just hamil. if
      // no overlap is present) --DRT
      if (h_rmin != 0.0)
	if ( (!hasoverlap) || (s_rmin != 0.0) )
	  inmode = (inmode | 2);

      if(l1 != L1 || l2 != L2) {// new L1-L2 pair is added
	//cout << "eps " << eps << endl;
	//will create [0,min(l,l')] function blocks
	// first add a new scaling function for l-l' pair
	if(inmode | 2) {// NRLscale + spline (Trinkle et atl)
	  vloc = Vh.size();
	  Vh.push_back(new vector<TiScaleFunction_t>);
	  SK[ib]->add(SKBlock<T,TiScaleFunction_t>
		      ::make(l1,l2,ia,ja,lt1,lt2,eps,Vh[vloc]));//
	} else {
	  vloc = V.size();
	  V.push_back(new vector<ScaleFunction_t>);
	  SK[ib]->add(SKBlock<T,ScaleFunction_t>
		      ::make(l1,l2,ia,ja,lt1,lt2,eps,V[vloc]));
	}

	SK[ib]->setRcut(rcut);
	
	if(hasoverlap) {// make SK for the overlap 
         switch (inmode) {
          case 0: // old overlap, no spline
            sloc = Vs_old.size();
            Vs_old.push_back(new vector<ScaleFunction_t>);
            SK_O[ib]->add(SKBlock<T,ScaleFunction_t>
                          ::make(l1,l2,ia,ja,lt1,lt2,1.0,Vs_old[sloc]));
            break;
          case 1: // new overlap, no spline
            if(ia == ja) {// for A-A bond relation
              sloc = Vs_new.size();
              Vs_new.push_back(new vector<ScaleFunction2_t>);
              SK_O[ib]->add(SKBlock<T,ScaleFunction2_t>
                            ::make(l1,l2,ia,ja,lt1,lt2,1.0,Vs_new[sloc]));
            } else { // for A-B bond, using the old format
              sloc = Vs_old.size();
              Vs_old.push_back(new vector<ScaleFunction_t>);
              SK_O[ib]->add(SKBlock<T,ScaleFunction_t>
                            ::make(l1,l2,ia,ja,lt1,lt2,1.0,Vs_old[sloc]));
            }
            break;
          case 2: // old overlap + spline (Trinkle et al)
            sloc = Vs.size();
            Vs.push_back(new vector<TiScaleFunction_t>);
            SK_O[ib]->add(SKBlock<T,TiScaleFunction_t>
                          ::make(l1,l2,ia,ja,lt1,lt2,1.0,Vs[sloc]));
            break;
          case 3: // new overlap + spline (Trinkle et al)
            if(ia == ja) {// for A-A bond relation
              sloc = Vs2.size();
              Vs2.push_back(new vector<TiScaleFunction2_t>);
              SK_O[ib]->add(SKBlock<T,TiScaleFunction2_t>
                            ::make(l1,l2,ia,ja,lt1,lt2,1.0,Vs2[sloc]));
            } else { // for A-B bond, using the old format
              sloc = Vs.size();
              Vs.push_back(new vector<TiScaleFunction_t>);
              SK_O[ib]->add(SKBlock<T,TiScaleFunction_t>
                          ::make(l1,l2,ia,ja,lt1,lt2,1.0,Vs[sloc]));
            }
            break;
          default: // Scream bloody murder
            cerr << "Bad value of inmode = " << inmode << endl;
          }
          SK_O[ib]->setRcut(rcut);
	} // overlap-matrix
      }

      L1 = l1; L2 = l2;
      if(m>=0) {
	if(inmode | 2) {
	  // Added h2 to SplineScale --DRT
	  Vh[vloc]->operator[](m).set(h0,h1,h2,h3,rcut,screen,h_rmin,h_sigma);
	} else {
	  V[vloc]->operator[](m).set(h0,h1,h2,h3,rcut,screen);
	}
	
	if(hasoverlap) {
          switch (inmode) {
          case 0: // old overlap, no spline
            Vs_old[sloc]->operator[](m).set(s0,s1,s2,s3,rcut,screen);
            break;
          case 1: // new overlap, no spline
            if(ia == ja) {
              T td = 0.0;
              if(l1 == l2) td = 1.0;
              Vs_new[sloc]->operator[](m).set(td,s0,s1,s2,s3, rcut,screen);
            } else {
              Vs_old[sloc]->operator[](m).set(s0,s1,s2,s3, rcut,screen);
            }
            break;
          case 2: // old overlap + spline
            Vs[sloc]->operator[](m).set(s0,s1,s2,s3,rcut,screen,
                                        s_rmin,s_sigma);
            break;
          case 3: // new overlap + spline
            if(ia == ja) {
              T td = 0.0;
              if(l1 == l2) td = 1.0;
              Vs2[sloc]->operator[](m).set(td,s0,s1,s2,s3,rcut,screen,
                                           s_rmin,s_sigma);
            } else {
              Vs[sloc]->operator[](m).set(s0,s1,s2,s3,rcut,screen,
                                          s_rmin,s_sigma);
            }
            break;
          }
	}
      }
    } else if(slist[0] == "</TBPARAM>")  {
      tofindend = false;
    }
  } while(tofindend);
}
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: NRLTBLibxmlReader.cpp 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
