//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
///////////////////////////////////////////////////////////////////////
// TBModelBase is an abstract class from which a model TB is derived.
//  pure virtial functions should be implemneted by a derived model.
//  returns the maximum angular momentum.
//  virtual int initialize(SKArray_t&)  =0;
//  virtual int initialize(SKArray_t&, SKArray_t&)  =0;
/////////////////////////////////////////////////////////////////////
#ifndef OHMMS_SLATERKOSTERTB_ZINCBLENDSURFACE_H
#define OHMMS_SLATERKOSTERTB_ZINCBLENDSURFACE_H

#include "SlaterKosterTB/TBModelBase.h"
#include "OhmmsPETE/TinyMatrix.h"

namespace OHMMS {
  /*!\class ZincBlendSurface
   *\author Jeongnim Kim
   *\breif Implements surface termination method to handle dangling bonds 
   * of structures based on ZincBlend bulk system.
   * Can be used with any tight-binding potential for ZB structure.
   */
  struct ZincBlendSurface: public TBOnsiteBase {

    ZincBlendSurface() { }
    ~ZincBlendSurface() { }

    void initialize();
    bool get(ostream&) const;
    bool put(istream&);
    void reset();
    bool put(xmlNodePtr cur);

    Scalar_t evalForceNN(PtclPairList_t& nnlist,
                         ParticlePos_t& f0,
                         unsigned stressmode) const { return 0.0;}
                                                                                                         
    Scalar_t evalEnergyNN(const PtclPairList_t& nnlist, Scalar_t rscale=1.0) const { return 0.0;}
                                                                                                         
    void diagonal(SparseMatrix_t& hmat, const PtclPairList_t& nnlist);

    int i00, i11, i22, i33, i44;
    int i01, i10, i23, i32;
    int i02, i13, i20, i31;
    int i03, i12, i21, i30;
    vector<Scalar_t> surfShift, ssShift;

    //Really temporary
    struct OnsiteMixtureType {
      vector<Scalar_t> DeltaX, DeltaY;
      int SpeciesX;
      int SpeciesY;
      OnsiteMixtureType():SpeciesX(-1), SpeciesY(-1) { }
      OnsiteMixtureType(const OnsiteMixtureType& c) { 
	DeltaX = c.DeltaY;
	SpeciesX = c.SpeciesX;
	SpeciesY = c.SpeciesY;
      }
      OnsiteMixtureType(int n): SpeciesX(-1), SpeciesY(-1) {
        DeltaX.resize(n,0.0);
        DeltaY.resize(n,0.0);
      }

      void setShift(int l, int loc, int x, int y, 
		    Scalar_t delx, Scalar_t dely);
      void add(Scalar_t* onsite, Scalar_t mix);
      void print(ostream& os);
    };

    vector<OnsiteMixtureType*>  diagMix;

#if defined(FIXEDTBMETHOD)
    vector<TinyMatrix<Scalar_t,TB_BLOCK,TB_BLOCK> > DiagBlocks, OffBlocks;
#endif
  };
}
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 330 $   $Date: 2005-05-26 17:44:25 -0500 (Thu, 26 May 2005) $
 * $Id: ZincBlendSurface.h 330 2005-05-26 22:44:25Z jnkim $ 
 ***************************************************************************/

