//////////////////////////////////////////////////////////////////
// (c) Copyright 2003- by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "SlaterKosterTB/SlaterKosterTB.h"
#include "SlaterKosterTB/SKTBLibxmlReader.h"
#include "OhmmsData/DOMProcessor.h"

using namespace OHMMS;
bool SKTBLibxmlReader::parse(const char* fname) {

  DOMProcessor reader;
  if(!reader.parse(fname.c_str())) {
    ERRORMSG("Failed to read TB parameter file")
    return false;
  }

  // make private references to Species
  SpeciesBase* mySpecies = SpeciesCollection::getSpecies();

  // make private references to Species
  int icharge = SpeciesCollection::addAttrib("q");
  int ins     = SpeciesCollection::addAttrib("ns");
  int imass   = SpeciesCollection::addAttrib("mass");
  int irmax   = SpeciesCollection::addAttrib("rmax");
  int Lmax = 0;
  
  LOGMSG("TB input files are in xml format")
    //check the species first
    int nspecies = 0; 
  xmlXPathObjectPtr result = reader.get("//Species");
  if(!xmlXPathNodeSetIsEmpty(result->nodesetval)) {
    nspecies = result->nodesetval->nodeNr;
    for(int is=0; is<nspecies; is++) {
      cur = result->nodesetval->nodeTab[is];
      string a((const char*)(xmlGetProp(cur,(const xmlChar *) "name")));
      int newid =  SpeciesCollection::addSpecies(a.c_str());
      cur = cur->xmlChildrenNode;
      while(cur != NULL) {
	if ((!xmlStrcmp(cur->name, (const xmlChar *)"Property"))) {
	  if(!xmlStrcmp(xmlGetProp(cur,(const xmlChar *)"name"),(const xmlChar*)"mass")) {
	    putContent((*mySpecies)(imass,newid),cur);
	  }
	  if(!xmlStrcmp(xmlGetProp(cur,(const xmlChar *)"name"),(const xmlChar*)"valence")) {
	    putContent((*mySpecies)(icharge,newid),cur);
	  }
	}	
	cur = cur->next;
      }
    }
  }
  xmlXPathFreeObject(result);
  
  TBBasisSetType skblock;
  result = reader.get("//Basis");
  if(!xmlXPathNodeSetIsEmpty(result->nodesetval)) {
    for(int isk=0; isk<result->nodesetval->nodeNr; isk++) {
      cur = result->nodesetval->nodeTab[isk];
      int ns = 0;
      int newid
	= SpeciesCollection::addSpecies((const char*)xmlGetProp(cur,(const xmlChar *)"species"));
      cur = cur->xmlChildrenNode;
      while(cur != NULL) {
	if ((!xmlStrcmp(cur->name, (const xmlChar *)"Rnl"))) {
	  int l = atoi((const char*)xmlGetProp(cur,(const xmlChar *)"l"));
	  int m = 0;
	  if((const char*)xmlHasProp(cur,(const xmlChar *)"m")) {
	    m = atoi((const char*)xmlGetProp(cur,(const xmlChar *)"m"));
	  }
	  Lmax = max(l,Lmax);
	  skblock[(const char*)xmlGetProp(cur,(const xmlChar *)"id")] = TinyVector<int,3>(l,m,ns);
	  ns += (2*l+1);
	}
	cur = cur->next;
      }
	(*mySpecies)(ins,newid)=ns;
    }
  }
  xmlXPathFreeObject(result);
  
  map<string,TinyVector<int,3> >::iterator it = skblock.begin();
  while(it != skblock.end()) {
    cout << (*it).first << " " << (*it).second << endl;it++;
  }
    
  result = reader.get("//onsite");
  if(!xmlXPathNodeSetIsEmpty(result->nodesetval)) {
    cur = result->nodesetval->nodeTab[0];
    string oname;
    if(xmlHasProp(cur,(const xmlChar *)"name")) {
      oname = (const char*)(xmlGetProp(cur,(const xmlChar *)"name"));
    }
    LOGMSG("Creating TB-onsite term " << oname)
    if(oname == "nrl") {
      NRLTBOnsite *aon = new NRLTBOnsite;
      DOMNode<NRLTBOnsite>  a(*aon,skblock);
      a.put(cur);
      Onsite = aon;
    }
  }
  xmlXPathFreeObject(result);
    
  /////////////////////////////////
  // bond information is reset, whenver new species are added
  /////////////////////////////////
  Bond.resize(nspecies, nspecies);
  Bond = -1; // no bond is connected

  //   result = reader.get("//onsite");
  //   if(!xmlXPathNodeSetIsEmpty(result->nodesetval)) {
  //     Onsite = TBOnsiteFactory::init(skblock, result->nodesetval->nodeTab[0]);
  //   }   
  //   xmlXPathFreeObject(result);
  
  //   PotentialBase* rep =NULL;
  //   result = reader.get("//TBPARAM");
  //   if(!xmlXPathNodeSetIsEmpty(result->nodesetval)) {
  //     rep= TBModelFactory::init(SK_h, SK_s, Bond, result->nodesetval->nodeTab[0]);
  //   }   
  //   xmlXPathFreeObject(result);
  if(SK_s.empty())  Identity = true;
  else              Identity = false;

  for(int ia=0; ia<nspecies; ia++) {
    for(int ja=0; ja<nspecies; ja++) {
      int ib = Bond(ia,ja);
      if(ib<0) continue;
      SK_h[ib]->reset(static_cast<int>((*mySpecies)(ins,ia)), 
		      static_cast<int>((*mySpecies)(ins,ja)), Lmax);
      if(!Identity) {
	SK_s[ib]->reset(static_cast<int>((*mySpecies)(ins,ia)), 
			static_cast<int>((*mySpecies)(ins,ja)), Lmax);
      }
    }
  }

  Rmax = 0.0e0;
  for(int ib=0; ib<SK_h.size(); ib++) {
    Rmax = max(Rmax, SK_h[ib]->rcut());
  }
  for(int ib=0; ib<SK_s.size(); ib++) {
    Rmax = max(Rmax, SK_s[ib]->rcut());
  }

  ref_.MaxAngMomentum = Lmax;
}
