//////////////////////////////////////////////////////////////////
//                        COPYRIGHT NOTICE
//   (c) Copyright 2003- Jeongnim Kim.
//   All rights reserved.
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "Message/Communicate.h"
#include "Particle/NNEngine.h"
#include "OhmmsNumeric/SparseEigenSolver.h"
#include "SparseMatrix/Matrix2SparseMatrixProduct.h"
#include "SparseMatrix/BlockMatrixFunctions.h"
#include "SlaterKosterTB/SlaterKosterTBGamma.h"
using namespace OHMMS;

SlaterKosterTBGamma::Scalar_t
SlaterKosterTBGamma::el_ph_coupling(const vector<int>& psi_i,  
                                    const PhononMode& phonon,
				    vector<int>& psi_f, Scalar_t del) {
// Scalar_t 
// SlaterKosterTBGamma::tbVep(const vector<int>& hband, 
// 			   const ParticlePos_t* pmode){

  //Scalar_t Vep = 0.0;

  const ParticleIndex_t& grID(myPtcl->GroupID);

  NNEngine *nnListEngine = myPtcl->getNNEngine();
  for(int myID=0; myID<nnListEngine->size(); myID++)  {

    typedef PtclPairList_t::PairData_t PairData_t;
    const SparseIndex_t& hIndex(*(Hmat.getIndex()));

    // construct electron-phonon coupling Hamiltonian
    Hmat.clear(0);
    if(!Identity) Smat.clear(0);

    const PtclPairList_t& nnlist = nnListEngine->getPairAttrib(myID);

    int maxbasis = hIndex.MaxRowBlock;
    Matrix<Scalar_t> 
      Tx(maxbasis,maxbasis), Ty(maxbasis,maxbasis), Tz(maxbasis,maxbasis), 
      Sx(maxbasis,maxbasis), Sy(maxbasis,maxbasis), Sz(maxbasis,maxbasis), 
      dT(maxbasis,maxbasis);
    
    int nnh = 0;  
    int ig = hIndex.first_diag(myID); // index for the sparse matrix
    bool ablock;    
    int iat(nnListEngine->first_id(myID));
    int nptcl(nnListEngine->last_id(myID)-iat);
    for(int iL=0; iL< nptcl; iL++, iat++, ig++) {
      int ishift(hIndex.row_loc(iat));
      PtclPairList_t::const_iterator nit(nnlist.begin(iL)),nit_end(nnlist.end(iL));
      int nni(0);
      while(nit != nit_end) {

	// logical index (alternatively, sparse matrix index)
	int jat(hIndex.iadj(iL, nni, myID)); 
	int jshift(hIndex.col_loc(jat));
        
	Scalar_t dpx = phonon.Z[iat][0] - phonon.Z[jat][0];
	Scalar_t dpy = phonon.Z[iat][1] - phonon.Z[jat][1];
	Scalar_t dpz = phonon.Z[iat][2] - phonon.Z[jat][2];

	int ij = SKP.pair_id(grID[iat],grID[jat]);

	if(ij>=0) {
    
          const PairData_t& cur(*nit);

	  Tx = 0.0; Ty = 0.0; Tz = 0.0;
	  if(Identity) {
	    ablock = SKP.getSKBlocks(ij,cur.sep, cur.displ,
	  			   Tx.data(), Ty.data(), Tz.data());
	  } else {
	    Sx = 0.0;  Sy = 0.0; Sz = 0.0;
	    ablock = SKP.getSKBlocks(ij,cur.sep, cur.displ,
	  			   Tx.data(), Ty.data(), Tz.data(),
	  			   Sx.data(), Sy.data(), Sz.data());
	  }
	  if(!ablock) continue;
          //int ij = Bond(grID[iG],grID[jG]);
          //Scalar_t sep = nnlist.R[nnh];
          //if( sep > SK_h[ij]->rcut() ) continue;
          //SingleParticlePos_t dR = nnlist.dR[nnh];
          //getPD(dR, pvec, pd, SK_h[ij]->maxL());
          //SK_h[ij]->getDeriv(sep, pvec, pd, Tx.data(), Ty.data(), Tz.data());
	
	  int nblock = hIndex.row_block(iat)*hIndex.row_block(jat);
	  dT = dpx*Tx + dpy*Ty + dpz*Tz;
	  copyadd(dT.data(),Hmat.offdiag(myID, nnh),nblock);
	  if(!Identity) {
	    dT = dpx*Sx + dpy*Sy + dpz*Sz;	
	    copyadd(dT.data(), Smat.offdiag(myID, nnh),nblock);
	  }
        }
        ++nit; ++nnh; ++nni;
      }
    }

  }
  Scalar_t eps = 0.0;
  for(int ih=0; ih<psi_i.size(); ih++) {
    eps += E[0][psi_i[ih]];
  }
  eps /= -static_cast<Scalar_t>(psi_i.size());

  RealMatrix_t HPsi(psi_i.size(), BasisDim), SPsi;
  for(int ih=0; ih<psi_i.size(); ih++) {
    Hmat.apply(thisSolver->Psi.data()+psi_i[ih]*BasisDim, HPsi.data()+ih*BasisDim);
    //Hmat.apply(Psi.data()+psi_i[ih]*BasisDim, HPsi.data()+ih*BasisDim);
  }

  if(!Identity) {
    SPsi.resize(psi_i.size(),BasisDim);
    for(int ih=0; ih<psi_i.size(); ih++) {
    Smat.apply(thisSolver->Psi.data()+psi_i[ih]*BasisDim, SPsi.data()+ih*BasisDim);
    //Smat.apply(Psi.data()+psi_i[ih]*BasisDim, SPsi.data()+ih*BasisDim);
    }
    HPsi += eps*SPsi;
  }

  Scalar_t DeltaE = (del<0.0)? phonon.E : del;

  if(DeltaE <1e-12) {
    psi_f = psi_i;
  } else {
    int bandmin =psi_i.back()+1;
    psi_f.reserve(BasisDim-psi_i.back());
    psi_f.erase(psi_f.begin(), psi_f.end());
    while(fabs(E(0,bandmin)-eps)<Sigma) {
      psi_f.push_back(bandmin); bandmin++;
    }
  }

  RealMatrix_t Vep; 
  if(psi_f.empty()) {
    Vep.resize(1,1); Vep = 0.0;
  } else {
    Vep.resize(psi_f.size(),psi_i.size());
    for(int ih=0; ih<psi_f.size(); ih++) {
      for(int jh=0; jh<psi_i.size(); jh++) {
	Vep(ih,jh) 
	  = OhmmsBlas::dot(BasisDim,thisSolver->Psi.data()+psi_f[ih]*BasisDim,
			   HPsi.data()+jh*BasisDim);
      }
    }
  }

  return OhmmsBlas::dot(Vep.size(), Vep.data(), Vep.data());
//       //Vep = sum_ij |<psi_i|dV|psi_j>|^2
//       for(int ih=0; ih<hband.size(); ih++) {
//         //int jh = ih;
// 	const Scalar_t* wfs_i = Psi.data()+hband[ih]*BasisDim;
//  	for(int jh=0; jh<hband.size(); jh++) {
// 	  const Scalar_t* wfs_j = Psi.data()+hband[jh]*BasisDim;
// 	  Scalar_t sumij = 0.0;
// 	  //iG = myIndex.first_diag(myID);
// 	  nnh = 0;
// 	  //<psi_i| dV | psi_j>
// 	  for(int iL=0; iL<hIndex.num_diag(myID); iL++) {
// 	    int iG = nnlist.I[iL];
// 	    for(int nni=0; nni< hIndex.nadj(iL, myID); nni++, nnh++){
// 	      // logical index (alternatively, sparse matrix index)
// 	      int jg = hIndex.iadj(iL, nni, myID); 
// 	      int jG = myPtcl->ID[jg];
// 	      sumij +=
// 		MakeTrace<Scalar_t,TB_BLOCK>::apply(wfs_i+hIndex.row_loc(ig), 
// 						    Hmat.offdiag(myID,nnh),
// 						    wfs_j+hIndex.col_loc(jg),
// 						    hIndex.row_block(ig), 
// 						    hIndex.col_block(jg));
//               if(!Identity) {
// 		sumij +=
// 		  eps*MakeTrace<Scalar_t,TB_BLOCK>
// 		  ::apply(wfs_i+hIndex.row_loc(ig), Smat.offdiag(myID,nnh),
// 			  wfs_j+hIndex.col_loc(jg),
// 			  hIndex.row_block(ig), hIndex.col_block(jg));

// 	      }
// 	    }
// 	  }
// 	  //////////////////////////
// 	  // to work with open mp, sumij has to be collected before | |^2
// 	  // vector<Scalar_t> sumij(hband.size(),hband.size());
// 	  // reduction(+:sumij) ???????????? (possible with guidec++ 4.0)
// 	  //////////////////////////
//           Vep += sumij*sumij;
// 	  /////////////////////////
// 	}
//       }
//     }
//   }
  ///////////////////
  // int ii=0;
  // for(int ih=0; ih<hband.size(); ih++)
  //   for(int jh=0; jh<hband.size(); jh++, ii++)
  //      Vep += sumij[ii]*sumij[ii];
  //////////////////////////////
//   return Vep;

///Partial density matrix. Error exists
//    if(pmode) {
//      // construct electron-phonon coupling Hamiltonian
//      H->reset(0);
//      if(!Identity) S->reset(0);
//      const PtclPairList_t& nnlist = nnListEngine->getPairAttrib()[myID];
//      Scalar_t pvec[8], pd[8][3];
//      int maxbasis = myIndex.MaxRowBlock;
//      Matrix<Scalar_t> Tx(maxbasis,maxbasis), Ty(maxbasis,maxbasis), 
//        Tz(maxbasis,maxbasis), dT(maxbasis,maxbasis);
//      int iG = myIndex.first_diag(myID);
//      int nn = 0;  

//      for(int iL=0; iL<myIndex.num_diag(myID); iL++, iG++) {

//        int ishift = myIndex.row_loc(iG);
//        for(int nni=0; nni< myIndex.nadj(iL, myID); nni++, nn++){

//  	int jG = myIndex.iadj(iL, nni, myID);
//  	int jshift = myIndex.col_loc(jG);
        
//  	Scalar_t dpx = (*pmode)[iG][0] - (*pmode)[jG][0];
//  	Scalar_t dpy = (*pmode)[iG][1] - (*pmode)[jG][1];
//  	Scalar_t dpz = (*pmode)[iG][2] - (*pmode)[jG][2];

//  	int ij = Bond(myPtcl->GroupID[iG],myPtcl->GroupID[jG]);
//  	Scalar_t sep = nnlist.R[nn];
//  	if( sep > SK[ij]->rcut() ) continue;

//  	SingleParticlePos_t dR = nnlist.dR[nn];
//  #ifdef TB_BLOCK
//  	const int nblock = TB_BLOCK*TB_BLOCK;
//  #else
//  	int nblock = myIndex.row_block(iG)*myIndex.row_block(jG);
//  #endif

//  	Tx = 0.0; Ty = 0.0; Tz = 0.0;
//  	getPD(dR, pvec, pd, SK[ij]->maxL());
//  	SK[ij]->getDeriv(sep, pvec, pd, Tx.begin(), Ty.begin(), Tz.begin());

//  	dT = dpx*Tx + dpy*Ty + dpz*Tz;
//  	copyadd(dT.begin(), H->offdiag(myID, nn),nblock);
//  	if(!Identity) {
//  	  Tx = 0.0; Ty = 0.0; Tz = 0.0;
//  	  SK_O[ij]->getDeriv(sep,pvec,pd,Tx.begin(),Ty.begin(),Tz.begin());
//  	  dT = dpx*Tx + dpy*Ty + dpz*Tz;	
//  	  copyadd(dT.begin(), S->offdiag(myID, nn),nblock);
//  	}
//        }
//      }

//      Scalar_t eps = 0.0;
//      for(int ih=0; ih<hband.size(); ih++) {
//        eps += eStates->E[0][hband[ih]];
//      }
//      eps /= static_cast<Scalar_t>(hband.size());

//      cout << "Average Band Energy " << eps << endl;

//      iG = myIndex.first_diag(myID);
//      nn = 0;
//      for(int iL=0; iL<myIndex.num_diag(myID); iL++, iG++) {
//        for(int nni=0; nni< myIndex.nadj(iL, myID); nni++, nn++){
//  	int jG = myIndex.iadj(iL, nni, myID);
//  #ifdef TB_BLOCK
//  	sum +=Trace<T,TB_BLOCK>::get(Rho->offdiag(myID,nn),H->offdiag(myID,nn));
//  #else
//  	sum +=Trace<T,0>::get(Rho->offdiag(myID,nn),H->offdiag(myID,nn),
//  			      myIndex.row_block(iG), myIndex.col_block(jG));
//  #endif
//  	if(!Identity) {
//  #ifdef TB_BLOCK
//  	  sum -=eps*
//  	    Trace<T,TB_BLOCK>::get(Rho->offdiag(myID,nn),S->offdiag(myID,nn));
//  #else
//  	  sum -=eps*
//  	    Trace<T,0>::get(Rho->offdiag(myID,nn),S->offdiag(myID,nn),
//  			    myIndex.row_block(iG), myIndex.col_block(jG));
//  #endif
//  	}
//        }
//      }
//    } else {
//      Rho->reset(0);
//      for(int ih=0; ih<hband.size(); ih++) {
//        for(int jh=0; jh<hband.size(); jh++) {
//          Rho->product(tbsolver->getwfs(hband[ih]), tbsolver->getwfs(hband[jh]));
//        }
//      }
//      const PtclPairList_t& nnlist = nnListEngine->getPairAttrib()[myID];
//      int iG = myIndex.first_diag(myID);
//      Scalar_t pdos = 0.0;
//      for(int iL=0; iL<myIndex.num_diag(myID); iL++, iG++) {
//  #ifdef TB_BLOCK
//        pdos += Trace<T,TB_BLOCK>::get(Rho->diag(myID, iL));
//  #else
//        int nb = myIndex.row_block(iG);
//        pdos += Trace<T,0>::get(Rho->diag(myID, iL), nb);
//  #endif
//      }
//      cout << "Trace of a partial density of states " << pdos << endl;
//     }
}

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 354 $   $Date: 2005-07-07 21:20:49 -0500 (Thu, 07 Jul 2005) $
 * $Id: SlaterKosterTBGamma.Vep.cpp 354 2005-07-08 02:20:49Z jnkim $ 
 ***************************************************************************/
