//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
///////////////////////////////////////////////////////////////////////
// TBModelBase is an abstract class from which a model TB is derived.
//  pure virtial functions should be implemneted by a derived model.
//  returns the maximum angular momentum.
//  virtual int initialize(SKArray_t&)  =0;
//  virtual int initialize(SKArray_t&, SKArray_t&)  =0;
/////////////////////////////////////////////////////////////////////
#ifndef OHMMS_SLATERKOSTERTB_MODELBASE_H
#define OHMMS_SLATERKOSTERTB_MODELBASE_H
#ifndef SLATERKOSTERARRAYBASE_H
#include "SlaterKoster/SlaterKosterBase.h"
#endif
#ifndef OHMMS_SPARSEMATRIX_H
#include "SparseMatrix/SparseMatrix.h"
#endif
#include "SlaterKosterTB/SKParameters.h"
#include "Particle/ParticleSet.h"
#include "Potentials/ShortRangePotential.h"

namespace OHMMS {

  /*!\class TBModelBase
   * \brief Base class to implement model-dependent slater-koster relations
   *
   */
  struct TBModelBase {

    typedef SKParameters::Scalar_t Scalar_t; 
    typedef SKParameters::SKList_t SKList_t;
    typedef ShortRangePotential::Particle_t Particle_t;

    virtual 
    ShortRangePotential* init(SKList_t&, 
			      SKList_t&, 
			      Matrix<int>&, 
			      std::istream&) = 0;
    TBModelBase(): myPtcl(0) { }
    virtual ~TBModelBase() { }
    void setParticleSet(Particle_t* ptclin) {
      myPtcl = ptclin;
    }

    Particle_t* myPtcl;
  };

  /*!\class TBOnsiteBase
   * \brief Base class to implement model-dependent onsite functions
   *
   */
  struct TBOnsiteBase: public ShortRangePotential {

    typedef SKParameters::Scalar_t Scalar_t; 
    typedef SKParameters::SKList_t SKList_t;

    TBOnsiteBase():SK_h(NULL), SK_s(NULL){}
    virtual ~TBOnsiteBase(){}
    virtual void diagonal(SparseMatrix_t& h, const PtclPairList_t& nnlist )=0;
    void setParticleSet(Particle_t* ptclset) {myPtcl = ptclset;}
    void resetParticleSet(Particle_t* ptclset) {myPtcl = ptclset;}

    SKList_t* SK_h;
    SKList_t* SK_s;
    SparseMatrix_t* Rho;
  };
}
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 364 $   $Date: 2005-07-13 13:33:56 -0500 (Wed, 13 Jul 2005) $
 * $Id: TBModelBase.h 364 2005-07-13 18:33:56Z jnkim $ 
 ***************************************************************************/
