//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#ifndef OHMMS_BLOCKMATRIXFUNCTIONS_H
#define OHMMS_BLOCKMATRIXFUNCTIONS_H

#include "OhmmsPETE/OhmmsMatrix.h"
#include "OhmmsNumeric/OhmmsBlasf.h"

template<class T, unsigned D>
struct MultAdd {
  static inline void apply(const T* restrict a, 
			   const T* restrict x, 
			   T* restrict y, 
			   int n, int m) {
    int ii=0;
    for(int i=0; i<D; i++) {
      T tmp = 0;
      for(int j=0; j<D; j++ ){
	tmp += a[ii++]*x[j];
      }
      y[i] += tmp;
    }
  }

  // x and y are doubled space for complex storage
  static inline void apply2(const T* restrict a, const T* restrict x, T* restrict y, 
			    int n, int m) {
    int ii=0;
    for(int i=0; i<D; i++) {
      T tmpr = 0;
      T tmpi = 0;
      for(int j=0; j<D; j++,ii++ ){
	tmpr += a[ii]*x[2*j];
	tmpi += a[ii]*x[2*j+1];
      }
      y[2*i] += tmpr;
      y[2*i+1] += tmpi;
    }
  }

  static inline void apply(const T* restrict a, const complex<T>* restrict x, 
			   complex<T>* restrict y,  int n, int m) {
    int ii=0;
    for(int i=0; i<D; i++) {
      complex<T> tmp = 0;
      for(int j=0; j<D; j++ ){
	tmp += a[ii++]*x[j];
      }
      y[i] += tmp;
    }
  }
};


template<class T>
struct MultAdd<T,4> {
  static inline void apply(const T* restrict a, const T* restrict x, 
			   T* restrict y, int n, int m) {
    y[0] += a[ 0]*x[0] + a[ 1]*x[1] + a[ 2]*x[2] + a[ 3]*x[3];
    y[1] += a[ 4]*x[0] + a[ 5]*x[1] + a[ 6]*x[2] + a[ 7]*x[3];
    y[2] += a[ 8]*x[0] + a[ 9]*x[1] + a[10]*x[2] + a[11]*x[3];
    y[3] += a[12]*x[0] + a[13]*x[1] + a[14]*x[2] + a[15]*x[3];
  }

  static inline void apply(const T* restrict a, 
			   const complex<T>* restrict x, 
			   complex<T>* restrict y, 
			   int n, int m) {
    y[0] += a[ 0]*x[0] + a[ 1]*x[1] + a[ 2]*x[2] + a[ 3]*x[3];
    y[1] += a[ 4]*x[0] + a[ 5]*x[1] + a[ 6]*x[2] + a[ 7]*x[3];
    y[2] += a[ 8]*x[0] + a[ 9]*x[1] + a[10]*x[2] + a[11]*x[3];
    y[3] += a[12]*x[0] + a[13]*x[1] + a[14]*x[2] + a[15]*x[3];
  }

  static inline void apply2(const T* restrict a, 
			    const T* restrict x, T* restrict y, 
			    int n, int m) {
    y[0] += a[ 0]*x[0] + a[ 1]*x[2] + a[ 2]*x[4] + a[ 3]*x[6];
    y[1] += a[ 0]*x[1] + a[ 1]*x[3] + a[ 2]*x[5] + a[ 3]*x[7];
    y[2] += a[ 4]*x[0] + a[ 5]*x[2] + a[ 6]*x[4] + a[ 7]*x[6];
    y[3] += a[ 4]*x[1] + a[ 5]*x[3] + a[ 6]*x[5] + a[ 7]*x[7];
    y[4] += a[ 8]*x[0] + a[ 9]*x[2] + a[10]*x[4] + a[11]*x[6];
    y[5] += a[ 9]*x[1] + a[ 9]*x[3] + a[10]*x[5] + a[11]*x[7];
    y[6] += a[12]*x[0] + a[13]*x[2] + a[14]*x[4] + a[15]*x[6];
    y[7] += a[12]*x[1] + a[13]*x[3] + a[14]*x[5] + a[15]*x[7];
  }
};

template<class T>
struct MultAdd<T,5> {
  static inline void apply(const T* restrict a, const T* restrict x, 
			   T* restrict y, int n, int m) {
    y[0] += a[ 0]*x[0] + a[ 1]*x[1] + a[ 2]*x[2] + a[ 3]*x[3] + a[ 4]*x[4];
    y[1] += a[ 5]*x[0] + a[ 6]*x[1] + a[ 7]*x[2] + a[ 8]*x[3] + a[ 9]*x[4];
    y[2] += a[10]*x[0] + a[11]*x[1] + a[12]*x[2] + a[13]*x[3] + a[14]*x[4];
    y[3] += a[15]*x[0] + a[16]*x[1] + a[17]*x[2] + a[18]*x[3] + a[19]*x[4];
    y[4] += a[20]*x[0] + a[21]*x[1] + a[22]*x[2] + a[23]*x[3] + a[24]*x[4];
  }

  static inline void apply2(const T* restrict a, const T* restrict x, 
			    T* restrict y, int n, int m) {
    y[0] += a[ 0]*x[0] + a[ 1]*x[2] + a[ 2]*x[4] + a[ 3]*x[6] + a[ 4]*x[8];
    y[1] += a[ 0]*x[1] + a[ 1]*x[3] + a[ 2]*x[5] + a[ 3]*x[7] + a[ 4]*x[9];
    y[2] += a[ 5]*x[0] + a[ 6]*x[2] + a[ 7]*x[4] + a[ 8]*x[6] + a[ 9]*x[8];
    y[3] += a[ 5]*x[1] + a[ 6]*x[3] + a[ 7]*x[5] + a[ 8]*x[7] + a[ 9]*x[9];
    y[4] += a[10]*x[0] + a[11]*x[2] + a[12]*x[4] + a[13]*x[6] + a[14]*x[8];
    y[5] += a[10]*x[1] + a[11]*x[3] + a[12]*x[5] + a[13]*x[7] + a[14]*x[9];
    y[6] += a[15]*x[0] + a[16]*x[2] + a[17]*x[4] + a[18]*x[6] + a[19]*x[8];
    y[7] += a[15]*x[1] + a[16]*x[3] + a[17]*x[5] + a[18]*x[7] + a[19]*x[9];
    y[8] += a[20]*x[0] + a[21]*x[2] + a[22]*x[4] + a[23]*x[6] + a[24]*x[8];
    y[9] += a[20]*x[1] + a[21]*x[3] + a[22]*x[5] + a[23]*x[7] + a[24]*x[9];
  }

  static inline void apply(const T* restrict a, const complex<T>* restrict x, 
			   complex<T>* restrict y, 
			   int n, int m) {
    y[0] += a[ 0]*x[0] + a[ 1]*x[1] + a[ 2]*x[2] + a[ 3]*x[3] + a[ 4]*x[4];
    y[1] += a[ 5]*x[0] + a[ 6]*x[1] + a[ 7]*x[2] + a[ 8]*x[3] + a[ 9]*x[4];
    y[2] += a[10]*x[0] + a[11]*x[1] + a[12]*x[2] + a[13]*x[3] + a[14]*x[4];
    y[3] += a[15]*x[0] + a[16]*x[1] + a[17]*x[2] + a[18]*x[3] + a[19]*x[4];
    y[4] += a[20]*x[0] + a[21]*x[1] + a[22]*x[2] + a[23]*x[3] + a[24]*x[4];
  }
};

template<class T>
struct MultAdd<T,0> {

  static inline void apply(const T* restrict a, const T* restrict x, 
			   T* restrict y, int n, int m) {
    int ii=0;
    for(int i=0; i<m; i++) {
      T tmp = 0;
      for(int j=0; j<n; j++ ){
	tmp += a[ii++]*x[j];
      }
      y[i] += tmp;
    }
  }

  static inline void apply2(const T* restrict a, const T* restrict x, 
			    T* restrict y, int n, int m) {
    int ii=0;
    for(int i=0; i<m; i++) {
      T tmpr = 0;
      T tmpi = 0;
      for(int j=0; j<n; j++,ii++ ){
	tmpr += a[ii]*x[2*j];
	tmpi += a[ii]*x[2*j+1];
      }
      y[2*i] += tmpr;
      y[2*i+1] += tmpi;
    }
  }

  static inline void apply(const T* restrict a, 
			   const complex<T>* restrict x, 
			   complex<T>* restrict y, 
                           int n, int m) {
    int ii=0;
    for(int i=0; i<m; i++) {
      complex<T> tmp = 0;
      for(int j=0; j<n; j++ ){
	tmp += a[ii++]*x[j];
      }
      y[i] += tmp;
    }
  }
};

template<class T, unsigned D>
struct MultAdd2{
  static inline void apply(const T* restrict a, 
			   const complex<T>* restrict x, 
			   complex<T>* restrict y) { 
    int kk = 0;
    for(int i=0; i<D; i++) {
      complex<T> z0;
      complex<T> z1;
      for(int j=0; j<D; j++, kk++) {
        z0 += a[kk]*x[j];
        z1 += a[kk]*x[j+D];
      }
      y[i]+= z0;
      y[i+D] += z1;
    }
  }

  static inline void apply(const T* restrict a, 
			   const T* restrict x, T* restrict y) { 
    int d2 = 2*D;
    int kk = 0;
    for(int i=0; i<D; i++) {
      T zur=0, zui=0, zdr=0,zdi=0;
      for(int j=0; j<d2; j+=2, kk++) {
        zur += a[kk]*x[j];
        zui += a[kk]*x[j+1];
        zdr += a[kk]*x[j+d2];
        zdi += a[kk]*x[j+1+d2];
      }
      y[2*i] += zur;
      y[2*i+1] += zui;
      y[2*i+d2] += zdr;
      y[2*i+1+d2] += zdi;
    }
  }
};

template<class T>
struct MultAdd2<T,0>{

  static inline void apply(const T* restrict a, 
			   const complex<T>* restrict x, 
			   complex<T>* restrict y, 
                           int n, int m) {
    int kk = 0;
    for(int i=0; i<m; i++) {
      complex<T> z0;
      complex<T> z1;
      for(int j=0; j<n; j++, kk++) {
        z0 += a[kk]*x[j];
        z1 += a[kk]*x[j+n];
      }
      y[i]+= z0;
      y[i+m] += z1;
    }
  }

  static inline void apply(const T* restrict a, 
			   const T* restrict x, 
			   T* restrict y, 
			   int n, int m) { 
    int d2 = 2*n;
    int kk = 0;
    for(int i=0; i<m; i++) {
      T zur=0, zui=0, zdr=0,zdi=0;
      for(int j=0; j<d2; j+=2, kk++) {
        zur += a[kk]*x[j];
        zui += a[kk]*x[j+1];
        zdr += a[kk]*x[j+d2];
        zdi += a[kk]*x[j+1+d2];
      }
      y[2*i] += zur;
      y[2*i+1] += zui;
      y[2*i+d2] += zdr;
      y[2*i+1+d2] += zdi;
    }
  }

};


template<class T, unsigned D>
struct Product {

  static inline void make(T* restrict block, int nbi, int nbj,
			  const Matrix<T>& a, int ioff,
			  const Matrix<T>& b, int joff, const vector<T>& w) {
    std::fill(block,block+D*D,0.0);
    int colsA(a.cols()),colsB(b.cols());
    const T* aPtr=a.data()+ioff;
    const T* bPtr=b.data()+joff;
    typename vector<T>::const_iterator wit(w.begin()),wit_end(w.end());
    while(wit != wit_end) {
      register T wx(*wit++);
      int kk(0);
      const T* aptr=aPtr;
      for(int ib=0; ib<D; ib++) {
        register T aw(wx*(*aptr++));
        const T* bptr=bPtr;
        for(int jb=0; jb<D; jb++) {
          block[kk++]+=aw*(*bptr++);
        }
      }
      aPtr+=colsA; bPtr+=colsB;
    }
  }

  static inline void make(T* restrict block, int nbi, int nbj,
			  const Matrix<T>& a, int ioff,
			  const Matrix<T>& b, int joff, T w) {
    std::fill(block,block+D*D,0.0);
    int rowsA(a.rows()),colsA(a.cols()),colsB(b.cols());
    const T* aPtr=a.data()+ioff;
    const T* bPtr=b.data()+joff;
    for(int m=0; m<rowsA; m++) {
      int kk(0);
      const T* aptr=aPtr;
      for(int ib=0; ib<D; ib++) {
        register T aw(w*(*aptr++));
        const T* bptr=bPtr;
        for(int jb=0; jb<D; jb++) {
          block[kk++]+=aw*(*bptr++);
        }
      }
      aPtr+=colsA; bPtr+=colsB;
    }
  }

  static inline void make(T* restrict block, const T* a, const T* b) {
    int kk = 0;
    for(int ib=0; ib<D; ib++) {
      T tmp = a[ib];
      for(int jb=0; jb<D; jb++)  block[kk++] = tmp*b[jb];
    }
  }

  static inline void make(T* restrict rblock, T* restrict iblock, 
			  int nbi, int nbj,
			  const Matrix<complex<T> >& a, int ioff,  
			  const Matrix<complex<T> >& b, int joff, 
			  const vector<T>& w) {
    std::fill(rblock,rblock+D*D,0.0);
    std::fill(iblock,iblock+D*D,0.0);
    int colsA(a.cols()),colsB(b.cols());
    const complex<T>* aPtr=a.data()+ioff;
    const complex<T>* bPtr=b.data()+joff;
    typename vector<T>::const_iterator wit(w.begin()),wit_end(w.end());
    while(wit != wit_end) {
      register T wx(*wit++);
      const complex<T>* aptr=aPtr;
      for(int ib=0,kk=0; ib<D; ib++) {
        register complex<T> aw(wx*(conj(*aptr++)));
        const complex<T>* bptr=bPtr;
        for(int jb=0; jb<D; jb++,kk++) {
          complex<T> t(aw*(*bptr++));
          rblock[kk]+=t.real();
          iblock[kk]+=t.imag();
        }
      }
      aPtr+=colsA; bPtr+=colsB;
    }
  }
};

/** specialization for blocks of any dimension
 */
template<class T>
struct Product<T,0> {

  static inline void make(T* restrict block, 
			  int n, int m, 
                          const Matrix<T>& a, int ioff,
			  const Matrix<T>& b, int joff, 
                          const vector<T>& w) {

    std::fill(block,block+n*m,0.0);
    int colsA(a.cols()),colsB(b.cols());
    const T* aPtr=a.data()+ioff;
    const T* bPtr=b.data()+joff;
    typename vector<T>::const_iterator wit(w.begin()),wit_end(w.end());
    while(wit != wit_end) {
      register T wx(*wit++);
      for(int ib=0,kk=0; ib<n; ib++) {
        register T aw(wx*(aPtr[ib]));
        for(int jb=0; jb<m; jb++) {
          block[kk++] +=aw*bPtr[jb];
        }
      }
      aPtr+=colsA; bPtr+=colsB;
    }
    /*
    typename vector<T>::const_iterator wit(w.begin()),wit_end(w.end());
    while(wit != wit_end) {
      register T wx(*wit++);
      T* restrict blockPtr=block;
      const T* aptr=aPtr;
      for(int ib=0; ib<n; ib++) {
        register T aw(wx*(*aptr++));
        const T* bptr=bPtr;
        for(int jb=0; jb<m; jb++) {
          *blockPtr++ +=aw*(*bptr++);
        }
      }
      aPtr+=colsA; bPtr+=colsB;
    }
    */
  }

  static inline void make(T* restrict block, int n, int m, 
			  const Matrix<T>& a, int ioff,
			  const Matrix<T>& b, int joff, T w) {
    std::fill(block,block+n*m,0.0);
    size_t rowsA(a.rows()),colsA(a.cols()),colsB(b.cols());
    const T* aPtr=a.data()+ioff;
    const T* bPtr=b.data()+joff;
    for(size_t m=0; m<rowsA; m++) {
      size_t kk(0);
      const T* aptr=aPtr;
      for(int ib=0; ib<n; ib++) {
        register T aw(w*(*aptr++));
        const T* bptr=bPtr;
        for(int jb=0; jb<m; jb++) {
          block[kk++]+=aw*(*bptr++);
        }
      }
      aPtr+=colsA; bPtr+=colsB;
    }
  }

  static inline void make(T* restrict block, int n, int m, const T* a, const T* b) {
    int kk = 0;
    for(int ib=0; ib<n; ib++) {
      T tmp(*a++);
      for(int jb=0; jb<m; jb++)  block[kk++] += tmp*b[jb];
    }
  }

  static inline void make(T* restrict rblock, T* restrict iblock, int n, int m, 
                          const Matrix<complex<T> >& a, int ioff,
			  const Matrix<complex<T> >& b, int joff, 
			  const vector<T>& w) {
    std::fill(rblock,rblock+n*m,0.0);
    std::fill(iblock,iblock+n*m,0.0);
    int colsA(a.cols()),colsB(b.cols());
    const complex<T>* aPtr=a.data()+ioff;
    const complex<T>* bPtr=b.data()+joff;
    typename vector<T>::const_iterator wit(w.begin()),wit_end(w.end());
    while(wit != wit_end) {
      register T wx(*wit++);
      int kk(0);
      const complex<T>* aptr=aPtr;
      for(int ib=0; ib<n; ib++) {
        register complex<T> aw(wx*(conj(*aptr++)));
        const complex<T>* bptr=bPtr;
        for(int jb=0; jb<m; jb++,kk++) {
          complex<T> t(aw*(*bptr++));
          rblock[kk]+=t.real();
          iblock[kk]+=t.imag();
        }
      }
      aPtr+=colsA; bPtr+=colsB;
    }
  }
};

/** specialization for 4x4 blocks
 */
template<class T>
struct Product<T,4> {

  static inline void make(T* restrict block,  int nbi, int nbj,
			  const Matrix<T>& a, int ioff,
			  const Matrix<T>& b, int joff, const vector<T>& w) {
    int colsA(a.cols()),colsB(b.cols());
    const T* restrict aPtr=a.data()+ioff;
    const T* restrict bPtr=b.data()+joff;
    typename vector<T>::const_iterator wit(w.begin()),wit_end(w.end());
    std::fill(block,block+16,0.0);
    while(wit != wit_end) {
      register T wx(*wit++);
      T s0(wx*aPtr[0]); T s1(wx*aPtr[1]); T s2(wx*aPtr[2]); T s3(wx*aPtr[3]);
      T x0(bPtr[0]); T x1(bPtr[1]); T x2(bPtr[2]); T x3(bPtr[3]);
      block[0] +=s0*x0; block[1] +=s0*x1; block[2] +=s0*x2; block[3] +=s0*x3;
      block[4] +=s1*x0; block[5] +=s1*x1; block[6] +=s1*x2; block[7] +=s1*x3;
      block[8] +=s2*x0; block[9] +=s2*x1; block[10]+=s2*x2; block[11]+=s2*x3;
      block[12]+=s3*x0; block[13]+=s3*x1; block[14]+=s3*x2; block[15]+=s3*x3;
      aPtr+=colsA; bPtr+=colsB;
    }
  }

  static inline void make(T* restrict block,  int nbi, int nbj,
			  const Matrix<T>& a, int ioff,
			  const Matrix<T>& b, int joff, T w) {

    std::fill(block,block+16,0.0);

    int colsA(a.cols()),rowsA(a.rows()),colsB(b.cols());
    const T* aPtr=a.data()+ioff;
    const T* bPtr=b.data()+joff;
    for(int m=0; m<rowsA; m++) {
      T s0(w*aPtr[0]); T s1(w*aPtr[1]); T s2(w*aPtr[2]); T s3(w*aPtr[3]);
      T x0(bPtr[0]); T x1(bPtr[1]); T x2(bPtr[2]); T x3(bPtr[3]);
      block[0] +=s0*x0; block[1] +=s0*x1; block[2] +=s0*x2; block[3] +=s0*x3;
      block[4] +=s1*x0; block[5] +=s1*x1; block[6] +=s1*x2; block[7] +=s1*x3;
      block[8] +=s2*x0; block[9] +=s2*x1; block[10]+=s2*x2; block[11]+=s2*x3;
      block[12]+=s3*x0; block[13]+=s3*x1; block[14]+=s3*x2; block[15]+=s3*x3;
      aPtr+=colsA; bPtr+=colsB;
    }
  }


  static inline void make(T* restrict block, const T* restrict a, 
			  const T* restrict b) {
    int kk = 0;
    for(int ib=0; ib<4; ib++) {
      T tmp = a[ib];
      for(int jb=0; jb<4; jb++)  block[kk++] = tmp*b[jb];
    }
  }

  static inline void make(T* restrict rblock, T* restrict iblock, 
			  int nbi, int nbj,
                          const Matrix<complex<T> >& a, int ioff,
			  const Matrix<complex<T> >& b, int joff, 
			  const vector<T>& w) {

    std::fill(rblock,rblock+16,0.0);
    std::fill(iblock,iblock+16,0.0);
    int colsA(a.cols()),colsB(b.cols());
    const complex<T>* aPtr=a.data()+ioff;
    const complex<T>* bPtr=b.data()+joff;
    typename vector<T>::const_iterator wit(w.begin()),wit_end(w.end());
    while(wit != wit_end) {
      register T wx(*wit++);
      const complex<T>* aptr=aPtr;
      for(int ib=0,kk=0; ib<4; ib++) {
        register complex<T> aw(wx*(conj(*aptr++)));
        const complex<T>* bptr=bPtr;
        for(int jb=0; jb<4; jb++,kk++) {
          complex<T> t(aw*(*bptr++));
          rblock[kk]+=t.real();
          iblock[kk]+=t.imag();
        }
      }
      aPtr+=colsA; bPtr+=colsB;
    }

    //complex<T> t00, t01, t02, t03, t10, t11, t12, t13, t20, t21, t22, t23, t30, t31, t32, t33;
    //for(int m=0; m<w.size();m++) {
    //  int ii = ioff;
    //  int jj = joff;
    //  t00 += s0*x0;      t01 += s0*x1;      t02 += s0*x2;      t03 += s0*x3;
    //  t10 += s1*x0;      t11 += s1*x1;      t12 += s1*x2;      t13 += s1*x3;
    //  t20 += s2*x0;      t21 += s2*x1;      t22 += s2*x2;      t23 += s2*x3;
    //  t30 += s3*x0;      t31 += s3*x1;      t32 += s3*x2;      t33 += s3*x3;
    //}
    //int kk = 0;
    //rblock[kk] = t00.real(); iblock[kk++] = t00.imag();
    //rblock[kk] = t01.real(); iblock[kk++] = t01.imag();
    //rblock[kk] = t02.real(); iblock[kk++] = t02.imag();
    //rblock[kk] = t03.real(); iblock[kk++] = t03.imag();
    //rblock[kk] = t10.real(); iblock[kk++] = t10.imag(); 
    //rblock[kk] = t11.real(); iblock[kk++] = t11.imag();
    //rblock[kk] = t12.real(); iblock[kk++] = t12.imag();
    //rblock[kk] = t13.real(); iblock[kk++] = t13.imag();
    //rblock[kk] = t20.real(); iblock[kk++] = t20.imag();
    //rblock[kk] = t21.real(); iblock[kk++] = t21.imag();
    //rblock[kk] = t22.real(); iblock[kk++] = t22.imag();
    //rblock[kk] = t23.real(); iblock[kk++] = t23.imag();
    //rblock[kk] = t30.real(); iblock[kk++] = t30.imag();
    //rblock[kk] = t31.real(); iblock[kk++] = t31.imag(); 
    //rblock[kk] = t32.real(); iblock[kk++] = t32.imag();
    //rblock[kk] = t33.real(); iblock[kk++] = t33.imag();
  }
};


template<class T>
inline void assign(T* restrict d, int n, T val) {
  for(int i=0; i<n; i++) *d++=val; //d[i] = val;
}

template<class T>
inline void cmult(complex<T>* restrict y, const complex<T> k, 
		  const T* restrict x, int n) {
  for(int i=0; i<n; i++) *y++ = k*(*x++);
  //for(int i=0; i<n; i++) y[i] = x[i]*k;
}

template<class T, unsigned D>
struct AddDiag {
  static inline void apply(T* restrict a, const T& val, int n) {
    for(int i=0; i<D; i++) {
      *(a+i*D+i) += val;
    }
  }
};

template<class T>
struct AddDiag<T,0> {
  static inline void apply(T* restrict a, const T& val, int n) {
    for(int i=0; i<n; i++) {
      *(a+i*n+i) += val; //a[i*n+i] += val;
    }
  }
};

////////////////////////////////////////////////////////////////////
// special block operators used for the dynamic and el-ph coupling matrix
////////////////////////////////////////////////////////////////////
template<class T, unsigned D>
struct MakeTrace {

  static inline T apply(const T* restrict x, const T* restrict a, 
			const T* restrict y, int n, int m) {
    T sum = 0.0;
    int ii= 0;
    for(int i=0; i<D; i++) {
      T psum = 0.0;
      for(int j=0; j<D; j++) {
        psum += a[ii++]*y[j];
      }
      sum += psum*x[i];
    }
    return sum;
  }
};
template<class T>
struct MakeTrace<T,0> {

  static inline T apply(const T* restrict x, const T* restrict a, 
			const T* restrict y, int n, int m) {
    T sum = 0.0;
    int ii= 0;
    for(int i=0; i<n; i++) {
      T psum = 0.0;
      for(int j=0; j<m; j++) {
        psum += a[ii++]*y[j];
      }
      sum += psum*x[i];
    }
    return sum;
  }
};

inline 
void copyadd(double* a, double* b, int n) {
  for(int i=0; i<n; i++) b[i] += a[i];
}



#endif



/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 354 $   $Date: 2005-07-07 21:20:49 -0500 (Thu, 07 Jul 2005) $
 * $Id: BlockMatrixFunctions.h 354 2005-07-08 02:20:49Z jnkim $ 
 ***************************************************************************/

