//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include <vector>
#include <set>
using namespace std;

#include "Message/OpenMP.h"
#include "SparseMatrix/SparseIndexOps.h"
#include "SparseMatrix/SparseIndex.h"
#include "Utilities/OhmmsInfo.h"
namespace OHMMS {

void 
copyIndex(const SparseIndex& inda, SparseIndex& res, int n) {

  ERRORMSG("OHMMS::copyIndex is incomplete. DONT June/24/05")
  res.TotalNum = inda.getTotNum();
  res.create(inda.NumThreads);

  res.RowBlock = inda.RowBlock;
  if(n) {
    res.RowBlock[0] = 0;
    for(int iat=0; iat<res.TotalNum; iat++) 
      res.RowBlock[iat+1] = res.RowBlock[iat]+n;
    res.MaxRowBlock = n;
  } else {
    res.MaxRowBlock = inda.MaxRowBlock;
  }

  res.ColBlock = inda.ColBlock;
  res.MaxColBlock = inda.MaxColBlock;

  res.MaxBlockSize = res.MaxRowBlock*res.MaxColBlock;
  
  /////////////////////////////////////////////////////
  // WARNING: Assuming each row and block size is identical
  /////////////////////////////////////////////////////
  res.TotalRows = res.RowBlock[res.TotalNum];
  res.TotalCols = res.ColBlock[res.TotalNum];
  //res.GroupID = inda.GroupID;

#pragma omp parallel
  {
    int np=res.NumThreads;
#pragma omp for nowait 
    for(int ip=0; ip< np; ip++) {
      res.update(*(inda.J[ip]), *(inda.M[ip]), inda.first_diag(ip),ip);
    }
  }

  res.checkBounds();
}

void expandIndex(SparseIndex& inda, int nshell) {
  ERRORMSG("OHMMS::expandIndex is incomplete. DONT June/24/05")
//Only used by O(N) method
//Incomplete: June/24/05
//  vector<vector<int>* > J;
//  vector<vector<int>* > M;
//
//  for(int myID=0; myID<inda.NumThreads; myID++) {
//    int TotalNum = inda.getTotNum();
//
//    Vector<int> Work(TotalNum);
//    J.push_back(new vector<int>);
//    M.push_back(new vector<int>(inda.numatoms(myID)+1));
//
//    J[myID]->reserve(TotalNum*inda.numatoms(myID));
//    (*M[myID])[0] = 0;
//    int iL=0;
//    int iG = inda.first_diag(myID);
//    for(; iL<inda.numatoms(myID); iL++, iG++) {
//
//      int nn=0; // per atom
//      Work =-1;
//      Work[iG] = iG;//exclude itself
//      set<int> newnn;
//      for(int jnn=0; jnn< inda.nadj(iL,myID); jnn++,nn++) {
//	int  jG = inda.iadj(iL, jnn, myID);
//	Work[jG]=jG;
//	J[myID]->push_back(jG);
//	newnn.insert(jG);
//      }
//
//      int ishell = 2;
//      while(nn < TotalNum && ishell <= nshell) { 
//	set<int>::iterator it = newnn.begin();
//	set<int> tmpnn;
//	while(it != newnn.end()) {
//	  int jG = (*it);
//	  int nodeid = inda.where(jG);
//	  int jL = jG - inda.first_diag(nodeid);
//	  for(int knn=0; knn< inda.nadj(jL,nodeid); knn++) {
//	    int kG = inda.iadj(jL, knn,nodeid);
//	    if(Work[kG] == -1) { 
//	      nn++;
//	      Work[kG] = kG;
//	      J[myID]->push_back(kG);
//	      tmpnn.insert(kG);
//	    }
//	  }
//	  it++;
//	}
//	newnn = tmpnn;
//	tmpnn.erase(tmpnn.begin(), tmpnn.end());
//	ishell++;
//      }
//      (*M[myID])[iL+1] = (*M[myID])[iL]+nn;
//    }
//  }
//
//  for(int myID=0; myID<inda.NumThreads; myID++) {
//    int nloc = inda.numatoms(myID);
//    for(int k=0;k<=nloc; k++) inda.M[myID][k] = (*M[myID])[k];
//
//    if((*M[myID])[nloc] > inda.maxpairs(myID)) {
//      int nn = J[myID]->size();
//      int nnmax = nn+nn/10; //add 10%
//      nnmax = min(nnmax,inda.getTotNum()*inda.getTotNum());
//      delete [] inda.J[myID];     
//      inda.J[myID] = new int[nnmax];
//      inda.myPairs[myID][3] = nnmax;
//      inda.myPairs[myID][2] = nn;
//      for(int j=0; j<nn; j++) inda.J[myID][j] = (*J[myID])[j];
//    }
//  }
//
//  inda.update(0);
//
//  for(int myID=0; myID<inda.NumThreads; myID++) {
//    delete J[myID];
//    delete M[myID];
//  }
}
void 
getUnion(const SparseIndex& inda, const SparseIndex& indb, 
	 SparseIndex& res, bool transa, bool transb) {

  ERRORMSG("OHMMS::getUnion is incomplete. DONT June/24/05")
//Only used by O(N) method
//Incomplete: June/24/05
//  if(!res.J) {
//
//    res.OwnIndices = true;
//    res.TotalNum = inda.getTotNum();
//    res.create(inda.NumThreads);
//
//    if(transa){
//      res.RowBlock = inda.ColBlock;
//      res.MaxRowBlock = inda.MaxColBlock;
//    } else {
//      res.RowBlock = inda.RowBlock;
//      res.MaxRowBlock = inda.MaxRowBlock;
//    }
//
//    if(transb){
//      res.ColBlock = indb.RowBlock;
//      res.MaxColBlock = indb.MaxRowBlock;
//    }else {
//      res.ColBlock = indb.ColBlock;
//      res.MaxColBlock = indb.MaxColBlock;
//    }
//
//    res.MaxBlockSize = res.MaxRowBlock*res.MaxColBlock;
//    
//    res.TotalRows = res.RowBlock[res.TotalNum];
//    res.TotalCols = res.ColBlock[res.TotalNum];
//    /////////////////////////////////////////////////////
//    // WARNING: Assuming each row and block size is identical
//    /////////////////////////////////////////////////////
//    //res.TotalRows = res.TotalNum*res.RowBlock[0];
//    //res.TotalCols = res.TotalNum*res.ColBlock[0];
//    //res.GroupID = inda.GroupID;
//  }
//
//
//  for(int myID=0; myID<res.NumThreads; myID++) {
//
//    res.myAtoms[myID][0] = inda.first_diag(myID);
//    res.myAtoms[myID][1] = inda.last_diag(myID);
//    res.myAtoms[myID][2] = inda.numatoms(myID);
//
//    int TotalNum = res.getTotNum();
//    vector<int> Work(TotalNum);
//
//    int nna = inda.numpairs(myID)/inda.numatoms(myID);
//    int nnb = indb.numpairs(myID)/indb.numatoms(myID);
//
//    int nnmax =  (nna*nnb)*inda.numatoms(myID);
//
//    if(nnmax > res.maxpairs(myID)) {
//      nnmax += nna*nnb;
//      if(res.J[myID]) delete [] res.J[myID];     
//      res.J[myID] = new int[nnmax];
//      res.myPairs[myID][3] = nnmax;
//    }
//    if(!res.M[myID]) { 
//      res.M[myID] = new int[inda.numatoms(myID)+1];
//    }
//
//    int iG = inda.first_diag(myID);
//    int nnloc = 0; // node sum
//    res.M[myID][0] = 0;
//    for(int iL=0; iL<inda.numatoms(myID); iL++, iG++) {
//
//      int nn=0; // per atom
//
//      for(int i=0; i<TotalNum; i++) Work[i] = -1;
//
//      Work[iG] = iG;
//      for(int jnn=0; jnn< inda.nadj(iL,myID); jnn++) {
//	int jG = inda.iadj(iL, jnn, myID);
//	res.J[myID][nnloc] = jG;
//	nnloc++; nn++;
//	Work[jG]=jG;
//      }
//
//      for(int jnn=0; jnn< inda.nadj(iL,myID); jnn++) {
//	int jG = inda.iadj(iL, jnn, myID);
//	int nodeid = indb.where(jG);
//
//	int jL = jG - indb.first_diag(nodeid);
//	
//	for(int knn=0; knn< indb.nadj(jL,nodeid); knn++) {
//	  int kG = indb.iadj(jL, knn,nodeid);
//	  if(Work[kG] > -1) continue;
//	  res.J[myID][nnloc] = kG;          
//	  nnloc++; nn++;
//	  Work[kG]=kG;
//	}
//      }
//      res.M[myID][iL+1] = res.M[myID][iL] + nn;
//    }
//    res.myPairs[myID][2] = nnloc;
//  }
//
//  res.update(0);
}
}

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 354 $   $Date: 2005-07-07 21:20:49 -0500 (Thu, 07 Jul 2005) $
 * OHMMS_VERSION: $Id: SparseIndexOps.cpp 354 2005-07-08 02:20:49Z jnkim $
 ***************************************************************************/
