//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#include "SparseMatrix/SparseMatrixOps.h"
#include "SparseMatrix/BlockMatrixFunctions.h"

void 
OHMMS::product(const ComplexMatrix_t& bra, 
	       const ComplexMatrix_t& ket, 
	       const vector<Scalar_t>& w,
	       SparseMatrix_t& mat_r, SparseMatrix_t& mat_i) {

#pragma omp parallel
  {
    int myID = omp_get_thread_num();
    typedef Scalar_t T;
    const SparseIndex_t* myIndex = mat_r.index();

    int iG=myIndex->first_diag(myID);
    int nn = 0;
    for(int iL=0; iL < myIndex->num_diag(myID); iL++,iG++){    

      int ioff = myIndex->row_loc(iG);

#ifdef TB_BLOCK
      Product<T,TB_BLOCK>
      ::make(mat_r.diag(myID,iL), mat_i.diag(myID,iL), 
	     bra, ioff, ket, ioff, w);
#else
      int nbi = myIndex->row_block(iG);
      Product<T,0>
	::make(mat_r.diag(myID,iL),mat_i.diag(myID,iL),
	       nbi,nbi,bra,ioff,ket,ioff,w);
#endif

      for(int nni=0; nni < myIndex->nadj(iL, myID); nni++, nn++) {
        int jG = myIndex->iadj(iL, nni, myID);
        if(iG == jG) continue;
#ifdef TB_BLOCK
        Product<T,TB_BLOCK>
	  ::make(mat_r.offdiag(myID,nn), mat_i.offdiag(myID,nn),
		 bra, ioff, ket, myIndex->col_loc(jG), w);
#else
        Product<T,0>
	  ::make(mat_r.offdiag(myID,nn), mat_i.offdiag(myID,nn),
		 nbi, myIndex->row_block(jG), 
		 bra, ioff, ket, myIndex->col_loc(jG),w);
#endif
      }// pairs
    }
  }// omp
}

void 
OHMMS::product(const RealMatrix_t& a, const RealMatrix_t& b,
	       const vector<Scalar_t>& w, SparseMatrix_t& mat) 
{
#pragma omp parallel
   {
    int myID = omp_get_thread_num();
    typedef Scalar_t T;
    const SparseIndex_t* myIndex = mat.index();

    int iG=myIndex->first_diag(myID);
    int nn = 0;
    for(int iL=0; iL < myIndex->num_diag(myID); iL++,iG++){    

      int ioff = myIndex->row_loc(iG);

#ifdef TB_BLOCK
      Product<T,TB_BLOCK>::make(mat.diag(myID,iL),a,ioff,b,ioff,w);
#else
      int nbi = myIndex->row_block(iG);
      Product<T,0>::make(mat.diag(myID,iL),nbi,nbi,a,ioff,b,ioff,w);
#endif

      for(int nni=0; nni < myIndex->nadj(iL, myID); nni++, nn++) {
        int jG = myIndex->iadj(iL, nni, myID);

        if(iG == jG) continue;
#ifdef TB_BLOCK
        Product<T,TB_BLOCK>
	  ::make(mat.offdiag(myID,nn),a,ioff,b,myIndex->col_loc(jG),w);
#else
        Product<T,0>::make(mat.offdiag(myID,nn),nbi,myIndex->row_block(jG),
                           a,ioff,b,myIndex->col_loc(jG),w);
#endif
      }// pair-list
    }
  }// closing pramga

}

void 
OHMMS::product(const Scalar_t* a, const Scalar_t* b, SparseMatrix_t& mat) 
{
#pragma omp parallel
   {
    int myID = omp_get_thread_num();
    typedef Scalar_t T;
    const SparseIndex_t* myIndex = mat.index();

    int iG=myIndex->first_diag(myID);
    int nn = 0;
    for(int iL=0; iL < myIndex->num_diag(myID); iL++,iG++){    

      int ioff = myIndex->row_loc(iG);

#ifdef TB_BLOCK
      Product<T,TB_BLOCK>::make(mat.diag(myID,iL),a+ioff,b+ioff);
#else
      int nbi = myIndex->row_block(iG);
      Product<T,0>::make(mat.diag(myID,iL),nbi,nbi,a+ioff,b+ioff);
#endif

      for(int nni=0; nni < myIndex->nadj(iL, myID); nni++, nn++) {
        int jG = myIndex->iadj(iL, nni, myID);

        if(iG == jG) continue;
#ifdef TB_BLOCK
        Product<T,TB_BLOCK>
	  ::make(mat.offdiag(myID,nn),a,ioff,b,myIndex->col_loc(jG),w);
#else
        Product<T,0>::make(mat.offdiag(myID,nn),nbi,myIndex->row_block(jG),
                           a+ioff,b+myIndex->col_loc(jG));
#endif
      }// pair-list
    }
  }// closing pramga

}

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: SparseMatrixOps.cpp 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/

