//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim, Seungwon Lee, and Lars Jonsson
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

// -*- C++ -*-
#ifndef OHMMS_SPARSEMATRIXSPINORBITMP_H
#define OHMMS_SPARSEMATRIXSPINORBITMP_H

#include <complex>
using namespace std;
#ifndef OHMMS_PTCLPAIRLISTBASE_H
#include "ParticleBase/PtclPairListBase.h"
#endif
#ifndef OHMMS_PETE_MATRIX_H
#include "OhmmsPETE/OhmmsMatrix.h"
#endif
#include "SparseMatrix/SparseVectorIndex.h"
#include "SlaterKoster/SpinOrbitBase.h"

class SparseMatrixMP {
public:
  typedef double             T;
  typedef complex<T>         dcomplex_t;
  typedef Matrix<dcomplex_t> FullMatrix_t;
#ifdef OHMMS_DIM
  typedef VectorIndex<PtclPairListBase<T,OHMMS_DIM> > Index_t;
#else
  typedef VectorIndex<PtclPairListBase<T,3> > Index_t;
#endif
  typedef vector<SpinOrbitComp<T>* >    SOArray_t;

  SparseMatrixMP();
  SparseMatrixMP(Index_t* ind);
  ~SparseMatrixMP(); 

  inline const Index_t* index() const { return myIndex;}
  void init(Index_t*);
  void reset(bool changediag = false);

  SparseMatrixMP& operator +=(T);

  void apply(const dcomplex_t* win, dcomplex_t* wout) const;

  void copy(Matrix<dcomplex_t>& mout, int packing=0) const;
  void copy(Matrix<dcomplex_t>& mout, const complex<T>*, int packing =0) const;

  void product(const FullMatrix_t&, const FullMatrix_t&, const vector<T>& w);
  void product(SparseMatrixMP&, const FullMatrix_t&, const FullMatrix_t&, 
               const vector<T>& w);

//    inline T* diag(int i) { return d_diag[i];}
//    inline T* offdiag(int i) { return d_offdiag[i];}
//    inline const T* diag(int i) const { return d_diag[i];}
//    inline const T* offdiag(int i) const { return d_offdiag[i];}

  inline T* diag(int myid, int i) { return d_diag[myid][i];}
  inline T* offdiag(int myid, int i) { return d_offdiag[myid][i];}
  inline const T* diag(int myid, int i) const { return d_diag[myid][i];}
  inline const T* offdiag(int myid, int i) const { return d_offdiag[myid][i];}

  inline int first_diag(int myid) const { return myAtoms[myid][0];}
  inline int last_diag(int myid) const { return myAtoms[myid][1];}
  inline int first_offdiag(int myid) const { return myPairs[myid][0];}
  inline int last_offdiag(int myid) const { return myPairs[myid][1];}

//    inline T* offdiag(int i, int j) { // expensive operation
//      if(i == j) return d_diag[i];
//      else       return d_offdiag[myIndex->loc(i,j)];
//    }
  
  //template<class SK_t, class Bond_t, class IonID_t>
  //void make(SK_t&, const Bond_t&, const IonID_t&);

  void print(ostream& os);

private:

  int Num; 
  int NumPairs;
  int WS_size;
  int Dmax;
  int NumThreads;

  Index_t* myIndex;
  T***  d_diag;
  T***  d_offdiag;
  int** myAtoms;
  int** myPairs;

  SOArray_t SO;

  void make_mp_map(int myID, int*, int*); 
  void read_so_parameters();
};
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: SparseMatrixSpinOrbitMP.h 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/

