//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim, Seungwon Lee, and Lars Jonsson
//
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

// -*- C++ -*-
#ifndef OHMMS_SPARSEMATRIXSPINORBIT_OMP_H
#define OHMMS_SPARSEMATRIXSPINORBIT_OMP_H

#include <complex>
using namespace std;
#ifndef OHMMS_PTCLPAIRLISTBASE_H
#include "ParticleBase/PtclPairListBase.h"
#endif
#ifndef OHMMS_PETE_MATRIX_H
#include "OhmmsPETE/OhmmsMatrix.h"
#endif
#include "SparseMatrix/SparseIndexOMP.h"
#include "SlaterKoster/SpinOrbitBase.h"

class SparseMatrixOMP {
public:

  typedef double             T;
  typedef complex<T>         dcomplex_t;
  typedef Matrix<dcomplex_t> FullMatrix_t;
  typedef SparseIndexOMP     Index_t;
  typedef vector<SpinOrbitComp<T>* >    SOArray_t;

  SparseMatrixOMP();
  SparseMatrixOMP(Index_t* ind);
  ~SparseMatrixOMP(); 

  inline const Index_t* index() const { return myIndex;}
  void init(Index_t*);
  void reset(int  = 0);

  SparseMatrixOMP& operator +=(T);

  void apply(T** win, const int, T** wout, const int, const int, const int) const;
  void apply(const dcomplex_t* win, dcomplex_t* wout) const;

  void copy(Matrix<double>& mout, int packing=0) const { }
  void copy(Matrix<double>& mout, const double*, int pacing=0) const { }

  void copy(FullMatrix_t& mout, int packing=0) const;
  void copy(FullMatrix_t& mout, const dcomplex_t*, int pacing=0) const;

  void product(const FullMatrix_t&, const FullMatrix_t&, const vector<T>& w);
  void product(SparseMatrixOMP&, const FullMatrix_t&, const FullMatrix_t&, 
               const vector<T>& w);

  inline T* diag(int myid, int i) { return d_diag[myid][i];}
  inline T* offdiag(int myid, int i) { return d_offdiag[myid][i];}
  inline const T* diag(int myid, int i) const { return d_diag[myid][i];}
  inline const T* offdiag(int myid, int i) const { return d_offdiag[myid][i];}

  // returns the size of column on myid: note the factor of two
  inline int numcol(int myid) const {
    return  2*(myIndex->col_loc(myIndex->last_diag(myid))
              -myIndex->col_loc(myIndex->first_diag(myid)));
  }

  void print(ostream& os);

private:
  int Num; 
  int WS_size;
  int Dmax;
  int NumThreads;

  Index_t* myIndex;
  int* MaxNumPairs;

  T***  d_diag;
  T***  d_offdiag;
  T*    diagWorkSpace;
  T*    offdiagWorkSpace;

  SOArray_t SO;
  void read_so_parameters();
};
#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: SparseMatrixSpinOrbitOMP.h 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/

