//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//   Jeongnim Kim
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//
//////////////////////////////////////////////////////////////////
// -*- c++ -*-
#ifndef OHMMS_SPARSE_SPARSE_OPERATIONS_H
#define OHMMS_SPARSE_SPARSE_OPERATIONS_H

#include "SparseMatrix/SparseMatrixOMP.h"

namespace OhmmsSparseMatrix {

#ifdef USE_SPARSE_THREADED
// B <- a*A + B
void axpy(double a, const SparseMatrixOMP& A, SparseMatrixOMP& B, int myID=0);

// B <- A
void copy(const SparseMatrixOMP& A, SparseMatrixOMP& B, int myID = 0);

// A += xI
void addDiagonal(SparseMatrixOMP& A, double x, int myID = 0);

// C = A*B
void product(const SparseMatrixOMP& B, const SparseMatrixOMP& A, SparseMatrixOMP& C,
	     int myID = 0);

// C = x A*B + y*C
void  
xAB_plus_yC(double x, const SparseMatrixOMP& A, const SparseMatrixOMP& B, 
	    double y, SparseMatrixOMP& C, int myID=0);

// C = x A*Bt + y*C
void  
xAAt_plus_yC(double x, const SparseMatrixOMP& A, double y, SparseMatrixOMP& C, int myID = 0);

// C = x A*Bt + y*C
void  
xABt_plus_yC(double x, const SparseMatrixOMP& A, const SparseMatrixOMP& B, 
	     double y, SparseMatrixOMP& C, int myID = 0);

void  
xABt_xBAt_plus_yC(double x, const SparseMatrixOMP& A, const SparseMatrixOMP& B, 
		  double y, SparseMatrixOMP& C, int myID = 0);

double Dot(const SparseMatrixOMP& A, const SparseMatrixOMP& B, int myID = 0);
#else
// B <- a*A + B
void axpy(double a, const SparseMatrixOMP& A, SparseMatrixOMP& B);

// B <- A
void copy(const SparseMatrixOMP& A, SparseMatrixOMP& B);

// A += xI
void addDiagonal(SparseMatrixOMP& A, double x);

// C = A*B
void product(const SparseMatrixOMP& B, const SparseMatrixOMP& A, SparseMatrixOMP& C);

// C = x A*B + y*C
void  
xAB_plus_yC(double x, const SparseMatrixOMP& A, const SparseMatrixOMP& B, 
	    double y, SparseMatrixOMP& C);

// C = x A*Bt + y*C
void  
xAAt_plus_yC(double x, const SparseMatrixOMP& A, double y, SparseMatrixOMP& C);

// C = x A*Bt + y*C
void  
xABt_plus_yC(double x, const SparseMatrixOMP& A, const SparseMatrixOMP& B, 
	     double y, SparseMatrixOMP& C);

void  
xABt_xBAt_plus_yC(double x, const SparseMatrixOMP& A, const SparseMatrixOMP& B, 
		  double y, SparseMatrixOMP& C);

double Dot(const SparseMatrixOMP& A, const SparseMatrixOMP& B);
#endif
}
#endif
/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: SparseSparseOps.h 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
