//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-
#ifndef OHMMS_RANDOMGENERATOR
#define OHMMS_RANDOMGENERATOR
#include <math.h>

#include <vector>
using std::vector;

#ifdef HAVE_CONFIG_H
#include "ohmms-config.h"
#endif

#include "Utilities/OhmmsInfo.h"
#ifdef HAVE_LIBBOOST
#include "Utilities/BoostRandom.h"
typedef BoostRandom<OHMMS_PRECISION> RandomGenerator_t;
#else
#ifdef USE_SPRNG
#include "Utilities/SprngRandom.h"
typedef SprngRandom<0> RandomGenerator_t;
#else
#include "Utilities/RandRandom.h"
typedef RandRandom RandomGenerator_t;
#endif
#endif

//namespace OHMMS {
  class GaussianRandom {

  public:
    typedef RandomGenerator_t::Return_t Return_t; 
    
    GaussianRandom(RandomGenerator_t& rg, Return_t sig=1.0, Return_t c0=0.0): 
      d_engine(rg), newpair(true){ Sigma2 = sig*sig; Center = c0;}  
    
    inline Return_t operator()(){
      if(newpair) {
	d_engine.bivariate(gauss0,gauss1);
	newpair = false;
	return Sigma2*gauss0+Center;
      } else {
	newpair = true;
	return Sigma2*gauss1+Center;
      }
    }
  private:
    RandomGenerator_t& d_engine;
    bool newpair;
    Return_t gauss0, gauss1, Sigma2, Center;
    
  };

  extern RandomGenerator_t Random;
//}

// generic templated class to generate a random vector
template<class VT, class RNG>
struct RandomVector { };


// generic templated class to generate a random sequence
template<class RA, class RNG>
struct RandomSequence { };

// specialized for vector<T> to generate a random sequence of type T
template<class T, class RNG>
struct RandomSequence<vector<T>, RNG> { 

  static void apply(vector<T>& v, RandomVector<T,RNG>& rnd) {
    typename vector<T>::iterator it= v.begin();
    while(it != v.end()) {
      (*it) = rnd(); it++;
    }
  }
};

// specialized for vector<double> to generate a random sequence of type T
template<class RNG>
struct RandomSequence<vector<double>, RNG> {

  static void apply(vector<double>& s, RNG& rnd) {
    typename vector<double>::iterator it= s.begin();
    while(it != s.end()) {
      (*it) = rnd(); it++;
    }
  }
};

#endif

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 312 $   $Date: 2005-03-01 17:06:19 -0600 (Tue, 01 Mar 2005) $
 * $Id: RandomGenerator.h 312 2005-03-01 23:06:19Z jnkim $ 
 ***************************************************************************/
