//////////////////////////////////////////////////////////////////
// (c) Copyright 1998-2002 by Jeongnim Kim
//
//   National Center for Supercomputing Applications &
//   Materials Computation Center
//   University of Illinois, Urbana-Champaign
//   Urbana, IL 61801
//   e-mail: jnkim@ncsa.uiuc.edu
//   Tel:    217-244-6319 (NCSA) 217-333-3324 (MCC)
//
// Supported by 
//   National Center for Supercomputing Applications, UIUC
//   Materials Computation Center, UIUC
//   Department of Physics, Ohio State University
//   Ohio Supercomputer Center
//////////////////////////////////////////////////////////////////
// -*- C++ -*-

#include <sstream>
#include <iosfwd>
using namespace std;
#include "Utilities/MDParameters.h"
#include "Utilities/SimpleParser.h"
#include "Utilities/OhmmsInfo.h"
#include "Message/CommCreate.h"
using namespace OHMMS;

MDParameters::MDParameters() {
  Name = "ohmms";
  SeqID =0;
  MDType = NONE;
  T = 0.0e0;
  E = 0.0e0;
  V = 0.0e0;
  P = 0.0e0;
  N = 0.0e0;
  Mu = 0.0e0;
  Friction = 0.0e0;
  Dt = 1.0e0;
  MDTime = 0.0e0;
  NodeGroupID =0;
  IONode = true;
  SaveStride = 10;

  // assign nodeid determined by Communcate
  Communicate* comm = CommCreate::get();
  NodeID = comm->getNodeID();
}

void MDParameters::set(int argc, char **argv) {
  vector<string> arglist;
  int i=0;
  while(i < argc) {
    arglist.push_back(argv[i++]);
  }
  set(arglist);
}

void MDParameters::set(vector<string>& argv) {
  int i=0;
  int argc = argv.size();
  while(i < argc) {
    if(argv[i] =="temperature") {
      i++;
      T = atof(argv[i].c_str());
      if(i+NodeID<argc) {
        if(isdigit(argv[i+NodeID][0]))  T = atof(argv[i+NodeID].c_str());
      }
      DEBUGMSG("temperautre = " << T << " Kelvin\n")
    } else if(argv[i] == "pressure") {
      P = atof(argv[++i].c_str())/pressure_to_kbar;
      DEBUGMSG("pressure = " << P << " KBar\n")
    } 
    else if(argv[i] =="friction") {
      Friction = atof(argv[++i].c_str());
      DEBUGMSG("friction = " << Friction << " fsec-1\n")
    } else if(argv[i] == "timestep") {
      Dt = atof(argv[++i].c_str());
      DEBUGMSG("timestep = " << Dt << " fsec\n")
    } else if(argv[i] == "mdclock") {
      MDTime = atof(argv[++i].c_str());
      DEBUGMSG("mdclock = " << MDTime << " fsec\n")
    } else if(argv[i] == "stress") {
      Stress(0,0) = atof(argv[++i].c_str());
    } else if(argv[i] == "mdsys") {
      Name = argv[++i];
      if(i+1 < argc && isdigit(argv[i+1][0])) SeqID = atoi(argv[++i].c_str());
    } else {
      if(i+1<argc && !isdigit(argv[i][0])) ExtraVar[argv[i]] = atof(argv[++i].c_str());
    }
    i++;
  }
}

void MDParameters::print(ostream& os) const {
  if(IONode) {
    os.setf(ios::scientific, ios::floatfield);
    os << "mdsys " << Name << " " << SeqID << endl;
    os << "timestep " << Dt << " fsec " << endl;
    os << "pressure " << P*pressure_to_kbar << " kBar " << endl;
    os << "temperature " << T << " kelvin " <<  endl;
    os << "friction " << Friction << " fsec-1 " << endl;
    os << "mdclock " << MDTime << " fsec " << endl;
    os << "stress "  << Stress << endl;
    map<string, Scalar_t>::const_iterator it = ExtraVar.begin();
    while(it != ExtraVar.end()) {
      os << (*it).first << " " << (*it).second << endl;
      it++;
    }
  }
}

void MDParameters::put(const string& s) {
  istringstream script(s);
  string t1;
  while(!script.eof()) {
    script >> t1;
    if(t1.find("timestep") < t1.size()) 
      script >> Dt;
    else if(t1.find("temperature") < t1.size()) 
      script >> T;
    else if(t1.find("pressure") < t1.size()) 
      script >> P;
    else if(t1.find("friction") < t1.size())
      script >> Friction;
    else if(t1.find("mdclock") < t1.size())
      script >> MDTime;
    else if(t1.find("mdsys") < t1.size())
      script >> Name;
    else if(t1.find("stress") < t1.size())
      script >> Stress;
    else if(t1.find("series") < t1.size())
      script >> SeqID;
  }
} 

/***************************************************************************
 * $RCSfile$   $Author: jnkim $
 * $Revision: 151 $   $Date: 2004-11-02 17:51:14 -0600 (Tue, 02 Nov 2004) $
 * $Id: MDParameters.cpp 151 2004-11-02 23:51:14Z jnkim $ 
 ***************************************************************************/
